@@ -5,9 +5,13 @@
 #include "base/debug/alias.h"
 #include "base/logging.h"
 #include "base/process/memory.h"
+#include "build/build_config.h"
 
 namespace base {
 
+// Defined in memory_win.cc for Windows.
+#if !defined(OS_WIN)
+
 namespace {
 
 // Breakpad server classifies base::`anonymous namespace'::OnNoMemory as
@@ -24,6 +28,8 @@ void TerminateBecauseOutOfMemory(size_t size) {
   OnNoMemory(size);
 }
 
+#endif
+
 // Defined in memory_mac.mm for Mac.
 #if !defined(OS_MACOSX)
 
