@@ -6,10 +6,10 @@
 #define BASE_SUPPORTS_USER_DATA_H_
 
 #include <map>
+#include <memory>
 
 #include "base/base_export.h"
 #include "base/macros.h"
-#include "base/memory/linked_ptr.h"
 #include "base/memory/ref_counted.h"
 #include "base/threading/thread_checker.h"
 
@@ -47,7 +47,7 @@ class BASE_EXPORT SupportsUserData {
   virtual ~SupportsUserData();
 
  private:
-  typedef std::map<const void*, linked_ptr<Data> > DataMap;
+  using DataMap = std::map<const void*, std::unique_ptr<Data>>;
 
   // Externally-defined data accessible by key.
   DataMap user_data_;
