@@ -5,6 +5,7 @@
 #include "base/task_scheduler/task_scheduler.h"
 
 #include "base/logging.h"
+#include "base/task_scheduler/task_scheduler_impl.h"
 
 namespace base {
 
@@ -15,6 +16,15 @@ TaskScheduler* g_task_scheduler = nullptr;
 
 }  // namespace
 
+// static
+void TaskScheduler::CreateAndSetDefaultTaskScheduler(
+    const std::vector<SchedulerWorkerPoolParams>& worker_pool_params_vector,
+    const WorkerPoolIndexForTraitsCallback&
+        worker_pool_index_for_traits_callback) {
+  SetInstance(internal::TaskSchedulerImpl::Create(
+      worker_pool_params_vector, worker_pool_index_for_traits_callback));
+}
+
 // static
 void TaskScheduler::SetInstance(std::unique_ptr<TaskScheduler> task_scheduler) {
   delete g_task_scheduler;
