@@ -5,10 +5,11 @@
 #ifndef CC_ANIMATION_TRANSFORM_OPERATIONS_H_
 #define CC_ANIMATION_TRANSFORM_OPERATIONS_H_
 
+#include <memory>
 #include <vector>
 
+#include "base/logging.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "cc/animation/transform_operation.h"
 #include "cc/base/cc_export.h"
 #include "ui/gfx/transform.h"
@@ -58,9 +59,6 @@ class CC_EXPORT TransformOperations {
                            SkMScalar max_progress,
                            gfx::BoxF* bounds) const;
 
-  // Returns true if these operations affect scale.
-  bool AffectsScale() const;
-
   // Returns true if these operations are only translations.
   bool IsTranslation() const;
 
@@ -76,10 +74,10 @@ class CC_EXPORT TransformOperations {
   // fails (this can happen if either matrix cannot be decomposed).
   bool CanBlendWith(const TransformOperations& other) const;
 
-  // If these operations have no more than one scale operation, and if the only
-  // other operations are translations, sets |scale| to the scale component
-  // of these operations. Otherwise, returns false.
-  bool ScaleComponent(gfx::Vector3dF* scale) const;
+  // If none of these operations have a perspective component, sets |scale| to
+  // be the product of the scale component of every operation. Otherwise,
+  // returns false.
+  bool ScaleComponent(SkMScalar* scale) const;
 
   void AppendTranslate(SkMScalar x, SkMScalar y, SkMScalar z);
   void AppendRotate(SkMScalar x, SkMScalar y, SkMScalar z, SkMScalar degrees);
@@ -90,6 +88,13 @@ class CC_EXPORT TransformOperations {
   void AppendIdentity();
   bool IsIdentity() const;
 
+  size_t size() const { return operations_.size(); }
+
+  const TransformOperation& at(size_t index) const {
+    DCHECK_LT(index, size());
+    return operations_[index];
+  }
+
  private:
   bool BlendInternal(const TransformOperations& from,
                      SkMScalar progress,
@@ -100,7 +105,7 @@ class CC_EXPORT TransformOperations {
   bool ComputeDecomposedTransform() const;
 
   // For efficiency, we cache the decomposed transform.
-  mutable scoped_ptr<gfx::DecomposedTransform> decomposed_transform_;
+  mutable std::unique_ptr<gfx::DecomposedTransform> decomposed_transform_;
   mutable bool decomposed_transform_dirty_;
 
   DISALLOW_ASSIGN(TransformOperations);
