@@ -187,10 +187,7 @@ bool TextureLayer::Update() {
   if (client_) {
     TextureMailbox mailbox;
     std::unique_ptr<SingleReleaseCallback> release_callback;
-    if (client_->PrepareTextureMailbox(
-            &mailbox,
-            &release_callback,
-            layer_tree_host()->UsingSharedMemoryResources())) {
+    if (client_->PrepareTextureMailbox(&mailbox, &release_callback)) {
       // Already within a commit, no need to do another one immediately.
       bool requires_commit = false;
       bool allow_mailbox_reuse = false;
@@ -203,7 +200,11 @@ bool TextureLayer::Update() {
   // SetTextureMailbox could be called externally and the same mailbox used for
   // different textures.  Such callers notify this layer that the texture has
   // changed by calling SetNeedsDisplay, so check for that here.
-  return updated || !update_rect_.IsEmpty();
+  return updated || !update_rect().IsEmpty();
+}
+
+bool TextureLayer::IsSnapped() {
+  return true;
 }
 
 void TextureLayer::PushPropertiesTo(LayerImpl* layer) {
@@ -260,8 +261,8 @@ std::unique_ptr<TextureLayer::TextureMailboxHolder::MainThreadReference>
 TextureLayer::TextureMailboxHolder::Create(
     const TextureMailbox& mailbox,
     std::unique_ptr<SingleReleaseCallback> release_callback) {
-  return base::WrapUnique(new MainThreadReference(
-      new TextureMailboxHolder(mailbox, std::move(release_callback))));
+  return base::MakeUnique<MainThreadReference>(
+      new TextureMailboxHolder(mailbox, std::move(release_callback)));
 }
 
 void TextureLayer::TextureMailboxHolder::Return(
