@@ -6,6 +6,7 @@
 
 #include <stdint.h>
 
+#include "base/bind.h"
 #include "base/memory/ptr_util.h"
 #include "base/run_loop.h"
 #include "cc/output/compositor_frame_metadata.h"
@@ -26,24 +27,26 @@
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/skia/include/core/SkCanvas.h"
+#include "ui/gfx/skia_util.h"
 
 namespace cc {
 namespace {
 
-class SoftwareRendererTest : public testing::Test, public RendererClient {
+class SoftwareRendererTest : public testing::Test {
  public:
   void InitializeRenderer(
       std::unique_ptr<SoftwareOutputDevice> software_output_device) {
     output_surface_ =
         FakeOutputSurface::CreateSoftware(std::move(software_output_device));
-    CHECK(output_surface_->BindToClient(&output_surface_client_));
+    output_surface_->BindToClient(&output_surface_client_);
 
     shared_bitmap_manager_.reset(new TestSharedBitmapManager());
-    resource_provider_ = FakeResourceProvider::Create(
-        output_surface_.get(), shared_bitmap_manager_.get());
-    renderer_ = SoftwareRenderer::Create(
-        this, &settings_, output_surface_.get(), resource_provider(),
-        true /* use_image_hijack_canvas */);
+    resource_provider_ =
+        FakeResourceProvider::Create(nullptr, shared_bitmap_manager_.get());
+    renderer_ = base::MakeUnique<SoftwareRenderer>(
+        &settings_, output_surface_.get(), resource_provider());
+    renderer_->Initialize();
+    renderer_->SetVisible(true);
   }
 
   ResourceProvider* resource_provider() const {
@@ -52,12 +55,9 @@ class SoftwareRendererTest : public testing::Test, public RendererClient {
 
   SoftwareRenderer* renderer() const { return renderer_.get(); }
 
-  // RendererClient implementation.
-  void SetFullRootLayerDamage() override {}
-
   std::unique_ptr<SkBitmap> DrawAndCopyOutput(RenderPassList* list,
                                               float device_scale_factor,
-                                              gfx::Rect device_viewport_rect) {
+                                              gfx::Size viewport_size) {
     std::unique_ptr<SkBitmap> bitmap_result;
     base::RunLoop loop;
 
@@ -68,7 +68,7 @@ class SoftwareRendererTest : public testing::Test, public RendererClient {
                        loop.QuitClosure())));
 
     renderer()->DrawFrame(list, device_scale_factor, gfx::ColorSpace(),
-                          device_viewport_rect, device_viewport_rect, false);
+                          viewport_size);
     loop.Run();
     return bitmap_result;
   }
@@ -127,9 +127,8 @@ TEST_F(SoftwareRendererTest, SolidColorQuad) {
   list.push_back(std::move(root_render_pass));
 
   float device_scale_factor = 1.f;
-  gfx::Rect device_viewport_rect(outer_size);
   std::unique_ptr<SkBitmap> output =
-      DrawAndCopyOutput(&list, device_scale_factor, device_viewport_rect);
+      DrawAndCopyOutput(&list, device_scale_factor, outer_size);
   EXPECT_EQ(outer_rect.width(), output->info().width());
   EXPECT_EQ(outer_rect.height(), output->info().height());
 
@@ -150,9 +149,11 @@ TEST_F(SoftwareRendererTest, TileQuad) {
   InitializeRenderer(base::WrapUnique(new SoftwareOutputDevice));
 
   ResourceId resource_yellow = resource_provider()->CreateResource(
-      outer_size, ResourceProvider::TEXTURE_HINT_IMMUTABLE, RGBA_8888);
+      outer_size, ResourceProvider::TEXTURE_HINT_IMMUTABLE, RGBA_8888,
+      gfx::ColorSpace());
   ResourceId resource_cyan = resource_provider()->CreateResource(
-      inner_size, ResourceProvider::TEXTURE_HINT_IMMUTABLE, RGBA_8888);
+      inner_size, ResourceProvider::TEXTURE_HINT_IMMUTABLE, RGBA_8888,
+      gfx::ColorSpace());
 
   SkBitmap yellow_tile;
   yellow_tile.allocN32Pixels(outer_size.width(), outer_size.height());
@@ -199,9 +200,8 @@ TEST_F(SoftwareRendererTest, TileQuad) {
   list.push_back(std::move(root_render_pass));
 
   float device_scale_factor = 1.f;
-  gfx::Rect device_viewport_rect(outer_size);
   std::unique_ptr<SkBitmap> output =
-      DrawAndCopyOutput(&list, device_scale_factor, device_viewport_rect);
+      DrawAndCopyOutput(&list, device_scale_factor, outer_size);
   EXPECT_EQ(outer_rect.width(), output->info().width());
   EXPECT_EQ(outer_rect.height(), output->info().height());
 
@@ -221,7 +221,8 @@ TEST_F(SoftwareRendererTest, TileQuadVisibleRect) {
   InitializeRenderer(base::WrapUnique(new SoftwareOutputDevice));
 
   ResourceId resource_cyan = resource_provider()->CreateResource(
-      tile_size, ResourceProvider::TEXTURE_HINT_IMMUTABLE, RGBA_8888);
+      tile_size, ResourceProvider::TEXTURE_HINT_IMMUTABLE, RGBA_8888,
+      gfx::ColorSpace());
 
   SkBitmap cyan_tile;  // The lowest five rows are yellow.
   cyan_tile.allocN32Pixels(tile_size.width(), tile_size.height());
@@ -261,9 +262,8 @@ TEST_F(SoftwareRendererTest, TileQuadVisibleRect) {
   list.push_back(std::move(root_render_pass));
 
   float device_scale_factor = 1.f;
-  gfx::Rect device_viewport_rect(tile_size);
   std::unique_ptr<SkBitmap> output =
-      DrawAndCopyOutput(&list, device_scale_factor, device_viewport_rect);
+      DrawAndCopyOutput(&list, device_scale_factor, tile_size);
   EXPECT_EQ(tile_rect.width(), output->info().width());
   EXPECT_EQ(tile_rect.height(), output->info().height());
 
@@ -289,7 +289,7 @@ TEST_F(SoftwareRendererTest, TileQuadVisibleRect) {
 
 TEST_F(SoftwareRendererTest, ShouldClearRootRenderPass) {
   float device_scale_factor = 1.f;
-  gfx::Rect device_viewport_rect(0, 0, 100, 100);
+  gfx::Size viewport_size(100, 100);
 
   settings_.should_clear_root_render_pass = false;
   InitializeRenderer(base::WrapUnique(new SoftwareOutputDevice));
@@ -299,20 +299,19 @@ TEST_F(SoftwareRendererTest, ShouldClearRootRenderPass) {
   // Draw a fullscreen green quad in a first frame.
   RenderPassId root_clear_pass_id(1, 0);
   RenderPass* root_clear_pass = AddRenderPass(
-      &list, root_clear_pass_id, device_viewport_rect, gfx::Transform());
-  AddQuad(root_clear_pass, device_viewport_rect, SK_ColorGREEN);
+      &list, root_clear_pass_id, gfx::Rect(viewport_size), gfx::Transform());
+  AddQuad(root_clear_pass, gfx::Rect(viewport_size), SK_ColorGREEN);
 
   renderer()->DecideRenderPassAllocationsForFrame(list);
 
   std::unique_ptr<SkBitmap> output =
-      DrawAndCopyOutput(&list, device_scale_factor, device_viewport_rect);
-  EXPECT_EQ(device_viewport_rect.width(), output->info().width());
-  EXPECT_EQ(device_viewport_rect.height(), output->info().height());
+      DrawAndCopyOutput(&list, device_scale_factor, viewport_size);
+  EXPECT_EQ(viewport_size.width(), output->info().width());
+  EXPECT_EQ(viewport_size.height(), output->info().height());
 
   EXPECT_EQ(SK_ColorGREEN, output->getColor(0, 0));
-  EXPECT_EQ(SK_ColorGREEN,
-            output->getColor(device_viewport_rect.width() - 1,
-                             device_viewport_rect.height() - 1));
+  EXPECT_EQ(SK_ColorGREEN, output->getColor(viewport_size.width() - 1,
+                                            viewport_size.height() - 1));
 
   list.clear();
 
@@ -322,20 +321,19 @@ TEST_F(SoftwareRendererTest, ShouldClearRootRenderPass) {
 
   RenderPassId root_smaller_pass_id(2, 0);
   RenderPass* root_smaller_pass = AddRenderPass(
-      &list, root_smaller_pass_id, device_viewport_rect, gfx::Transform());
+      &list, root_smaller_pass_id, gfx::Rect(viewport_size), gfx::Transform());
   AddQuad(root_smaller_pass, smaller_rect, SK_ColorMAGENTA);
 
   renderer()->DecideRenderPassAllocationsForFrame(list);
 
-  output = DrawAndCopyOutput(&list, device_scale_factor, device_viewport_rect);
-  EXPECT_EQ(device_viewport_rect.width(), output->info().width());
-  EXPECT_EQ(device_viewport_rect.height(), output->info().height());
+  output = DrawAndCopyOutput(&list, device_scale_factor, viewport_size);
+  EXPECT_EQ(viewport_size.width(), output->info().width());
+  EXPECT_EQ(viewport_size.height(), output->info().height());
 
   // If we didn't clear, the borders should still be green.
   EXPECT_EQ(SK_ColorGREEN, output->getColor(0, 0));
-  EXPECT_EQ(SK_ColorGREEN,
-            output->getColor(device_viewport_rect.width() - 1,
-                             device_viewport_rect.height() - 1));
+  EXPECT_EQ(SK_ColorGREEN, output->getColor(viewport_size.width() - 1,
+                                            viewport_size.height() - 1));
 
   EXPECT_EQ(SK_ColorMAGENTA,
             output->getColor(smaller_rect.x(), smaller_rect.y()));
@@ -346,7 +344,7 @@ TEST_F(SoftwareRendererTest, ShouldClearRootRenderPass) {
 
 TEST_F(SoftwareRendererTest, RenderPassVisibleRect) {
   float device_scale_factor = 1.f;
-  gfx::Rect device_viewport_rect(0, 0, 100, 100);
+  gfx::Size viewport_size(100, 100);
   InitializeRenderer(base::WrapUnique(new SoftwareOutputDevice));
 
   RenderPassList list;
@@ -361,9 +359,9 @@ TEST_F(SoftwareRendererTest, RenderPassVisibleRect) {
   // Root pass is green.
   RenderPassId root_clear_pass_id(1, 0);
   RenderPass* root_clear_pass = AddRenderPass(
-      &list, root_clear_pass_id, device_viewport_rect, gfx::Transform());
+      &list, root_clear_pass_id, gfx::Rect(viewport_size), gfx::Transform());
   AddRenderPassQuad(root_clear_pass, smaller_pass);
-  AddQuad(root_clear_pass, device_viewport_rect, SK_ColorGREEN);
+  AddQuad(root_clear_pass, gfx::Rect(viewport_size), SK_ColorGREEN);
 
   // Interior pass quad has smaller visible rect.
   gfx::Rect interior_visible_rect(30, 30, 40, 40);
@@ -372,14 +370,13 @@ TEST_F(SoftwareRendererTest, RenderPassVisibleRect) {
   renderer()->DecideRenderPassAllocationsForFrame(list);
 
   std::unique_ptr<SkBitmap> output =
-      DrawAndCopyOutput(&list, device_scale_factor, device_viewport_rect);
-  EXPECT_EQ(device_viewport_rect.width(), output->info().width());
-  EXPECT_EQ(device_viewport_rect.height(), output->info().height());
+      DrawAndCopyOutput(&list, device_scale_factor, viewport_size);
+  EXPECT_EQ(viewport_size.width(), output->info().width());
+  EXPECT_EQ(viewport_size.height(), output->info().height());
 
   EXPECT_EQ(SK_ColorGREEN, output->getColor(0, 0));
-  EXPECT_EQ(SK_ColorGREEN,
-            output->getColor(device_viewport_rect.width() - 1,
-                             device_viewport_rect.height() - 1));
+  EXPECT_EQ(SK_ColorGREEN, output->getColor(viewport_size.width() - 1,
+                                            viewport_size.height() - 1));
 
   // Part outside visible rect should remain green.
   EXPECT_EQ(SK_ColorGREEN,
@@ -396,5 +393,60 @@ TEST_F(SoftwareRendererTest, RenderPassVisibleRect) {
                              interior_visible_rect.bottom() - 1));
 }
 
+class PartialSwapSoftwareOutputDevice : public SoftwareOutputDevice {
+ public:
+  // SoftwareOutputDevice overrides.
+  SkCanvas* BeginPaint(const gfx::Rect& damage_rect) override {
+    damage_rect_at_start_ = damage_rect;
+    canvas_ = SoftwareOutputDevice::BeginPaint(damage_rect);
+    return canvas_;
+  }
+  void EndPaint() override {
+    SkIRect clip_device_bounds;
+    canvas_->getClipDeviceBounds(&clip_device_bounds);
+    clip_rect_at_end_ = gfx::SkIRectToRect(clip_device_bounds);
+    SoftwareOutputDevice::EndPaint();
+  }
+
+  gfx::Rect damage_rect_at_start() const { return damage_rect_at_start_; }
+  gfx::Rect clip_rect_at_end() const { return clip_rect_at_end_; }
+
+ private:
+  SkCanvas* canvas_ = nullptr;
+  gfx::Rect damage_rect_at_start_;
+  gfx::Rect clip_rect_at_end_;
+};
+
+TEST_F(SoftwareRendererTest, PartialSwap) {
+  float device_scale_factor = 1.f;
+  gfx::Size viewport_size(100, 100);
+
+  settings_.partial_swap_enabled = true;
+
+  auto device_owned = base::MakeUnique<PartialSwapSoftwareOutputDevice>();
+  auto* device = device_owned.get();
+  InitializeRenderer(std::move(device_owned));
+
+  RenderPassList list;
+
+  RenderPassId root_pass_id(1, 0);
+  RenderPass* root_pass = AddRenderPass(
+      &list, root_pass_id, gfx::Rect(viewport_size), gfx::Transform());
+  AddQuad(root_pass, gfx::Rect(viewport_size), SK_ColorGREEN);
+
+  // Partial frame, we should pass this rect to the SoftwareOutputDevice.
+  // partial swap is enabled.
+  root_pass->damage_rect = gfx::Rect(2, 2, 3, 3);
+
+  renderer()->DecideRenderPassAllocationsForFrame(list);
+  renderer()->DrawFrame(&list, device_scale_factor, gfx::ColorSpace(),
+                        viewport_size);
+
+  // The damage rect should be reported to the SoftwareOutputDevice.
+  EXPECT_EQ(gfx::Rect(2, 2, 3, 3), device->damage_rect_at_start());
+  // The SkCanvas should be clipped to the damage rect.
+  EXPECT_EQ(gfx::Rect(2, 2, 3, 3), device->clip_rect_at_end());
+}
+
 }  // namespace
 }  // namespace cc
