@@ -60,8 +60,8 @@ class MaskContentLayerClient : public ContentLayerClient {
     }
 
     scoped_refptr<DisplayItemList> display_list =
-        DisplayItemList::Create(PaintableRegion(), DisplayItemListSettings());
-    display_list->CreateAndAppendItem<DrawingDisplayItem>(
+        DisplayItemList::Create(DisplayItemListSettings());
+    display_list->CreateAndAppendDrawingItem<DrawingDisplayItem>(
         PaintableRegion(), recorder.finishRecordingAsPicture());
 
     display_list->Finalize();
@@ -150,184 +150,6 @@ TEST_P(LayerTreeHostMasksPixelTest, MaskOfClippedLayer) {
       base::FilePath(FILE_PATH_LITERAL("mask_of_clipped_layer.png")));
 }
 
-TEST_P(LayerTreeHostMasksPixelTest, MaskWithReplica) {
-  scoped_refptr<SolidColorLayer> background = CreateSolidColorLayer(
-      gfx::Rect(100, 100), SK_ColorWHITE);
-
-  gfx::Size mask_bounds(50, 50);
-  MaskContentLayerClient client(mask_bounds);
-  scoped_refptr<PictureLayer> mask = PictureLayer::Create(&client);
-  mask->SetBounds(mask_bounds);
-  mask->SetIsDrawable(true);
-  mask->SetIsMask(true);
-
-  scoped_refptr<SolidColorLayer> green = CreateSolidColorLayerWithBorder(
-      gfx::Rect(0, 0, 50, 50), kCSSGreen, 1, SK_ColorBLACK);
-  background->AddChild(green);
-  green->SetMaskLayer(mask.get());
-
-  gfx::Transform replica_transform;
-  replica_transform.Rotate(-90.0);
-
-  scoped_refptr<Layer> replica = Layer::Create();
-  replica->SetTransformOrigin(gfx::Point3F(25.f, 25.f, 0.f));
-  replica->SetPosition(gfx::PointF(50.f, 50.f));
-  replica->SetTransform(replica_transform);
-  green->SetReplicaLayer(replica.get());
-
-  RunPixelResourceTest(
-      background, base::FilePath(FILE_PATH_LITERAL("mask_with_replica.png")));
-}
-
-// Test collapsing of render passes containing a single tile quad and
-// drawing them directly with masks and replicas.
-TEST_P(LayerTreeHostMasksPixelTest, ContentLayerMaskWithReplica) {
-  scoped_refptr<SolidColorLayer> background =
-      CreateSolidColorLayer(gfx::Rect(100, 100), SK_ColorWHITE);
-
-  gfx::Size mask_bounds(50, 50);
-  MaskContentLayerClient client(mask_bounds);
-  scoped_refptr<PictureLayer> mask = PictureLayer::Create(&client);
-  mask->SetBounds(mask_bounds);
-  mask->SetIsDrawable(true);
-  mask->SetIsMask(true);
-
-  // Use a border to defeat solid color detection to force a tile quad.
-  SolidColorContentLayerClient green_client(kCSSGreen, mask_bounds, 1,
-                                            SK_ColorBLACK);
-  scoped_refptr<PictureLayer> green = PictureLayer::Create(&green_client);
-  green->SetBounds(mask_bounds);
-  green->SetIsDrawable(true);
-  background->AddChild(green);
-  green->SetMaskLayer(mask.get());
-
-  gfx::Transform replica_transform;
-  replica_transform.Rotate(-90.0);
-
-  scoped_refptr<Layer> replica = Layer::Create();
-  replica->SetTransformOrigin(gfx::Point3F(25.f, 25.f, 0.f));
-  replica->SetPosition(gfx::PointF(50.f, 50.f));
-  replica->SetTransform(replica_transform);
-  green->SetReplicaLayer(replica.get());
-
-  RunPixelResourceTest(
-      background, base::FilePath(FILE_PATH_LITERAL("mask_with_replica.png")));
-}
-
-TEST_P(LayerTreeHostMasksPixelTest, MaskWithReplicaOfClippedLayer) {
-  scoped_refptr<SolidColorLayer> background = CreateSolidColorLayer(
-      gfx::Rect(100, 100), SK_ColorWHITE);
-
-  gfx::Size mask_bounds(50, 50);
-  MaskContentLayerClient client(mask_bounds);
-  scoped_refptr<PictureLayer> mask = PictureLayer::Create(&client);
-  mask->SetBounds(mask_bounds);
-  mask->SetIsDrawable(true);
-  mask->SetIsMask(true);
-
-  // Clip to the bottom half of the green layer, and the left half of the
-  // replica.
-  scoped_refptr<Layer> clip = Layer::Create();
-  clip->SetPosition(gfx::PointF(0.f, 25.f));
-  clip->SetBounds(gfx::Size(75, 75));
-  clip->SetMasksToBounds(true);
-  background->AddChild(clip);
-
-  scoped_refptr<SolidColorLayer> green = CreateSolidColorLayerWithBorder(
-      gfx::Rect(0, -25, 50, 50), kCSSGreen, 1, SK_ColorBLACK);
-  clip->AddChild(green);
-  green->SetMaskLayer(mask.get());
-
-  gfx::Transform replica_transform;
-  replica_transform.Rotate(-90.0);
-
-  scoped_refptr<Layer> replica = Layer::Create();
-  replica->SetTransformOrigin(gfx::Point3F(25.f, 25.f, 0.f));
-  replica->SetPosition(gfx::PointF(50.f, 50.f));
-  replica->SetTransform(replica_transform);
-  green->SetReplicaLayer(replica.get());
-
-  RunPixelResourceTest(background,
-                       base::FilePath(FILE_PATH_LITERAL(
-                           "mask_with_replica_of_clipped_layer.png")));
-}
-
-TEST_P(LayerTreeHostMasksPixelTest, MaskOfReplica) {
-  scoped_refptr<SolidColorLayer> background = CreateSolidColorLayer(
-      gfx::Rect(100, 100), SK_ColorWHITE);
-
-  gfx::Size mask_bounds(50, 50);
-  MaskContentLayerClient client(mask_bounds);
-  scoped_refptr<PictureLayer> mask = PictureLayer::Create(&client);
-  mask->SetBounds(mask_bounds);
-  mask->SetIsDrawable(true);
-  mask->SetIsMask(true);
-
-  scoped_refptr<SolidColorLayer> green = CreateSolidColorLayerWithBorder(
-      gfx::Rect(25, 0, 50, 50), kCSSGreen, 1, SK_ColorBLACK);
-  background->AddChild(green);
-
-  scoped_refptr<SolidColorLayer> orange = CreateSolidColorLayer(
-      gfx::Rect(-25, 25, 25, 25), kCSSOrange);
-  green->AddChild(orange);
-
-  gfx::Transform replica_transform;
-  replica_transform.Rotate(180.0);
-  replica_transform.Translate(50.0, 0.0);
-
-  scoped_refptr<Layer> replica = Layer::Create();
-  replica->SetTransformOrigin(gfx::Point3F(50.f, 50.f, 0.f));
-  replica->SetPosition(gfx::PointF());
-  replica->SetTransform(replica_transform);
-  replica->SetMaskLayer(mask.get());
-  green->SetReplicaLayer(replica.get());
-
-  RunPixelResourceTest(
-      background, base::FilePath(FILE_PATH_LITERAL("mask_of_replica.png")));
-}
-
-TEST_P(LayerTreeHostMasksPixelTest, MaskOfReplicaOfClippedLayer) {
-  scoped_refptr<SolidColorLayer> background = CreateSolidColorLayer(
-      gfx::Rect(100, 100), SK_ColorWHITE);
-
-  gfx::Size mask_bounds(50, 50);
-  MaskContentLayerClient client(mask_bounds);
-  scoped_refptr<PictureLayer> mask = PictureLayer::Create(&client);
-  mask->SetBounds(mask_bounds);
-  mask->SetIsDrawable(true);
-  mask->SetIsMask(true);
-
-  // Clip to the bottom 3/4 of the green layer, and the top 3/4 of the replica.
-  scoped_refptr<Layer> clip = Layer::Create();
-  clip->SetPosition(gfx::PointF(0.f, 12.f));
-  clip->SetBounds(gfx::Size(100, 75));
-  clip->SetMasksToBounds(true);
-  background->AddChild(clip);
-
-  scoped_refptr<SolidColorLayer> green = CreateSolidColorLayerWithBorder(
-      gfx::Rect(25, -12, 50, 50), kCSSGreen, 1, SK_ColorBLACK);
-  clip->AddChild(green);
-
-  scoped_refptr<SolidColorLayer> orange = CreateSolidColorLayer(
-      gfx::Rect(-25, 25, 25, 25), kCSSOrange);
-  green->AddChild(orange);
-
-  gfx::Transform replica_transform;
-  replica_transform.Rotate(180.0);
-  replica_transform.Translate(50.0, 0.0);
-
-  scoped_refptr<Layer> replica = Layer::Create();
-  replica->SetTransformOrigin(gfx::Point3F(50.f, 50.f, 0.f));
-  replica->SetPosition(gfx::PointF());
-  replica->SetTransform(replica_transform);
-  replica->SetMaskLayer(mask.get());
-  green->SetReplicaLayer(replica.get());
-
-  RunPixelResourceTest(background,
-                       base::FilePath(FILE_PATH_LITERAL(
-                           "mask_of_replica_of_clipped_layer.png")));
-}
-
 class CheckerContentLayerClient : public ContentLayerClient {
  public:
   CheckerContentLayerClient(const gfx::Size& bounds,
@@ -360,8 +182,8 @@ class CheckerContentLayerClient : public ContentLayerClient {
     }
 
     scoped_refptr<DisplayItemList> display_list =
-        DisplayItemList::Create(PaintableRegion(), DisplayItemListSettings());
-    display_list->CreateAndAppendItem<DrawingDisplayItem>(
+        DisplayItemList::Create(DisplayItemListSettings());
+    display_list->CreateAndAppendDrawingItem<DrawingDisplayItem>(
         PaintableRegion(), recorder.finishRecordingAsPicture());
 
     display_list->Finalize();
@@ -398,8 +220,8 @@ class CircleContentLayerClient : public ContentLayerClient {
                        paint);
 
     scoped_refptr<DisplayItemList> display_list =
-        DisplayItemList::Create(PaintableRegion(), DisplayItemListSettings());
-    display_list->CreateAndAppendItem<DrawingDisplayItem>(
+        DisplayItemList::Create(DisplayItemListSettings());
+    display_list->CreateAndAppendDrawingItem<DrawingDisplayItem>(
         PaintableRegion(), recorder.finishRecordingAsPicture());
 
     display_list->Finalize();
