@@ -145,9 +145,9 @@ void DeclarativeContentApiTest::CheckIncognito(IncognitoMode mode,
   ExtensionTestMessageListener ready("ready", false);
   ExtensionTestMessageListener ready_incognito("ready (split)", false);
 
-  const Extension* extension = is_enabled_in_incognito ?
-      LoadExtensionIncognito(ext_dir_.unpacked_path()) :
-      LoadExtension(ext_dir_.unpacked_path());
+  const Extension* extension =
+      is_enabled_in_incognito ? LoadExtensionIncognito(ext_dir_.UnpackedPath())
+                              : LoadExtension(ext_dir_.UnpackedPath());
   ASSERT_TRUE(extension);
 
   Browser* incognito_browser = CreateIncognitoBrowser();
@@ -207,7 +207,7 @@ void DeclarativeContentApiTest::CheckBookmarkEvents(bool match_is_bookmarked) {
       browser()->tab_strip_model()->GetWebContentsAt(0);
   const int tab_id = ExtensionTabUtil::GetTabId(tab);
 
-  const Extension* extension = LoadExtension(ext_dir_.unpacked_path());
+  const Extension* extension = LoadExtension(ext_dir_.UnpackedPath());
   ASSERT_TRUE(extension);
   const ExtensionAction* page_action = ExtensionActionManager::Get(
       browser()->profile())->GetPageAction(*extension);
@@ -230,7 +230,7 @@ void DeclarativeContentApiTest::CheckBookmarkEvents(bool match_is_bookmarked) {
 
   // Check rule evaluation on add/remove bookmark.
   bookmarks::BookmarkModel* bookmark_model =
-      BookmarkModelFactory::GetForProfile(browser()->profile());
+      BookmarkModelFactory::GetForBrowserContext(browser()->profile());
   const bookmarks::BookmarkNode* node =
       bookmark_model->AddURL(bookmark_model->other_node(), 0,
                              base::ASCIIToUTF16("title"),
@@ -278,7 +278,7 @@ IN_PROC_BROWSER_TEST_F(DeclarativeContentApiTest, DISABLED_Overview) {
       "  });\n"
       "});\n");
   ExtensionTestMessageListener ready("ready", false);
-  const Extension* extension = LoadExtension(ext_dir_.unpacked_path());
+  const Extension* extension = LoadExtension(ext_dir_.UnpackedPath());
   ASSERT_TRUE(extension);
   const ExtensionAction* page_action =
       ExtensionActionManager::Get(browser()->profile())->
@@ -358,7 +358,7 @@ IN_PROC_BROWSER_TEST_F(DeclarativeContentApiTest, ReusedActionInstance) {
       "  });\n"
       "});\n");
   ExtensionTestMessageListener ready("ready", false);
-  const Extension* extension = LoadExtension(ext_dir_.unpacked_path());
+  const Extension* extension = LoadExtension(ext_dir_.UnpackedPath());
   ASSERT_TRUE(extension);
   const ExtensionAction* page_action =
       ExtensionActionManager::Get(browser()->profile())
@@ -380,7 +380,7 @@ IN_PROC_BROWSER_TEST_F(DeclarativeContentApiTest, ReusedActionInstance) {
 IN_PROC_BROWSER_TEST_F(DeclarativeContentApiTest, RulesEvaluatedOnAddRemove) {
   ext_dir_.WriteManifest(kDeclarativeContentManifest);
   ext_dir_.WriteFile(FILE_PATH_LITERAL("background.js"), kBackgroundHelpers);
-  const Extension* extension = LoadExtension(ext_dir_.unpacked_path());
+  const Extension* extension = LoadExtension(ext_dir_.UnpackedPath());
   ASSERT_TRUE(extension);
   const ExtensionAction* page_action =
       ExtensionActionManager::Get(browser()->profile())->
@@ -450,7 +450,7 @@ IN_PROC_BROWSER_TEST_F(DeclarativeContentApiTest, RulesAddedFromManifest) {
       "  }]\n"
       "}\n";
   ext_dir_.WriteManifest(manifest);
-  const Extension* extension = LoadExtension(ext_dir_.unpacked_path());
+  const Extension* extension = LoadExtension(ext_dir_.UnpackedPath());
   ASSERT_TRUE(extension);
   const ExtensionAction* page_action =
       ExtensionActionManager::Get(browser()->profile())
@@ -509,7 +509,7 @@ IN_PROC_BROWSER_TEST_F(DeclarativeContentApiTest,
   ext_dir_.WriteFile(FILE_PATH_LITERAL("background.js"),
                      kIncognitoSpecificBackground);
   ExtensionTestMessageListener ready("ready", false);
-  const Extension* extension = LoadExtensionIncognito(ext_dir_.unpacked_path());
+  const Extension* extension = LoadExtensionIncognito(ext_dir_.UnpackedPath());
   ASSERT_TRUE(extension);
   ASSERT_TRUE(ready.WaitUntilSatisfied());
 
@@ -599,7 +599,7 @@ IN_PROC_BROWSER_TEST_F(DeclarativeContentApiTest,
                        MAYBE_UninstallWhileActivePageAction) {
   ext_dir_.WriteManifest(kDeclarativeContentManifest);
   ext_dir_.WriteFile(FILE_PATH_LITERAL("background.js"), kBackgroundHelpers);
-  const Extension* extension = LoadExtension(ext_dir_.unpacked_path());
+  const Extension* extension = LoadExtension(ext_dir_.UnpackedPath());
   ASSERT_TRUE(extension);
   const std::string extension_id = extension->id();
   const ExtensionAction* page_action = ExtensionActionManager::Get(
@@ -658,7 +658,7 @@ IN_PROC_BROWSER_TEST_F(DeclarativeContentApiTest,
   ASSERT_TRUE(content::ExecuteScript(
       tab, "document.body.innerHTML = '<span class=\"foo\">';"));
 
-  const Extension* extension = LoadExtension(ext_dir_.unpacked_path());
+  const Extension* extension = LoadExtension(ext_dir_.UnpackedPath());
   ASSERT_TRUE(extension);
   const ExtensionAction* page_action = ExtensionActionManager::Get(
       browser()->profile())->GetPageAction(*extension);
@@ -716,7 +716,7 @@ class ShowPageActionWithoutPageActionTest : public DeclarativeContentApiTest {
     ext_dir_.WriteManifest(manifest_without_page_action);
     ext_dir_.WriteFile(FILE_PATH_LITERAL("background.js"), kBackgroundHelpers);
 
-    const Extension* extension = LoadExtension(ext_dir_.unpacked_path());
+    const Extension* extension = LoadExtension(ext_dir_.UnpackedPath());
     ASSERT_TRUE(extension);
 
     const char kScript[] =
@@ -749,20 +749,12 @@ class ShowPageActionWithoutPageActionTest : public DeclarativeContentApiTest {
  private:
   void SetUpCommandLine(base::CommandLine* command_line) override {
     DeclarativeContentApiTest::SetUpCommandLine(command_line);
-    // If disabling the redesign, we need to disable Media Router since having
-    // Media Router enabled will result in auto-enabling the redesign and
-    // breaking the test.
-    if (!enable_redesign_) {
-      override_media_router_.reset(new FeatureSwitch::ScopedOverride(
-          FeatureSwitch::media_router(), false));
-    }
     override_toolbar_redesign_.reset(new FeatureSwitch::ScopedOverride(
         FeatureSwitch::extension_action_redesign(), enable_redesign_));
   }
 
   bool enable_redesign_;
   std::unique_ptr<FeatureSwitch::ScopedOverride> override_toolbar_redesign_;
-  std::unique_ptr<FeatureSwitch::ScopedOverride> override_media_router_;
   DISALLOW_COPY_AND_ASSIGN(ShowPageActionWithoutPageActionTest);
 };
 
@@ -799,7 +791,7 @@ IN_PROC_BROWSER_TEST_F(DeclarativeContentApiTest,
       "function Return(obj) {\n"
       "  window.domAutomationController.send('' + obj);\n"
       "}\n");
-  const Extension* extension = LoadExtension(ext_dir_.unpacked_path());
+  const Extension* extension = LoadExtension(ext_dir_.UnpackedPath());
   ASSERT_TRUE(extension);
 
   EXPECT_EQ("input[type=\"password\"]",
@@ -876,7 +868,7 @@ IN_PROC_BROWSER_TEST_F(DeclarativeContentApiTest,
   // trigger OnExtensionLoaded.
   ext_dir_.WriteManifest(kDeclarativeContentManifest);
   ext_dir_.WriteFile(FILE_PATH_LITERAL("background.js"), kBackgroundHelpers);
-  ASSERT_TRUE(LoadExtension(ext_dir_.unpacked_path()));
+  ASSERT_TRUE(LoadExtension(ext_dir_.UnpackedPath()));
 }
 
 // https://crbug.com/501225
@@ -884,7 +876,7 @@ IN_PROC_BROWSER_TEST_F(DeclarativeContentApiTest,
                        PendingWebContentsClearedOnRemoveRules) {
   ext_dir_.WriteManifest(kDeclarativeContentManifest);
   ext_dir_.WriteFile(FILE_PATH_LITERAL("background.js"), kBackgroundHelpers);
-  const Extension* extension = LoadExtension(ext_dir_.unpacked_path());
+  const Extension* extension = LoadExtension(ext_dir_.UnpackedPath());
   ASSERT_TRUE(extension);
   const ExtensionAction* page_action = ExtensionActionManager::Get(
       browser()->profile())->GetPageAction(*extension);
@@ -943,7 +935,7 @@ IN_PROC_BROWSER_TEST_F(DeclarativeContentApiTest,
   ext_dir_.WriteFile(FILE_PATH_LITERAL("background.js"), kBackgroundHelpers);
 
   // Load the extension, add a rule, then uninstall the extension.
-  const Extension* extension = LoadExtension(ext_dir_.unpacked_path());
+  const Extension* extension = LoadExtension(ext_dir_.UnpackedPath());
   ASSERT_TRUE(extension);
 
   const std::string kAddTestRule =
@@ -967,7 +959,7 @@ IN_PROC_BROWSER_TEST_F(DeclarativeContentApiTest,
   ASSERT_EQ(base::ASCIIToUTF16(""), error);
 
   // Reload the extension, then add and remove a rule.
-  extension = LoadExtension(ext_dir_.unpacked_path());
+  extension = LoadExtension(ext_dir_.UnpackedPath());
   ASSERT_TRUE(extension);
 
   EXPECT_EQ("add_rule",
