@@ -19,7 +19,7 @@
 #include "chrome/browser/chromeos/login/users/mock_user_manager.h"
 #include "chrome/browser/chromeos/login/users/scoped_user_manager_enabler.h"
 #include "chrome/browser/chromeos/policy/browser_policy_connector_chromeos.h"
-#include "chrome/browser/chromeos/policy/stub_enterprise_install_attributes.h"
+#include "chrome/browser/chromeos/settings/stub_install_attributes.h"
 #include "extensions/common/extension_builder.h"
 #endif
 #include "chrome/browser/extensions/api/identity/identity_api.h"
@@ -78,45 +78,6 @@ namespace utils = extension_function_test_utils;
 static const char kAccessToken[] = "auth_token";
 static const char kExtensionId[] = "ext_id";
 
-// This helps us be able to wait until an UIThreadExtensionFunction calls
-// SendResponse.
-class SendResponseDelegate
-    : public UIThreadExtensionFunction::DelegateForTests {
- public:
-  SendResponseDelegate() : should_post_quit_(false) {}
-
-  virtual ~SendResponseDelegate() {}
-
-  void set_should_post_quit(bool should_quit) {
-    should_post_quit_ = should_quit;
-  }
-
-  bool HasResponse() {
-    return response_.get() != NULL;
-  }
-
-  bool GetResponse() {
-    EXPECT_TRUE(HasResponse());
-    return *response_.get();
-  }
-
-  void OnSendResponse(UIThreadExtensionFunction* function,
-                      bool success,
-                      bool bad_message) override {
-    ASSERT_FALSE(bad_message);
-    ASSERT_FALSE(HasResponse());
-    response_.reset(new bool);
-    *response_ = success;
-    if (should_post_quit_) {
-      base::MessageLoopForUI::current()->QuitWhenIdle();
-    }
-  }
-
- private:
-  std::unique_ptr<bool> response_;
-  bool should_post_quit_;
-};
-
 class AsyncExtensionBrowserTest : public ExtensionBrowserTest {
  protected:
   // Asynchronous function runner allows tests to manipulate the browser window
@@ -124,8 +85,7 @@ class AsyncExtensionBrowserTest : public ExtensionBrowserTest {
   void RunFunctionAsync(
       UIThreadExtensionFunction* function,
       const std::string& args) {
-    response_delegate_.reset(new SendResponseDelegate);
-    function->set_test_delegate(response_delegate_.get());
+    response_delegate_.reset(new api_test_utils::SendResponseHelper(function));
     std::unique_ptr<base::ListValue> parsed_args(utils::ParseList(args));
     EXPECT_TRUE(parsed_args.get()) <<
         "Could not parse extension function arguments: " << args;
@@ -144,7 +104,8 @@ class AsyncExtensionBrowserTest : public ExtensionBrowserTest {
 
   std::string WaitForError(UIThreadExtensionFunction* function) {
     RunMessageLoopUntilResponse();
-    EXPECT_FALSE(function->GetResultList()) << "Did not expect a result";
+    CHECK(function->response_type());
+    EXPECT_EQ(ExtensionFunction::FAILED, *function->response_type());
     return function->GetError();
   }
 
@@ -162,16 +123,11 @@ class AsyncExtensionBrowserTest : public ExtensionBrowserTest {
 
  private:
   void RunMessageLoopUntilResponse() {
-    // If the RunAsync of |function| didn't already call SendResponse, run the
-    // message loop until they do.
-    if (!response_delegate_->HasResponse()) {
-      response_delegate_->set_should_post_quit(true);
-      content::RunMessageLoop();
-    }
-    EXPECT_TRUE(response_delegate_->HasResponse());
+    response_delegate_->WaitForResponse();
+    EXPECT_TRUE(response_delegate_->has_response());
   }
 
-  std::unique_ptr<SendResponseDelegate> response_delegate_;
+  std::unique_ptr<api_test_utils::SendResponseHelper> response_delegate_;
 };
 
 class TestHangOAuth2MintTokenFlow : public OAuth2MintTokenFlow {
@@ -313,7 +269,7 @@ class FakeGetAuthTokenFunction : public IdentityGetAuthTokenFunction {
 
   void set_mint_token_result(TestOAuth2MintTokenFlow::ResultType result_type) {
     set_mint_token_flow(
-        base::WrapUnique(new TestOAuth2MintTokenFlow(result_type, this)));
+        base::MakeUnique<TestOAuth2MintTokenFlow>(result_type, this));
   }
 
   void set_scope_ui_failure(GaiaWebAuthFlow::Failure failure) {
@@ -626,7 +582,7 @@ class IdentityGetProfileUserInfoFunctionTest : public IdentityTestWithSignin {
     func->set_extension(test_util::CreateEmptyExtension(kExtensionId).get());
     std::unique_ptr<base::Value> value(
         utils::RunFunctionAndReturnSingleResult(func.get(), "[]", browser()));
-    return api::identity::ProfileUserInfo::FromValue(*value.get());
+    return api::identity::ProfileUserInfo::FromValue(*value);
   }
 
   std::unique_ptr<api::identity::ProfileUserInfo>
@@ -636,7 +592,7 @@ class IdentityGetProfileUserInfoFunctionTest : public IdentityTestWithSignin {
     func->set_extension(CreateExtensionWithEmailPermission());
     std::unique_ptr<base::Value> value(
         utils::RunFunctionAndReturnSingleResult(func.get(), "[]", browser()));
-    return api::identity::ProfileUserInfo::FromValue(*value.get());
+    return api::identity::ProfileUserInfo::FromValue(*value);
   }
 
  private:
@@ -1249,8 +1205,7 @@ IN_PROC_BROWSER_TEST_F(GetAuthTokenFunctionTest, NoninteractiveShutdown) {
   scoped_refptr<FakeGetAuthTokenFunction> func(new FakeGetAuthTokenFunction());
   func->set_extension(extension.get());
 
-  func->set_mint_token_flow(
-      base::WrapUnique(new TestHangOAuth2MintTokenFlow()));
+  func->set_mint_token_flow(base::MakeUnique<TestHangOAuth2MintTokenFlow>());
   RunFunctionAsync(func.get(), "[{\"interactive\": false}]");
 
   // After the request is canceled, the function will complete.
@@ -1570,8 +1525,8 @@ IN_PROC_BROWSER_TEST_F(GetAuthTokenFunctionTest, ScopesDefault) {
 
   const ExtensionTokenKey* token_key = func->GetExtensionTokenKeyForTest();
   EXPECT_EQ(2ul, token_key->scopes.size());
-  EXPECT_TRUE(ContainsKey(token_key->scopes, "scope1"));
-  EXPECT_TRUE(ContainsKey(token_key->scopes, "scope2"));
+  EXPECT_TRUE(base::ContainsKey(token_key->scopes, "scope1"));
+  EXPECT_TRUE(base::ContainsKey(token_key->scopes, "scope2"));
 }
 
 IN_PROC_BROWSER_TEST_F(GetAuthTokenFunctionTest, ScopesEmpty) {
@@ -1599,7 +1554,7 @@ IN_PROC_BROWSER_TEST_F(GetAuthTokenFunctionTest, ScopesEmail) {
 
   const ExtensionTokenKey* token_key = func->GetExtensionTokenKeyForTest();
   EXPECT_EQ(1ul, token_key->scopes.size());
-  EXPECT_TRUE(ContainsKey(token_key->scopes, "email"));
+  EXPECT_TRUE(base::ContainsKey(token_key->scopes, "email"));
 }
 
 IN_PROC_BROWSER_TEST_F(GetAuthTokenFunctionTest, ScopesEmailFooBar) {
@@ -1616,9 +1571,9 @@ IN_PROC_BROWSER_TEST_F(GetAuthTokenFunctionTest, ScopesEmailFooBar) {
 
   const ExtensionTokenKey* token_key = func->GetExtensionTokenKeyForTest();
   EXPECT_EQ(3ul, token_key->scopes.size());
-  EXPECT_TRUE(ContainsKey(token_key->scopes, "email"));
-  EXPECT_TRUE(ContainsKey(token_key->scopes, "foo"));
-  EXPECT_TRUE(ContainsKey(token_key->scopes, "bar"));
+  EXPECT_TRUE(base::ContainsKey(token_key->scopes, "email"));
+  EXPECT_TRUE(base::ContainsKey(token_key->scopes, "foo"));
+  EXPECT_TRUE(base::ContainsKey(token_key->scopes, "bar"));
 }
 
 
@@ -1640,10 +1595,9 @@ class GetAuthTokenFunctionPublicSessionTest : public GetAuthTokenFunctionTest {
 
     // Set up fake install attributes to make the device appeared as
     // enterprise-managed.
-     std::unique_ptr<policy::StubEnterpriseInstallAttributes> attributes(
-         new policy::StubEnterpriseInstallAttributes());
-     attributes->SetDomain("example.com");
-     attributes->SetRegistrationUser("user@example.com");
+     std::unique_ptr<chromeos::StubInstallAttributes> attributes
+         = base::MakeUnique<chromeos::StubInstallAttributes>();
+     attributes->SetEnterprise("example.com", "fake-id");
      policy::BrowserPolicyConnectorChromeOS::SetInstallAttributesForTesting(
          attributes.release());
   }
