@@ -5,7 +5,9 @@
 #include "chrome/browser/printing/cloud_print/privet_http_impl.h"
 
 #include <stddef.h>
+
 #include <algorithm>
+#include <memory>
 #include <utility>
 #include <vector>
 
@@ -15,16 +17,17 @@
 #include "base/rand_util.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/utf_string_conversions.h"
-#include "base/thread_task_runner_handle.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "chrome/browser/extensions/api/gcd_private/privet_v3_context_getter.h"
 #include "chrome/browser/printing/cloud_print/privet_constants.h"
 #include "chrome/common/chrome_content_client.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/cloud_print/cloud_print_constants.h"
 #include "net/base/url_util.h"
+#include "printing/features/features.h"
 #include "url/gurl.h"
 
-#if defined(ENABLE_PRINT_PREVIEW)
+#if BUILDFLAG(ENABLE_PRINT_PREVIEW)
 #include "chrome/browser/printing/pwg_raster_converter.h"
 #include "components/cloud_devices/common/printer_description.h"
 #include "printing/pdf_render_settings.h"
@@ -42,7 +45,7 @@ const char kPrivetRegisterUserArgName[] = "user";
 
 const int kPrivetCancelationTimeoutSeconds = 3;
 
-#if defined(ENABLE_PRINT_PREVIEW)
+#if BUILDFLAG(ENABLE_PRINT_PREVIEW)
 const char kPrivetURLKeyUserName[] = "user_name";
 const char kPrivetURLKeyClientName[] = "client_name";
 const char kPrivetURLKeyJobname[] = "job_name";
@@ -392,7 +395,7 @@ void PrivetJSONOperationImpl::OnNeedPrivetToken(
   privet_client_->RefreshPrivetToken(callback);
 }
 
-#if defined(ENABLE_PRINT_PREVIEW)
+#if BUILDFLAG(ENABLE_PRINT_PREVIEW)
 PrivetLocalPrintOperationImpl::PrivetLocalPrintOperationImpl(
     PrivetHTTPClient* privet_client,
     PrivetLocalPrintOperation::Delegate* delegate)
@@ -690,7 +693,7 @@ void PrivetLocalPrintOperationImpl::SetPageSize(const gfx::Size& page_size) {
 }
 
 void PrivetLocalPrintOperationImpl::SetPWGRasterConverterForTesting(
-    scoped_ptr<printing::PWGRasterConverter> pwg_raster_converter) {
+    std::unique_ptr<printing::PWGRasterConverter> pwg_raster_converter) {
   pwg_raster_converter_ = std::move(pwg_raster_converter);
 }
 #endif  // ENABLE_PRINT_PREVIEW
@@ -708,13 +711,13 @@ const std::string& PrivetHTTPClientImpl::GetName() {
   return name_;
 }
 
-scoped_ptr<PrivetJSONOperation> PrivetHTTPClientImpl::CreateInfoOperation(
+std::unique_ptr<PrivetJSONOperation> PrivetHTTPClientImpl::CreateInfoOperation(
     const PrivetJSONOperation::ResultCallback& callback) {
-  return scoped_ptr<PrivetJSONOperation>(
+  return std::unique_ptr<PrivetJSONOperation>(
       new PrivetInfoOperationImpl(this, callback));
 }
 
-scoped_ptr<PrivetURLFetcher> PrivetHTTPClientImpl::CreateURLFetcher(
+std::unique_ptr<PrivetURLFetcher> PrivetHTTPClientImpl::CreateURLFetcher(
     const GURL& url,
     net::URLFetcher::RequestType request_type,
     PrivetURLFetcher::Delegate* delegate) {
@@ -723,7 +726,7 @@ scoped_ptr<PrivetURLFetcher> PrivetHTTPClientImpl::CreateURLFetcher(
   replacements.SetHostStr(host);
   std::string port = base::UintToString(host_port_.port());
   replacements.SetPortStr(port);
-  return scoped_ptr<PrivetURLFetcher>(
+  return std::unique_ptr<PrivetURLFetcher>(
       new PrivetURLFetcher(url.ReplaceComponents(replacements), request_type,
                            context_getter_, delegate));
 }
@@ -761,7 +764,7 @@ void PrivetHTTPClientImpl::OnPrivetInfoDone(
 }
 
 PrivetV1HTTPClientImpl::PrivetV1HTTPClientImpl(
-    scoped_ptr<PrivetHTTPClient> info_client)
+    std::unique_ptr<PrivetHTTPClient> info_client)
     : info_client_(std::move(info_client)) {}
 
 PrivetV1HTTPClientImpl::~PrivetV1HTTPClientImpl() {
@@ -771,34 +774,35 @@ const std::string& PrivetV1HTTPClientImpl::GetName() {
   return info_client()->GetName();
 }
 
-scoped_ptr<PrivetJSONOperation> PrivetV1HTTPClientImpl::CreateInfoOperation(
+std::unique_ptr<PrivetJSONOperation>
+PrivetV1HTTPClientImpl::CreateInfoOperation(
     const PrivetJSONOperation::ResultCallback& callback) {
   return info_client()->CreateInfoOperation(callback);
 }
 
-scoped_ptr<PrivetRegisterOperation>
+std::unique_ptr<PrivetRegisterOperation>
 PrivetV1HTTPClientImpl::CreateRegisterOperation(
     const std::string& user,
     PrivetRegisterOperation::Delegate* delegate) {
-  return scoped_ptr<PrivetRegisterOperation>(
+  return std::unique_ptr<PrivetRegisterOperation>(
       new PrivetRegisterOperationImpl(info_client(), user, delegate));
 }
 
-scoped_ptr<PrivetJSONOperation>
+std::unique_ptr<PrivetJSONOperation>
 PrivetV1HTTPClientImpl::CreateCapabilitiesOperation(
     const PrivetJSONOperation::ResultCallback& callback) {
-  return scoped_ptr<PrivetJSONOperation>(new PrivetJSONOperationImpl(
+  return std::unique_ptr<PrivetJSONOperation>(new PrivetJSONOperationImpl(
       info_client(), kPrivetCapabilitiesPath, "", callback));
 }
 
-scoped_ptr<PrivetLocalPrintOperation>
+std::unique_ptr<PrivetLocalPrintOperation>
 PrivetV1HTTPClientImpl::CreateLocalPrintOperation(
     PrivetLocalPrintOperation::Delegate* delegate) {
-#if defined(ENABLE_PRINT_PREVIEW)
-  return scoped_ptr<PrivetLocalPrintOperation>(
+#if BUILDFLAG(ENABLE_PRINT_PREVIEW)
+  return std::unique_ptr<PrivetLocalPrintOperation>(
       new PrivetLocalPrintOperationImpl(info_client(), delegate));
 #else
-  return scoped_ptr<PrivetLocalPrintOperation>();
+  return std::unique_ptr<PrivetLocalPrintOperation>();
 #endif  // ENABLE_PRINT_PREVIEW
 }
 
