@@ -18,24 +18,17 @@
 #include "components/printing/browser/print_manager_utils.h"
 #include "components/printing/common/print_messages.h"
 #include "content/public/browser/browser_thread.h"
-#include "content/public/browser/render_view_host.h"
+#include "content/public/browser/render_frame_host.h"
 #include "content/public/browser/web_contents.h"
 #include "content/public/common/child_process_host.h"
+#include "printing/features/features.h"
 
-#if defined(ENABLE_PRINT_PREVIEW)
+#if BUILDFLAG(ENABLE_PRINT_PREVIEW)
 #include "chrome/browser/ui/webui/print_preview/print_preview_ui.h"
 #endif
 
-#if defined(OS_CHROMEOS)
-#include <fcntl.h>
-
-#include <map>
-
-#include "base/files/file_util.h"
-#include "base/lazy_instance.h"
-#endif
-
 #if defined(OS_ANDROID)
+#include "base/file_descriptor_posix.h"
 #include "base/strings/string_number_conversions.h"
 #include "chrome/browser/printing/print_view_manager_basic.h"
 #endif
@@ -47,19 +40,19 @@ namespace printing {
 namespace {
 
 #if defined(OS_ANDROID)
-content::WebContents* GetWebContentsForRenderView(int render_process_id,
-                                                  int render_view_id) {
+content::WebContents* GetWebContentsForRenderFrame(int render_process_id,
+                                                   int render_frame_id) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  content::RenderViewHost* view = content::RenderViewHost::FromID(
-      render_process_id, render_view_id);
-  return view ? content::WebContents::FromRenderViewHost(view) : nullptr;
+  content::RenderFrameHost* frame =
+      content::RenderFrameHost::FromID(render_process_id, render_frame_id);
+  return frame ? content::WebContents::FromRenderFrameHost(frame) : nullptr;
 }
 
 PrintViewManagerBasic* GetPrintManager(int render_process_id,
-                                       int render_view_id) {
+                                       int render_frame_id) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   content::WebContents* web_contents =
-      GetWebContentsForRenderView(render_process_id, render_view_id);
+      GetWebContentsForRenderFrame(render_process_id, render_frame_id);
   return web_contents ? PrintViewManagerBasic::FromWebContents(web_contents)
                       : nullptr;
 }
@@ -76,7 +69,7 @@ PrintingMessageFilter::PrintingMessageFilter(int render_process_id,
   DCHECK(queue_.get());
   is_printing_enabled_->Init(prefs::kPrintingEnabled, profile->GetPrefs());
   is_printing_enabled_->MoveToThread(
-      BrowserThread::GetMessageLoopProxyForThread(BrowserThread::IO));
+      BrowserThread::GetTaskRunnerForThread(BrowserThread::IO));
 }
 
 PrintingMessageFilter::~PrintingMessageFilter() {
@@ -101,13 +94,12 @@ bool PrintingMessageFilter::OnMessageReceived(const IPC::Message& message) {
     IPC_MESSAGE_HANDLER(PrintHostMsg_TempFileForPrintingWritten,
                         OnTempFileForPrintingWritten)
 #endif
-    IPC_MESSAGE_HANDLER(PrintHostMsg_IsPrintingEnabled, OnIsPrintingEnabled)
     IPC_MESSAGE_HANDLER_DELAY_REPLY(PrintHostMsg_GetDefaultPrintSettings,
                                     OnGetDefaultPrintSettings)
     IPC_MESSAGE_HANDLER_DELAY_REPLY(PrintHostMsg_ScriptedPrint, OnScriptedPrint)
     IPC_MESSAGE_HANDLER_DELAY_REPLY(PrintHostMsg_UpdatePrintSettings,
                                     OnUpdatePrintSettings)
-#if defined(ENABLE_PRINT_PREVIEW)
+#if BUILDFLAG(ENABLE_PRINT_PREVIEW)
     IPC_MESSAGE_HANDLER(PrintHostMsg_CheckForCancel, OnCheckForCancel)
 #endif
     IPC_MESSAGE_UNHANDLED(handled = false)
@@ -117,35 +109,31 @@ bool PrintingMessageFilter::OnMessageReceived(const IPC::Message& message) {
 
 #if defined(OS_ANDROID)
 void PrintingMessageFilter::OnAllocateTempFileForPrinting(
-    int render_view_id,
+    int render_frame_id,
     base::FileDescriptor* temp_file_fd,
     int* sequence_number) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   PrintViewManagerBasic* print_view_manager =
-      GetPrintManager(render_process_id_, render_view_id);
+      GetPrintManager(render_process_id_, render_frame_id);
   if (!print_view_manager)
     return;
+
   // The file descriptor is originally created in & passed from the Android
   // side, and it will handle the closing.
   temp_file_fd->fd = print_view_manager->file_descriptor().fd;
   temp_file_fd->auto_close = false;
 }
 
-void PrintingMessageFilter::OnTempFileForPrintingWritten(int render_view_id,
+void PrintingMessageFilter::OnTempFileForPrintingWritten(int render_frame_id,
                                                          int sequence_number) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   PrintViewManagerBasic* print_view_manager =
-      GetPrintManager(render_process_id_, render_view_id);
+      GetPrintManager(render_process_id_, render_frame_id);
   if (print_view_manager)
     print_view_manager->PdfWritingDone(true);
 }
 #endif  // defined(OS_ANDROID)
 
-void PrintingMessageFilter::OnIsPrintingEnabled(bool* is_enabled) {
-  DCHECK_CURRENTLY_ON(BrowserThread::IO);
-  *is_enabled = is_printing_enabled_->GetValue();
-}
-
 void PrintingMessageFilter::OnGetDefaultPrintSettings(IPC::Message* reply_msg) {
   DCHECK_CURRENTLY_ON(BrowserThread::IO);
   scoped_refptr<PrinterQuery> printer_query;
@@ -163,15 +151,10 @@ void PrintingMessageFilter::OnGetDefaultPrintSettings(IPC::Message* reply_msg) {
   // Loads default settings. This is asynchronous, only the IPC message sender
   // will hang until the settings are retrieved.
   printer_query->GetSettings(
-      PrinterQuery::GetSettingsAskParam::DEFAULTS,
-      0,
-      false,
-      DEFAULT_MARGINS,
-      false,
-      base::Bind(&PrintingMessageFilter::OnGetDefaultPrintSettingsReply,
-                 this,
-                 printer_query,
-                 reply_msg));
+      PrinterQuery::GetSettingsAskParam::DEFAULTS, 0, false, DEFAULT_MARGINS,
+      false, false,
+      base::Bind(&PrintingMessageFilter::OnGetDefaultPrintSettingsReply, this,
+                 printer_query, reply_msg));
 }
 
 void PrintingMessageFilter::OnGetDefaultPrintSettingsReply(
@@ -208,15 +191,11 @@ void PrintingMessageFilter::OnScriptedPrint(
         queue_->CreatePrinterQuery(render_process_id_, reply_msg->routing_id());
   }
   printer_query->GetSettings(
-      PrinterQuery::GetSettingsAskParam::ASK_USER,
-      params.expected_pages_count,
-      params.has_selection,
-      params.margin_type,
-      params.is_scripted,
-      base::Bind(&PrintingMessageFilter::OnScriptedPrintReply,
-                 this,
-                 printer_query,
-                 reply_msg));
+      PrinterQuery::GetSettingsAskParam::ASK_USER, params.expected_pages_count,
+      params.has_selection, params.margin_type, params.is_scripted,
+      params.is_modifiable,
+      base::Bind(&PrintingMessageFilter::OnScriptedPrintReply, this,
+                 printer_query, reply_msg));
 }
 
 void PrintingMessageFilter::OnScriptedPrintReply(
@@ -256,10 +235,10 @@ void PrintingMessageFilter::OnScriptedPrintReply(
 }
 
 #if defined(OS_ANDROID)
-void PrintingMessageFilter::UpdateFileDescriptor(int render_view_id, int fd) {
+void PrintingMessageFilter::UpdateFileDescriptor(int render_frame_id, int fd) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   PrintViewManagerBasic* print_view_manager =
-      GetPrintManager(render_process_id_, render_view_id);
+      GetPrintManager(render_process_id_, render_frame_id);
   if (print_view_manager)
     print_view_manager->set_file_descriptor(base::FileDescriptor(fd, false));
 }
@@ -278,14 +257,12 @@ void PrintingMessageFilter::OnUpdatePrintSettings(
   }
   printer_query = queue_->PopPrinterQuery(document_cookie);
   if (!printer_query.get()) {
-    int host_id = render_process_id_;
-    int routing_id = reply_msg->routing_id();
-    if (!new_settings->GetInteger(printing::kPreviewInitiatorHostId,
-                                  &host_id) ||
-        !new_settings->GetInteger(printing::kPreviewInitiatorRoutingId,
-                                  &routing_id)) {
+    int host_id;
+    int routing_id;
+    if (!new_settings->GetInteger(kPreviewInitiatorHostId, &host_id) ||
+        !new_settings->GetInteger(kPreviewInitiatorRoutingId, &routing_id)) {
       host_id = content::ChildProcessHost::kInvalidUniqueID;
-      routing_id = content::ChildProcessHost::kInvalidUniqueID;
+      routing_id = MSG_ROUTING_NONE;
     }
     printer_query = queue_->CreatePrinterQuery(host_id, routing_id);
   }
@@ -307,11 +284,10 @@ void PrintingMessageFilter::OnUpdatePrintSettingsReply(
     params.params.document_cookie = printer_query->cookie();
     params.pages = PageRange::GetPages(printer_query->settings().ranges());
   }
-  PrintHostMsg_UpdatePrintSettings::WriteReplyParams(
-      reply_msg,
-      params,
-      printer_query.get() &&
-          (printer_query->last_status() == printing::PrintingContext::CANCEL));
+  bool canceled = printer_query.get() &&
+                  (printer_query->last_status() == PrintingContext::CANCEL);
+  PrintHostMsg_UpdatePrintSettings::WriteReplyParams(reply_msg, params,
+                                                     canceled);
   Send(reply_msg);
   // If user hasn't cancelled.
   if (printer_query.get()) {
@@ -323,7 +299,7 @@ void PrintingMessageFilter::OnUpdatePrintSettingsReply(
   }
 }
 
-#if defined(ENABLE_PRINT_PREVIEW)
+#if BUILDFLAG(ENABLE_PRINT_PREVIEW)
 void PrintingMessageFilter::OnCheckForCancel(int32_t preview_ui_id,
                                              int preview_request_id,
                                              bool* cancel) {
