@@ -7,15 +7,36 @@ cr.define('downloads', function() {
     is: 'downloads-manager',
 
     properties: {
+      /** @private */
       hasDownloads_: {
         observer: 'hasDownloadsChanged_',
         type: Boolean,
       },
 
+      /** @private */
+      hasShadow_: {
+        type: Boolean,
+        value: false,
+        reflectToAttribute: true,
+      },
+
+      /** @private */
+      inSearchMode_: {
+        type: Boolean,
+        value: false,
+      },
+
+      /** @private {!Array<!downloads.Data>} */
       items_: {
         type: Array,
         value: function() { return []; },
       },
+
+      /** @private */
+      spinnerActive_: {
+        type: Boolean,
+        notify: true,
+      },
     },
 
     hostAttributes: {
@@ -24,6 +45,7 @@ cr.define('downloads', function() {
 
     listeners: {
       'downloads-list.scroll': 'onListScroll_',
+      'toolbar.search-changed': 'onSearchChanged_',
     },
 
     observers: [
@@ -40,14 +62,8 @@ cr.define('downloads', function() {
       if (loadTimeData.getBoolean('allowDeletingHistory'))
         this.$.toolbar.downloadsShowing = this.hasDownloads_;
 
-      if (this.hasDownloads_) {
+      if (this.hasDownloads_)
         this.$['downloads-list'].fire('iron-resize');
-      } else {
-        var isSearching = downloads.ActionService.getInstance().isSearching();
-        var messageToShow = isSearching ? 'noSearchResults' : 'noDownloads';
-        this.$['no-downloads'].querySelector('span').textContent =
-            loadTimeData.getString(messageToShow);
-      }
     },
 
     /**
@@ -59,6 +75,7 @@ cr.define('downloads', function() {
       this.splice.apply(this, ['items_', index, 0].concat(list));
       this.updateHideDates_(index, index + list.length);
       this.removeAttribute('loading');
+      this.spinnerActive_ = false;
     },
 
     /** @private */
@@ -66,6 +83,15 @@ cr.define('downloads', function() {
       this.hasDownloads_ = this.items_.length > 0;
     },
 
+    /**
+     * @return {string} The text to show when no download items are showing.
+     * @private
+     */
+    noDownloadsText_: function() {
+      return loadTimeData.getString(
+          this.inSearchMode_ ? 'noSearchResults' : 'noDownloads');
+    },
+
     /**
      * @param {Event} e
      * @private
@@ -105,6 +131,7 @@ cr.define('downloads', function() {
         // Approaching the end of the scrollback. Attempt to load more items.
         downloads.ActionService.getInstance().loadMore();
       }
+      this.hasShadow_ = list.scrollTop > 0;
     },
 
     /** @private */
@@ -116,6 +143,11 @@ cr.define('downloads', function() {
       downloads.ActionService.getInstance().loadMore();
     },
 
+    /** @private */
+    onSearchChanged_: function() {
+      this.inSearchMode_ = downloads.ActionService.getInstance().isSearching();
+    },
+
     /**
      * @param {number} index
      * @private
@@ -137,7 +169,8 @@ cr.define('downloads', function() {
         if (!current)
           continue;
         var prev = this.items_[i - 1];
-        current.hideDate = !!prev && prev.date_string == current.date_string;
+        var hideDate = !!prev && prev.date_string == current.date_string;
+        this.set('items_.' + i + '.hideDate', hideDate);
       }
     },
 
