@@ -63,8 +63,7 @@ cr.define('options.supervisedUserOptions', function() {
       // Add the avatar.
       var iconElement = this.ownerDocument.createElement('img');
       iconElement.className = 'profile-img';
-      iconElement.style.content = cr.icon.getProfileAvatarIcon(
-          supervisedUser.iconURL);
+      iconElement.style.content = cr.icon.getImage(supervisedUser.iconURL);
       this.appendChild(iconElement);
 
       // Add the profile name.
