@@ -14,6 +14,36 @@ function getIntersectionHeight(rect1, rect2) {
       Math.max(rect1.y, rect2.y));
 }
 
+/**
+ * Makes sure that the scale level doesn't get out of the limits.
+ * @param {number} scale The new scale level.
+ * @return {number} The scale clamped within the limits.
+ */
+function clampScale(scale) {
+  return Math.min(5, Math.max(0.25, scale));
+}
+
+/**
+ * Computes vector between two points.
+ * @param {!Object} p1 The first point.
+ * @param {!Object} p2 The second point.
+ * @return {!Object} The vector.
+ */
+function vectorDelta(p1, p2) {
+  return {
+    x: p2.x - p1.x,
+    y: p2.y - p1.y
+  };
+}
+
+function frameToPluginCoordinate(coordinateInFrame) {
+  var container = $('plugin');
+  return {
+    x: coordinateInFrame.x - container.getBoundingClientRect().left,
+    y: coordinateInFrame.y - container.getBoundingClientRect().top
+  };
+}
+
 /**
  * Create a new viewport.
  * @constructor
@@ -25,6 +55,8 @@ function getIntersectionHeight(rect1, rect2) {
  * @param {Function} afterZoomCallback is run after a change in zoom
  * @param {number} scrollbarWidth the width of scrollbars on the page
  * @param {number} defaultZoom The default zoom level.
+ * @param {number} topToolbarHeight The number of pixels that should initially
+ *     be left blank above the document for the toolbar.
  */
 function Viewport(window,
                   sizer,
@@ -32,7 +64,8 @@ function Viewport(window,
                   beforeZoomCallback,
                   afterZoomCallback,
                   scrollbarWidth,
-                  defaultZoom) {
+                  defaultZoom,
+                  topToolbarHeight) {
   this.window_ = window;
   this.sizer_ = sizer;
   this.viewportChangedCallback_ = viewportChangedCallback;
@@ -45,6 +78,12 @@ function Viewport(window,
   this.scrollbarWidth_ = scrollbarWidth;
   this.fittingType_ = Viewport.FittingType.NONE;
   this.defaultZoom_ = defaultZoom;
+  this.topToolbarHeight_ = topToolbarHeight;
+  this.prevScale_ = 1;
+  this.pinchPhase_ = Viewport.PinchPhase.PINCH_NONE;
+  this.pinchPanVector_ = null;
+  this.pinchCenter_ = null;
+  this.firstPinchCenterInFrame_ = null;
 
   window.addEventListener('scroll', this.updateViewport_.bind(this));
   window.addEventListener('resize', this.resize_.bind(this));
@@ -60,6 +99,19 @@ Viewport.FittingType = {
   FIT_TO_WIDTH: 'fit-to-width'
 };
 
+/**
+ * Enumeration of pinch states.
+ * This should match PinchPhase enum in pdf/out_of_process_instance.h
+ * @enum {number}
+ */
+Viewport.PinchPhase = {
+  PINCH_NONE: 0,
+  PINCH_START: 1,
+  PINCH_UPDATE_ZOOM_OUT: 2,
+  PINCH_UPDATE_ZOOM_IN: 3,
+  PINCH_END: 4
+};
+
 /**
  * The increment to scroll a page by in pixels when up/down/left/right arrow
  * keys are pressed. Usually we just let the browser handle scrolling on the
@@ -70,11 +122,12 @@ Viewport.SCROLL_INCREMENT = 40;
 
 /**
  * Predefined zoom factors to be used when zooming in/out. These are in
- * ascending order. This should match the list in
- * components/ui/zoom/page_zoom_constants.h
+ * ascending order. This should match the lists in
+ * components/ui/zoom/page_zoom_constants.h and
+ * chrome/browser/resources/settings/appearance_page/appearance_page.js
  */
-Viewport.ZOOM_FACTORS = [0.25, 0.333, 0.5, 0.666, 0.75, 0.9, 1,
-                         1.1, 1.25, 1.5, 1.75, 2, 2.5, 3, 4, 5];
+Viewport.ZOOM_FACTORS = [0.25, 1 / 3, 0.5, 2 / 3, 0.75, 0.8, 0.9,
+                         1, 1.1, 1.25, 1.5, 1.75, 2, 2.5, 3, 4, 5];
 
 /**
  * The minimum and maximum range to be used to clip zoom factor.
@@ -90,6 +143,24 @@ Viewport.ZOOM_FACTOR_RANGE = {
 Viewport.PAGE_SHADOW = {top: 3, bottom: 7, left: 5, right: 5};
 
 Viewport.prototype = {
+  /**
+   * Returns the zoomed and rounded document dimensions for the given zoom.
+   * Rounding is necessary when interacting with the renderer which tends to
+   * operate in integral values (for example for determining if scrollbars
+   * should be shown).
+   * @param {number} zoom The zoom to use to compute the scaled dimensions.
+   * @return {Object} A dictionary with scaled 'width'/'height' of the document.
+   * @private
+   */
+  getZoomedDocumentDimensions_: function(zoom) {
+    if (!this.documentDimensions_)
+      return null;
+    return {
+      width: Math.round(this.documentDimensions_.width * zoom),
+      height: Math.round(this.documentDimensions_.height * zoom)
+    };
+  },
+
   /**
    * @private
    * Returns true if the document needs scrollbars at the given zoom level.
@@ -99,25 +170,25 @@ Viewport.prototype = {
    *     respectively.
    */
   documentNeedsScrollbars_: function(zoom) {
-    if (!this.documentDimensions_) {
+    var zoomedDimensions = this.getZoomedDocumentDimensions_(zoom);
+    if (!zoomedDimensions) {
       return {
         horizontal: false,
         vertical: false
       };
     }
-    var documentWidth = this.documentDimensions_.width * zoom;
-    var documentHeight = this.documentDimensions_.height * zoom;
 
     // If scrollbars are required for one direction, expand the document in the
     // other direction to take the width of the scrollbars into account when
     // deciding whether the other direction needs scrollbars.
-    if (documentWidth > this.window_.innerWidth)
-      documentHeight += this.scrollbarWidth_;
-    else if (documentHeight > this.window_.innerHeight)
-      documentWidth += this.scrollbarWidth_;
+    if (zoomedDimensions.width > this.window_.innerWidth)
+      zoomedDimensions.height += this.scrollbarWidth_;
+    else if (zoomedDimensions.height > this.window_.innerHeight)
+      zoomedDimensions.width += this.scrollbarWidth_;
     return {
-      horizontal: documentWidth > this.window_.innerWidth,
-      vertical: documentHeight > this.window_.innerHeight
+      horizontal: zoomedDimensions.width > this.window_.innerWidth,
+      vertical: zoomedDimensions.height + this.topToolbarHeight_ >
+          this.window_.innerHeight
     };
   },
 
@@ -136,11 +207,11 @@ Viewport.prototype = {
    * Helper function called when the zoomed document size changes.
    */
   contentSizeChanged_: function() {
-    if (this.documentDimensions_) {
-      this.sizer_.style.width =
-          this.documentDimensions_.width * this.zoom_ + 'px';
-      this.sizer_.style.height =
-          this.documentDimensions_.height * this.zoom_ + 'px';
+    var zoomedDimensions = this.getZoomedDocumentDimensions_(this.zoom_);
+    if (zoomedDimensions) {
+      this.sizer_.style.width = zoomedDimensions.width + 'px';
+      this.sizer_.style.height = zoomedDimensions.height +
+          this.topToolbarHeight_ + 'px';
     }
   },
 
@@ -171,7 +242,7 @@ Viewport.prototype = {
   get position() {
     return {
       x: this.window_.pageXOffset,
-      y: this.window_.pageYOffset
+      y: this.window_.pageYOffset - this.topToolbarHeight_
     };
   },
 
@@ -180,7 +251,7 @@ Viewport.prototype = {
    * @type {Object} position the position to scroll to.
    */
   set position(position) {
-    this.window_.scrollTo(position.x, position.y);
+    this.window_.scrollTo(position.x, position.y + this.topToolbarHeight_);
   },
 
   /**
@@ -203,6 +274,29 @@ Viewport.prototype = {
     return this.zoom_;
   },
 
+  /**
+   * @type {Viewport.PinchPhase} The phase of the current pinch gesture for
+   *    the viewport.
+   */
+  get pinchPhase() {
+    return this.pinchPhase_;
+  },
+
+  /**
+   * @type {Object} The panning caused by the current pinch gesture (as
+   *    the deltas of the x and y coordinates).
+   */
+  get pinchPanVector() {
+    return this.pinchPanVector_;
+  },
+
+  /**
+   * @type {Object} The coordinates of the center of the current pinch gesture.
+   */
+  get pinchCenter() {
+    return this.pinchCenter_;
+  },
+
   /**
    * @private
    * Used to wrap a function that might perform zooming on the viewport. This is
@@ -229,15 +323,65 @@ Viewport.prototype = {
             'Viewport.mightZoom_.';
     }
     // Record the scroll position (relative to the top-left of the window).
-    var currentScrollPos = [
-      this.window_.pageXOffset / this.zoom_,
-      this.window_.pageYOffset / this.zoom_
-    ];
+    var currentScrollPos = {
+      x: this.position.x / this.zoom_,
+      y: this.position.y / this.zoom_
+    };
     this.zoom_ = newZoom;
     this.contentSizeChanged_();
     // Scroll to the scaled scroll position.
-    this.window_.scrollTo(currentScrollPos[0] * newZoom,
-                          currentScrollPos[1] * newZoom);
+    this.position = {
+      x: currentScrollPos.x * newZoom,
+      y: currentScrollPos.y * newZoom
+    };
+  },
+
+  /**
+   * @private
+   * Sets the zoom of the viewport.
+   * Same as setZoomInternal_ but for pinch zoom we have some more operations.
+   * @param {number} scaleDelta The zoom delta.
+   * @param {!Object} center The pinch center in content coordinates.
+   */
+  setPinchZoomInternal_: function(scaleDelta, center) {
+    assert(this.allowedToChangeZoom_,
+        'Called Viewport.setPinchZoomInternal_ without calling ' +
+        'Viewport.mightZoom_.');
+    this.zoom_ = clampScale(this.zoom_ * scaleDelta);
+
+    var newCenterInContent = this.frameToContent(center);
+    var delta = {
+      x: (newCenterInContent.x - this.oldCenterInContent.x),
+      y: (newCenterInContent.y - this.oldCenterInContent.y)
+    };
+
+    // Record the scroll position (relative to the pinch center).
+    var currentScrollPos = {
+      x: this.position.x - delta.x * this.zoom_,
+      y: this.position.y - delta.y * this.zoom_
+    };
+
+    this.contentSizeChanged_();
+    // Scroll to the scaled scroll position.
+    this.position = {
+      x: currentScrollPos.x,
+      y: currentScrollPos.y
+    };
+  },
+
+  /**
+   *  @private
+   *  Converts a point from frame to content coordinates.
+   *  @param {!Object} framePoint The frame coordinates.
+   *  @return {!Object} The content coordinates.
+   */
+  frameToContent: function(framePoint) {
+    // TODO(mcnee) Add a helper Point class to avoid duplicating operations
+    // on plain {x,y} objects.
+    return {
+      x: (framePoint.x + this.position.x) / this.zoom_,
+      y: (framePoint.y + this.position.y) / this.zoom_
+    };
   },
 
   /**
@@ -353,10 +497,7 @@ Viewport.prototype = {
     if (!needsScrollbars.horizontal && !needsScrollbars.vertical)
       return zoom;
 
-    var zoomedDimensions = {
-      width: this.documentDimensions_.width * zoom,
-      height: this.documentDimensions_.height * zoom
-    };
+    var zoomedDimensions = this.getZoomedDocumentDimensions_(zoom);
 
     // Check if adding a scrollbar will result in needing the other scrollbar.
     var scrollbarWidth = this.scrollbarWidth_;
@@ -426,8 +567,12 @@ Viewport.prototype = {
         height: this.pageDimensions_[page].height,
       };
       this.setZoomInternal_(this.computeFittingZoom_(dimensions, false));
-      if (scrollToTopOfPage)
-        this.window_.scrollTo(0, this.pageDimensions_[page].y * this.zoom_);
+      if (scrollToTopOfPage) {
+        this.position = {
+          x: 0,
+          y: this.pageDimensions_[page].y * this.zoom_
+        };
+      }
       this.updateViewport_();
     }.bind(this));
   },
@@ -472,6 +617,79 @@ Viewport.prototype = {
     }.bind(this));
   },
 
+  /**
+   * Pinch zoom event handler.
+   * @param {!Object} e The pinch event.
+   */
+  pinchZoom: function(e) {
+    this.mightZoom_(function() {
+      this.pinchPhase_ = e.direction == 'out' ?
+                         Viewport.PinchPhase.PINCH_UPDATE_ZOOM_OUT :
+                         Viewport.PinchPhase.PINCH_UPDATE_ZOOM_IN;
+
+      var scaleDelta = e.startScaleRatio / this.prevScale_;
+      this.pinchPanVector_ =
+          vectorDelta(e.center, this.firstPinchCenterInFrame_);
+
+      var needsScrollbars = this.documentNeedsScrollbars_(
+          clampScale(this.zoom_ * scaleDelta));
+
+      this.pinchCenter_ = e.center;
+
+      // If there's no horizontal scrolling, keep the content centered so the
+      // user can't zoom in on the non-content area.
+      // TODO(mcnee) Investigate other ways of scaling when we don't have
+      // horizontal scrolling. We want to keep the document centered,
+      // but this causes a potentially awkward transition when we start
+      // using the gesture center.
+      if (!needsScrollbars.horizontal) {
+        this.pinchCenter_ = {
+          x: this.window_.innerWidth / 2,
+          y: this.window_.innerHeight / 2
+        };
+      } else if (this.keepContentCentered_) {
+        this.oldCenterInContent =
+            this.frameToContent(frameToPluginCoordinate(e.center));
+        this.keepContentCentered_ = false;
+      }
+
+      this.setPinchZoomInternal_(
+          scaleDelta, frameToPluginCoordinate(e.center));
+      this.updateViewport_();
+      this.prevScale_ = e.startScaleRatio;
+    }.bind(this));
+  },
+
+  pinchZoomStart: function(e) {
+    this.pinchPhase_ = Viewport.PinchPhase.PINCH_START;
+    this.prevScale_ = 1;
+    this.oldCenterInContent =
+        this.frameToContent(frameToPluginCoordinate(e.center));
+
+    var needsScrollbars = this.documentNeedsScrollbars_(this.zoom_);
+    this.keepContentCentered_ = !needsScrollbars.horizontal;
+    // We keep track of begining of the pinch.
+    // By doing so we will be able to compute the pan distance.
+    this.firstPinchCenterInFrame_ = e.center;
+  },
+
+  pinchZoomEnd: function(e) {
+    this.mightZoom_(function() {
+      this.pinchPhase_ = Viewport.PinchPhase.PINCH_END;
+      var scaleDelta = e.startScaleRatio / this.prevScale_;
+      this.pinchCenter_ = e.center;
+
+      this.setPinchZoomInternal_(
+          scaleDelta, frameToPluginCoordinate(e.center));
+      this.updateViewport_();
+    }.bind(this));
+
+    this.pinchPhase_ = Viewport.PinchPhase.PINCH_NONE;
+    this.pinchPanVector_ = null;
+    this.pinchCenter_ = null;
+    this.firstPinchCenterInFrame_ = null;
+  },
+
   /**
    * Go to the given page index.
    * @param {number} page the index of the page to go to. zero-based.
@@ -485,8 +703,16 @@ Viewport.prototype = {
       if (page >= this.pageDimensions_.length)
         page = this.pageDimensions_.length - 1;
       var dimensions = this.pageDimensions_[page];
-      this.window_.scrollTo(dimensions.x * this.zoom_,
-                            dimensions.y * this.zoom_);
+      var toolbarOffset = 0;
+      // Unless we're in fit to page mode, scroll above the page by
+      // |this.topToolbarHeight_| so that the toolbar isn't covering it
+      // initially.
+      if (this.fittingType_ != Viewport.FittingType.FIT_TO_PAGE)
+        toolbarOffset = this.topToolbarHeight_;
+      this.position = {
+        x: dimensions.x * this.zoom_,
+        y: dimensions.y * this.zoom_ - toolbarOffset
+      };
       this.updateViewport_();
     }.bind(this));
   },
@@ -504,7 +730,10 @@ Viewport.prototype = {
         this.setZoomInternal_(
             Math.min(this.defaultZoom_,
                      this.computeFittingZoom_(this.documentDimensions_, true)));
-        this.window_.scrollTo(0, 0);
+        this.position = {
+          x: 0,
+          y: -this.topToolbarHeight_
+        };
       }
       this.contentSizeChanged_();
       this.resize_();
