@@ -9,11 +9,10 @@
 Polymer({
   is: 'settings-search-engines-list',
 
+  behaviors: [settings.GlobalScrollTargetBehavior],
+
   properties: {
     /** @type {!Array<!SearchEngine>} */
-    engines: {
-      type: Array,
-      value: function() { return []; }
-    }
+    engines: Array,
   },
 });
