@@ -7,7 +7,7 @@
 
 #include <string>
 
-#include "net/quic/quic_protocol.h"
+#include "net/quic/core/quic_protocol.h"
 
 // N.B. This file and the .cc are separate from util.h/.cc so that they can be
 // independently updated by folks working on QUIC when new errors are added.
