@@ -10,14 +10,17 @@
 #include "base/command_line.h"
 #include "base/json/json_reader.h"
 #include "base/json/json_writer.h"
-#include "base/metrics/histogram.h"
+#include "base/memory/ptr_util.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/optional.h"
-#include "base/stl_util.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/values.h"
+#include "components/history/core/browser/history_service_observer.h"
+#include "components/history/core/browser/web_history_service_observer.h"
 #include "components/signin/core/browser/signin_manager.h"
-#include "components/sync_driver/sync_util.h"
+#include "components/sync/driver/sync_util.h"
+#include "components/sync/protocol/history_status.pb.h"
 #include "google_apis/gaia/gaia_urls.h"
 #include "google_apis/gaia/google_service_auth_error.h"
 #include "google_apis/gaia/oauth2_token_service.h"
@@ -28,7 +31,6 @@
 #include "net/url_request/url_fetcher.h"
 #include "net/url_request/url_fetcher_delegate.h"
 #include "net/url_request/url_request_context_getter.h"
-#include "sync/protocol/history_status.pb.h"
 #include "ui/base/device_form_factor.h"
 #include "url/gurl.h"
 
@@ -302,11 +304,11 @@ GURL GetQueryUrl(const base::string16& text_query,
 // Creates a DictionaryValue to hold the parameters for a deletion.
 // Ownership is passed to the caller.
 // |url| may be empty, indicating a time-range deletion.
-base::DictionaryValue* CreateDeletion(
+std::unique_ptr<base::DictionaryValue> CreateDeletion(
     const std::string& min_time,
     const std::string& max_time,
     const GURL& url) {
-  base::DictionaryValue* deletion = new base::DictionaryValue;
+  std::unique_ptr<base::DictionaryValue> deletion(new base::DictionaryValue);
   deletion->SetString("type", "CHROME_HISTORY");
   if (url.is_valid())
     deletion->SetString("url", url.spec());
@@ -334,10 +336,14 @@ WebHistoryService::WebHistoryService(
 }
 
 WebHistoryService::~WebHistoryService() {
-  STLDeleteElements(&pending_expire_requests_);
-  STLDeleteElements(&pending_audio_history_requests_);
-  STLDeleteElements(&pending_web_and_app_activity_requests_);
-  STLDeleteElements(&pending_other_forms_of_browsing_history_requests_);
+}
+
+void WebHistoryService::AddObserver(WebHistoryServiceObserver* observer) {
+  observer_list_.AddObserver(observer);
+}
+
+void WebHistoryService::RemoveObserver(WebHistoryServiceObserver* observer) {
+  observer_list_.RemoveObserver(observer);
 }
 
 WebHistoryService::Request* WebHistoryService::CreateRequest(
@@ -419,8 +425,9 @@ void WebHistoryService::ExpireHistory(
 
   std::unique_ptr<Request> request(CreateRequest(url, completion_callback));
   request->SetPostData(post_data);
-  request->Start();
-  pending_expire_requests_.insert(request.release());
+  Request* request_ptr = request.get();
+  pending_expire_requests_[request_ptr] = std::move(request);
+  request_ptr->Start();
 }
 
 void WebHistoryService::ExpireHistoryBetween(
@@ -446,7 +453,8 @@ void WebHistoryService::GetAudioHistoryEnabled(
   GURL url(kHistoryAudioHistoryUrl);
   std::unique_ptr<Request> request(CreateRequest(url, completion_callback));
   request->Start();
-  pending_audio_history_requests_.insert(request.release());
+  Request* request_ptr = request.get();
+  pending_audio_history_requests_[request_ptr] = std::move(request);
 }
 
 void WebHistoryService::SetAudioHistoryEnabled(
@@ -470,7 +478,8 @@ void WebHistoryService::SetAudioHistoryEnabled(
   request->SetPostData(post_data);
 
   request->Start();
-  pending_audio_history_requests_.insert(request.release());
+  Request* request_ptr = request.get();
+  pending_audio_history_requests_[request_ptr] = std::move(request);
 }
 
 size_t WebHistoryService::GetNumberOfPendingAudioHistoryRequests() {
@@ -487,7 +496,7 @@ void WebHistoryService::QueryWebAndAppActivity(
 
   GURL url(kQueryWebAndAppActivityUrl);
   Request* request = CreateRequest(url, completion_callback);
-  pending_web_and_app_activity_requests_.insert(request);
+  pending_web_and_app_activity_requests_[request] = base::WrapUnique(request);
   request->Start();
 }
 
@@ -501,8 +510,8 @@ void WebHistoryService::QueryOtherFormsOfBrowsingHistory(
       callback);
 
   // Find the Sync request URL.
-  GURL url =
-      GetSyncServiceURL(*base::CommandLine::ForCurrentProcess(), channel);
+  GURL url = syncer::GetSyncServiceURL(*base::CommandLine::ForCurrentProcess(),
+                                       channel);
   GURL::Replacements replace_path;
   std::string new_path =
       url.path() + kQueryOtherFormsOfBrowsingHistoryUrlSuffix;
@@ -513,11 +522,12 @@ void WebHistoryService::QueryOtherFormsOfBrowsingHistory(
   Request* request = CreateRequest(url, completion_callback);
 
   // Set the Sync-specific user agent.
-  std::string user_agent = MakeUserAgentForSync(
+  std::string user_agent = syncer::MakeUserAgentForSync(
       channel, ui::GetDeviceFormFactor() == ui::DEVICE_FORM_FACTOR_TABLET);
   request->SetUserAgent(user_agent);
 
-  pending_other_forms_of_browsing_history_requests_.insert(request);
+  pending_other_forms_of_browsing_history_requests_[request] =
+      base::WrapUnique(request);
 
   // Set the request protobuf.
   sync_pb::HistoryStatusRequest request_proto;
@@ -543,8 +553,9 @@ void WebHistoryService::ExpireHistoryCompletionCallback(
     const WebHistoryService::ExpireWebHistoryCallback& callback,
     WebHistoryService::Request* request,
     bool success) {
+  std::unique_ptr<Request> request_ptr =
+      std::move(pending_expire_requests_[request]);
   pending_expire_requests_.erase(request);
-  std::unique_ptr<Request> request_ptr(request);
 
   std::unique_ptr<base::DictionaryValue> response_value;
   if (success) {
@@ -552,6 +563,13 @@ void WebHistoryService::ExpireHistoryCompletionCallback(
     if (response_value)
       response_value->GetString("version_info", &server_version_info_);
   }
+
+  // Inform the observers about the history deletion.
+  if (response_value.get() && success) {
+    for (WebHistoryServiceObserver& observer : observer_list_)
+      observer.OnWebHistoryDeleted();
+  }
+
   callback.Run(response_value.get() && success);
 }
 
@@ -559,13 +577,14 @@ void WebHistoryService::AudioHistoryCompletionCallback(
     const WebHistoryService::AudioWebHistoryCallback& callback,
     WebHistoryService::Request* request,
     bool success) {
+  std::unique_ptr<Request> request_ptr =
+      std::move(pending_audio_history_requests_[request]);
   pending_audio_history_requests_.erase(request);
-  std::unique_ptr<WebHistoryService::Request> request_ptr(request);
 
   std::unique_ptr<base::DictionaryValue> response_value;
   bool enabled_value = false;
   if (success) {
-    response_value = ReadResponse(request_ptr.get());
+    response_value = ReadResponse(request);
     if (response_value)
       response_value->GetBoolean("history_recording_enabled", &enabled_value);
   }
@@ -580,8 +599,9 @@ void WebHistoryService::QueryWebAndAppActivityCompletionCallback(
     const WebHistoryService::QueryWebAndAppActivityCallback& callback,
     WebHistoryService::Request* request,
     bool success) {
+  std::unique_ptr<Request> request_ptr =
+      std::move(pending_web_and_app_activity_requests_[request]);
   pending_web_and_app_activity_requests_.erase(request);
-  std::unique_ptr<Request> request_ptr(request);
 
   std::unique_ptr<base::DictionaryValue> response_value;
   bool web_and_app_activity_enabled = false;
@@ -601,8 +621,9 @@ void WebHistoryService::QueryOtherFormsOfBrowsingHistoryCompletionCallback(
     const WebHistoryService::QueryOtherFormsOfBrowsingHistoryCallback& callback,
     WebHistoryService::Request* request,
     bool success) {
+  std::unique_ptr<Request> request_ptr =
+      std::move(pending_other_forms_of_browsing_history_requests_[request]);
   pending_other_forms_of_browsing_history_requests_.erase(request);
-  std::unique_ptr<Request> request_ptr(request);
 
   bool has_other_forms_of_browsing_history = false;
   if (success && request->GetResponseCode() == net::HTTP_OK) {
