@@ -33,9 +33,12 @@
 using autofill::PasswordForm;
 using base::ASCIIToUTF16;
 using ::testing::Eq;
+using ::testing::Pointee;
+using ::testing::UnorderedElementsAre;
 
 namespace password_manager {
 namespace {
+
 PasswordStoreChangeList AddChangeForForm(const PasswordForm& form) {
   return PasswordStoreChangeList(
       1, PasswordStoreChange(PasswordStoreChange::ADD, form));
@@ -55,7 +58,6 @@ void GenerateExamplePasswordForm(PasswordForm* form) {
   form->password_value = ASCIIToUTF16("test");
   form->submit_element = ASCIIToUTF16("signIn");
   form->signon_realm = "http://www.google.com/";
-  form->ssl_valid = false;
   form->preferred = false;
   form->scheme = PasswordForm::SCHEME_HTML;
   form->times_used = 1;
@@ -88,18 +90,19 @@ template<> std::string GetFirstColumn(const sql::Statement& s) {
 }
 
 bool AddZeroClickableLogin(LoginDatabase* db,
-                           const std::string& unique_string) {
+                           const std::string& unique_string,
+                           const GURL& origin) {
   // Example password form.
   PasswordForm form;
-  form.origin = GURL("https://example.com/");
+  form.origin = origin;
   form.username_element = ASCIIToUTF16(unique_string);
   form.username_value = ASCIIToUTF16(unique_string);
   form.password_element = ASCIIToUTF16(unique_string);
   form.submit_element = ASCIIToUTF16("signIn");
   form.signon_realm = form.origin.spec();
   form.display_name = ASCIIToUTF16(unique_string);
-  form.icon_url = GURL("https://example.com/");
-  form.federation_origin = url::Origin(GURL("https://example.com/"));
+  form.icon_url = origin;
+  form.federation_origin = url::Origin(origin);
   form.date_created = base::Time::Now();
 
   form.skip_zero_click = false;
@@ -107,6 +110,24 @@ bool AddZeroClickableLogin(LoginDatabase* db,
   return db->AddLogin(form) == AddChangeForForm(form);
 }
 
+MATCHER(IsGoogle1Account, "") {
+  return arg.origin.spec() == "https://accounts.google.com/ServiceLogin" &&
+         arg.action.spec() == "https://accounts.google.com/ServiceLoginAuth" &&
+         arg.username_value == ASCIIToUTF16("theerikchen") &&
+         arg.scheme == PasswordForm::SCHEME_HTML;
+}
+
+MATCHER(IsGoogle2Account, "") {
+  return arg.origin.spec() == "https://accounts.google.com/ServiceLogin" &&
+         arg.action.spec() == "https://accounts.google.com/ServiceLoginAuth" &&
+         arg.username_value == ASCIIToUTF16("theerikchen2") &&
+         arg.scheme == PasswordForm::SCHEME_HTML;
+}
+
+MATCHER(IsBasicAuthAccount, "") {
+  return arg.scheme == PasswordForm::SCHEME_BASIC;
+}
+
 }  // namespace
 
 // Serialization routines for vectors implemented in login_database.cc.
@@ -117,7 +138,7 @@ class LoginDatabaseTest : public testing::Test {
  protected:
   void SetUp() override {
     ASSERT_TRUE(temp_dir_.CreateUniqueTempDir());
-    file_ = temp_dir_.path().AppendASCII("TestMetadataStoreMacDatabase");
+    file_ = temp_dir_.GetPath().AppendASCII("TestMetadataStoreMacDatabase");
     OSCryptMocker::SetUpWithSingleton();
 
     db_.reset(new LoginDatabase(file_));
@@ -129,7 +150,7 @@ class LoginDatabaseTest : public testing::Test {
   LoginDatabase& db() { return *db_; }
 
   void TestNonHTMLFormPSLMatching(const PasswordForm::Scheme& scheme) {
-    ScopedVector<autofill::PasswordForm> result;
+    std::vector<std::unique_ptr<PasswordForm>> result;
 
     base::Time now = base::Time::Now();
 
@@ -160,16 +181,17 @@ class LoginDatabaseTest : public testing::Test {
     EXPECT_EQ(2U, result.size());
     result.clear();
 
-    PasswordForm second_non_html_auth(non_html_auth);
-    second_non_html_auth.origin = GURL("http://second.example.com");
-    second_non_html_auth.signon_realm = "http://second.example.com/Realm";
+    PasswordStore::FormDigest second_non_html_auth = {
+        scheme, "http://second.example.com/Realm",
+        GURL("http://second.example.com")};
 
     // This shouldn't match anything.
     EXPECT_TRUE(db().GetLogins(second_non_html_auth, &result));
     EXPECT_EQ(0U, result.size());
 
     // non-html auth still matches against itself.
-    EXPECT_TRUE(db().GetLogins(non_html_auth, &result));
+    EXPECT_TRUE(
+        db().GetLogins(PasswordStore::FormDigest(non_html_auth), &result));
     ASSERT_EQ(1U, result.size());
     EXPECT_EQ(result[0]->signon_realm, "http://example.com/Realm");
 
@@ -181,7 +203,7 @@ class LoginDatabaseTest : public testing::Test {
   // retrieved from the database.
   void TestRetrievingIPAddress(const PasswordForm::Scheme& scheme) {
     SCOPED_TRACE(testing::Message() << "scheme = " << scheme);
-    ScopedVector<autofill::PasswordForm> result;
+    std::vector<std::unique_ptr<PasswordForm>> result;
 
     base::Time now = base::Time::Now();
     std::string origin("http://56.7.8.90");
@@ -195,7 +217,7 @@ class LoginDatabaseTest : public testing::Test {
     ip_form.date_created = now;
 
     EXPECT_EQ(AddChangeForForm(ip_form), db().AddLogin(ip_form));
-    EXPECT_TRUE(db().GetLogins(ip_form, &result));
+    EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(ip_form), &result));
     ASSERT_EQ(1U, result.size());
     EXPECT_EQ(result[0]->signon_realm, origin);
 
@@ -209,7 +231,7 @@ class LoginDatabaseTest : public testing::Test {
 };
 
 TEST_F(LoginDatabaseTest, Logins) {
-  ScopedVector<autofill::PasswordForm> result;
+  std::vector<std::unique_ptr<PasswordForm>> result;
 
   // Verify the database is empty.
   EXPECT_TRUE(db().GetAutofillableLogins(&result));
@@ -228,7 +250,7 @@ TEST_F(LoginDatabaseTest, Logins) {
   result.clear();
 
   // Match against an exact copy.
-  EXPECT_TRUE(db().GetLogins(form, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form), &result));
   ASSERT_EQ(1U, result.size());
   EXPECT_EQ(form, *result[0]);
   result.clear();
@@ -239,7 +261,7 @@ TEST_F(LoginDatabaseTest, Logins) {
   form2.submit_element = ASCIIToUTF16("reallySignIn");
 
   // Match against an inexact copy
-  EXPECT_TRUE(db().GetLogins(form2, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form2), &result));
   EXPECT_EQ(1U, result.size());
   result.clear();
 
@@ -248,17 +270,16 @@ TEST_F(LoginDatabaseTest, Logins) {
   form3.action = GURL("http://www.google.com/new/accounts/Login");
 
   // signon_realm is the same, should match.
-  EXPECT_TRUE(db().GetLogins(form3, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form3), &result));
   EXPECT_EQ(1U, result.size());
   result.clear();
 
   // Imagine the site moves to a secure server for login.
   PasswordForm form4(form3);
   form4.signon_realm = "https://www.google.com/";
-  form4.ssl_valid = true;
 
   // We have only an http record, so no match for this.
-  EXPECT_TRUE(db().GetLogins(form4, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form4), &result));
   EXPECT_EQ(0U, result.size());
 
   // Let's imagine the user logs into the secure site.
@@ -268,7 +289,7 @@ TEST_F(LoginDatabaseTest, Logins) {
   result.clear();
 
   // Now the match works
-  EXPECT_TRUE(db().GetLogins(form4, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form4), &result));
   EXPECT_EQ(1U, result.size());
   result.clear();
 
@@ -279,38 +300,28 @@ TEST_F(LoginDatabaseTest, Logins) {
   result.clear();
 
   // The old form wont match the new site (http vs https).
-  EXPECT_TRUE(db().GetLogins(form, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form), &result));
   EXPECT_EQ(0U, result.size());
 
-  // The user's request for the HTTPS site is intercepted
-  // by an attacker who presents an invalid SSL cert.
-  PasswordForm form5(form4);
-  form5.ssl_valid = 0;
-
-  // It will match in this case.
-  EXPECT_TRUE(db().GetLogins(form5, &result));
-  EXPECT_EQ(1U, result.size());
-  result.clear();
-
   // User changes their password.
-  PasswordForm form6(form5);
-  form6.password_value = ASCIIToUTF16("test6");
-  form6.preferred = true;
+  PasswordForm form5(form4);
+  form5.password_value = ASCIIToUTF16("test6");
+  form5.preferred = true;
 
   // We update, and check to make sure it matches the
   // old form, and there is only one record.
-  EXPECT_EQ(UpdateChangeForForm(form6), db().UpdateLogin(form6));
+  EXPECT_EQ(UpdateChangeForForm(form5), db().UpdateLogin(form5));
   // matches
-  EXPECT_TRUE(db().GetLogins(form5, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form5), &result));
   EXPECT_EQ(1U, result.size());
   result.clear();
   // Only one record.
   EXPECT_TRUE(db().GetAutofillableLogins(&result));
   EXPECT_EQ(1U, result.size());
   // Password element was updated.
-  EXPECT_EQ(form6.password_value, result[0]->password_value);
+  EXPECT_EQ(form5.password_value, result[0]->password_value);
   // Preferred login.
-  EXPECT_TRUE(form6.preferred);
+  EXPECT_TRUE(form5.preferred);
   result.clear();
 
   // Make sure everything can disappear.
@@ -320,7 +331,7 @@ TEST_F(LoginDatabaseTest, Logins) {
 }
 
 TEST_F(LoginDatabaseTest, TestPublicSuffixDomainMatching) {
-  ScopedVector<autofill::PasswordForm> result;
+  std::vector<std::unique_ptr<PasswordForm>> result;
 
   // Verify the database is empty.
   EXPECT_TRUE(db().GetAutofillableLogins(&result));
@@ -336,7 +347,6 @@ TEST_F(LoginDatabaseTest, TestPublicSuffixDomainMatching) {
   form.password_value = ASCIIToUTF16("test");
   form.submit_element = ASCIIToUTF16("");
   form.signon_realm = "https://foo.com/";
-  form.ssl_valid = true;
   form.preferred = false;
   form.scheme = PasswordForm::SCHEME_HTML;
 
@@ -347,7 +357,7 @@ TEST_F(LoginDatabaseTest, TestPublicSuffixDomainMatching) {
   result.clear();
 
   // Match against an exact copy.
-  EXPECT_TRUE(db().GetLogins(form, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form), &result));
   EXPECT_EQ(1U, result.size());
   result.clear();
 
@@ -358,7 +368,7 @@ TEST_F(LoginDatabaseTest, TestPublicSuffixDomainMatching) {
   form2.signon_realm = "https://mobile.foo.com/";
 
   // Match against the mobile site.
-  EXPECT_TRUE(db().GetLogins(form2, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form2), &result));
   EXPECT_EQ(1U, result.size());
   EXPECT_EQ("https://foo.com/", result[0]->signon_realm);
   EXPECT_TRUE(result[0]->is_public_suffix_match);
@@ -367,13 +377,13 @@ TEST_F(LoginDatabaseTest, TestPublicSuffixDomainMatching) {
   EXPECT_FALSE(db().RemoveLogin(*result[0]));
   result.clear();
   // Ensure that the original form is still there
-  EXPECT_TRUE(db().GetLogins(form, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form), &result));
   EXPECT_EQ(1U, result.size());
   result.clear();
 }
 
 TEST_F(LoginDatabaseTest, TestFederatedMatching) {
-  ScopedVector<autofill::PasswordForm> result;
+  std::vector<std::unique_ptr<PasswordForm>> result;
 
   // Example password form.
   PasswordForm form;
@@ -382,7 +392,6 @@ TEST_F(LoginDatabaseTest, TestFederatedMatching) {
   form.username_value = ASCIIToUTF16("test@gmail.com");
   form.password_value = ASCIIToUTF16("test");
   form.signon_realm = "https://foo.com/";
-  form.ssl_valid = true;
   form.preferred = false;
   form.scheme = PasswordForm::SCHEME_HTML;
 
@@ -402,20 +411,17 @@ TEST_F(LoginDatabaseTest, TestFederatedMatching) {
   EXPECT_EQ(2U, result.size());
 
   // Match against desktop.
-  PasswordForm form_request;
-  form_request.origin = GURL("https://foo.com/");
-  form_request.signon_realm = "https://foo.com/";
-  form_request.scheme = PasswordForm::SCHEME_HTML;
+  PasswordStore::FormDigest form_request = {
+      PasswordForm::SCHEME_HTML, "https://foo.com/", GURL("https://foo.com/")};
   EXPECT_TRUE(db().GetLogins(form_request, &result));
-  EXPECT_THAT(result, testing::ElementsAre(testing::Pointee(form)));
+  EXPECT_THAT(result, testing::ElementsAre(Pointee(form)));
 
   // Match against the mobile site.
   form_request.origin = GURL("https://mobile.foo.com/");
   form_request.signon_realm = "https://mobile.foo.com/";
   EXPECT_TRUE(db().GetLogins(form_request, &result));
   form.is_public_suffix_match = true;
-  EXPECT_THAT(result, testing::UnorderedElementsAre(testing::Pointee(form),
-                                                    testing::Pointee(form2)));
+  EXPECT_THAT(result, UnorderedElementsAre(Pointee(form), Pointee(form2)));
 }
 
 TEST_F(LoginDatabaseTest, TestPublicSuffixDisabledForNonHTMLForms) {
@@ -441,7 +447,7 @@ TEST_F(LoginDatabaseTest, TestIPAddressMatches_other) {
 }
 
 TEST_F(LoginDatabaseTest, TestPublicSuffixDomainMatchingShouldMatchingApply) {
-  ScopedVector<autofill::PasswordForm> result;
+  std::vector<std::unique_ptr<PasswordForm>> result;
 
   // Verify the database is empty.
   EXPECT_TRUE(db().GetAutofillableLogins(&result));
@@ -457,7 +463,6 @@ TEST_F(LoginDatabaseTest, TestPublicSuffixDomainMatchingShouldMatchingApply) {
   form.password_value = ASCIIToUTF16("test");
   form.submit_element = ASCIIToUTF16("");
   form.signon_realm = "https://accounts.google.com/";
-  form.ssl_valid = true;
   form.preferred = false;
   form.scheme = PasswordForm::SCHEME_HTML;
 
@@ -468,15 +473,14 @@ TEST_F(LoginDatabaseTest, TestPublicSuffixDomainMatchingShouldMatchingApply) {
   result.clear();
 
   // Match against an exact copy.
-  EXPECT_TRUE(db().GetLogins(form, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form), &result));
   EXPECT_EQ(1U, result.size());
   result.clear();
 
   // We go to a different site on the same domain where feature is not needed.
-  PasswordForm form2(form);
-  form2.origin = GURL("https://some.other.google.com/");
-  form2.action = GURL("https://some.other.google.com/login");
-  form2.signon_realm = "https://some.other.google.com/";
+  PasswordStore::FormDigest form2 = {PasswordForm::SCHEME_HTML,
+                                     "https://some.other.google.com/",
+                                     GURL("https://some.other.google.com/")};
 
   // Match against the other site. Should not match since feature should not be
   // enabled for this domain.
@@ -488,7 +492,7 @@ TEST_F(LoginDatabaseTest, TestPublicSuffixDomainMatchingShouldMatchingApply) {
 }
 
 TEST_F(LoginDatabaseTest, TestFederatedMatchingWithoutPSLMatching) {
-  ScopedVector<autofill::PasswordForm> result;
+  std::vector<std::unique_ptr<PasswordForm>> result;
 
   // Example password form.
   PasswordForm form;
@@ -497,7 +501,6 @@ TEST_F(LoginDatabaseTest, TestFederatedMatchingWithoutPSLMatching) {
   form.username_value = ASCIIToUTF16("test@gmail.com");
   form.password_value = ASCIIToUTF16("test");
   form.signon_realm = "https://accounts.google.com/";
-  form.ssl_valid = true;
   form.preferred = false;
   form.scheme = PasswordForm::SCHEME_HTML;
 
@@ -517,12 +520,10 @@ TEST_F(LoginDatabaseTest, TestFederatedMatchingWithoutPSLMatching) {
   EXPECT_EQ(2U, result.size());
 
   // Match against the first one.
-  PasswordForm form_request;
-  form_request.origin = form.origin;
-  form_request.signon_realm = form.signon_realm;
-  form_request.scheme = PasswordForm::SCHEME_HTML;
+  PasswordStore::FormDigest form_request = {PasswordForm::SCHEME_HTML,
+                                            form.signon_realm, form.origin};
   EXPECT_TRUE(db().GetLogins(form_request, &result));
-  EXPECT_THAT(result, testing::ElementsAre(testing::Pointee(form)));
+  EXPECT_THAT(result, testing::ElementsAre(Pointee(form)));
 
   // Match against the second one.
   ASSERT_FALSE(ShouldPSLDomainMatchingApply(
@@ -531,14 +532,14 @@ TEST_F(LoginDatabaseTest, TestFederatedMatchingWithoutPSLMatching) {
   form_request.signon_realm = form2.signon_realm;
   EXPECT_TRUE(db().GetLogins(form_request, &result));
   form.is_public_suffix_match = true;
-  EXPECT_THAT(result, testing::ElementsAre(testing::Pointee(form2)));
+  EXPECT_THAT(result, testing::ElementsAre(Pointee(form2)));
 }
 
 // This test fails if the implementation of GetLogins uses GetCachedStatement
 // instead of GetUniqueStatement, since REGEXP is in use. See
 // http://crbug.com/248608.
 TEST_F(LoginDatabaseTest, TestPublicSuffixDomainMatchingDifferentSites) {
-  ScopedVector<autofill::PasswordForm> result;
+  std::vector<std::unique_ptr<PasswordForm>> result;
 
   // Verify the database is empty.
   EXPECT_TRUE(db().GetAutofillableLogins(&result));
@@ -554,7 +555,6 @@ TEST_F(LoginDatabaseTest, TestPublicSuffixDomainMatchingDifferentSites) {
   form.password_value = ASCIIToUTF16("test");
   form.submit_element = ASCIIToUTF16("");
   form.signon_realm = "https://foo.com/";
-  form.ssl_valid = true;
   form.preferred = false;
   form.scheme = PasswordForm::SCHEME_HTML;
 
@@ -565,14 +565,13 @@ TEST_F(LoginDatabaseTest, TestPublicSuffixDomainMatchingDifferentSites) {
   result.clear();
 
   // Match against an exact copy.
-  EXPECT_TRUE(db().GetLogins(form, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form), &result));
   EXPECT_EQ(1U, result.size());
   result.clear();
 
   // We go to the mobile site.
-  PasswordForm form2(form);
+  PasswordStore::FormDigest form2(form);
   form2.origin = GURL("https://mobile.foo.com/");
-  form2.action = GURL("https://mobile.foo.com/login");
   form2.signon_realm = "https://mobile.foo.com/";
 
   // Match against the mobile site.
@@ -591,7 +590,6 @@ TEST_F(LoginDatabaseTest, TestPublicSuffixDomainMatchingDifferentSites) {
   form.password_value = ASCIIToUTF16("test");
   form.submit_element = ASCIIToUTF16("");
   form.signon_realm = "https://baz.com/";
-  form.ssl_valid = true;
   form.preferred = false;
   form.scheme = PasswordForm::SCHEME_HTML;
 
@@ -602,9 +600,8 @@ TEST_F(LoginDatabaseTest, TestPublicSuffixDomainMatchingDifferentSites) {
   result.clear();
 
   // We go to the mobile site of baz.com.
-  PasswordForm form3(form);
+  PasswordStore::FormDigest form3(form);
   form3.origin = GURL("https://m.baz.com/login/");
-  form3.action = GURL("https://m.baz.com/login/");
   form3.signon_realm = "https://m.baz.com/";
 
   // Match against the mobile site of baz.com.
@@ -625,7 +622,7 @@ PasswordForm GetFormWithNewSignonRealm(PasswordForm form,
 }
 
 TEST_F(LoginDatabaseTest, TestPublicSuffixDomainMatchingRegexp) {
-  ScopedVector<autofill::PasswordForm> result;
+  std::vector<std::unique_ptr<PasswordForm>> result;
 
   // Verify the database is empty.
   EXPECT_TRUE(db().GetAutofillableLogins(&result));
@@ -641,7 +638,6 @@ TEST_F(LoginDatabaseTest, TestPublicSuffixDomainMatchingRegexp) {
   form.password_value = ASCIIToUTF16("test");
   form.submit_element = ASCIIToUTF16("");
   form.signon_realm = "http://foo.com/";
-  form.ssl_valid = false;
   form.preferred = false;
   form.scheme = PasswordForm::SCHEME_HTML;
 
@@ -662,75 +658,75 @@ TEST_F(LoginDatabaseTest, TestPublicSuffixDomainMatchingRegexp) {
   result.clear();
 
   // Match against an exact copy.
-  EXPECT_TRUE(db().GetLogins(form, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form), &result));
   EXPECT_EQ(1U, result.size());
   result.clear();
 
   // www.foo.com should match.
   PasswordForm form2 = GetFormWithNewSignonRealm(form, "http://www.foo.com/");
-  EXPECT_TRUE(db().GetLogins(form2, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form2), &result));
   EXPECT_EQ(1U, result.size());
   result.clear();
 
   // a.b.foo.com should match.
   form2 = GetFormWithNewSignonRealm(form, "http://a.b.foo.com/");
-  EXPECT_TRUE(db().GetLogins(form2, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form2), &result));
   EXPECT_EQ(1U, result.size());
   result.clear();
 
   // a-b.foo.com should match.
   form2 = GetFormWithNewSignonRealm(form, "http://a-b.foo.com/");
-  EXPECT_TRUE(db().GetLogins(form2, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form2), &result));
   EXPECT_EQ(1U, result.size());
   result.clear();
 
   // foo-bar.com should match.
   form2 = GetFormWithNewSignonRealm(form, "http://foo-bar.com/");
-  EXPECT_TRUE(db().GetLogins(form2, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form2), &result));
   EXPECT_EQ(1U, result.size());
   result.clear();
 
   // www.foo-bar.com should match.
   form2 = GetFormWithNewSignonRealm(form, "http://www.foo-bar.com/");
-  EXPECT_TRUE(db().GetLogins(form2, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form2), &result));
   EXPECT_EQ(1U, result.size());
   result.clear();
 
   // a.b.foo-bar.com should match.
   form2 = GetFormWithNewSignonRealm(form, "http://a.b.foo-bar.com/");
-  EXPECT_TRUE(db().GetLogins(form2, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form2), &result));
   EXPECT_EQ(1U, result.size());
   result.clear();
 
   // a-b.foo-bar.com should match.
   form2 = GetFormWithNewSignonRealm(form, "http://a-b.foo-bar.com/");
-  EXPECT_TRUE(db().GetLogins(form2, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form2), &result));
   EXPECT_EQ(1U, result.size());
   result.clear();
 
   // foo.com with port 1337 should not match.
   form2 = GetFormWithNewSignonRealm(form, "http://foo.com:1337/");
-  EXPECT_TRUE(db().GetLogins(form2, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form2), &result));
   EXPECT_EQ(0U, result.size());
 
   // http://foo.com should not match since the scheme is wrong.
   form2 = GetFormWithNewSignonRealm(form, "https://foo.com/");
-  EXPECT_TRUE(db().GetLogins(form2, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form2), &result));
   EXPECT_EQ(0U, result.size());
 
   // notfoo.com should not match.
   form2 = GetFormWithNewSignonRealm(form, "http://notfoo.com/");
-  EXPECT_TRUE(db().GetLogins(form2, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form2), &result));
   EXPECT_EQ(0U, result.size());
 
   // baz.com should not match.
   form2 = GetFormWithNewSignonRealm(form, "http://baz.com/");
-  EXPECT_TRUE(db().GetLogins(form2, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form2), &result));
   EXPECT_EQ(0U, result.size());
 
   // foo-baz.com should not match.
   form2 = GetFormWithNewSignonRealm(form, "http://foo-baz.com/");
-  EXPECT_TRUE(db().GetLogins(form2, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form2), &result));
   EXPECT_EQ(0U, result.size());
 }
 
@@ -760,7 +756,7 @@ static bool AddTimestampedLogin(LoginDatabase* db,
 }
 
 TEST_F(LoginDatabaseTest, ClearPrivateData_SavedPasswords) {
-  ScopedVector<autofill::PasswordForm> result;
+  std::vector<std::unique_ptr<PasswordForm>> result;
 
   // Verify the database is empty.
   EXPECT_TRUE(db().GetAutofillableLogins(&result));
@@ -784,10 +780,13 @@ TEST_F(LoginDatabaseTest, ClearPrivateData_SavedPasswords) {
   EXPECT_EQ(4U, result.size());
   result.clear();
 
+  // TODO(crbug.com/555132) Replace |result_scopedvector| back with |result|.
+  ScopedVector<PasswordForm> result_scopedvector;
   // Get everything from today's date and on.
-  EXPECT_TRUE(db().GetLoginsCreatedBetween(now, base::Time(), &result));
-  EXPECT_EQ(2U, result.size());
-  result.clear();
+  EXPECT_TRUE(
+      db().GetLoginsCreatedBetween(now, base::Time(), &result_scopedvector));
+  EXPECT_EQ(2U, result_scopedvector.size());
+  result_scopedvector.clear();
 
   // Delete everything from today's date and on.
   db().RemoveLoginsCreatedBetween(now, base::Time());
@@ -806,7 +805,7 @@ TEST_F(LoginDatabaseTest, ClearPrivateData_SavedPasswords) {
 }
 
 TEST_F(LoginDatabaseTest, RemoveLoginsSyncedBetween) {
-  ScopedVector<autofill::PasswordForm> result;
+  std::vector<std::unique_ptr<PasswordForm>> result;
 
   base::Time now = base::Time::Now();
   base::TimeDelta one_day = base::TimeDelta::FromDays(1);
@@ -826,12 +825,15 @@ TEST_F(LoginDatabaseTest, RemoveLoginsSyncedBetween) {
   EXPECT_EQ(4U, result.size());
   result.clear();
 
+  // TODO(crbug.com/555132) Replace |result_scopedvector| back with |result|.
+  ScopedVector<PasswordForm> result_scopedvector;
   // Get everything from today's date and on.
-  EXPECT_TRUE(db().GetLoginsSyncedBetween(now, base::Time(), &result));
-  ASSERT_EQ(2U, result.size());
-  EXPECT_EQ("http://3.com", result[0]->signon_realm);
-  EXPECT_EQ("http://4.com", result[1]->signon_realm);
-  result.clear();
+  EXPECT_TRUE(
+      db().GetLoginsSyncedBetween(now, base::Time(), &result_scopedvector));
+  ASSERT_EQ(2U, result_scopedvector.size());
+  EXPECT_EQ("http://3.com", result_scopedvector[0]->signon_realm);
+  EXPECT_EQ("http://4.com", result_scopedvector[1]->signon_realm);
+  result_scopedvector.clear();
 
   // Delete everything from today's date and on.
   db().RemoveLoginsSyncedBetween(now, base::Time());
@@ -852,43 +854,53 @@ TEST_F(LoginDatabaseTest, RemoveLoginsSyncedBetween) {
 }
 
 TEST_F(LoginDatabaseTest, GetAutoSignInLogins) {
-  ScopedVector<autofill::PasswordForm> result;
+  ScopedVector<PasswordForm> result;
 
-  EXPECT_TRUE(AddZeroClickableLogin(&db(), "foo1"));
-  EXPECT_TRUE(AddZeroClickableLogin(&db(), "foo2"));
-  EXPECT_TRUE(AddZeroClickableLogin(&db(), "foo3"));
-  EXPECT_TRUE(AddZeroClickableLogin(&db(), "foo4"));
+  GURL origin("https://example.com");
+  EXPECT_TRUE(AddZeroClickableLogin(&db(), "foo1", origin));
+  EXPECT_TRUE(AddZeroClickableLogin(&db(), "foo2", origin));
+  EXPECT_TRUE(AddZeroClickableLogin(&db(), "foo3", origin));
+  EXPECT_TRUE(AddZeroClickableLogin(&db(), "foo4", origin));
 
   EXPECT_TRUE(db().GetAutoSignInLogins(&result));
   EXPECT_EQ(4U, result.size());
-  for (const auto& form : result)
+  for (const auto* form : result)
     EXPECT_FALSE(form->skip_zero_click);
 
-  EXPECT_TRUE(db().DisableAutoSignInForAllLogins());
+  EXPECT_TRUE(db().DisableAutoSignInForOrigin(origin));
   EXPECT_TRUE(db().GetAutoSignInLogins(&result));
   EXPECT_EQ(0U, result.size());
 }
 
-TEST_F(LoginDatabaseTest, DisableAutoSignInForAllLogins) {
-  ScopedVector<autofill::PasswordForm> result;
+TEST_F(LoginDatabaseTest, DisableAutoSignInForOrigin) {
+  std::vector<std::unique_ptr<PasswordForm>> result;
 
-  EXPECT_TRUE(AddZeroClickableLogin(&db(), "foo1"));
-  EXPECT_TRUE(AddZeroClickableLogin(&db(), "foo2"));
-  EXPECT_TRUE(AddZeroClickableLogin(&db(), "foo3"));
-  EXPECT_TRUE(AddZeroClickableLogin(&db(), "foo4"));
+  GURL origin1("https://google.com");
+  GURL origin2("https://chrome.com");
+  GURL origin3("http://example.com");
+  GURL origin4("http://localhost");
+  EXPECT_TRUE(AddZeroClickableLogin(&db(), "foo1", origin1));
+  EXPECT_TRUE(AddZeroClickableLogin(&db(), "foo2", origin2));
+  EXPECT_TRUE(AddZeroClickableLogin(&db(), "foo3", origin3));
+  EXPECT_TRUE(AddZeroClickableLogin(&db(), "foo4", origin4));
 
   EXPECT_TRUE(db().GetAutofillableLogins(&result));
   for (const auto& form : result)
     EXPECT_FALSE(form->skip_zero_click);
 
-  EXPECT_TRUE(db().DisableAutoSignInForAllLogins());
+  EXPECT_TRUE(db().DisableAutoSignInForOrigin(origin1));
+  EXPECT_TRUE(db().DisableAutoSignInForOrigin(origin3));
   EXPECT_TRUE(db().GetAutofillableLogins(&result));
-  for (const auto& form : result)
-    EXPECT_TRUE(form->skip_zero_click);
+  for (const auto& form : result) {
+    if (form->origin == origin1 || form->origin == origin3)
+      EXPECT_TRUE(form->skip_zero_click);
+    else
+      EXPECT_FALSE(form->skip_zero_click);
+  }
 }
 
 TEST_F(LoginDatabaseTest, BlacklistedLogins) {
-  ScopedVector<autofill::PasswordForm> result;
+  std::vector<std::unique_ptr<PasswordForm>> result;
 
   // Verify the database is empty.
   EXPECT_TRUE(db().GetBlacklistLogins(&result));
@@ -902,7 +914,6 @@ TEST_F(LoginDatabaseTest, BlacklistedLogins) {
   form.password_element = ASCIIToUTF16("Passwd");
   form.submit_element = ASCIIToUTF16("signIn");
   form.signon_realm = "http://www.google.com/";
-  form.ssl_valid = false;
   form.preferred = true;
   form.blacklisted_by_user = true;
   form.scheme = PasswordForm::SCHEME_HTML;
@@ -918,12 +929,12 @@ TEST_F(LoginDatabaseTest, BlacklistedLogins) {
   ASSERT_EQ(0U, result.size());
 
   // GetLogins should give the blacklisted result.
-  EXPECT_TRUE(db().GetLogins(form, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form), &result));
   ASSERT_EQ(1U, result.size());
   EXPECT_EQ(form, *result[0]);
   result.clear();
 
-  // So should GetAllBlacklistedLogins.
+  // So should GetBlacklistedLogins.
   EXPECT_TRUE(db().GetBlacklistLogins(&result));
   ASSERT_EQ(1U, result.size());
   EXPECT_EQ(form, *result[0]);
@@ -948,7 +959,7 @@ TEST_F(LoginDatabaseTest, VectorSerialization) {
 }
 
 TEST_F(LoginDatabaseTest, UpdateIncompleteCredentials) {
-  ScopedVector<autofill::PasswordForm> result;
+  std::vector<std::unique_ptr<PasswordForm>> result;
   // Verify the database is empty.
   EXPECT_TRUE(db().GetAutofillableLogins(&result));
   ASSERT_EQ(0U, result.size());
@@ -962,7 +973,6 @@ TEST_F(LoginDatabaseTest, UpdateIncompleteCredentials) {
   incomplete_form.signon_realm = "http://accounts.google.com/";
   incomplete_form.username_value = ASCIIToUTF16("my_username");
   incomplete_form.password_value = ASCIIToUTF16("my_password");
-  incomplete_form.ssl_valid = false;
   incomplete_form.preferred = true;
   incomplete_form.blacklisted_by_user = false;
   incomplete_form.scheme = PasswordForm::SCHEME_HTML;
@@ -978,14 +988,14 @@ TEST_F(LoginDatabaseTest, UpdateIncompleteCredentials) {
   encountered_form.submit_element = ASCIIToUTF16("signIn");
 
   // Get matches for encountered_form.
-  EXPECT_TRUE(db().GetLogins(encountered_form, &result));
+  EXPECT_TRUE(
+      db().GetLogins(PasswordStore::FormDigest(encountered_form), &result));
   ASSERT_EQ(1U, result.size());
   EXPECT_EQ(incomplete_form.origin, result[0]->origin);
   EXPECT_EQ(incomplete_form.signon_realm, result[0]->signon_realm);
   EXPECT_EQ(incomplete_form.username_value, result[0]->username_value);
   EXPECT_EQ(incomplete_form.password_value, result[0]->password_value);
   EXPECT_TRUE(result[0]->preferred);
-  EXPECT_FALSE(result[0]->ssl_valid);
 
   // We should return empty 'action', 'username_element', 'password_element'
   // and 'submit_element' as we can't be sure if the credentials were entered
@@ -1008,7 +1018,8 @@ TEST_F(LoginDatabaseTest, UpdateIncompleteCredentials) {
   EXPECT_TRUE(db().RemoveLogin(incomplete_form));
 
   // Get matches for encountered_form again.
-  EXPECT_TRUE(db().GetLogins(encountered_form, &result));
+  EXPECT_TRUE(
+      db().GetLogins(PasswordStore::FormDigest(encountered_form), &result));
   ASSERT_EQ(1U, result.size());
 
   // This time we should have all the info available.
@@ -1027,7 +1038,6 @@ TEST_F(LoginDatabaseTest, UpdateOverlappingCredentials) {
   incomplete_form.signon_realm = "http://accounts.google.com/";
   incomplete_form.username_value = ASCIIToUTF16("my_username");
   incomplete_form.password_value = ASCIIToUTF16("my_password");
-  incomplete_form.ssl_valid = false;
   incomplete_form.preferred = true;
   incomplete_form.blacklisted_by_user = false;
   incomplete_form.scheme = PasswordForm::SCHEME_HTML;
@@ -1047,7 +1057,7 @@ TEST_F(LoginDatabaseTest, UpdateOverlappingCredentials) {
   EXPECT_EQ(AddChangeForForm(complete_form), db().AddLogin(complete_form));
 
   // Make sure both passwords exist.
-  ScopedVector<autofill::PasswordForm> result;
+  std::vector<std::unique_ptr<PasswordForm>> result;
   EXPECT_TRUE(db().GetAutofillableLogins(&result));
   ASSERT_EQ(2U, result.size());
   result.clear();
@@ -1074,7 +1084,6 @@ TEST_F(LoginDatabaseTest, DoubleAdd) {
   form.signon_realm = "http://accounts.google.com/";
   form.username_value = ASCIIToUTF16("my_username");
   form.password_value = ASCIIToUTF16("my_password");
-  form.ssl_valid = false;
   form.preferred = true;
   form.blacklisted_by_user = false;
   form.scheme = PasswordForm::SCHEME_HTML;
@@ -1095,7 +1104,6 @@ TEST_F(LoginDatabaseTest, AddWrongForm) {
   form.signon_realm = "http://accounts.google.com/";
   form.username_value = ASCIIToUTF16("my_username");
   form.password_value = ASCIIToUTF16("my_password");
-  form.ssl_valid = false;
   form.preferred = true;
   form.blacklisted_by_user = false;
   form.scheme = PasswordForm::SCHEME_HTML;
@@ -1113,7 +1121,6 @@ TEST_F(LoginDatabaseTest, UpdateLogin) {
   form.signon_realm = "http://accounts.google.com/";
   form.username_value = ASCIIToUTF16("my_username");
   form.password_value = ASCIIToUTF16("my_password");
-  form.ssl_valid = false;
   form.preferred = true;
   form.blacklisted_by_user = false;
   form.scheme = PasswordForm::SCHEME_HTML;
@@ -1121,7 +1128,6 @@ TEST_F(LoginDatabaseTest, UpdateLogin) {
 
   form.action = GURL("http://accounts.google.com/login");
   form.password_value = ASCIIToUTF16("my_new_password");
-  form.ssl_valid = true;
   form.preferred = false;
   form.other_possible_usernames.push_back(ASCIIToUTF16("my_new_username"));
   form.times_used = 20;
@@ -1137,8 +1143,8 @@ TEST_F(LoginDatabaseTest, UpdateLogin) {
   form.skip_zero_click = true;
   EXPECT_EQ(UpdateChangeForForm(form), db().UpdateLogin(form));
 
-  ScopedVector<autofill::PasswordForm> result;
-  EXPECT_TRUE(db().GetLogins(form, &result));
+  std::vector<std::unique_ptr<PasswordForm>> result;
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form), &result));
   ASSERT_EQ(1U, result.size());
   EXPECT_EQ(form, *result[0]);
 }
@@ -1150,7 +1156,6 @@ TEST_F(LoginDatabaseTest, RemoveWrongForm) {
   form.signon_realm = "http://accounts.google.com/";
   form.username_value = ASCIIToUTF16("my_username");
   form.password_value = ASCIIToUTF16("my_password");
-  form.ssl_valid = false;
   form.preferred = true;
   form.blacklisted_by_user = false;
   form.scheme = PasswordForm::SCHEME_HTML;
@@ -1459,8 +1464,8 @@ TEST_F(LoginDatabaseTest, ClearPasswordValues) {
   form.password_value = ASCIIToUTF16("12345");
   EXPECT_EQ(AddChangeForForm(form), db().AddLogin(form));
 
-  ScopedVector<autofill::PasswordForm> result;
-  EXPECT_TRUE(db().GetLogins(form, &result));
+  std::vector<std::unique_ptr<PasswordForm>> result;
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form), &result));
   ASSERT_EQ(1U, result.size());
   PasswordForm expected_form = form;
   expected_form.password_value.clear();
@@ -1469,7 +1474,7 @@ TEST_F(LoginDatabaseTest, ClearPasswordValues) {
   // Update the password, it should stay empty.
   form.password_value = ASCIIToUTF16("password");
   EXPECT_EQ(UpdateChangeForForm(form), db().UpdateLogin(form));
-  EXPECT_TRUE(db().GetLogins(form, &result));
+  EXPECT_TRUE(db().GetLogins(PasswordStore::FormDigest(form), &result));
   ASSERT_EQ(1U, result.size());
   EXPECT_EQ(expected_form, *result[0]);
 
@@ -1491,6 +1496,30 @@ TEST_F(LoginDatabaseTest, FilePermissions) {
 }
 #endif  // defined(OS_POSIX)
 
+// If the database initialisation fails, the initialisation transaction should
+// roll back without crashing.
+TEST(LoginDatabaseFailureTest, Init_NoCrashOnFailedRollback) {
+  base::ScopedTempDir temp_dir;
+  ASSERT_TRUE(temp_dir.CreateUniqueTempDir());
+  base::FilePath database_path = temp_dir.GetPath().AppendASCII("test.db");
+
+  // To cause an init failure, set the compatible version to be higher than the
+  // current version (in reality, this could happen if, e.g., someone opened a
+  // Canary-created profile with Chrome Stable.
+  {
+    sql::Connection connection;
+    sql::MetaTable meta_table;
+    ASSERT_TRUE(connection.Open(database_path));
+    ASSERT_TRUE(meta_table.Init(&connection, kCurrentVersionNumber + 1,
+                                kCompatibleVersionNumber + 1));
+  }
+
+  // Now try to init the database with the file. The test succeeds if it does
+  // not crash.
+  LoginDatabase db(database_path);
+  EXPECT_FALSE(db.Init());
+}
+
 // Test the migration from GetParam() version to kCurrentVersionNumber.
 class LoginDatabaseMigrationTest : public testing::TestWithParam<int> {
  protected:
@@ -1500,7 +1529,7 @@ class LoginDatabaseMigrationTest : public testing::TestWithParam<int> {
                                   .AppendASCII("test")
                                   .AppendASCII("data")
                                   .AppendASCII("password_manager");
-    database_path_ = temp_dir_.path().AppendASCII("test.db");
+    database_path_ = temp_dir_.GetPath().AppendASCII("test.db");
     OSCryptMocker::SetUpWithSingleton();
   }
 
@@ -1567,16 +1596,21 @@ void LoginDatabaseMigrationTest::MigrationToVCurrent(
   CreateDatabase(sql_file);
   // Original date, in seconds since UTC epoch.
   std::vector<int64_t> date_created(GetValues<int64_t>("date_created"));
-  ASSERT_EQ(2U, date_created.size());
+  if (version() == 10)  // Version 10 has a duplicate entry.
+    ASSERT_EQ(4U, date_created.size());
+  else
+    ASSERT_EQ(3U, date_created.size());
   // Migration to version 8 performs changes dates to the new format.
   // So for versions less of equal to 8 create date should be in old
   // format before migration and in new format after.
   if (version() <= 8) {
     ASSERT_EQ(1402955745, date_created[0]);
     ASSERT_EQ(1402950000, date_created[1]);
+    ASSERT_EQ(1402950000, date_created[2]);
   } else {
     ASSERT_EQ(13047429345000000, date_created[0]);
     ASSERT_EQ(13047423600000000, date_created[1]);
+    ASSERT_EQ(13047423600000000, date_created[2]);
   }
 
   {
@@ -1584,6 +1618,14 @@ void LoginDatabaseMigrationTest::MigrationToVCurrent(
     // to current version.
     LoginDatabase db(database_path_);
     ASSERT_TRUE(db.Init());
+
+    // Check that the contents was preserved.
+    std::vector<std::unique_ptr<PasswordForm>> result;
+    EXPECT_TRUE(db.GetAutofillableLogins(&result));
+    EXPECT_THAT(result, UnorderedElementsAre(Pointee(IsGoogle1Account()),
+                                             Pointee(IsGoogle2Account()),
+                                             Pointee(IsBasicAuthAccount())));
+
     // Verifies that the final version can save all the appropriate fields.
     PasswordForm form;
     GenerateExamplePasswordForm(&form);
@@ -1594,28 +1636,25 @@ void LoginDatabaseMigrationTest::MigrationToVCurrent(
     list.push_back(PasswordStoreChange(PasswordStoreChange::ADD, form));
     EXPECT_EQ(list, db.AddLogin(form));
 
-    ScopedVector<autofill::PasswordForm> result;
-    EXPECT_TRUE(db.GetLogins(form, &result));
+    result.clear();
+    EXPECT_TRUE(db.GetLogins(PasswordStore::FormDigest(form), &result));
     ASSERT_EQ(1U, result.size());
     EXPECT_EQ(form, *result[0]);
     EXPECT_TRUE(db.RemoveLogin(form));
   }
   // New date, in microseconds since platform independent epoch.
   std::vector<int64_t> new_date_created(GetValues<int64_t>("date_created"));
+  ASSERT_EQ(3U, new_date_created.size());
   if (version() <= 8) {
-    ASSERT_EQ(2U, new_date_created.size());
     // Check that the two dates match up.
     for (size_t i = 0; i < date_created.size(); ++i) {
       EXPECT_EQ(base::Time::FromInternalValue(new_date_created[i]),
                 base::Time::FromTimeT(date_created[i]));
     }
-  } else if (version() == 10) {
-    // The test data is setup on this version to cause a unique key collision.
-    EXPECT_EQ(1U, new_date_created.size());
   } else {
-    ASSERT_EQ(2U, new_date_created.size());
     ASSERT_EQ(13047429345000000, new_date_created[0]);
     ASSERT_EQ(13047423600000000, new_date_created[1]);
+    ASSERT_EQ(13047423600000000, new_date_created[2]);
   }
 
   if (version() >= 7 && version() <= 13) {
@@ -1624,15 +1663,8 @@ void LoginDatabaseMigrationTest::MigrationToVCurrent(
     // to >= 14 should not break theses URLs.
     std::vector<std::string> urls(GetValues<std::string>("icon_url"));
 
-    if (version() == 10) {
-      // The testcase for version 10 tests duplicate entries, so we only expect
-      // one URL.
-      EXPECT_THAT(urls, testing::ElementsAre("https://www.google.com/icon"));
-    } else {
-      // Otherwise, we expect one empty and one valid URL.
-      EXPECT_THAT(
-          urls, testing::ElementsAre("", "https://www.google.com/icon"));
-    }
+    EXPECT_THAT(urls, UnorderedElementsAre("", "https://www.google.com/icon",
+                                           "https://www.google.com/icon"));
   }
 
   {
