@@ -10,11 +10,10 @@
 #include <map>
 #include <utility>
 
-#include "base/command_line.h"
 #include "base/feature_list.h"
 #include "base/memory/ptr_util.h"
-#include "base/metrics/field_trial.h"
 #include "base/metrics/histogram_macros.h"
+#include "base/metrics/user_metrics.h"
 #include "base/strings/string16.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
@@ -23,11 +22,9 @@
 #include "components/autofill/core/browser/autofill_manager.h"
 #include "components/autofill/core/browser/proto/server.pb.h"
 #include "components/autofill/core/browser/validation.h"
-#include "components/autofill/core/common/autofill_switches.h"
 #include "components/autofill/core/common/password_form.h"
 #include "components/password_manager/core/browser/affiliation_utils.h"
 #include "components/password_manager/core/browser/browser_save_password_progress_logger.h"
-#include "components/password_manager/core/browser/credentials_filter.h"
 #include "components/password_manager/core/browser/form_saver.h"
 #include "components/password_manager/core/browser/log_manager.h"
 #include "components/password_manager/core/browser/password_manager.h"
@@ -41,7 +38,6 @@
 
 using autofill::FormStructure;
 using autofill::PasswordForm;
-using autofill::PasswordFormMap;
 using base::Time;
 
 // Shorten the name to spare line breaks. The code provides enough context
@@ -52,27 +48,6 @@ namespace password_manager {
 
 namespace {
 
-PasswordForm CopyAndModifySSLValidity(const PasswordForm& orig,
-                                      bool ssl_valid) {
-  PasswordForm result(orig);
-  result.ssl_valid = ssl_valid;
-  return result;
-}
-
-// Returns true if user-typed username and password field values match with one
-// of the password form within |credentials| map; otherwise false.
-bool DoesUsenameAndPasswordMatchCredentials(
-    const base::string16& typed_username,
-    const base::string16& typed_password,
-    const autofill::PasswordFormMap& credentials) {
-  for (const auto& match : credentials) {
-    if (match.second->username_value == typed_username &&
-        match.second->password_value == typed_password)
-      return true;
-  }
-  return false;
-}
-
 std::vector<std::string> SplitPathToSegments(const std::string& path) {
   return base::SplitString(path, "/", base::TRIM_WHITESPACE,
                            base::SPLIT_WANT_ALL);
@@ -97,27 +72,6 @@ bool IsProbablyNotUsername(const base::string16& s) {
   return !s.empty() && DoesStringContainOnlyDigits(s) && s.size() < 3;
 }
 
-// Splits federated matches from |store_results| into a separate vector and
-// returns that.
-std::vector<std::unique_ptr<autofill::PasswordForm>> SplitFederatedMatches(
-    ScopedVector<PasswordForm>* store_results) {
-  auto first_federated =
-      std::partition(store_results->begin(), store_results->end(),
-                     [](const PasswordForm* form) {
-                       return form->federation_origin.unique();
-                     });
-
-  std::vector<std::unique_ptr<autofill::PasswordForm>> federated_matches;
-  federated_matches.reserve(store_results->end() - first_federated);
-  for (auto federated = first_federated; federated != store_results->end();
-       ++federated) {
-    federated_matches.push_back(base::WrapUnique(*federated));
-    *federated = nullptr;
-  }
-  store_results->weak_erase(first_federated, store_results->end());
-  return federated_matches;
-}
-
 bool ShouldShowInitialPasswordAccountSuggestions() {
   return base::FeatureList::IsEnabled(
       password_manager::features::kFillOnAccountSelect);
@@ -134,12 +88,13 @@ void UpdateMetadataForUsage(PasswordForm* credential) {
 
 // Returns true iff |best_matches| contain a preferred credential with a
 // username other than |preferred_username|.
-bool DidPreferenceChange(const autofill::PasswordFormMap& best_matches,
-                         const base::string16& preferred_username) {
+bool DidPreferenceChange(
+    const std::map<base::string16, const PasswordForm*>& best_matches,
+    const base::string16& preferred_username) {
   for (const auto& key_value_pair : best_matches) {
-    const auto& form = key_value_pair.second;
-    if (form->preferred && !form->is_public_suffix_match &&
-        form->username_value != preferred_username) {
+    const PasswordForm& form = *key_value_pair.second;
+    if (form.preferred && !form.is_public_suffix_match &&
+        form.username_value != preferred_username) {
       return true;
     }
   }
@@ -165,6 +120,20 @@ void SanitizePossibleUsernames(PasswordForm* form) {
   usernames.erase(new_end, usernames.end());
 }
 
+// Copies field properties masks from the form |from| to the form |to|.
+void CopyFieldPropertiesMasks(const PasswordForm& from, PasswordForm* to) {
+  // Skip copying if the number of fields is different.
+  if (from.form_data.fields.size() != to->form_data.fields.size())
+    return;
+
+  for (size_t i = 0; i < from.form_data.fields.size(); ++i) {
+    to->form_data.fields[i].properties_mask =
+        to->form_data.fields[i].name == from.form_data.fields[i].name
+            ? from.form_data.fields[i].properties_mask
+            : autofill::FieldPropertiesFlags::ERROR_OCCURRED;
+  }
+}
+
 }  // namespace
 
 PasswordFormManager::PasswordFormManager(
@@ -172,9 +141,8 @@ PasswordFormManager::PasswordFormManager(
     PasswordManagerClient* client,
     const base::WeakPtr<PasswordManagerDriver>& driver,
     const PasswordForm& observed_form,
-    bool ssl_valid,
     std::unique_ptr<FormSaver> form_saver)
-    : observed_form_(CopyAndModifySSLValidity(observed_form, ssl_valid)),
+    : observed_form_(observed_form),
       provisionally_saved_form_(nullptr),
       other_possible_username_action_(
           PasswordFormManager::IGNORE_OTHER_POSSIBLE_USERNAMES),
@@ -192,26 +160,28 @@ PasswordFormManager::PasswordFormManager(
       generation_available_(false),
       password_manager_(password_manager),
       preferred_match_(nullptr),
-      is_ignorable_change_password_form_(false),
       is_possible_change_password_form_without_username_(
           observed_form.IsPossibleChangePasswordFormWithoutUsername()),
-      state_(PRE_MATCHING_PHASE),
       client_(client),
       manager_action_(kManagerActionNone),
       user_action_(kUserActionNone),
       submit_result_(kSubmitResultNotSubmitted),
       form_type_(kFormTypeUnspecified),
       need_to_refetch_(false),
-      form_saver_(std::move(form_saver)) {
+      form_saver_(std::move(form_saver)),
+      form_fetcher_impl_(client),
+      form_fetcher_(&form_fetcher_impl_) {
   DCHECK_EQ(observed_form.scheme == PasswordForm::SCHEME_HTML,
             driver != nullptr);
   if (driver)
     drivers_.push_back(driver);
+  FetchDataFromPasswordStore();
+  form_fetcher_->AddConsumer(this);
 }
 
 PasswordFormManager::~PasswordFormManager() {
-  UMA_HISTOGRAM_ENUMERATION(
-      "PasswordManager.ActionsTakenV3", GetActionsTaken(), kMaxNumActionsTaken);
+  UMA_HISTOGRAM_ENUMERATION("PasswordManager.ActionsTakenV3", GetActionsTaken(),
+                            kMaxNumActionsTaken);
   if (submit_result_ == kSubmitResultNotSubmitted) {
     if (has_generated_password_)
       metrics_util::LogPasswordGenerationSubmissionEvent(
@@ -266,10 +236,10 @@ PasswordFormManager::MatchResultMask PasswordFormManager::DoesManage(
   // redirects to HTTPS (as in http://example.org -> https://example.org/auth).
   if (!origins_match && !observed_form_.origin.SchemeIsCryptographic() &&
       form.origin.SchemeIsCryptographic()) {
-    const std::string& old_path = observed_form_.origin.path();
-    const std::string& new_path = form.origin.path();
+    const base::StringPiece& old_path = observed_form_.origin.path_piece();
+    const base::StringPiece& new_path = form.origin.path_piece();
     origins_match =
-        observed_form_.origin.host() == form.origin.host() &&
+        observed_form_.origin.host_piece() == form.origin.host_piece() &&
         observed_form_.origin.port() == form.origin.port() &&
         base::StartsWith(new_path, old_path, base::CompareCase::SENSITIVE);
   }
@@ -298,21 +268,23 @@ PasswordFormManager::MatchResultMask PasswordFormManager::DoesManage(
 }
 
 bool PasswordFormManager::IsBlacklisted() const {
-  DCHECK_EQ(state_, POST_MATCHING_PHASE);
+  DCHECK_EQ(FormFetcher::State::NOT_WAITING, form_fetcher_->GetState());
   return !blacklisted_matches_.empty();
 }
 
 void PasswordFormManager::PermanentlyBlacklist() {
-  DCHECK_EQ(state_, POST_MATCHING_PHASE);
+  DCHECK_EQ(FormFetcher::State::NOT_WAITING, form_fetcher_->GetState());
   DCHECK(!client_->IsOffTheRecord());
 
-  blacklisted_matches_.push_back(
-      base::WrapUnique(new autofill::PasswordForm(observed_form_)));
-  form_saver_->PermanentlyBlacklist(blacklisted_matches_.back().get());
+  if (!new_blacklisted_) {
+    new_blacklisted_ = base::MakeUnique<PasswordForm>(observed_form_);
+    blacklisted_matches_.push_back(new_blacklisted_.get());
+  }
+  form_saver_->PermanentlyBlacklist(new_blacklisted_.get());
 }
 
 bool PasswordFormManager::IsNewLogin() const {
-  DCHECK_EQ(state_, POST_MATCHING_PHASE);
+  DCHECK_EQ(FormFetcher::State::NOT_WAITING, form_fetcher_->GetState());
   return is_new_login_;
 }
 
@@ -323,7 +295,6 @@ bool PasswordFormManager::IsPendingCredentialsPublicSuffixMatch() const {
 void PasswordFormManager::ProvisionallySave(
     const PasswordForm& credentials,
     OtherPossibleUsernamesAction action) {
-  DCHECK(state_ == MATCHING_PHASE || state_ == POST_MATCHING_PHASE) << state_;
   DCHECK_NE(RESULT_NO_MATCH, DoesManage(credentials));
 
   std::unique_ptr<autofill::PasswordForm> mutable_provisionally_saved_form(
@@ -344,12 +315,12 @@ void PasswordFormManager::ProvisionallySave(
 }
 
 void PasswordFormManager::Save() {
-  DCHECK_EQ(state_, POST_MATCHING_PHASE);
+  DCHECK_EQ(FormFetcher::State::NOT_WAITING, form_fetcher_->GetState());
   DCHECK(!client_->IsOffTheRecord());
 
   if ((user_action_ == kUserActionNone) &&
       DidPreferenceChange(best_matches_, pending_credentials_.username_value)) {
-    user_action_ = kUserActionChoose;
+    SetUserAction(kUserActionChoose);
   }
   base::Optional<PasswordForm> old_primary_key;
   if (is_new_login_) {
@@ -360,7 +331,7 @@ void PasswordFormManager::Save() {
                       old_primary_key ? &old_primary_key.value() : nullptr);
   } else {
     ProcessUpdate();
-    std::vector<const PasswordForm*> credentials_to_update;
+    std::vector<PasswordForm> credentials_to_update;
     old_primary_key = UpdatePendingAndGetOldKey(&credentials_to_update);
     form_saver_->Update(pending_credentials_, best_matches_,
                         &credentials_to_update,
@@ -382,7 +353,7 @@ void PasswordFormManager::Update(
   if (observed_form_.IsPossibleChangePasswordForm()) {
     FormStructure form_structure(credentials_to_update.form_data);
     UploadChangePasswordForm(autofill::NEW_PASSWORD,
-                             form_structure.FormSignature());
+                             form_structure.FormSignatureAsStr());
   }
   base::string16 password_to_save = pending_credentials_.password_value;
   bool skip_zero_click = pending_credentials_.skip_zero_click;
@@ -392,7 +363,7 @@ void PasswordFormManager::Update(
   pending_credentials_.preferred = true;
   is_new_login_ = false;
   ProcessUpdate();
-  std::vector<const PasswordForm*> more_credentials_to_update;
+  std::vector<PasswordForm> more_credentials_to_update;
   base::Optional<PasswordForm> old_primary_key =
       UpdatePendingAndGetOldKey(&more_credentials_to_update);
   form_saver_->Update(pending_credentials_, best_matches_,
@@ -401,7 +372,7 @@ void PasswordFormManager::Update(
 }
 
 void PasswordFormManager::FetchDataFromPasswordStore() {
-  if (state_ == MATCHING_PHASE) {
+  if (form_fetcher_->GetState() == FormFetcher::State::WAITING) {
     // There is currently a password store query in progress, need to re-fetch
     // store results later.
     need_to_refetch_ = true;
@@ -413,20 +384,23 @@ void PasswordFormManager::FetchDataFromPasswordStore() {
     logger.reset(
         new BrowserSavePasswordProgressLogger(client_->GetLogManager()));
     logger->LogMessage(Logger::STRING_FETCH_LOGINS_METHOD);
-    logger->LogNumber(Logger::STRING_FORM_MANAGER_STATE, state_);
+    logger->LogNumber(Logger::STRING_FORM_MANAGER_STATE,
+                      static_cast<int>(form_fetcher_->GetState()));
   }
 
   provisionally_saved_form_.reset();
-  state_ = MATCHING_PHASE;
+  form_fetcher_impl_.set_state(FormFetcher::State::WAITING);
 
   PasswordStore* password_store = client_->GetPasswordStore();
   if (!password_store) {
     if (logger)
       logger->LogMessage(Logger::STRING_NO_STORE);
-    NOTREACHED();
+    // TODO(crbug.com/621355): The store might be empty in some tests. Start
+    // enforcing the presence of a (non-null) PasswordStore once FormFetcher is
+    // fetching the credentials instead of PasswordFormManager.
     return;
   }
-  password_store->GetLogins(observed_form_, this);
+  password_store->GetLogins(PasswordStore::FormDigest(observed_form_), this);
 
 // The statistics isn't needed on mobile, only on desktop. Let's save some
 // processor cycles.
@@ -437,28 +411,18 @@ void PasswordFormManager::FetchDataFromPasswordStore() {
 }
 
 bool PasswordFormManager::HasCompletedMatching() const {
-  return state_ == POST_MATCHING_PHASE;
+  return form_fetcher_->GetState() == FormFetcher::State::NOT_WAITING;
 }
 
 void PasswordFormManager::SetSubmittedForm(const autofill::PasswordForm& form) {
   bool is_change_password_form =
       !form.new_password_value.empty() && !form.password_value.empty();
-  is_ignorable_change_password_form_ =
-      is_change_password_form && !form.username_marked_by_site &&
-      !DoesUsenameAndPasswordMatchCredentials(
-          form.username_value, form.password_value, best_matches_) &&
-      !client_->IsUpdatePasswordUIEnabled();
   bool is_signup_form =
       !form.new_password_value.empty() && form.password_value.empty();
   bool no_username = form.username_element.empty();
 
   if (form.layout == PasswordForm::Layout::LAYOUT_LOGIN_AND_SIGNUP) {
     form_type_ = kFormTypeLoginAndSignup;
-  } else if (is_ignorable_change_password_form_) {
-    if (no_username)
-      form_type_ = kFormTypeChangePasswordNoUsername;
-    else
-      form_type_ = kFormTypeChangePasswordDisabled;
   } else if (is_change_password_form) {
     form_type_ = kFormTypeChangePasswordEnabled;
   } else if (is_signup_form) {
@@ -473,91 +437,109 @@ void PasswordFormManager::SetSubmittedForm(const autofill::PasswordForm& form) {
   }
 }
 
-void PasswordFormManager::OnRequestDone(
-    ScopedVector<PasswordForm> logins_result) {
+void PasswordFormManager::ScoreMatches(
+    const std::vector<const PasswordForm*>& matches) {
+  DCHECK(std::all_of(
+      matches.begin(), matches.end(),
+      [](const PasswordForm* match) { return !match->blacklisted_by_user; }));
+
   preferred_match_ = nullptr;
   best_matches_.clear();
-  blacklisted_matches_.clear();
-  const size_t logins_result_size = logins_result.size();
+  not_best_matches_.clear();
 
-  std::unique_ptr<BrowserSavePasswordProgressLogger> logger;
-  if (password_manager_util::IsLoggingActive(client_)) {
-    logger.reset(
-        new BrowserSavePasswordProgressLogger(client_->GetLogManager()));
-    logger->LogMessage(Logger::STRING_ON_REQUEST_DONE_METHOD);
-  }
-
-  // Remove credentials which need to be ignored from |logins_result|.
-  if (!observed_form_.ssl_valid) {
-    logins_result.erase(
-        std::partition(logins_result.begin(), logins_result.end(),
-                       [](PasswordForm* form) { return !form->ssl_valid; }),
-        logins_result.end());
-  }
-  logins_result =
-      client_->GetStoreResultFilter()->FilterResults(std::move(logins_result));
-
-  // Deal with blacklisted forms.
-  auto begin_blacklisted = std::partition(
-      logins_result.begin(), logins_result.end(),
-      [](PasswordForm* form) { return !form->blacklisted_by_user; });
-  for (auto it = begin_blacklisted; it != logins_result.end(); ++it) {
-    if (IsBlacklistMatch(**it)) {
-      blacklisted_matches_.push_back(base::WrapUnique(*it));
-      *it = nullptr;
-    }
-  }
-  logins_result.erase(begin_blacklisted, logins_result.end());
-  if (logins_result.empty())
+  if (matches.empty())
     return;
 
-  // Now compute scores for the remaining credentials in |login_result|.
-  std::vector<uint32_t> credential_scores;
-  credential_scores.reserve(logins_result.size());
-  uint32_t best_score = 0;
-  std::map<base::string16, uint32_t> best_scores;
-  for (const PasswordForm* login : logins_result) {
-    uint32_t current_score = ScoreResult(*login);
-    best_score = std::max(best_score, current_score);
-    best_scores[login->username_value] =
-        std::max(best_scores[login->username_value], current_score);
-    credential_scores.push_back(current_score);
+  // Compute scores.
+  std::vector<uint32_t> credential_scores(matches.size());
+  std::transform(
+      matches.begin(), matches.end(), credential_scores.begin(),
+      [this](const PasswordForm* match) { return ScoreResult(*match); });
+
+  const uint32_t best_score =
+      *std::max_element(credential_scores.begin(), credential_scores.end());
+
+  std::map<base::string16, uint32_t> best_scores;  // best scores for usernames
+
+  for (size_t i = 0; i < matches.size(); ++i) {
+    uint32_t& score = best_scores[matches[i]->username_value];
+    score = std::max(score, credential_scores[i]);
   }
 
+  // Assign best, non-best and preferred matches.
+  not_best_matches_.reserve(matches.size() - best_scores.size());
   // Fill |best_matches_| with the best-scoring credentials for each username.
-  for (size_t i = 0; i < logins_result.size(); ++i) {
-    // Take ownership of the PasswordForm from the ScopedVector.
-    std::unique_ptr<PasswordForm> login(logins_result[i]);
-    logins_result[i] = nullptr;
-    DCHECK(!login->blacklisted_by_user);
-    const base::string16& username = login->username_value;
+  for (size_t i = 0; i < matches.size(); ++i) {
+    const PasswordForm* const match = matches[i];
+    const base::string16& username = match->username_value;
 
     if (credential_scores[i] < best_scores[username]) {
-      not_best_matches_.push_back(std::move(login));
+      not_best_matches_.push_back(match);
       continue;
     }
 
     if (!preferred_match_ && credential_scores[i] == best_score)
-      preferred_match_ = login.get();
+      preferred_match_ = match;
 
     // If there is another best-score match for the same username then leave it
     // and add the current form to |not_best_matches_|.
     auto best_match_username = best_matches_.find(username);
     if (best_match_username == best_matches_.end()) {
-      best_matches_.insert(std::make_pair(username, std::move(login)));
+      best_matches_.insert(std::make_pair(username, match));
     } else {
-      not_best_matches_.push_back(std::move(login));
+      not_best_matches_.push_back(match);
     }
   }
+}
 
-  UMA_HISTOGRAM_COUNTS("PasswordManager.NumPasswordsNotShown",
-                       logins_result_size - best_matches_.size());
+void PasswordFormManager::ProcessMatches(
+    const std::vector<const PasswordForm*>& non_federated,
+    size_t filtered_count) {
+  blacklisted_matches_.clear();
+  new_blacklisted_.reset();
+
+  std::unique_ptr<BrowserSavePasswordProgressLogger> logger;
+  if (password_manager_util::IsLoggingActive(client_)) {
+    logger.reset(
+        new BrowserSavePasswordProgressLogger(client_->GetLogManager()));
+    logger->LogMessage(Logger::STRING_PROCESS_MATCHES_METHOD);
+  }
+
+  // Copy out and score non-blacklisted matches.
+  std::vector<const PasswordForm*> matches(std::count_if(
+      non_federated.begin(), non_federated.end(),
+      [this](const PasswordForm* form) { return IsMatch(*form); }));
+  std::copy_if(non_federated.begin(), non_federated.end(), matches.begin(),
+               [this](const PasswordForm* form) { return IsMatch(*form); });
+  ScoreMatches(matches);
+
+  // Copy out blacklisted matches.
+  blacklisted_matches_.resize(std::count_if(
+      non_federated.begin(), non_federated.end(),
+      [this](const PasswordForm* form) { return IsBlacklistMatch(*form); }));
+  std::copy_if(
+      non_federated.begin(), non_federated.end(), blacklisted_matches_.begin(),
+      [this](const PasswordForm* form) { return IsBlacklistMatch(*form); });
+
+  UMA_HISTOGRAM_COUNTS(
+      "PasswordManager.NumPasswordsNotShown",
+      non_federated.size() + filtered_count - best_matches_.size());
+
+  // If password store was slow and provisionally saved form is already here
+  // then create pending credentials (see http://crbug.com/470322).
+  if (provisionally_saved_form_)
+    CreatePendingCredentials();
+
+  for (auto const& driver : drivers_)
+    ProcessFrameInternal(driver);
+  if (observed_form_.scheme != PasswordForm::SCHEME_HTML)
+    ProcessLoginPrompt();
 }
 
 void PasswordFormManager::ProcessFrame(
     const base::WeakPtr<PasswordManagerDriver>& driver) {
   DCHECK_EQ(PasswordForm::SCHEME_HTML, observed_form_.scheme);
-  if (state_ == POST_MATCHING_PHASE)
+  if (form_fetcher_->GetState() == FormFetcher::State::NOT_WAITING)
     ProcessFrameInternal(driver);
 
   for (auto const& old_driver : drivers_) {
@@ -573,10 +555,9 @@ void PasswordFormManager::ProcessFrame(
 void PasswordFormManager::ProcessFrameInternal(
     const base::WeakPtr<PasswordManagerDriver>& driver) {
   DCHECK_EQ(PasswordForm::SCHEME_HTML, observed_form_.scheme);
-  if (!driver || manager_action_ == kManagerActionBlacklisted)
+  if (!driver)
     return;
 
-  // Allow generation for any non-blacklisted form.
   driver->AllowPasswordGenerationForForm(observed_form_);
 
   if (best_matches_.empty())
@@ -597,24 +578,26 @@ void PasswordFormManager::ProcessFrameInternal(
             preferred_match_->action.GetWithEmptyPath() ||
         preferred_match_->is_public_suffix_match ||
         observed_form_.IsPossibleChangePasswordForm()));
-  if (wait_for_username)
+  if (wait_for_username) {
     manager_action_ = kManagerActionNone;
-  else
+  } else {
     manager_action_ = kManagerActionAutofilled;
+    base::RecordAction(base::UserMetricsAction("PasswordManager_Autofilled"));
+  }
   if (ShouldShowInitialPasswordAccountSuggestions()) {
     // This is for the fill-on-account-select experiment. Instead of autofilling
     // found usernames and passwords on load, this instructs the renderer to
     // return with any found password forms so a list of password account
     // suggestions can be drawn.
     password_manager_->ShowInitialPasswordAccountSuggestions(
-        driver.get(), observed_form_, best_matches_, federated_matches_,
-        *preferred_match_, wait_for_username);
+        driver.get(), observed_form_, best_matches_, *preferred_match_,
+        wait_for_username);
   } else {
     // If fill-on-account-select is not enabled, continue with autofilling any
     // password forms as traditionally has been done.
     password_manager_->Autofill(driver.get(), observed_form_, best_matches_,
-                                federated_matches_, *preferred_match_,
-                                wait_for_username);
+                                form_fetcher_->GetFederatedMatches(),
+                                *preferred_match_, wait_for_username);
   }
 }
 
@@ -628,12 +611,12 @@ void PasswordFormManager::ProcessLoginPrompt() {
 }
 
 void PasswordFormManager::OnGetPasswordStoreResults(
-    ScopedVector<PasswordForm> results) {
-  DCHECK_EQ(state_, MATCHING_PHASE);
+    std::vector<std::unique_ptr<autofill::PasswordForm>> results) {
+  DCHECK_EQ(FormFetcher::State::WAITING, form_fetcher_->GetState());
 
   if (need_to_refetch_) {
     // The received results are no longer up to date, need to re-request.
-    state_ = PRE_MATCHING_PHASE;
+    form_fetcher_impl_.set_state(FormFetcher::State::NOT_WAITING);
     FetchDataFromPasswordStore();
     need_to_refetch_ = false;
     return;
@@ -647,34 +630,18 @@ void PasswordFormManager::OnGetPasswordStoreResults(
     logger->LogNumber(Logger::STRING_NUMBER_RESULTS, results.size());
   }
 
-  federated_matches_ = SplitFederatedMatches(&results);
-  if (!results.empty())
-    OnRequestDone(std::move(results));
-  state_ = POST_MATCHING_PHASE;
-
-  // If password store was slow and provisionally saved form is already here
-  // then create pending credentials (see http://crbug.com/470322).
-  if (provisionally_saved_form_)
-    CreatePendingCredentials();
-
-  if (manager_action_ != kManagerActionBlacklisted) {
-    for (auto const& driver : drivers_)
-      ProcessFrameInternal(driver);
-    if (observed_form_.scheme != PasswordForm::SCHEME_HTML)
-      ProcessLoginPrompt();
-  }
+  form_fetcher_impl_.SetResults(std::move(results));
 }
 
 void PasswordFormManager::OnGetSiteStatistics(
-    std::unique_ptr<std::vector<std::unique_ptr<InteractionsStats>>> stats) {
+    std::vector<std::unique_ptr<InteractionsStats>> stats) {
   // On Windows the password request may be resolved after the statistics due to
   // importing from IE.
-  DCHECK(state_ == MATCHING_PHASE || state_ == POST_MATCHING_PHASE) << state_;
-  interactions_stats_.swap(*stats);
+  form_fetcher_impl_.SetStats(std::move(stats));
 }
 
 void PasswordFormManager::ProcessUpdate() {
-  DCHECK_EQ(state_, POST_MATCHING_PHASE);
+  DCHECK_EQ(FormFetcher::State::NOT_WAITING, form_fetcher_->GetState());
   DCHECK(preferred_match_ || !pending_credentials_.federation_origin.unique());
   // If we're doing an Update, we either autofilled correctly and need to
   // update the stats, or the user typed in a new password for autofilled
@@ -685,7 +652,8 @@ void PasswordFormManager::ProcessUpdate() {
 
   UpdateMetadataForUsage(&pending_credentials_);
 
-  client_->GetStoreResultFilter()->ReportFormUsed(pending_credentials_);
+  base::RecordAction(
+      base::UserMetricsAction("PasswordManager_LoginFollowingAutofill"));
 
   // Check to see if this form is a candidate for password generation.
   // Do not send votes on change password forms, since they were already sent in
@@ -696,11 +664,11 @@ void PasswordFormManager::ProcessUpdate() {
 
 bool PasswordFormManager::UpdatePendingCredentialsIfOtherPossibleUsername(
     const base::string16& username) {
-  for (PasswordFormMap::const_iterator it = best_matches_.begin();
-       it != best_matches_.end(); ++it) {
-    for (size_t i = 0; i < it->second->other_possible_usernames.size(); ++i) {
-      if (it->second->other_possible_usernames[i] == username) {
-        pending_credentials_ = *it->second;
+  for (const auto& key_value : best_matches_) {
+    const PasswordForm& match = *key_value.second;
+    for (size_t i = 0; i < match.other_possible_usernames.size(); ++i) {
+      if (match.other_possible_usernames[i] == username) {
+        pending_credentials_ = match;
         return true;
       }
     }
@@ -708,9 +676,8 @@ bool PasswordFormManager::UpdatePendingCredentialsIfOtherPossibleUsername(
   return false;
 }
 
-void PasswordFormManager::SendAutofillVotes(
-    const PasswordForm& observed,
-    PasswordForm* pending) {
+void PasswordFormManager::SendAutofillVotes(const PasswordForm& observed,
+                                            PasswordForm* pending) {
   if (pending->form_data.fields.empty())
     return;
 
@@ -722,7 +689,8 @@ void PasswordFormManager::SendAutofillVotes(
   // to PasswordForm. Even without this check, these FormStructure's won't
   // be uploaded, but it makes it hard to see if we are encountering
   // unexpected errors.
-  if (pending_structure.FormSignature() != observed_structure.FormSignature()) {
+  if (pending_structure.FormSignatureAsStr() !=
+      observed_structure.FormSignatureAsStr()) {
     // Only upload if this is the first time the password has been used.
     // Otherwise the credentials have been used on the same field before so
     // they aren't from an account creation form.
@@ -732,7 +700,7 @@ void PasswordFormManager::SendAutofillVotes(
     if (pending->times_used == 1 && selected_username_.empty()) {
       if (UploadPasswordForm(pending->form_data, pending->username_element,
                              autofill::ACCOUNT_CREATION_PASSWORD,
-                             observed_structure.FormSignature())) {
+                             observed_structure.FormSignatureAsStr())) {
         pending->generation_upload_status =
             autofill::PasswordForm::POSITIVE_SIGNAL_SENT;
       }
@@ -809,6 +777,11 @@ bool PasswordFormManager::UploadPasswordForm(
   // sure to receive them.
   form_structure.set_upload_required(UPLOAD_REQUIRED);
 
+  if (password_manager_util::IsLoggingActive(client_)) {
+    BrowserSavePasswordProgressLogger logger(client_->GetLogManager());
+    logger.LogFormStructure(Logger::STRING_FORM_VOTES, form_structure);
+  }
+
   bool success = autofill_manager->download_manager()->StartUploadRequest(
       form_structure, false /* was_autofilled */, available_field_types,
       login_form_signature, true /* observed_submission */);
@@ -892,6 +865,11 @@ bool PasswordFormManager::UploadChangePasswordForm(
   // always pass.
   form_structure.set_upload_required(UPLOAD_REQUIRED);
 
+  if (password_manager_util::IsLoggingActive(client_)) {
+    BrowserSavePasswordProgressLogger logger(client_->GetLogManager());
+    logger.LogFormStructure(Logger::STRING_FORM_VOTES, form_structure);
+  }
+
   return autofill_manager->download_manager()->StartUploadRequest(
       form_structure, false /* was_autofilled */, available_field_types,
       login_form_signature, true /* observed_submission */);
@@ -960,7 +938,7 @@ void PasswordFormManager::CreatePendingCredentials() {
   // autofilled ones, as they may have changed if the user experienced a login
   // failure.
   // Look for these credentials in the list containing auto-fill entries.
-  PasswordForm* saved_form =
+  const PasswordForm* saved_form =
       FindBestSavedMatch(provisionally_saved_form_.get());
   if (saved_form != nullptr) {
     // The user signed in with a login we autofilled.
@@ -972,8 +950,8 @@ void PasswordFormManager::CreatePendingCredentials() {
       // from Android apps store a copy with the current origin and signon
       // realm. This ensures that on the next visit, a precise match is found.
       is_new_login_ = true;
-      user_action_ = password_overridden_ ? kUserActionOverridePassword
-                                          : kUserActionChoosePslMatch;
+      SetUserAction(password_overridden_ ? kUserActionOverridePassword
+                                         : kUserActionChoosePslMatch);
 
       // Since this credential will not overwrite a previously saved credential,
       // username_value can be updated now.
@@ -1024,7 +1002,7 @@ void PasswordFormManager::CreatePendingCredentials() {
     } else {  // Not a PSL match.
       is_new_login_ = false;
       if (password_overridden_)
-        user_action_ = kUserActionOverridePassword;
+        SetUserAction(kUserActionOverridePassword);
     }
   } else if (other_possible_username_action_ ==
                  ALLOW_OTHER_POSSIBLE_USERNAMES &&
@@ -1036,13 +1014,13 @@ void PasswordFormManager::CreatePendingCredentials() {
     // credential.
     selected_username_ = provisionally_saved_form_->username_value;
     is_new_login_ = false;
-  } else if (client_->IsUpdatePasswordUIEnabled() && !best_matches_.empty() &&
+  } else if (!best_matches_.empty() &&
              provisionally_saved_form_->type !=
                  autofill::PasswordForm::TYPE_API &&
              (provisionally_saved_form_
                   ->IsPossibleChangePasswordFormWithoutUsername() ||
               provisionally_saved_form_->username_element.empty())) {
-    PasswordForm* best_update_match = FindBestMatchForUpdatePassword(
+    const PasswordForm* best_update_match = FindBestMatchForUpdatePassword(
         provisionally_saved_form_->password_value);
 
     retry_password_form_password_update_ =
@@ -1077,6 +1055,7 @@ void PasswordFormManager::CreatePendingCredentials() {
 
   pending_credentials_.password_value = password_to_save;
   pending_credentials_.preferred = provisionally_saved_form_->preferred;
+  CopyFieldPropertiesMasks(*provisionally_saved_form_, &pending_credentials_);
 
   // If we're dealing with an API-driven provisionally saved form, then take
   // the server provided values. We don't do this for non-API forms, as
@@ -1107,13 +1086,11 @@ void PasswordFormManager::CreatePendingCredentials() {
   // |provisionally_saved_form_| in order to be able to determine which field is
   // password and which is a new password during sending a vote in other cases
   // we can reset |provisionally_saved_form_|.
-  if (!client_->IsUpdatePasswordUIEnabled() ||
-      !provisionally_saved_form_->IsPossibleChangePasswordForm())
+  if (!provisionally_saved_form_->IsPossibleChangePasswordForm())
     provisionally_saved_form_.reset();
 }
 
 uint32_t PasswordFormManager::ScoreResult(const PasswordForm& candidate) const {
-  DCHECK_EQ(state_, MATCHING_PHASE);
   DCHECK(!candidate.blacklisted_by_user);
   // For scoring of candidate login data:
   // The most important element that should match is the signon_realm followed
@@ -1179,16 +1156,23 @@ uint32_t PasswordFormManager::ScoreResult(const PasswordForm& candidate) const {
   return score;
 }
 
+bool PasswordFormManager::IsMatch(const autofill::PasswordForm& form) const {
+  return !form.blacklisted_by_user && form.scheme == observed_form_.scheme;
+}
+
 bool PasswordFormManager::IsBlacklistMatch(
     const autofill::PasswordForm& blacklisted_form) const {
-  DCHECK(blacklisted_form.blacklisted_by_user);
+  if (!blacklisted_form.blacklisted_by_user ||
+      blacklisted_form.is_public_suffix_match ||
+      blacklisted_form.scheme != observed_form_.scheme ||
+      blacklisted_form.origin.GetOrigin() !=
+          observed_form_.origin.GetOrigin()) {
+    return false;
+  }
 
-  if (blacklisted_form.is_public_suffix_match)
-    return false;
-  if (blacklisted_form.origin.GetOrigin() != observed_form_.origin.GetOrigin())
-    return false;
   if (observed_form_.scheme == PasswordForm::SCHEME_HTML) {
-    return (blacklisted_form.origin.path() == observed_form_.origin.path()) ||
+    return (blacklisted_form.origin.path_piece() ==
+            observed_form_.origin.path_piece()) ||
            (AreStringsEqualOrEmpty(blacklisted_form.submit_element,
                                    observed_form_.submit_element) &&
             AreStringsEqualOrEmpty(blacklisted_form.password_element,
@@ -1199,28 +1183,30 @@ bool PasswordFormManager::IsBlacklistMatch(
   return true;
 }
 
-PasswordForm* PasswordFormManager::FindBestMatchForUpdatePassword(
+const PasswordForm* PasswordFormManager::FindBestMatchForUpdatePassword(
     const base::string16& password) const {
   if (best_matches_.size() == 1 && !has_generated_password_) {
     // In case when the user has only one credential and the current password is
     // not generated, consider it the same as is being saved.
-    return best_matches_.begin()->second.get();
+    return best_matches_.begin()->second;
   }
   if (password.empty())
     return nullptr;
 
-  for (auto it = best_matches_.begin(); it != best_matches_.end(); ++it) {
-    if (it->second->password_value == password)
-      return it->second.get();
+  for (const auto& key_value : best_matches_) {
+    if (key_value.second->password_value == password)
+      return key_value.second;
   }
   return nullptr;
 }
 
-PasswordForm* PasswordFormManager::FindBestSavedMatch(
+const PasswordForm* PasswordFormManager::FindBestSavedMatch(
     const PasswordForm* form) const {
-  PasswordFormMap::const_iterator it = best_matches_.find(form->username_value);
+  if (!form->federation_origin.unique())
+    return nullptr;
+  auto it = best_matches_.find(form->username_value);
   if (it != best_matches_.end())
-    return it->second.get();
+    return it->second;
   if (form->type == autofill::PasswordForm::TYPE_API)
     // Match Credential API forms only by username.
     return nullptr;
@@ -1228,14 +1214,14 @@ PasswordForm* PasswordFormManager::FindBestSavedMatch(
     return nullptr;
   for (const auto& stored_match : best_matches_) {
     if (stored_match.second->password_value == form->password_value)
-      return stored_match.second.get();
+      return stored_match.second;
   }
   return nullptr;
 }
 
 void PasswordFormManager::CreatePendingCredentialsForNewCredentials() {
   // User typed in a new, unknown username.
-  user_action_ = kUserActionOverrideUsernameAndPassword;
+  SetUserAction(kUserActionOverrideUsernameAndPassword);
   pending_credentials_ = observed_form_;
   if (provisionally_saved_form_->was_parsed_using_autofill_predictions)
     pending_credentials_.username_element =
@@ -1276,6 +1262,7 @@ void PasswordFormManager::LogSubmitPassed() {
           metrics_util::PASSWORD_SUBMITTED);
     }
   }
+  base::RecordAction(base::UserMetricsAction("PasswordManager_LoginPassed"));
   submit_result_ = kSubmitResultPassed;
 }
 
@@ -1287,12 +1274,11 @@ void PasswordFormManager::LogSubmitFailed() {
     metrics_util::LogPasswordGenerationAvailableSubmissionEvent(
         metrics_util::PASSWORD_SUBMISSION_FAILED);
   }
+  base::RecordAction(base::UserMetricsAction("PasswordManager_LoginFailed"));
   submit_result_ = kSubmitResultFailed;
 }
 
 void PasswordFormManager::WipeStoreCopyIfOutdated() {
-  DCHECK_NE(PRE_MATCHING_PHASE, state_);
-
   UMA_HISTOGRAM_BOOLEAN("PasswordManager.StoreReadyWhenWiping",
                         HasCompletedMatching());
 
@@ -1329,8 +1315,27 @@ void PasswordFormManager::SendVotesOnSave() {
   }
 }
 
+void PasswordFormManager::SetUserAction(UserAction user_action) {
+  if (user_action == kUserActionChoose) {
+    base::RecordAction(
+        base::UserMetricsAction("PasswordManager_UsedNonDefaultUsername"));
+  } else if (user_action == kUserActionChoosePslMatch) {
+    base::RecordAction(
+        base::UserMetricsAction("PasswordManager_ChoseSubdomainPassword"));
+  } else if (user_action == kUserActionOverridePassword) {
+    base::RecordAction(
+        base::UserMetricsAction("PasswordManager_LoggedInWithNewPassword"));
+  } else if (user_action == kUserActionOverrideUsernameAndPassword) {
+    base::RecordAction(
+        base::UserMetricsAction("PasswordManager_LoggedInWithNewUsername"));
+  } else {
+    NOTREACHED();
+  }
+  user_action_ = user_action;
+}
+
 base::Optional<PasswordForm> PasswordFormManager::UpdatePendingAndGetOldKey(
-    std::vector<const PasswordForm*>* credentials_to_update) {
+    std::vector<PasswordForm>* credentials_to_update) {
   base::Optional<PasswordForm> old_primary_key;
   bool update_related_credentials = false;
 
@@ -1368,7 +1373,7 @@ base::Optional<PasswordForm> PasswordFormManager::UpdatePendingAndGetOldKey(
   // If this was a password update, then update all non-best matches entries
   // with the same username and the same old password.
   if (update_related_credentials) {
-    PasswordFormMap::const_iterator updated_password_it =
+    auto updated_password_it =
         best_matches_.find(pending_credentials_.username_value);
     DCHECK(best_matches_.end() != updated_password_it);
     const base::string16& old_password =
@@ -1377,8 +1382,9 @@ base::Optional<PasswordForm> PasswordFormManager::UpdatePendingAndGetOldKey(
       if (not_best_match->username_value ==
               pending_credentials_.username_value &&
           not_best_match->password_value == old_password) {
-        not_best_match->password_value = pending_credentials_.password_value;
-        credentials_to_update->push_back(not_best_match.get());
+        credentials_to_update->push_back(*not_best_match);
+        credentials_to_update->back().password_value =
+            pending_credentials_.password_value;
       }
     }
   }
