@@ -2,7 +2,6 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include <openssl/hmac.h>
 #include <stddef.h>
 #include <stdint.h>
 
@@ -20,6 +19,7 @@
 #include "crypto/secure_util.h"
 #include "third_party/WebKit/public/platform/WebCryptoAlgorithmParams.h"
 #include "third_party/WebKit/public/platform/WebCryptoKeyAlgorithm.h"
+#include "third_party/boringssl/src/include/openssl/hmac.h"
 
 namespace webcrypto {
 
@@ -96,15 +96,16 @@ Status SignHmac(const std::vector<uint8_t>& raw_key,
   size_t hmac_expected_length = EVP_MD_size(digest_algorithm);
 
   buffer->resize(hmac_expected_length);
-  crypto::ScopedOpenSSLSafeSizeBuffer<EVP_MAX_MD_SIZE> hmac_result(
-      buffer->data(), hmac_expected_length);
 
   unsigned int hmac_actual_length;
-  unsigned char* const success =
-      HMAC(digest_algorithm, raw_key.data(), raw_key.size(), data.bytes(),
-           data.byte_length(), hmac_result.safe_buffer(), &hmac_actual_length);
-  if (!success || hmac_actual_length != hmac_expected_length)
+  if (!HMAC(digest_algorithm, raw_key.data(), raw_key.size(), data.bytes(),
+            data.byte_length(), buffer->data(), &hmac_actual_length)) {
     return Status::OperationError();
+  }
+
+  // HMAC() promises to use at most EVP_MD_CTX_size(). If this was not the
+  // case then memory corruption may have just occurred.
+  CHECK_EQ(hmac_expected_length, hmac_actual_length);
 
   return Status::Success();
 }
@@ -117,7 +118,7 @@ class HmacImplementation : public AlgorithmImplementation {
                      bool extractable,
                      blink::WebCryptoKeyUsageMask usages,
                      GenerateKeyResult* result) const override {
-    Status status = CheckSecretKeyCreationUsages(kAllKeyUsages, usages);
+    Status status = CheckKeyCreationUsages(kAllKeyUsages, usages);
     if (status.IsError())
       return status;
 
@@ -141,29 +142,50 @@ class HmacImplementation : public AlgorithmImplementation {
                                       extractable, usages, keylen_bits, result);
   }
 
-  Status VerifyKeyUsagesBeforeImportKey(
-      blink::WebCryptoKeyFormat format,
-      blink::WebCryptoKeyUsageMask usages) const override {
+  Status ImportKey(blink::WebCryptoKeyFormat format,
+                   const CryptoData& key_data,
+                   const blink::WebCryptoAlgorithm& algorithm,
+                   bool extractable,
+                   blink::WebCryptoKeyUsageMask usages,
+                   blink::WebCryptoKey* key) const override {
     switch (format) {
       case blink::WebCryptoKeyFormatRaw:
+        return ImportKeyRaw(key_data, algorithm, extractable, usages, key);
       case blink::WebCryptoKeyFormatJwk:
-        return CheckSecretKeyCreationUsages(kAllKeyUsages, usages);
+        return ImportKeyJwk(key_data, algorithm, extractable, usages, key);
       default:
         return Status::ErrorUnsupportedImportKeyFormat();
     }
   }
 
+  Status ExportKey(blink::WebCryptoKeyFormat format,
+                   const blink::WebCryptoKey& key,
+                   std::vector<uint8_t>* buffer) const override {
+    switch (format) {
+      case blink::WebCryptoKeyFormatRaw:
+        return ExportKeyRaw(key, buffer);
+      case blink::WebCryptoKeyFormatJwk:
+        return ExportKeyJwk(key, buffer);
+      default:
+        return Status::ErrorUnsupportedExportKeyFormat();
+    }
+  }
+
   Status ImportKeyRaw(const CryptoData& key_data,
                       const blink::WebCryptoAlgorithm& algorithm,
                       bool extractable,
                       blink::WebCryptoKeyUsageMask usages,
-                      blink::WebCryptoKey* key) const override {
+                      blink::WebCryptoKey* key) const {
+    Status status = CheckKeyCreationUsages(kAllKeyUsages, usages);
+    if (status.IsError())
+      return status;
+
     const blink::WebCryptoHmacImportParams* params =
         algorithm.hmacImportParams();
 
     unsigned int keylen_bits = 0;
-    Status status = GetHmacImportKeyLengthBits(params, key_data.byte_length(),
-                                               &keylen_bits);
+    status = GetHmacImportKeyLengthBits(params, key_data.byte_length(),
+                                        &keylen_bits);
     if (status.IsError())
       return status;
 
@@ -189,7 +211,11 @@ class HmacImplementation : public AlgorithmImplementation {
                       const blink::WebCryptoAlgorithm& algorithm,
                       bool extractable,
                       blink::WebCryptoKeyUsageMask usages,
-                      blink::WebCryptoKey* key) const override {
+                      blink::WebCryptoKey* key) const {
+    Status status = CheckKeyCreationUsages(kAllKeyUsages, usages);
+    if (status.IsError())
+      return status;
+
     const char* algorithm_name =
         GetJwkHmacAlgorithmName(algorithm.hmacImportParams()->hash().id());
     if (!algorithm_name)
@@ -197,8 +223,8 @@ class HmacImplementation : public AlgorithmImplementation {
 
     std::vector<uint8_t> raw_data;
     JwkReader jwk;
-    Status status = ReadSecretKeyNoExpectedAlgJwk(key_data, extractable, usages,
-                                                  &raw_data, &jwk);
+    status = ReadSecretKeyNoExpectedAlgJwk(key_data, extractable, usages,
+                                           &raw_data, &jwk);
     if (status.IsError())
       return status;
     status = jwk.VerifyAlg(algorithm_name);
@@ -210,13 +236,13 @@ class HmacImplementation : public AlgorithmImplementation {
   }
 
   Status ExportKeyRaw(const blink::WebCryptoKey& key,
-                      std::vector<uint8_t>* buffer) const override {
+                      std::vector<uint8_t>* buffer) const {
     *buffer = GetSymmetricKeyData(key);
     return Status::Success();
   }
 
   Status ExportKeyJwk(const blink::WebCryptoKey& key,
-                      std::vector<uint8_t>* buffer) const override {
+                      std::vector<uint8_t>* buffer) const {
     const std::vector<uint8_t>& raw_data = GetSymmetricKeyData(key);
 
     const char* algorithm_name =
