@@ -79,8 +79,26 @@ class AppCacheRequestHandlerTest : public testing::Test {
   // exercise fallback code paths.
 
   class MockURLRequestDelegate : public net::URLRequest::Delegate {
-    void OnResponseStarted(net::URLRequest* request) override {}
-    void OnReadCompleted(net::URLRequest* request, int bytes_read) override {}
+   public:
+    MockURLRequestDelegate() : request_status_(1) {}
+
+    void OnResponseStarted(net::URLRequest* request, int net_error) override {
+      DCHECK_NE(net::ERR_IO_PENDING, net_error);
+      request_status_ = net_error;
+    }
+
+    void OnReadCompleted(net::URLRequest* request, int bytes_read) override {
+      DCHECK_NE(net::ERR_IO_PENDING, bytes_read);
+      if (bytes_read >= 0)
+        request_status_ = net::OK;
+      else
+        request_status_ = bytes_read;
+    }
+
+    int request_status() const { return request_status_; }
+
+   private:
+    int request_status_;
   };
 
   class MockURLRequestJob : public net::URLRequestJob {
@@ -195,7 +213,7 @@ class AppCacheRequestHandlerTest : public testing::Test {
   }
 
   void SetUpTest() {
-    DCHECK(base::MessageLoop::current() == io_thread_->message_loop());
+    DCHECK(io_thread_->task_runner()->BelongsToCurrentThread());
     mock_service_.reset(new MockAppCacheService);
     mock_service_->set_request_context(&empty_context_);
     mock_policy_.reset(new MockAppCachePolicy);
@@ -212,7 +230,7 @@ class AppCacheRequestHandlerTest : public testing::Test {
   }
 
   void TearDownTest() {
-    DCHECK(base::MessageLoop::current() == io_thread_->message_loop());
+    DCHECK(io_thread_->task_runner()->BelongsToCurrentThread());
     job_ = NULL;
     handler_.reset();
     request_.reset();
@@ -227,7 +245,7 @@ class AppCacheRequestHandlerTest : public testing::Test {
   void TestFinished() {
     // We unwind the stack prior to finishing up to let stack
     // based objects get deleted.
-    DCHECK(base::MessageLoop::current() == io_thread_->message_loop());
+    DCHECK(io_thread_->task_runner()->BelongsToCurrentThread());
     base::ThreadTaskRunnerHandle::Get()->PostTask(
         FROM_HERE, base::Bind(&AppCacheRequestHandlerTest::TestFinishedUnwound,
                               base::Unretained(this)));
@@ -243,7 +261,7 @@ class AppCacheRequestHandlerTest : public testing::Test {
   }
 
   void ScheduleNextTask() {
-    DCHECK(base::MessageLoop::current() == io_thread_->message_loop());
+    DCHECK(io_thread_->task_runner()->BelongsToCurrentThread());
     if (task_stack_.empty()) {
       TestFinished();
       return;
@@ -380,18 +398,18 @@ class AppCacheRequestHandlerTest : public testing::Test {
   }
 
   void SimulateResponseCode(int response_code) {
-    job_factory_->SetJob(base::WrapUnique(new MockURLRequestJob(
+    job_factory_->SetJob(base::MakeUnique<MockURLRequestJob>(
         request_.get(), request_->context()->network_delegate(),
-        response_code)));
+        response_code));
     request_->Start();
     // All our simulation needs  to satisfy are the following two DCHECKs
-    DCHECK(request_->status().is_success());
+    DCHECK_EQ(net::OK, delegate_.request_status());
     DCHECK_EQ(response_code, request_->GetResponseCode());
   }
 
   void SimulateResponseInfo(const net::HttpResponseInfo& info) {
-    job_factory_->SetJob(base::WrapUnique(new MockURLRequestJob(
-        request_.get(), request_->context()->network_delegate(), info)));
+    job_factory_->SetJob(base::MakeUnique<MockURLRequestJob>(
+        request_.get(), request_->context()->network_delegate(), info));
     request_->Start();
   }
 
@@ -780,6 +798,72 @@ class AppCacheRequestHandlerTest : public testing::Test {
     TestFinished();
   }
 
+  // DestroyedService -----------------------------
+
+  void DestroyedService() {
+    host_->AssociateCompleteCache(MakeNewCache());
+
+    mock_storage()->SimulateFindSubResource(
+        AppCacheEntry(AppCacheEntry::EXPLICIT, 1), AppCacheEntry(), false);
+
+    request_ = empty_context_.CreateRequest(
+        GURL("http://blah/"), net::DEFAULT_PRIORITY, &delegate_);
+    handler_.reset(host_->CreateRequestHandler(request_.get(),
+                                               RESOURCE_TYPE_SUB_RESOURCE,
+                                               false));
+    EXPECT_TRUE(handler_.get());
+    job_.reset(handler_->MaybeLoadResource(
+        request_.get(), request_->context()->network_delegate()));
+    EXPECT_TRUE(job_.get());
+
+    backend_impl_.reset();
+    mock_frontend_.reset();
+    mock_service_.reset();
+    mock_policy_.reset();
+    host_ = NULL;
+
+    EXPECT_TRUE(job_->has_been_killed());
+    EXPECT_FALSE(handler_->MaybeLoadResource(
+        request_.get(), request_->context()->network_delegate()));
+    EXPECT_FALSE(handler_->MaybeLoadFallbackForRedirect(
+        request_.get(),
+        request_->context()->network_delegate(),
+        GURL("http://blah/redirect")));
+    EXPECT_FALSE(handler_->MaybeLoadFallbackForResponse(
+        request_.get(), request_->context()->network_delegate()));
+
+    TestFinished();
+  }
+
+  void DestroyedServiceWithCrossSiteNav() {
+    request_ = empty_context_.CreateRequest(
+        GURL("http://blah/"), net::DEFAULT_PRIORITY, &delegate_);
+    handler_.reset(host_->CreateRequestHandler(request_.get(),
+                                               RESOURCE_TYPE_MAIN_FRAME,
+                                               false));
+    EXPECT_TRUE(handler_.get());
+    handler_->PrepareForCrossSiteTransfer(backend_impl_->process_id());
+    EXPECT_TRUE(handler_->host_for_cross_site_transfer_.get());
+
+    backend_impl_.reset();
+    mock_frontend_.reset();
+    mock_service_.reset();
+    mock_policy_.reset();
+    host_ = NULL;
+
+    EXPECT_FALSE(handler_->host_for_cross_site_transfer_.get());
+    EXPECT_FALSE(handler_->MaybeLoadResource(
+        request_.get(), request_->context()->network_delegate()));
+    EXPECT_FALSE(handler_->MaybeLoadFallbackForRedirect(
+        request_.get(),
+        request_->context()->network_delegate(),
+        GURL("http://blah/redirect")));
+    EXPECT_FALSE(handler_->MaybeLoadFallbackForResponse(
+        request_.get(), request_->context()->network_delegate()));
+
+    TestFinished();
+  }
+
   // UnsupportedScheme -----------------------------
 
   void UnsupportedScheme() {
@@ -918,7 +1002,7 @@ class AppCacheRequestHandlerTest : public testing::Test {
     EXPECT_TRUE(handler_->found_manifest_url_.is_empty());
     EXPECT_TRUE(host_->preferred_manifest_url().is_empty());
     EXPECT_TRUE(host_->main_resource_blocked_);
-    EXPECT_TRUE(host_->blocked_manifest_url_ == GURL("http://blah/manifest/"));
+    EXPECT_EQ(host_->blocked_manifest_url_, "http://blah/manifest/");
 
     TestFinished();
   }
@@ -1020,6 +1104,15 @@ TEST_F(AppCacheRequestHandlerTest, DestroyedHostWithWaitingJob) {
   RunTestOnIOThread(&AppCacheRequestHandlerTest::DestroyedHostWithWaitingJob);
 }
 
+TEST_F(AppCacheRequestHandlerTest, DestroyedService) {
+  RunTestOnIOThread(&AppCacheRequestHandlerTest::DestroyedService);
+}
+
+TEST_F(AppCacheRequestHandlerTest, DestroyedServiceWithCrossSiteNav) {
+  RunTestOnIOThread(
+      &AppCacheRequestHandlerTest::DestroyedServiceWithCrossSiteNav);
+}
+
 TEST_F(AppCacheRequestHandlerTest, UnsupportedScheme) {
   RunTestOnIOThread(&AppCacheRequestHandlerTest::UnsupportedScheme);
 }
