@@ -10,9 +10,10 @@
 
 #include "base/bind.h"
 #include "base/bind_helpers.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
-#include "content/browser/bluetooth/bluetooth_blacklist.h"
+#include "content/browser/bluetooth/bluetooth_blocklist.h"
 #include "content/browser/bluetooth/bluetooth_metrics.h"
 #include "content/browser/bluetooth/web_bluetooth_service_impl.h"
 #include "content/public/browser/browser_thread.h"
@@ -25,30 +26,60 @@
 #include "device/bluetooth/bluetooth_discovery_session.h"
 
 using device::BluetoothUUID;
+using UUIDSet = device::BluetoothDevice::UUIDSet;
+
+namespace {
+
+// Anything worse than or equal to this will show 0 bars.
+const int kMinRSSI = -100;
+// Anything better than or equal to this will show the maximum bars.
+const int kMaxRSSI = -55;
+// Number of RSSI levels used in the signal strength image.
+const int kNumSignalStrengthLevels = 5;
+
+const content::UMARSSISignalStrengthLevel kRSSISignalStrengthEnumTable[] = {
+    content::UMARSSISignalStrengthLevel::LEVEL_0,
+    content::UMARSSISignalStrengthLevel::LEVEL_1,
+    content::UMARSSISignalStrengthLevel::LEVEL_2,
+    content::UMARSSISignalStrengthLevel::LEVEL_3,
+    content::UMARSSISignalStrengthLevel::LEVEL_4};
+
+}  // namespace
 
 namespace content {
 
+bool BluetoothDeviceChooserController::use_test_scan_duration_ = false;
+
 namespace {
 constexpr size_t kMaxLengthForDeviceName =
     29;  // max length of device name in filter.
 
+// The duration of a Bluetooth Scan in seconds.
+constexpr int kScanDuration = 60;
+constexpr int kTestScanDuration = 0;
+
 void LogRequestDeviceOptions(
     const blink::mojom::WebBluetoothRequestDeviceOptionsPtr& options) {
   VLOG(1) << "requestDevice called with the following filters: ";
+  VLOG(1) << "acceptAllDevices: " << options->accept_all_devices;
+
+  if (!options->filters)
+    return;
+
   int i = 0;
-  for (const auto& filter : options->filters) {
+  for (const auto& filter : options->filters.value()) {
     VLOG(1) << "Filter #" << ++i;
-    if (!filter->name.is_null())
-      VLOG(1) << "Name: " << filter->name;
+    if (filter->name)
+      VLOG(1) << "Name: " << filter->name.value();
 
-    if (!filter->name_prefix.is_null())
-      VLOG(1) << "Name Prefix: " << filter->name_prefix;
+    if (filter->name_prefix)
+      VLOG(1) << "Name Prefix: " << filter->name_prefix.value();
 
-    if (!filter->services.is_null()) {
+    if (filter->services) {
       VLOG(1) << "Services: ";
       VLOG(1) << "\t[";
-      for (const auto& service : filter->services)
-        VLOG(1) << "\t\t" << service->canonical_value();
+      for (const auto& service : filter->services.value())
+        VLOG(1) << "\t\t" << service.canonical_value();
       VLOG(1) << "\t]";
     }
   }
@@ -57,52 +88,65 @@ void LogRequestDeviceOptions(
 bool IsEmptyOrInvalidFilter(
     const blink::mojom::WebBluetoothScanFilterPtr& filter) {
   // At least one member needs to be present.
-  if (filter->name.is_null() && filter->name_prefix.is_null() &&
-      filter->services.is_null())
+  if (!filter->name && !filter->name_prefix && !filter->services)
     return true;
 
   // The renderer will never send a name or a name_prefix longer than
   // kMaxLengthForDeviceName.
-  if (!filter->name.is_null() && filter->name.size() > kMaxLengthForDeviceName)
+  if (filter->name && filter->name->size() > kMaxLengthForDeviceName)
     return true;
-  if (!filter->name_prefix.is_null() &&
-      filter->name_prefix.size() > kMaxLengthForDeviceName)
+  if (filter->name_prefix && filter->name_prefix->size() == 0)
+    return true;
+  if (filter->name_prefix &&
+      filter->name_prefix->size() > kMaxLengthForDeviceName)
     return true;
 
   return false;
 }
 
 bool HasEmptyOrInvalidFilter(
-    const mojo::Array<blink::mojom::WebBluetoothScanFilterPtr>& filters) {
-  return filters.empty()
+    const base::Optional<std::vector<blink::mojom::WebBluetoothScanFilterPtr>>&
+        filters) {
+  if (!filters) {
+    return true;
+  }
+
+  return filters->empty()
              ? true
-             : filters.end() != std::find_if(filters.begin(), filters.end(),
-                                             IsEmptyOrInvalidFilter);
+             : filters->end() != std::find_if(filters->begin(), filters->end(),
+                                              IsEmptyOrInvalidFilter);
 }
 
-bool MatchesFilter(const device::BluetoothDevice& device,
+bool IsOptionsInvalid(
+    const blink::mojom::WebBluetoothRequestDeviceOptionsPtr& options) {
+  if (options->accept_all_devices) {
+    return options->filters.has_value();
+  } else {
+    return HasEmptyOrInvalidFilter(options->filters);
+  }
+}
+
+bool MatchesFilter(const std::string* device_name,
+                   const UUIDSet& device_uuids,
                    const blink::mojom::WebBluetoothScanFilterPtr& filter) {
-  DCHECK(!IsEmptyOrInvalidFilter(filter));
-
-  // TODO(615720): Use the upcoming GetName (was GetDeviceName).
-  const std::string device_name = base::UTF16ToUTF8(device.GetNameForDisplay());
-
-  if (!filter->name.is_null() && (device_name != filter->name)) {
-    return false;
+  if (filter->name) {
+    if (device_name == nullptr)
+      return false;
+    if (filter->name.value() != *device_name)
+      return false;
   }
 
-  if (!filter->name_prefix.is_null() &&
-      (!base::StartsWith(device_name, filter->name_prefix.get(),
-                         base::CompareCase::SENSITIVE))) {
-    return false;
+  if (filter->name_prefix && filter->name_prefix->size()) {
+    if (device_name == nullptr)
+      return false;
+    if (!base::StartsWith(*device_name, filter->name_prefix.value(),
+                          base::CompareCase::SENSITIVE))
+      return false;
   }
 
-  if (!filter->services.is_null()) {
-    const auto& device_uuid_list = device.GetUUIDs();
-    const std::unordered_set<BluetoothUUID, device::BluetoothUUIDHash>
-        device_uuids(device_uuid_list.begin(), device_uuid_list.end());
-    for (const base::Optional<BluetoothUUID>& service : filter->services) {
-      if (!ContainsKey(device_uuids, service.value())) {
+  if (filter->services) {
+    for (const auto& service : filter->services.value()) {
+      if (!base::ContainsKey(device_uuids, service)) {
         return false;
       }
     }
@@ -112,11 +156,13 @@ bool MatchesFilter(const device::BluetoothDevice& device,
 }
 
 bool MatchesFilters(
-    const device::BluetoothDevice& device,
-    const mojo::Array<blink::mojom::WebBluetoothScanFilterPtr>& filters) {
+    const std::string* device_name,
+    const UUIDSet& device_uuids,
+    const base::Optional<std::vector<blink::mojom::WebBluetoothScanFilterPtr>>&
+        filters) {
   DCHECK(!HasEmptyOrInvalidFilter(filters));
-  for (const auto& filter : filters) {
-    if (MatchesFilter(device, filter)) {
+  for (const auto& filter : filters.value()) {
+    if (MatchesFilter(device_name, device_uuids, filter)) {
       return true;
     }
   }
@@ -124,15 +170,27 @@ bool MatchesFilters(
 }
 
 std::unique_ptr<device::BluetoothDiscoveryFilter> ComputeScanFilter(
-    const mojo::Array<blink::mojom::WebBluetoothScanFilterPtr>& filters) {
+    const base::Optional<std::vector<blink::mojom::WebBluetoothScanFilterPtr>>&
+        filters) {
   std::unordered_set<BluetoothUUID, device::BluetoothUUIDHash> services;
-  for (const auto& filter : filters) {
-    for (const base::Optional<BluetoothUUID>& service : filter->services) {
-      services.insert(service.value());
+
+  if (filters) {
+    for (const auto& filter : filters.value()) {
+      if (!filter->services) {
+        continue;
+      }
+      for (const auto& service : filter->services.value()) {
+        services.insert(service);
+      }
     }
   }
+
+  // There isn't much support for GATT over BR/EDR from neither platforms nor
+  // devices so performing a Dual scan will find devices that the API is not
+  // able to interact with. To avoid wasting power and confusing users with
+  // devices they are not able to interact with, we only perform an LE Scan.
   auto discovery_filter = base::MakeUnique<device::BluetoothDiscoveryFilter>(
-      device::BLUETOOTH_TRANSPORT_DUAL);
+      device::BLUETOOTH_TRANSPORT_LE);
   for (const BluetoothUUID& service : services) {
     discovery_filter->AddUUID(service);
   }
@@ -167,22 +225,23 @@ UMARequestDeviceOutcome OutcomeFromChooserEvent(BluetoothChooser::Event event) {
       NOTREACHED();
       return UMARequestDeviceOutcome::SUCCESS;
     case BluetoothChooser::Event::RESCAN:
-      // Rescanning doesn't result in a IPC message for the request being sent
-      // so no need to histogram it.
-      NOTREACHED();
-      return UMARequestDeviceOutcome::SUCCESS;
+      return UMARequestDeviceOutcome::BLUETOOTH_CHOOSER_RESCAN;
   }
   NOTREACHED();
   return UMARequestDeviceOutcome::SUCCESS;
 }
 
+void RecordScanningDuration(const base::TimeDelta& duration) {
+  UMA_HISTOGRAM_LONG_TIMES("Bluetooth.Web.RequestDevice.ScanningDuration",
+                           duration);
+}
+
 }  // namespace
 
 BluetoothDeviceChooserController::BluetoothDeviceChooserController(
     WebBluetoothServiceImpl* web_bluetooth_service,
     RenderFrameHost* render_frame_host,
-    device::BluetoothAdapter* adapter,
-    base::TimeDelta scan_duration)
+    device::BluetoothAdapter* adapter)
     : adapter_(adapter),
       web_bluetooth_service_(web_bluetooth_service),
       render_frame_host_(render_frame_host),
@@ -191,7 +250,8 @@ BluetoothDeviceChooserController::BluetoothDeviceChooserController(
           FROM_HERE,
           // TODO(jyasskin): Add a way for tests to control the dialog
           // directly, and change this to a reasonable discovery timeout.
-          scan_duration,
+          base::TimeDelta::FromSeconds(
+              use_test_scan_duration_ ? kTestScanDuration : kScanDuration),
           base::Bind(&BluetoothDeviceChooserController::StopDeviceDiscovery,
                      // base::Timer guarantees it won't call back after its
                      // destructor starts.
@@ -202,9 +262,14 @@ BluetoothDeviceChooserController::BluetoothDeviceChooserController(
 }
 
 BluetoothDeviceChooserController::~BluetoothDeviceChooserController() {
+  if (scanning_start_time_) {
+    RecordScanningDuration(base::TimeTicks::Now() -
+                           scanning_start_time_.value());
+  }
+
   if (chooser_) {
     DCHECK(!error_callback_.is_null());
-    error_callback_.Run(blink::mojom::WebBluetoothError::CHOOSER_CANCELLED);
+    error_callback_.Run(blink::mojom::WebBluetoothResult::CHOOSER_CANCELLED);
   }
 }
 
@@ -221,24 +286,25 @@ void BluetoothDeviceChooserController::GetDevice(
   success_callback_ = success_callback;
   error_callback_ = error_callback;
 
-  // The renderer should never send empty filters.
-  if (HasEmptyOrInvalidFilter(options->filters)) {
+  // The renderer should never send invalid options.
+  if (IsOptionsInvalid(options)) {
     web_bluetooth_service_->CrashRendererAndClosePipe(
-        bad_message::BDH_EMPTY_OR_INVALID_FILTERS);
+        bad_message::BDH_INVALID_OPTIONS);
     return;
   }
   options_ = std::move(options);
   LogRequestDeviceOptions(options_);
 
-  // Check blacklist to reject invalid filters and adjust optional_services.
-  if (BluetoothBlacklist::Get().IsExcluded(options_->filters)) {
+  // Check blocklist to reject invalid filters and adjust optional_services.
+  if (options_->filters &&
+      BluetoothBlocklist::Get().IsExcluded(options_->filters.value())) {
     RecordRequestDeviceOutcome(
-        UMARequestDeviceOutcome::BLACKLISTED_SERVICE_IN_FILTER);
+        UMARequestDeviceOutcome::BLOCKLISTED_SERVICE_IN_FILTER);
     PostErrorCallback(
-        blink::mojom::WebBluetoothError::REQUEST_DEVICE_WITH_BLACKLISTED_UUID);
+        blink::mojom::WebBluetoothResult::REQUEST_DEVICE_WITH_BLOCKLISTED_UUID);
     return;
   }
-  BluetoothBlacklist::Get().RemoveExcludedUUIDs(options_.get());
+  BluetoothBlocklist::Get().RemoveExcludedUUIDs(options_.get());
 
   const url::Origin requesting_origin =
       render_frame_host_->GetLastCommittedOrigin();
@@ -249,7 +315,7 @@ void BluetoothDeviceChooserController::GetDevice(
   // matching origins. When relaxing this, take care to handle non-sandboxed
   // unique origins.
   if (!embedding_origin.IsSameOriginWith(requesting_origin)) {
-    PostErrorCallback(blink::mojom::WebBluetoothError::
+    PostErrorCallback(blink::mojom::WebBluetoothResult::
                           REQUEST_DEVICE_FROM_CROSS_ORIGIN_IFRAME);
     return;
   }
@@ -261,7 +327,7 @@ void BluetoothDeviceChooserController::GetDevice(
     VLOG(1) << "Bluetooth Adapter not present. Can't serve requestDevice.";
     RecordRequestDeviceOutcome(
         UMARequestDeviceOutcome::BLUETOOTH_ADAPTER_NOT_PRESENT);
-    PostErrorCallback(blink::mojom::WebBluetoothError::NO_BLUETOOTH_ADAPTER);
+    PostErrorCallback(blink::mojom::WebBluetoothResult::NO_BLUETOOTH_ADAPTER);
     return;
   }
 
@@ -271,7 +337,7 @@ void BluetoothDeviceChooserController::GetDevice(
     case ContentBrowserClient::AllowWebBluetoothResult::BLOCK_POLICY: {
       RecordRequestDeviceOutcome(
           UMARequestDeviceOutcome::BLUETOOTH_CHOOSER_POLICY_DISABLED);
-      PostErrorCallback(blink::mojom::WebBluetoothError::
+      PostErrorCallback(blink::mojom::WebBluetoothResult::
                             CHOOSER_NOT_SHOWN_API_LOCALLY_DISABLED);
       return;
     }
@@ -284,7 +350,7 @@ void BluetoothDeviceChooserController::GetDevice(
       // Block requests.
       RecordRequestDeviceOutcome(
           UMARequestDeviceOutcome::BLUETOOTH_GLOBALLY_DISABLED);
-      PostErrorCallback(blink::mojom::WebBluetoothError::
+      PostErrorCallback(blink::mojom::WebBluetoothResult::
                             CHOOSER_NOT_SHOWN_API_GLOBALLY_DISABLED);
       return;
     }
@@ -303,7 +369,8 @@ void BluetoothDeviceChooserController::GetDevice(
 
   if (!chooser_.get()) {
     PostErrorCallback(
-        blink::mojom::WebBluetoothError::WEB_BLUETOOTH_NOT_SUPPORTED);
+        blink::mojom::WebBluetoothResult::WEB_BLUETOOTH_NOT_SUPPORTED);
+    return;
   }
 
   if (!chooser_->CanAskForScanningPermission()) {
@@ -313,7 +380,7 @@ void BluetoothDeviceChooserController::GetDevice(
     return;
   }
 
-  PopulateFoundDevices();
+  PopulateConnectedDevices();
   if (!chooser_.get()) {
     // If the dialog's closing, no need to do any of the rest of this.
     return;
@@ -330,9 +397,18 @@ void BluetoothDeviceChooserController::GetDevice(
 
 void BluetoothDeviceChooserController::AddFilteredDevice(
     const device::BluetoothDevice& device) {
-  if (chooser_.get() && MatchesFilters(device, options_->filters)) {
-    VLOG(1) << "Adding device to chooser: " << device.GetAddress();
-    chooser_->AddDevice(device.GetAddress(), device.GetNameForDisplay());
+  base::Optional<std::string> device_name = device.GetName();
+  if (chooser_.get()) {
+    if (options_->accept_all_devices ||
+        MatchesFilters(device_name ? &device_name.value() : nullptr,
+                       device.GetUUIDs(), options_->filters)) {
+      base::Optional<int8_t> rssi = device.GetInquiryRSSI();
+      chooser_->AddOrUpdateDevice(
+          device.GetAddress(), !!device.GetName() /* should_update_name */,
+          device.GetNameForDisplay(), device.IsGattConnected(),
+          web_bluetooth_service_->IsDevicePaired(device.GetAddress()),
+          rssi ? CalculateSignalStrengthLevel(rssi.value()) : -1);
+    }
   }
 }
 
@@ -356,11 +432,39 @@ void BluetoothDeviceChooserController::AdapterPoweredChanged(bool powered) {
   }
 }
 
-void BluetoothDeviceChooserController::PopulateFoundDevices() {
-  VLOG(1) << "Populating " << adapter_->GetDevices().size()
-          << " devices in chooser.";
+int BluetoothDeviceChooserController::CalculateSignalStrengthLevel(
+    int8_t rssi) {
+  RecordRSSISignalStrength(rssi);
+
+  if (rssi <= kMinRSSI) {
+    RecordRSSISignalStrengthLevel(
+        UMARSSISignalStrengthLevel::LESS_THAN_OR_EQUAL_TO_MIN_RSSI);
+    return 0;
+  }
+
+  if (rssi >= kMaxRSSI) {
+    RecordRSSISignalStrengthLevel(
+        UMARSSISignalStrengthLevel::GREATER_THAN_OR_EQUAL_TO_MAX_RSSI);
+    return kNumSignalStrengthLevels - 1;
+  }
+
+  double input_range = kMaxRSSI - kMinRSSI;
+  double output_range = kNumSignalStrengthLevels - 1;
+  int level = static_cast<int>((rssi - kMinRSSI) * output_range / input_range);
+  DCHECK(kNumSignalStrengthLevels == arraysize(kRSSISignalStrengthEnumTable));
+  RecordRSSISignalStrengthLevel(kRSSISignalStrengthEnumTable[level]);
+  return level;
+}
+
+void BluetoothDeviceChooserController::SetTestScanDurationForTesting() {
+  BluetoothDeviceChooserController::use_test_scan_duration_ = true;
+}
+
+void BluetoothDeviceChooserController::PopulateConnectedDevices() {
   for (const device::BluetoothDevice* device : adapter_->GetDevices()) {
-    AddFilteredDevice(*device);
+    if (device->IsGattConnected()) {
+      AddFilteredDevice(*device);
+    }
   }
 }
 
@@ -373,6 +477,8 @@ void BluetoothDeviceChooserController::StartDeviceDiscovery() {
     return;
   }
 
+  scanning_start_time_ = base::TimeTicks::Now();
+
   chooser_->ShowDiscoveryState(BluetoothChooser::DiscoveryState::DISCOVERING);
   adapter_->StartDiscoverySessionWithFilter(
       ComputeScanFilter(options_->filters),
@@ -386,6 +492,13 @@ void BluetoothDeviceChooserController::StartDeviceDiscovery() {
 
 void BluetoothDeviceChooserController::StopDeviceDiscovery() {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
+
+  if (scanning_start_time_) {
+    RecordScanningDuration(base::TimeTicks::Now() -
+                           scanning_start_time_.value());
+    scanning_start_time_.reset();
+  }
+
   StopDiscoverySession(std::move(discovery_session_));
   if (chooser_) {
     chooser_->ShowDiscoveryState(BluetoothChooser::DiscoveryState::IDLE);
@@ -420,36 +533,39 @@ void BluetoothDeviceChooserController::OnBluetoothChooserEvent(
 
   switch (event) {
     case BluetoothChooser::Event::RESCAN:
-      PopulateFoundDevices();
+      RecordRequestDeviceOutcome(OutcomeFromChooserEvent(event));
+      PopulateConnectedDevices();
       DCHECK(chooser_);
       StartDeviceDiscovery();
       // No need to close the chooser so we return.
       return;
     case BluetoothChooser::Event::DENIED_PERMISSION:
       RecordRequestDeviceOutcome(OutcomeFromChooserEvent(event));
-      PostErrorCallback(blink::mojom::WebBluetoothError::
+      PostErrorCallback(blink::mojom::WebBluetoothResult::
                             CHOOSER_NOT_SHOWN_USER_DENIED_PERMISSION_TO_SCAN);
       break;
     case BluetoothChooser::Event::CANCELLED:
       RecordRequestDeviceOutcome(OutcomeFromChooserEvent(event));
-      PostErrorCallback(blink::mojom::WebBluetoothError::CHOOSER_CANCELLED);
+      PostErrorCallback(blink::mojom::WebBluetoothResult::CHOOSER_CANCELLED);
       break;
     case BluetoothChooser::Event::SHOW_OVERVIEW_HELP:
       VLOG(1) << "Overview Help link pressed.";
       RecordRequestDeviceOutcome(OutcomeFromChooserEvent(event));
-      PostErrorCallback(blink::mojom::WebBluetoothError::CHOOSER_CANCELLED);
+      PostErrorCallback(blink::mojom::WebBluetoothResult::CHOOSER_CANCELLED);
       break;
     case BluetoothChooser::Event::SHOW_ADAPTER_OFF_HELP:
       VLOG(1) << "Adapter Off Help link pressed.";
       RecordRequestDeviceOutcome(OutcomeFromChooserEvent(event));
-      PostErrorCallback(blink::mojom::WebBluetoothError::CHOOSER_CANCELLED);
+      PostErrorCallback(blink::mojom::WebBluetoothResult::CHOOSER_CANCELLED);
       break;
     case BluetoothChooser::Event::SHOW_NEED_LOCATION_HELP:
       VLOG(1) << "Need Location Help link pressed.";
       RecordRequestDeviceOutcome(OutcomeFromChooserEvent(event));
-      PostErrorCallback(blink::mojom::WebBluetoothError::CHOOSER_CANCELLED);
+      PostErrorCallback(blink::mojom::WebBluetoothResult::CHOOSER_CANCELLED);
       break;
     case BluetoothChooser::Event::SELECTED:
+      // RecordRequestDeviceOutcome is called in the callback, because the
+      // device may have vanished.
       PostSuccessCallback(device_address);
       break;
   }
@@ -468,7 +584,7 @@ void BluetoothDeviceChooserController::PostSuccessCallback(
 }
 
 void BluetoothDeviceChooserController::PostErrorCallback(
-    blink::mojom::WebBluetoothError error) {
+    blink::mojom::WebBluetoothResult error) {
   if (!base::ThreadTaskRunnerHandle::Get()->PostTask(
           FROM_HERE, base::Bind(error_callback_, error))) {
     LOG(WARNING) << "No TaskRunner.";
