@@ -14,6 +14,7 @@
 #include "base/lazy_instance.h"
 #include "base/macros.h"
 #include "base/profiler/scoped_tracker.h"
+#include "base/run_loop.h"
 #include "base/single_thread_task_runner.h"
 #include "base/threading/platform_thread.h"
 #include "base/threading/sequenced_worker_pool.h"
@@ -100,16 +101,24 @@ base::LazyInstance<BrowserThreadTaskRunners>::Leaky g_task_runners =
 
 struct BrowserThreadGlobals {
   BrowserThreadGlobals()
-      : blocking_pool(new base::SequencedWorkerPool(3, "BrowserBlocking")) {
+      : blocking_pool(
+            new base::SequencedWorkerPool(3,
+                                          "BrowserBlocking",
+                                          base::TaskPriority::USER_VISIBLE)) {
     memset(threads, 0, BrowserThread::ID_COUNT * sizeof(threads[0]));
+    memset(thread_ids, 0, BrowserThread::ID_COUNT * sizeof(thread_ids[0]));
     memset(thread_delegates, 0,
            BrowserThread::ID_COUNT * sizeof(thread_delegates[0]));
   }
 
-  // This lock protects |threads|. Do not read or modify that array
-  // without holding this lock. Do not block while holding this lock.
+  // This lock protects |threads| and |thread_ids|. Do not read or modify those
+  // arrays without holding this lock. Do not block while holding this lock.
   base::Lock lock;
 
+  // This array is protected by |lock|. IDs in this array are populated as soon
+  // as their respective thread is started and are never reset.
+  base::PlatformThreadId thread_ids[BrowserThread::ID_COUNT];
+
   // This array is protected by |lock|. The threads are not owned by this
   // array. Typically, the threads are owned on the UI thread by
   // BrowserMainLoop. BrowserThreadImpl objects remove themselves from this
@@ -131,13 +140,26 @@ base::LazyInstance<BrowserThreadGlobals>::Leaky
 BrowserThreadImpl::BrowserThreadImpl(ID identifier)
     : Thread(GetThreadName(identifier)), identifier_(identifier) {
   Initialize();
+
+  // Unit tests may create multiple TestBrowserThreadBundles, causing the same
+  // BrowserThread ID to be reinitialized. We explicitly clear the thread ID
+  // here so Start() can sanity check.
+  BrowserThreadGlobals& globals = g_globals.Get();
+  base::AutoLock lock(globals.lock);
+  globals.thread_ids[identifier] = base::kInvalidThreadId;
 }
 
 BrowserThreadImpl::BrowserThreadImpl(ID identifier,
                                      base::MessageLoop* message_loop)
     : Thread(GetThreadName(identifier)), identifier_(identifier) {
-  set_message_loop(message_loop);
+  SetMessageLoop(message_loop);
   Initialize();
+
+  // If constructed with an explicit message loop, this is a fake BrowserThread
+  // which runs on the current thread.
+  BrowserThreadGlobals& globals = g_globals.Get();
+  base::AutoLock lock(globals.lock);
+  globals.thread_ids[identifier] = base::PlatformThread::CurrentId();
 }
 
 // static
@@ -154,7 +176,7 @@ void BrowserThreadImpl::ShutdownThreadPool() {
 // static
 void BrowserThreadImpl::FlushThreadPoolHelperForTesting() {
   // We don't want to create a pool if none exists.
-  if (g_globals == NULL)
+  if (g_globals == nullptr)
     return;
   g_globals.Get().blocking_pool->FlushForTesting();
   disk_cache::SimpleBackendImpl::FlushWorkerPoolForTesting();
@@ -163,6 +185,16 @@ void BrowserThreadImpl::FlushThreadPoolHelperForTesting() {
 void BrowserThreadImpl::Init() {
   BrowserThreadGlobals& globals = g_globals.Get();
 
+  if (BrowserThread::CurrentlyOn(BrowserThread::DB) ||
+      BrowserThread::CurrentlyOn(BrowserThread::FILE) ||
+      BrowserThread::CurrentlyOn(BrowserThread::FILE_USER_BLOCKING) ||
+      BrowserThread::CurrentlyOn(BrowserThread::PROCESS_LAUNCHER) ||
+      BrowserThread::CurrentlyOn(BrowserThread::CACHE)) {
+    base::MessageLoop* message_loop = base::MessageLoop::current();
+    message_loop->DisallowNesting();
+    message_loop->DisallowTaskObservers();
+  }
+
   using base::subtle::AtomicWord;
   AtomicWord* storage =
       reinterpret_cast<AtomicWord*>(&globals.thread_delegates[identifier_]);
@@ -178,56 +210,54 @@ void BrowserThreadImpl::Init() {
 MSVC_DISABLE_OPTIMIZE()
 MSVC_PUSH_DISABLE_WARNING(4748)
 
-NOINLINE void BrowserThreadImpl::UIThreadRun(base::MessageLoop* message_loop) {
+NOINLINE void BrowserThreadImpl::UIThreadRun(base::RunLoop* run_loop) {
   volatile int line_number = __LINE__;
-  Thread::Run(message_loop);
+  Thread::Run(run_loop);
   CHECK_GT(line_number, 0);
 }
 
-NOINLINE void BrowserThreadImpl::DBThreadRun(base::MessageLoop* message_loop) {
+NOINLINE void BrowserThreadImpl::DBThreadRun(base::RunLoop* run_loop) {
   volatile int line_number = __LINE__;
-  Thread::Run(message_loop);
+  Thread::Run(run_loop);
   CHECK_GT(line_number, 0);
 }
 
-NOINLINE void BrowserThreadImpl::FileThreadRun(
-    base::MessageLoop* message_loop) {
+NOINLINE void BrowserThreadImpl::FileThreadRun(base::RunLoop* run_loop) {
   volatile int line_number = __LINE__;
-  Thread::Run(message_loop);
+  Thread::Run(run_loop);
   CHECK_GT(line_number, 0);
 }
 
 NOINLINE void BrowserThreadImpl::FileUserBlockingThreadRun(
-    base::MessageLoop* message_loop) {
+    base::RunLoop* run_loop) {
   volatile int line_number = __LINE__;
-  Thread::Run(message_loop);
+  Thread::Run(run_loop);
   CHECK_GT(line_number, 0);
 }
 
 NOINLINE void BrowserThreadImpl::ProcessLauncherThreadRun(
-    base::MessageLoop* message_loop) {
+    base::RunLoop* run_loop) {
   volatile int line_number = __LINE__;
-  Thread::Run(message_loop);
+  Thread::Run(run_loop);
   CHECK_GT(line_number, 0);
 }
 
-NOINLINE void BrowserThreadImpl::CacheThreadRun(
-    base::MessageLoop* message_loop) {
+NOINLINE void BrowserThreadImpl::CacheThreadRun(base::RunLoop* run_loop) {
   volatile int line_number = __LINE__;
-  Thread::Run(message_loop);
+  Thread::Run(run_loop);
   CHECK_GT(line_number, 0);
 }
 
-NOINLINE void BrowserThreadImpl::IOThreadRun(base::MessageLoop* message_loop) {
+NOINLINE void BrowserThreadImpl::IOThreadRun(base::RunLoop* run_loop) {
   volatile int line_number = __LINE__;
-  Thread::Run(message_loop);
+  Thread::Run(run_loop);
   CHECK_GT(line_number, 0);
 }
 
 MSVC_POP_WARNING()
 MSVC_ENABLE_OPTIMIZE();
 
-void BrowserThreadImpl::Run(base::MessageLoop* message_loop) {
+void BrowserThreadImpl::Run(base::RunLoop* run_loop) {
 #if defined(OS_ANDROID)
   // Not to reset thread name to "Thread-???" by VM, attach VM with thread name.
   // Though it may create unnecessary VM thread objects, keeping thread name
@@ -240,23 +270,22 @@ void BrowserThreadImpl::Run(base::MessageLoop* message_loop) {
   BrowserThread::ID thread_id = ID_COUNT;
   CHECK(GetCurrentThreadIdentifier(&thread_id));
   CHECK_EQ(identifier_, thread_id);
-  CHECK_EQ(Thread::message_loop(), message_loop);
 
   switch (identifier_) {
     case BrowserThread::UI:
-      return UIThreadRun(message_loop);
+      return UIThreadRun(run_loop);
     case BrowserThread::DB:
-      return DBThreadRun(message_loop);
+      return DBThreadRun(run_loop);
     case BrowserThread::FILE:
-      return FileThreadRun(message_loop);
+      return FileThreadRun(run_loop);
     case BrowserThread::FILE_USER_BLOCKING:
-      return FileUserBlockingThreadRun(message_loop);
+      return FileUserBlockingThreadRun(run_loop);
     case BrowserThread::PROCESS_LAUNCHER:
-      return ProcessLauncherThreadRun(message_loop);
+      return ProcessLauncherThreadRun(run_loop);
     case BrowserThread::CACHE:
-      return CacheThreadRun(message_loop);
+      return CacheThreadRun(run_loop);
     case BrowserThread::IO:
-      return IOThreadRun(message_loop);
+      return IOThreadRun(run_loop);
     case BrowserThread::ID_COUNT:
       CHECK(false);  // This shouldn't actually be reached!
       break;
@@ -285,15 +314,16 @@ void BrowserThreadImpl::CleanUp() {
   // to prevent a race with accessing the message loop in PostTaskHelper(),
   // remove this thread from the global array now.
   base::AutoLock lock(globals.lock);
-  globals.threads[identifier_] = NULL;
+  globals.threads[identifier_] = nullptr;
 }
 
 void BrowserThreadImpl::Initialize() {
   BrowserThreadGlobals& globals = g_globals.Get();
 
   base::AutoLock lock(globals.lock);
-  DCHECK(identifier_ >= 0 && identifier_ < ID_COUNT);
-  DCHECK(globals.threads[identifier_] == NULL);
+  DCHECK_GE(identifier_, 0);
+  DCHECK_LT(identifier_, ID_COUNT);
+  DCHECK_EQ(globals.threads[identifier_], nullptr);
   globals.threads[identifier_] = this;
 }
 
@@ -305,7 +335,7 @@ BrowserThreadImpl::~BrowserThreadImpl() {
 
   BrowserThreadGlobals& globals = g_globals.Get();
   base::AutoLock lock(globals.lock);
-  globals.threads[identifier_] = NULL;
+  globals.threads[identifier_] = nullptr;
 #ifndef NDEBUG
   // Double check that the threads are ordered correctly in the enumeration.
   for (int i = identifier_ + 1; i < ID_COUNT; ++i) {
@@ -315,15 +345,28 @@ BrowserThreadImpl::~BrowserThreadImpl() {
 #endif
 }
 
+bool BrowserThreadImpl::Start() {
+  return StartWithOptions(base::Thread::Options());
+}
+
 bool BrowserThreadImpl::StartWithOptions(const Options& options) {
   // The global thread table needs to be locked while a new thread is
   // starting, as the new thread can asynchronously start touching the
   // table (and other thread's message_loop).
   BrowserThreadGlobals& globals = g_globals.Get();
   base::AutoLock lock(globals.lock);
-  return Thread::StartWithOptions(options);
+  DCHECK_EQ(globals.thread_ids[identifier_], base::kInvalidThreadId);
+  bool result = Thread::StartWithOptions(options);
+  globals.thread_ids[identifier_] = GetThreadId();
+  return result;
 }
 
+bool BrowserThreadImpl::StartAndWaitForTesting() {
+  if (!Start())
+    return false;
+  WaitUntilThreadStarted();
+  return true;
+}
 // static
 bool BrowserThreadImpl::PostTaskHelper(
     BrowserThread::ID identifier,
@@ -331,7 +374,8 @@ bool BrowserThreadImpl::PostTaskHelper(
     const base::Closure& task,
     base::TimeDelta delay,
     bool nestable) {
-  DCHECK(identifier >= 0 && identifier < ID_COUNT);
+  DCHECK_GE(identifier, 0);
+  DCHECK_LT(identifier, ID_COUNT);
   // Optimization: to avoid unnecessary locks, we listed the ID enumeration in
   // order of lifetime.  So no need to lock if we know that the target thread
   // outlives current thread.
@@ -349,7 +393,7 @@ bool BrowserThreadImpl::PostTaskHelper(
 
   base::MessageLoop* message_loop =
       globals.threads[identifier] ? globals.threads[identifier]->message_loop()
-                                  : NULL;
+                                  : nullptr;
   if (message_loop) {
     if (nestable) {
       message_loop->task_runner()->PostDelayedTask(from_here, task, delay);
@@ -406,23 +450,23 @@ base::SequencedWorkerPool* BrowserThread::GetBlockingPool() {
 
 // static
 bool BrowserThread::IsThreadInitialized(ID identifier) {
-  if (g_globals == NULL)
+  if (g_globals == nullptr)
     return false;
 
   BrowserThreadGlobals& globals = g_globals.Get();
   base::AutoLock lock(globals.lock);
-  DCHECK(identifier >= 0 && identifier < ID_COUNT);
-  return globals.threads[identifier] != NULL;
+  DCHECK_GE(identifier, 0);
+  DCHECK_LT(identifier, ID_COUNT);
+  return globals.threads[identifier] != nullptr;
 }
 
 // static
 bool BrowserThread::CurrentlyOn(ID identifier) {
   BrowserThreadGlobals& globals = g_globals.Get();
   base::AutoLock lock(globals.lock);
-  DCHECK(identifier >= 0 && identifier < ID_COUNT);
-  return globals.threads[identifier] &&
-         globals.threads[identifier]->message_loop() ==
-             base::MessageLoop::current();
+  DCHECK_GE(identifier, 0);
+  DCHECK_LT(identifier, ID_COUNT);
+  return base::PlatformThread::CurrentId() == globals.thread_ids[identifier];
 }
 
 // static
@@ -441,12 +485,13 @@ std::string BrowserThread::GetDCheckCurrentlyOnErrorMessage(ID expected) {
 
 // static
 bool BrowserThread::IsMessageLoopValid(ID identifier) {
-  if (g_globals == NULL)
+  if (g_globals == nullptr)
     return false;
 
   BrowserThreadGlobals& globals = g_globals.Get();
   base::AutoLock lock(globals.lock);
-  DCHECK(identifier >= 0 && identifier < ID_COUNT);
+  DCHECK_GE(identifier, 0);
+  DCHECK_LT(identifier, ID_COUNT);
   return globals.threads[identifier] &&
          globals.threads[identifier]->message_loop();
 }
@@ -493,14 +538,13 @@ bool BrowserThread::PostTaskAndReply(
     const tracked_objects::Location& from_here,
     const base::Closure& task,
     const base::Closure& reply) {
-  return GetMessageLoopProxyForThread(identifier)->PostTaskAndReply(from_here,
-                                                                    task,
-                                                                    reply);
+  return GetTaskRunnerForThread(identifier)
+      ->PostTaskAndReply(from_here, task, reply);
 }
 
 // static
 bool BrowserThread::GetCurrentThreadIdentifier(ID* identifier) {
-  if (g_globals == NULL)
+  if (g_globals == nullptr)
     return false;
 
   base::MessageLoop* cur_message_loop = base::MessageLoop::current();
@@ -523,24 +567,10 @@ bool BrowserThread::GetCurrentThreadIdentifier(ID* identifier) {
 
 // static
 scoped_refptr<base::SingleThreadTaskRunner>
-BrowserThread::GetMessageLoopProxyForThread(ID identifier) {
+BrowserThread::GetTaskRunnerForThread(ID identifier) {
   return g_task_runners.Get().proxies[identifier];
 }
 
-// static
-base::MessageLoop* BrowserThread::UnsafeGetMessageLoopForThread(ID identifier) {
-  if (g_globals == NULL)
-    return NULL;
-
-  BrowserThreadGlobals& globals = g_globals.Get();
-  base::AutoLock lock(globals.lock);
-  base::Thread* thread = globals.threads[identifier];
-  DCHECK(thread);
-  base::MessageLoop* loop = thread->message_loop();
-  return loop;
-}
-
-// static
 void BrowserThread::SetDelegate(ID identifier,
                                 BrowserThreadDelegate* delegate) {
   using base::subtle::AtomicWord;
