@@ -8,6 +8,7 @@
 #include <stdint.h>
 
 #include <memory>
+#include <set>
 #include <utility>
 
 #include "base/files/file_path.h"
@@ -62,8 +63,8 @@ std::unique_ptr<storage::BlobProtocolHandler> CreateMockBlobProtocolHandler(
 // A disk_cache::Backend wrapper that can delay operations.
 class DelayableBackend : public disk_cache::Backend {
  public:
-  DelayableBackend(std::unique_ptr<disk_cache::Backend> backend)
-      : backend_(std::move(backend)), delay_open_(false) {}
+  explicit DelayableBackend(std::unique_ptr<disk_cache::Backend> backend)
+      : backend_(std::move(backend)), delay_doom_(false) {}
 
   // disk_cache::Backend overrides
   net::CacheType GetCacheType() const override {
@@ -73,15 +74,9 @@ class DelayableBackend : public disk_cache::Backend {
   int OpenEntry(const std::string& key,
                 disk_cache::Entry** entry,
                 const CompletionCallback& callback) override {
-    if (delay_open_) {
-      open_entry_callback_ =
-          base::Bind(&DelayableBackend::OpenEntryDelayedImpl,
-                     base::Unretained(this), key, entry, callback);
-      return net::ERR_IO_PENDING;
-    }
-
     return backend_->OpenEntry(key, entry, callback);
   }
+
   int CreateEntry(const std::string& key,
                   disk_cache::Entry** entry,
                   const CompletionCallback& callback) override {
@@ -89,6 +84,12 @@ class DelayableBackend : public disk_cache::Backend {
   }
   int DoomEntry(const std::string& key,
                 const CompletionCallback& callback) override {
+    if (delay_doom_) {
+      doom_entry_callback_ = base::Bind(&DelayableBackend::DoomEntryDelayedImpl,
+                                        base::Unretained(this), key, callback);
+      return net::ERR_IO_PENDING;
+    }
+
     return backend_->DoomEntry(key, callback);
   }
   int DoomAllEntries(const CompletionCallback& callback) override {
@@ -117,26 +118,25 @@ class DelayableBackend : public disk_cache::Backend {
     return backend_->OnExternalCacheHit(key);
   }
 
-  // Call to continue a delayed open.
-  void OpenEntryContinue() {
-    EXPECT_FALSE(open_entry_callback_.is_null());
-    open_entry_callback_.Run();
+  // Call to continue a delayed doom.
+  void DoomEntryContinue() {
+    EXPECT_FALSE(doom_entry_callback_.is_null());
+    doom_entry_callback_.Run();
   }
 
-  void set_delay_open(bool value) { delay_open_ = value; }
+  void set_delay_doom(bool value) { delay_doom_ = value; }
 
  private:
-  void OpenEntryDelayedImpl(const std::string& key,
-                            disk_cache::Entry** entry,
+  void DoomEntryDelayedImpl(const std::string& key,
                             const CompletionCallback& callback) {
-    int rv = backend_->OpenEntry(key, entry, callback);
+    int rv = backend_->DoomEntry(key, callback);
     if (rv != net::ERR_IO_PENDING)
       callback.Run(rv);
   }
 
   std::unique_ptr<disk_cache::Backend> backend_;
-  bool delay_open_;
-  base::Closure open_entry_callback_;
+  bool delay_doom_;
+  base::Closure doom_entry_callback_;
 };
 
 void CopyBody(const storage::BlobDataHandle& blob_handle, std::string* output) {
@@ -305,6 +305,8 @@ class TestCacheStorageCache : public CacheStorageCache {
     return delayable_backend;
   }
 
+  void Init() { InitBackend(); }
+
  private:
   std::unique_ptr<CacheStorageCacheHandle> CreateCacheHandle() override {
     // Returns an empty handle. There is no need for CacheStorage and its
@@ -330,13 +332,16 @@ class CacheStorageCacheTest : public testing::Test {
     base::RunLoop().RunUntilIdle();
     blob_storage_context_ = blob_storage_context->context();
 
-    if (!MemoryOnly())
+    const bool is_incognito = MemoryOnly();
+    base::FilePath temp_dir_path;
+    if (!is_incognito) {
       ASSERT_TRUE(temp_dir_.CreateUniqueTempDir());
+      temp_dir_path = temp_dir_.GetPath();
+    }
 
     quota_policy_ = new MockSpecialStoragePolicy;
     mock_quota_manager_ = new MockQuotaManager(
-        MemoryOnly() /* is incognito */, temp_dir_.path(),
-        base::ThreadTaskRunnerHandle::Get().get(),
+        is_incognito, temp_dir_path, base::ThreadTaskRunnerHandle::Get().get(),
         base::ThreadTaskRunnerHandle::Get().get(), quota_policy_.get());
     mock_quota_manager_->SetQuota(GURL(kOrigin), storage::kStorageTypeTemporary,
                                   1024 * 1024 * 100);
@@ -357,10 +362,11 @@ class CacheStorageCacheTest : public testing::Test {
     CreateRequests(blob_storage_context);
 
     cache_ = base::MakeUnique<TestCacheStorageCache>(
-        GURL(kOrigin), kCacheName, temp_dir_.path(), nullptr /* CacheStorage */,
+        GURL(kOrigin), kCacheName, temp_dir_path, nullptr /* CacheStorage */,
         BrowserContext::GetDefaultStoragePartition(&browser_context_)
             ->GetURLRequestContext(),
         quota_manager_proxy_, blob_storage_context->context()->AsWeakPtr());
+    cache_->Init();
   }
 
   void TearDown() override {
@@ -381,6 +387,9 @@ class CacheStorageCacheTest : public testing::Test {
     no_body_request_ =
         ServiceWorkerFetchRequest(GURL("http://example.com/no_body.html"),
                                   "GET", headers, Referrer(), false);
+    body_head_request_ =
+        ServiceWorkerFetchRequest(GURL("http://example.com/body.html"), "HEAD",
+                                  headers, Referrer(), false);
 
     std::string expected_response;
     for (int i = 0; i < 100; ++i)
@@ -422,9 +431,9 @@ class CacheStorageCacheTest : public testing::Test {
 
   std::unique_ptr<ServiceWorkerFetchRequest> CopyFetchRequest(
       const ServiceWorkerFetchRequest& request) {
-    return base::WrapUnique(new ServiceWorkerFetchRequest(
+    return base::MakeUnique<ServiceWorkerFetchRequest>(
         request.url, request.method, request.headers, request.referrer,
-        request.is_reload));
+        request.is_reload);
   }
 
   CacheStorageError BatchOperation(
@@ -454,11 +463,13 @@ class CacheStorageCacheTest : public testing::Test {
     return error == CACHE_STORAGE_OK;
   }
 
-  bool Match(const ServiceWorkerFetchRequest& request) {
+  bool Match(const ServiceWorkerFetchRequest& request,
+             const CacheStorageCacheQueryParams& match_params =
+                 CacheStorageCacheQueryParams()) {
     std::unique_ptr<base::RunLoop> loop(new base::RunLoop());
 
     cache_->Match(
-        CopyFetchRequest(request),
+        CopyFetchRequest(request), match_params,
         base::Bind(&CacheStorageCacheTest::ResponseAndErrorCallback,
                    base::Unretained(this), base::Unretained(loop.get())));
     loop->Run();
@@ -501,12 +512,16 @@ class CacheStorageCacheTest : public testing::Test {
     return error == CACHE_STORAGE_OK;
   }
 
-  bool Keys() {
+  bool Keys(
+      const ServiceWorkerFetchRequest& request = ServiceWorkerFetchRequest(),
+      const CacheStorageCacheQueryParams& match_params =
+          CacheStorageCacheQueryParams()) {
     std::unique_ptr<base::RunLoop> loop(new base::RunLoop());
 
-    cache_->Keys(base::Bind(&CacheStorageCacheTest::RequestsCallback,
-                            base::Unretained(this),
-                            base::Unretained(loop.get())));
+    cache_->Keys(
+        CopyFetchRequest(request), match_params,
+        base::Bind(&CacheStorageCacheTest::RequestsCallback,
+                   base::Unretained(this), base::Unretained(loop.get())));
     loop->Run();
 
     return callback_error_ == CACHE_STORAGE_OK;
@@ -635,21 +650,6 @@ class CacheStorageCacheTest : public testing::Test {
       run_loop->Quit();
   }
 
-  bool VerifyKeys(const std::vector<std::string>& expected_keys) {
-    if (expected_keys.size() != callback_strings_.size())
-      return false;
-
-    std::set<std::string> found_set;
-    for (int i = 0, max = callback_strings_.size(); i < max; ++i)
-      found_set.insert(callback_strings_[i]);
-
-    for (int i = 0, max = expected_keys.size(); i < max; ++i) {
-      if (found_set.find(expected_keys[i]) == found_set.end())
-        return false;
-    }
-    return true;
-  }
-
   bool TestResponseType(blink::WebServiceWorkerResponseType response_type) {
     body_response_.response_type = response_type;
     EXPECT_TRUE(Put(body_request_, body_response_));
@@ -667,6 +667,10 @@ class CacheStorageCacheTest : public testing::Test {
 
   virtual bool MemoryOnly() { return false; }
 
+  void SetMaxQuerySizeBytes(size_t max_bytes) {
+    cache_->max_query_size_bytes_ = max_bytes;
+  }
+
  protected:
   base::ScopedTempDir temp_dir_;
   TestBrowserThreadBundle browser_thread_bundle_;
@@ -685,6 +689,7 @@ class CacheStorageCacheTest : public testing::Test {
   ServiceWorkerResponse body_response_with_query_;
   ServiceWorkerFetchRequest no_body_request_;
   ServiceWorkerResponse no_body_response_;
+  ServiceWorkerFetchRequest body_head_request_;
   std::unique_ptr<storage::BlobDataHandle> blob_handle_;
   std::string expected_blob_data_;
 
@@ -742,6 +747,69 @@ TEST_P(CacheStorageCacheTestP, PutBody_Multiple) {
   EXPECT_TRUE(Match(operation3.request));
 }
 
+TEST_P(CacheStorageCacheTestP, MatchLimit) {
+  EXPECT_TRUE(Put(no_body_request_, no_body_response_));
+  EXPECT_TRUE(Match(no_body_request_));
+
+  size_t max_size = no_body_request_.EstimatedStructSize() +
+                    callback_response_->EstimatedStructSize();
+  SetMaxQuerySizeBytes(max_size);
+  EXPECT_TRUE(Match(no_body_request_));
+
+  SetMaxQuerySizeBytes(max_size - 1);
+  EXPECT_FALSE(Match(no_body_request_));
+  EXPECT_EQ(CACHE_STORAGE_ERROR_QUERY_TOO_LARGE, callback_error_);
+}
+
+TEST_P(CacheStorageCacheTestP, MatchAllLimit) {
+  EXPECT_TRUE(Put(body_request_, no_body_response_));
+  EXPECT_TRUE(Put(body_request_with_query_, no_body_response_));
+  EXPECT_TRUE(Match(body_request_));
+
+  size_t body_request_size = body_request_.EstimatedStructSize() +
+                             callback_response_->EstimatedStructSize();
+  size_t query_request_size = body_request_with_query_.EstimatedStructSize() +
+                              callback_response_->EstimatedStructSize();
+
+  std::unique_ptr<CacheStorageCache::Responses> responses;
+  std::unique_ptr<CacheStorageCache::BlobDataHandles> body_handles;
+  CacheStorageCacheQueryParams match_params;
+
+  // There is enough room for both requests and responses
+  SetMaxQuerySizeBytes(body_request_size + query_request_size);
+  EXPECT_TRUE(MatchAll(body_request_, match_params, &responses, &body_handles));
+  EXPECT_EQ(1u, responses->size());
+
+  match_params.ignore_search = true;
+  EXPECT_TRUE(MatchAll(body_request_, match_params, &responses, &body_handles));
+  EXPECT_EQ(2u, responses->size());
+
+  // There is not enough room for both requests and responses
+  SetMaxQuerySizeBytes(body_request_size);
+  match_params.ignore_search = false;
+  EXPECT_TRUE(MatchAll(body_request_, match_params, &responses, &body_handles));
+  EXPECT_EQ(1u, responses->size());
+
+  match_params.ignore_search = true;
+  EXPECT_FALSE(
+      MatchAll(body_request_, match_params, &responses, &body_handles));
+  EXPECT_EQ(CACHE_STORAGE_ERROR_QUERY_TOO_LARGE, callback_error_);
+}
+
+TEST_P(CacheStorageCacheTestP, KeysLimit) {
+  EXPECT_TRUE(Put(no_body_request_, no_body_response_));
+  EXPECT_TRUE(Put(body_request_, body_response_));
+
+  size_t max_size = no_body_request_.EstimatedStructSize() +
+                    body_request_.EstimatedStructSize();
+  SetMaxQuerySizeBytes(max_size);
+  EXPECT_TRUE(Keys());
+
+  SetMaxQuerySizeBytes(no_body_request_.EstimatedStructSize());
+  EXPECT_FALSE(Keys());
+  EXPECT_EQ(CACHE_STORAGE_ERROR_QUERY_TOO_LARGE, callback_error_);
+}
+
 // TODO(nhiroki): Add a test for the case where one of PUT operations fails.
 // Currently there is no handy way to fail only one operation in a batch.
 // This could be easily achieved after adding some security checks in the
@@ -798,7 +866,7 @@ TEST_P(CacheStorageCacheTestP, PutReplace) {
   EXPECT_FALSE(callback_response_data_);
 }
 
-TEST_P(CacheStorageCacheTestP, PutReplcaceInBatch) {
+TEST_P(CacheStorageCacheTestP, PutReplaceInBatch) {
   CacheStorageBatchOperation operation1;
   operation1.operation_type = CACHE_STORAGE_CACHE_OPERATION_TYPE_PUT;
   operation1.request = body_request_;
@@ -837,6 +905,11 @@ TEST_P(CacheStorageCacheTestP, MatchBody) {
       ResponseBodiesEqual(expected_blob_data_, *callback_response_data_));
 }
 
+TEST_P(CacheStorageCacheTestP, MatchBodyHead) {
+  EXPECT_TRUE(Put(body_request_, body_response_));
+  EXPECT_FALSE(Match(body_head_request_));
+}
+
 TEST_P(CacheStorageCacheTestP, MatchAll_Empty) {
   std::unique_ptr<CacheStorageCache::Responses> responses;
   std::unique_ptr<CacheStorageCache::BlobDataHandles> body_handles;
@@ -855,7 +928,7 @@ TEST_P(CacheStorageCacheTestP, MatchAll_NoBody) {
   ASSERT_EQ(1u, responses->size());
   EXPECT_TRUE(
       ResponseMetadataEqual(SetCacheName(no_body_response_), responses->at(0)));
-  EXPECT_TRUE(body_handles->empty());
+  EXPECT_FALSE(body_handles->at(0));
 }
 
 TEST_P(CacheStorageCacheTestP, MatchAll_Body) {
@@ -869,7 +942,7 @@ TEST_P(CacheStorageCacheTestP, MatchAll_Body) {
   ASSERT_EQ(1u, body_handles->size());
   EXPECT_TRUE(
       ResponseMetadataEqual(SetCacheName(body_response_), responses->at(0)));
-  EXPECT_TRUE(ResponseBodiesEqual(expected_blob_data_, body_handles->at(0)));
+  EXPECT_TRUE(ResponseBodiesEqual(expected_blob_data_, *body_handles->at(0)));
 }
 
 TEST_P(CacheStorageCacheTestP, MatchAll_TwoResponsesThenOne) {
@@ -879,25 +952,14 @@ TEST_P(CacheStorageCacheTestP, MatchAll_TwoResponsesThenOne) {
   std::unique_ptr<CacheStorageCache::Responses> responses;
   std::unique_ptr<CacheStorageCache::BlobDataHandles> body_handles;
   EXPECT_TRUE(MatchAll(&responses, &body_handles));
-  ASSERT_EQ(2u, responses->size());
-  ASSERT_EQ(1u, body_handles->size());
+  ASSERT_TRUE(body_handles->at(1));
 
-  // Order of returned responses is not guaranteed.
-  std::set<std::string> matched_set;
-  for (const ServiceWorkerResponse& response : *responses) {
-    if (response.url.spec() == "http://example.com/no_body.html") {
-      EXPECT_TRUE(
-          ResponseMetadataEqual(SetCacheName(no_body_response_), response));
-      matched_set.insert(response.url.spec());
-    } else if (response.url.spec() == "http://example.com/body.html") {
-      EXPECT_TRUE(
-          ResponseMetadataEqual(SetCacheName(body_response_), response));
-      EXPECT_TRUE(
-          ResponseBodiesEqual(expected_blob_data_, body_handles->at(0)));
-      matched_set.insert(response.url.spec());
-    }
-  }
-  EXPECT_EQ(2u, matched_set.size());
+  EXPECT_TRUE(
+      ResponseMetadataEqual(SetCacheName(no_body_response_), responses->at(0)));
+  EXPECT_FALSE(body_handles->at(0));
+  EXPECT_TRUE(
+      ResponseMetadataEqual(SetCacheName(body_response_), responses->at(1)));
+  EXPECT_TRUE(ResponseBodiesEqual(expected_blob_data_, *body_handles->at(1)));
 
   responses->clear();
   body_handles->clear();
@@ -908,7 +970,157 @@ TEST_P(CacheStorageCacheTestP, MatchAll_TwoResponsesThenOne) {
   ASSERT_EQ(1u, responses->size());
   EXPECT_TRUE(
       ResponseMetadataEqual(SetCacheName(no_body_response_), responses->at(0)));
-  EXPECT_TRUE(body_handles->empty());
+  ASSERT_EQ(1u, body_handles->size());
+  EXPECT_FALSE(body_handles->at(0));
+}
+
+TEST_P(CacheStorageCacheTestP, Match_IgnoreSearch) {
+  EXPECT_TRUE(Put(body_request_with_query_, body_response_with_query_));
+
+  EXPECT_FALSE(Match(body_request_));
+  CacheStorageCacheQueryParams match_params;
+  match_params.ignore_search = true;
+  EXPECT_TRUE(Match(body_request_, match_params));
+}
+
+TEST_P(CacheStorageCacheTestP, Match_IgnoreMethod) {
+  EXPECT_TRUE(Put(body_request_, body_response_));
+
+  ServiceWorkerFetchRequest post_request = body_request_;
+  post_request.method = "POST";
+  EXPECT_FALSE(Match(post_request));
+
+  CacheStorageCacheQueryParams match_params;
+  match_params.ignore_method = true;
+  EXPECT_TRUE(Match(post_request, match_params));
+}
+
+TEST_P(CacheStorageCacheTestP, Match_IgnoreVary) {
+  body_request_.headers["vary_foo"] = "foo";
+  body_response_.headers["vary"] = "vary_foo";
+  EXPECT_TRUE(Put(body_request_, body_response_));
+  EXPECT_TRUE(Match(body_request_));
+
+  body_request_.headers["vary_foo"] = "bar";
+  EXPECT_FALSE(Match(body_request_));
+
+  CacheStorageCacheQueryParams match_params;
+  match_params.ignore_vary = true;
+  EXPECT_TRUE(Match(body_request_, match_params));
+}
+
+TEST_P(CacheStorageCacheTestP, Keys_IgnoreSearch) {
+  EXPECT_TRUE(Put(body_request_with_query_, body_response_with_query_));
+
+  EXPECT_TRUE(Keys(body_request_));
+  EXPECT_EQ(0u, callback_strings_.size());
+
+  CacheStorageCacheQueryParams match_params;
+  match_params.ignore_search = true;
+  EXPECT_TRUE(Keys(body_request_, match_params));
+  EXPECT_EQ(1u, callback_strings_.size());
+}
+
+TEST_P(CacheStorageCacheTestP, Keys_IgnoreMethod) {
+  EXPECT_TRUE(Put(body_request_, body_response_));
+
+  ServiceWorkerFetchRequest post_request = body_request_;
+  post_request.method = "POST";
+  EXPECT_TRUE(Keys(post_request));
+  EXPECT_EQ(0u, callback_strings_.size());
+
+  CacheStorageCacheQueryParams match_params;
+  match_params.ignore_method = true;
+  EXPECT_TRUE(Keys(post_request, match_params));
+  EXPECT_EQ(1u, callback_strings_.size());
+}
+
+TEST_P(CacheStorageCacheTestP, Keys_IgnoreVary) {
+  body_request_.headers["vary_foo"] = "foo";
+  body_response_.headers["vary"] = "vary_foo";
+  EXPECT_TRUE(Put(body_request_, body_response_));
+  EXPECT_TRUE(Keys(body_request_));
+  EXPECT_EQ(1u, callback_strings_.size());
+
+  body_request_.headers["vary_foo"] = "bar";
+  EXPECT_TRUE(Keys(body_request_));
+  EXPECT_EQ(0u, callback_strings_.size());
+
+  CacheStorageCacheQueryParams match_params;
+  match_params.ignore_vary = true;
+  EXPECT_TRUE(Keys(body_request_, match_params));
+  EXPECT_EQ(1u, callback_strings_.size());
+}
+
+TEST_P(CacheStorageCacheTestP, Delete_IgnoreSearch) {
+  EXPECT_TRUE(Put(body_request_with_query_, body_response_with_query_));
+
+  EXPECT_FALSE(Delete(body_request_));
+  CacheStorageCacheQueryParams match_params;
+  match_params.ignore_search = true;
+  EXPECT_TRUE(Delete(body_request_, match_params));
+}
+
+TEST_P(CacheStorageCacheTestP, Delete_IgnoreMethod) {
+  EXPECT_TRUE(Put(body_request_, body_response_));
+
+  ServiceWorkerFetchRequest post_request = body_request_;
+  post_request.method = "POST";
+  EXPECT_FALSE(Delete(post_request));
+
+  CacheStorageCacheQueryParams match_params;
+  match_params.ignore_method = true;
+  EXPECT_TRUE(Delete(post_request, match_params));
+}
+
+TEST_P(CacheStorageCacheTestP, Delete_IgnoreVary) {
+  body_request_.headers["vary_foo"] = "foo";
+  body_response_.headers["vary"] = "vary_foo";
+  EXPECT_TRUE(Put(body_request_, body_response_));
+
+  body_request_.headers["vary_foo"] = "bar";
+  EXPECT_FALSE(Delete(body_request_));
+
+  CacheStorageCacheQueryParams match_params;
+  match_params.ignore_vary = true;
+  EXPECT_TRUE(Delete(body_request_, match_params));
+}
+
+TEST_P(CacheStorageCacheTestP, MatchAll_IgnoreMethod) {
+  EXPECT_TRUE(Put(body_request_, body_response_));
+
+  ServiceWorkerFetchRequest post_request = body_request_;
+  post_request.method = "POST";
+  std::unique_ptr<CacheStorageCache::Responses> responses;
+  std::unique_ptr<CacheStorageCache::BlobDataHandles> body_handles;
+  CacheStorageCacheQueryParams match_params;
+
+  EXPECT_TRUE(MatchAll(post_request, match_params, &responses, &body_handles));
+  EXPECT_EQ(0u, responses->size());
+
+  match_params.ignore_method = true;
+  EXPECT_TRUE(MatchAll(post_request, match_params, &responses, &body_handles));
+  EXPECT_EQ(1u, responses->size());
+}
+
+TEST_P(CacheStorageCacheTestP, MatchAll_IgnoreVary) {
+  body_request_.headers["vary_foo"] = "foo";
+  body_response_.headers["vary"] = "vary_foo";
+  EXPECT_TRUE(Put(body_request_, body_response_));
+  std::unique_ptr<CacheStorageCache::Responses> responses;
+  std::unique_ptr<CacheStorageCache::BlobDataHandles> body_handles;
+  CacheStorageCacheQueryParams match_params;
+
+  EXPECT_TRUE(MatchAll(body_request_, match_params, &responses, &body_handles));
+  EXPECT_EQ(1u, responses->size());
+  body_request_.headers["vary_foo"] = "bar";
+
+  EXPECT_TRUE(MatchAll(body_request_, match_params, &responses, &body_handles));
+  EXPECT_EQ(0u, responses->size());
+
+  match_params.ignore_vary = true;
+  EXPECT_TRUE(MatchAll(body_request_, match_params, &responses, &body_handles));
+  EXPECT_EQ(1u, responses->size());
 }
 
 TEST_P(CacheStorageCacheTestP, MatchAll_IgnoreSearch) {
@@ -941,6 +1153,28 @@ TEST_P(CacheStorageCacheTestP, MatchAll_IgnoreSearch) {
   EXPECT_EQ(2u, matched_set.size());
 }
 
+TEST_P(CacheStorageCacheTestP, MatchAll_Head) {
+  EXPECT_TRUE(Put(body_request_, body_response_));
+
+  std::unique_ptr<CacheStorageCache::Responses> responses;
+  std::unique_ptr<CacheStorageCache::BlobDataHandles> body_handles;
+  CacheStorageCacheQueryParams match_params;
+  match_params.ignore_search = true;
+  EXPECT_TRUE(
+      MatchAll(body_head_request_, match_params, &responses, &body_handles));
+  EXPECT_TRUE(responses->empty());
+  EXPECT_TRUE(body_handles->empty());
+
+  match_params.ignore_method = true;
+  EXPECT_TRUE(
+      MatchAll(body_head_request_, match_params, &responses, &body_handles));
+  ASSERT_EQ(1u, responses->size());
+  ASSERT_EQ(1u, body_handles->size());
+  EXPECT_TRUE(
+      ResponseMetadataEqual(SetCacheName(body_response_), responses->at(0)));
+  EXPECT_TRUE(ResponseBodiesEqual(expected_blob_data_, *body_handles->at(0)));
+}
+
 TEST_P(CacheStorageCacheTestP, Vary) {
   body_request_.headers["vary_foo"] = "foo";
   body_response_.headers["vary"] = "vary_foo";
@@ -1010,29 +1244,54 @@ TEST_P(CacheStorageCacheTestP, TwoKeys) {
   EXPECT_TRUE(Put(no_body_request_, no_body_response_));
   EXPECT_TRUE(Put(body_request_, body_response_));
   EXPECT_TRUE(Keys());
-  EXPECT_EQ(2u, callback_strings_.size());
-  std::vector<std::string> expected_keys;
-  expected_keys.push_back(no_body_request_.url.spec());
-  expected_keys.push_back(body_request_.url.spec());
-  EXPECT_TRUE(VerifyKeys(expected_keys));
+  std::vector<std::string> expected_keys{no_body_request_.url.spec(),
+                                         body_request_.url.spec()};
+  EXPECT_EQ(expected_keys, callback_strings_);
 }
 
 TEST_P(CacheStorageCacheTestP, TwoKeysThenOne) {
   EXPECT_TRUE(Put(no_body_request_, no_body_response_));
   EXPECT_TRUE(Put(body_request_, body_response_));
   EXPECT_TRUE(Keys());
-  EXPECT_EQ(2u, callback_strings_.size());
-  std::vector<std::string> expected_keys;
-  expected_keys.push_back(no_body_request_.url.spec());
-  expected_keys.push_back(body_request_.url.spec());
-  EXPECT_TRUE(VerifyKeys(expected_keys));
+  std::vector<std::string> expected_keys{no_body_request_.url.spec(),
+                                         body_request_.url.spec()};
+  EXPECT_EQ(expected_keys, callback_strings_);
 
   EXPECT_TRUE(Delete(body_request_));
   EXPECT_TRUE(Keys());
-  EXPECT_EQ(1u, callback_strings_.size());
-  std::vector<std::string> expected_key;
-  expected_key.push_back(no_body_request_.url.spec());
-  EXPECT_TRUE(VerifyKeys(expected_key));
+  std::vector<std::string> expected_keys2{no_body_request_.url.spec()};
+  EXPECT_EQ(expected_keys2, callback_strings_);
+}
+
+TEST_P(CacheStorageCacheTestP, KeysWithIgnoreSearchTrue) {
+  EXPECT_TRUE(Put(no_body_request_, no_body_response_));
+  EXPECT_TRUE(Put(body_request_, body_response_));
+  EXPECT_TRUE(Put(body_request_with_query_, body_response_with_query_));
+
+  CacheStorageCacheQueryParams match_params;
+  match_params.ignore_search = true;
+
+  EXPECT_TRUE(Keys(body_request_with_query_, match_params));
+  std::vector<std::string> expected_keys = {
+      body_request_.url.spec(), body_request_with_query_.url.spec()};
+  EXPECT_EQ(expected_keys, callback_strings_);
+}
+
+TEST_P(CacheStorageCacheTestP, KeysWithIgnoreSearchFalse) {
+  EXPECT_TRUE(Put(no_body_request_, no_body_response_));
+  EXPECT_TRUE(Put(body_request_, body_response_));
+  EXPECT_TRUE(Put(body_request_with_query_, body_response_with_query_));
+
+  // Default value of ignore_search is false.
+  CacheStorageCacheQueryParams match_params;
+  match_params.ignore_search = false;
+  EXPECT_EQ(match_params.ignore_search,
+            CacheStorageCacheQueryParams().ignore_search);
+
+  EXPECT_TRUE(Keys(body_request_with_query_, match_params));
+  std::vector<std::string> expected_keys = {
+      body_request_with_query_.url.spec()};
+  EXPECT_EQ(expected_keys, callback_strings_);
 }
 
 TEST_P(CacheStorageCacheTestP, DeleteNoBody) {
@@ -1063,12 +1322,10 @@ TEST_P(CacheStorageCacheTestP, DeleteWithIgnoreSearchTrue) {
   EXPECT_TRUE(Put(body_request_with_query_, body_response_with_query_));
 
   EXPECT_TRUE(Keys());
-  EXPECT_EQ(3u, callback_strings_.size());
-  std::vector<std::string> expected_keys;
-  expected_keys.push_back(no_body_request_.url.spec());
-  expected_keys.push_back(body_request_.url.spec());
-  expected_keys.push_back(body_request_with_query_.url.spec());
-  EXPECT_TRUE(VerifyKeys(expected_keys));
+  std::vector<std::string> expected_keys{no_body_request_.url.spec(),
+                                         body_request_.url.spec(),
+                                         body_request_with_query_.url.spec()};
+  EXPECT_EQ(expected_keys, callback_strings_);
 
   // The following delete operation will remove both of body_request_ and
   // body_request_with_query_ from cache storage.
@@ -1077,10 +1334,9 @@ TEST_P(CacheStorageCacheTestP, DeleteWithIgnoreSearchTrue) {
   EXPECT_TRUE(Delete(body_request_with_query_, match_params));
 
   EXPECT_TRUE(Keys());
-  EXPECT_EQ(1u, callback_strings_.size());
   expected_keys.clear();
-  expected_keys.push_back(no_body_request_.url.spec());
-  EXPECT_TRUE(VerifyKeys(expected_keys));
+  std::vector<std::string> expected_keys2{no_body_request_.url.spec()};
+  EXPECT_EQ(expected_keys2, callback_strings_);
 }
 
 TEST_P(CacheStorageCacheTestP, DeleteWithIgnoreSearchFalse) {
@@ -1089,12 +1345,10 @@ TEST_P(CacheStorageCacheTestP, DeleteWithIgnoreSearchFalse) {
   EXPECT_TRUE(Put(body_request_with_query_, body_response_with_query_));
 
   EXPECT_TRUE(Keys());
-  EXPECT_EQ(3u, callback_strings_.size());
-  std::vector<std::string> expected_keys;
-  expected_keys.push_back(no_body_request_.url.spec());
-  expected_keys.push_back(body_request_.url.spec());
-  expected_keys.push_back(body_request_with_query_.url.spec());
-  EXPECT_TRUE(VerifyKeys(expected_keys));
+  std::vector<std::string> expected_keys{no_body_request_.url.spec(),
+                                         body_request_.url.spec(),
+                                         body_request_with_query_.url.spec()};
+  EXPECT_EQ(expected_keys, callback_strings_);
 
   // Default value of ignore_search is false.
   CacheStorageCacheQueryParams match_params;
@@ -1105,11 +1359,9 @@ TEST_P(CacheStorageCacheTestP, DeleteWithIgnoreSearchFalse) {
   EXPECT_TRUE(Delete(body_request_with_query_, match_params));
 
   EXPECT_TRUE(Keys());
-  EXPECT_EQ(2u, callback_strings_.size());
-  expected_keys.clear();
-  expected_keys.push_back(no_body_request_.url.spec());
-  expected_keys.push_back(body_request_.url.spec());
-  EXPECT_TRUE(VerifyKeys(expected_keys));
+  std::vector<std::string> expected_keys2{no_body_request_.url.spec(),
+                                          body_request_.url.spec()};
+  EXPECT_EQ(expected_keys2, callback_strings_);
 }
 
 TEST_P(CacheStorageCacheTestP, QuickStressNoBody) {
@@ -1321,13 +1573,6 @@ TEST_P(CacheStorageCacheTestP, GetSizeThenClose) {
   VerifyAllOpsFail();
 }
 
-TEST_P(CacheStorageCacheTestP, OpsFailOnClosedBackendNeverCreated) {
-  cache_->set_delay_backend_creation(
-      true);  // Will hang the test if a backend is created.
-  EXPECT_TRUE(Close());
-  VerifyAllOpsFail();
-}
-
 TEST_P(CacheStorageCacheTestP, OpsFailOnClosedBackend) {
   // Create the backend and put something in it.
   EXPECT_TRUE(Put(body_request_, body_response_));
@@ -1340,7 +1585,7 @@ TEST_P(CacheStorageCacheTestP, VerifySerialScheduling) {
   // second should wait for the first.
   EXPECT_TRUE(Keys());  // Opens the backend.
   DelayableBackend* delayable_backend = cache_->UseDelayableBackend();
-  delayable_backend->set_delay_open(true);
+  delayable_backend->set_delay_doom(true);
 
   int sequence_out = -1;
 
@@ -1355,7 +1600,7 @@ TEST_P(CacheStorageCacheTestP, VerifySerialScheduling) {
       base::Bind(&CacheStorageCacheTest::SequenceCallback,
                  base::Unretained(this), 1, &sequence_out, close_loop1.get()));
 
-  // Blocks on opening the cache entry.
+  // Blocks on creating the cache entry.
   base::RunLoop().RunUntilIdle();
 
   CacheStorageBatchOperation operation2;
@@ -1363,7 +1608,7 @@ TEST_P(CacheStorageCacheTestP, VerifySerialScheduling) {
   operation2.request = body_request_;
   operation2.response = body_response_;
 
-  delayable_backend->set_delay_open(false);
+  delayable_backend->set_delay_doom(false);
   std::unique_ptr<base::RunLoop> close_loop2(new base::RunLoop());
   cache_->BatchOperation(
       std::vector<CacheStorageBatchOperation>(1, operation2),
@@ -1374,7 +1619,7 @@ TEST_P(CacheStorageCacheTestP, VerifySerialScheduling) {
   base::RunLoop().RunUntilIdle();
   EXPECT_FALSE(callback_response_);
 
-  delayable_backend->OpenEntryContinue();
+  delayable_backend->DoomEntryContinue();
   close_loop1->Run();
   EXPECT_EQ(1, sequence_out);
   close_loop2->Run();
