@@ -141,6 +141,8 @@ TEST_F(ChildProcessSecurityPolicyTest, IsPseudoSchemeTest) {
   EXPECT_TRUE(p->IsPseudoScheme(url::kAboutScheme));
   EXPECT_TRUE(p->IsPseudoScheme(url::kJavaScriptScheme));
   EXPECT_TRUE(p->IsPseudoScheme(kViewSourceScheme));
+  EXPECT_TRUE(p->IsPseudoScheme(url::kHttpSuboriginScheme));
+  EXPECT_TRUE(p->IsPseudoScheme(url::kHttpsSuboriginScheme));
 
   EXPECT_FALSE(p->IsPseudoScheme("registered-pseudo-scheme"));
   p->RegisterPseudoScheme("registered-pseudo-scheme");
@@ -331,6 +333,36 @@ TEST_F(ChildProcessSecurityPolicyTest, JavaScriptTest) {
   p->Remove(kRendererID);
 }
 
+TEST_F(ChildProcessSecurityPolicyTest, SuboriginTest) {
+  ChildProcessSecurityPolicyImpl* p =
+      ChildProcessSecurityPolicyImpl::GetInstance();
+
+  p->Add(kRendererID);
+
+  // Suborigin URLs are not requestable or committable.
+  EXPECT_FALSE(
+      p->CanRequestURL(kRendererID, GURL("http-so://foobar.example.com")));
+  EXPECT_FALSE(
+      p->CanRequestURL(kRendererID, GURL("https-so://foobar.example.com")));
+  EXPECT_FALSE(
+      p->CanCommitURL(kRendererID, GURL("http-so://foobar.example.com")));
+  EXPECT_FALSE(
+      p->CanCommitURL(kRendererID, GURL("https-so://foobar.example.com")));
+
+  // It's not possible to grant suborigins requestable status.
+  p->GrantRequestURL(kRendererID, GURL("https-so://foobar.example.com"));
+  EXPECT_FALSE(
+      p->CanCommitURL(kRendererID, GURL("https-so://foobar.example.com")));
+
+  // Suborigin URLs are valid origin headers.
+  EXPECT_TRUE(p->CanSetAsOriginHeader(kRendererID,
+                                      GURL("http-so://foobar.example.com")));
+  EXPECT_TRUE(p->CanSetAsOriginHeader(kRendererID,
+                                      GURL("https-so://foobar.example.com")));
+
+  p->Remove(kRendererID);
+}
+
 TEST_F(ChildProcessSecurityPolicyTest, RegisterWebSafeSchemeTest) {
   ChildProcessSecurityPolicyImpl* p =
       ChildProcessSecurityPolicyImpl::GetInstance();
