@@ -19,7 +19,7 @@ InspectorHandler::InspectorHandler()
 InspectorHandler::~InspectorHandler() {
 }
 
-void InspectorHandler::SetClient(scoped_ptr<Client> client) {
+void InspectorHandler::SetClient(std::unique_ptr<Client> client) {
   client_.swap(client);
 }
 
@@ -38,9 +38,12 @@ void InspectorHandler::TargetDetached(const std::string& reason) {
 Response InspectorHandler::Enable() {
   if (host_ && !host_->IsRenderFrameLive())
     client_->TargetCrashed(TargetCrashedParams::Create());
-  return Response::FallThrough();
+  return Response::OK();
 }
 
+Response InspectorHandler::Disable() {
+  return Response::OK();
+}
 
 }  // namespace inspector
 }  // namespace devtools
