@@ -14,21 +14,23 @@
 #include "base/strings/utf_string_conversions.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "base/threading/worker_pool.h"
+#include "content/browser/devtools/page_navigation_throttle.h"
 #include "content/browser/devtools/protocol/color_picker.h"
 #include "content/browser/renderer_host/render_widget_host_impl.h"
 #include "content/browser/renderer_host/render_widget_host_view_base.h"
 #include "content/browser/web_contents/web_contents_impl.h"
+#include "content/browser/web_contents/web_contents_view.h"
 #include "content/common/view_messages.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/javascript_dialog_manager.h"
 #include "content/public/browser/navigation_controller.h"
 #include "content/public/browser/navigation_entry.h"
+#include "content/public/browser/navigation_handle.h"
 #include "content/public/browser/notification_service.h"
 #include "content/public/browser/notification_types.h"
 #include "content/public/browser/storage_partition.h"
 #include "content/public/browser/web_contents_delegate.h"
 #include "content/public/common/referrer.h"
-#include "third_party/WebKit/public/platform/WebScreenInfo.h"
 #include "third_party/skia/include/core/SkBitmap.h"
 #include "ui/base/page_transition_types.h"
 #include "ui/gfx/codec/jpeg_codec.h"
@@ -102,11 +104,12 @@ PageHandler::PageHandler()
       session_id_(0),
       frame_counter_(0),
       frames_in_flight_(0),
-      color_picker_(new ColorPicker(base::Bind(
-          &PageHandler::OnColorPicked, base::Unretained(this)))),
+      color_picker_(new ColorPicker(
+          base::Bind(&PageHandler::OnColorPicked, base::Unretained(this)))),
+      navigation_throttle_enabled_(false),
+      next_navigation_id_(0),
       host_(nullptr),
-      weak_factory_(this) {
-}
+      weak_factory_(this) {}
 
 PageHandler::~PageHandler() {
 }
@@ -201,6 +204,7 @@ Response PageHandler::Enable() {
 Response PageHandler::Disable() {
   enabled_ = false;
   screencast_enabled_ = false;
+  SetControlNavigations(false);
   color_picker_->SetEnabled(false);
   return Response::FallThrough();
 }
@@ -314,8 +318,8 @@ Response PageHandler::StartScreencast(const std::string* format,
     if (has_compositor_frame_metadata_) {
       InnerSwapCompositorFrame();
     } else {
-      widget_host->Send(
-          new ViewMsg_ForceRedraw(widget_host->GetRoutingID(), 0));
+      widget_host->Send(new ViewMsg_ForceRedraw(widget_host->GetRoutingID(),
+                                                ui::LatencyInfo()));
     }
   }
   return Response::FallThrough();
@@ -373,6 +377,121 @@ Response PageHandler::RequestAppBanner() {
   return Response::OK();
 }
 
+Response PageHandler::SetControlNavigations(bool enabled) {
+  navigation_throttle_enabled_ = enabled;
+  // We don't own the page PageNavigationThrottles so we can't delete them, but
+  // we can turn them into NOPs.
+  for (auto& pair : navigation_throttles_) {
+    pair.second->AlwaysProceed();
+  }
+  navigation_throttles_.clear();
+  return Response::OK();
+}
+
+Response PageHandler::ProcessNavigation(const std::string& response,
+                                        int navigation_id) {
+  auto it = navigation_throttles_.find(navigation_id);
+  if (it == navigation_throttles_.end())
+    return Response::InvalidParams("Unknown navigation id");
+
+  if (response == kNavigationResponseProceed) {
+    it->second->Resume();
+    return Response::OK();
+  } else if (response == kNavigationResponseCancel) {
+    it->second->CancelDeferredNavigation(content::NavigationThrottle::CANCEL);
+    return Response::OK();
+  } else if (response == kNavigationResponseCancelAndIgnore) {
+    it->second->CancelDeferredNavigation(
+        content::NavigationThrottle::CANCEL_AND_IGNORE);
+    return Response::OK();
+  }
+
+  return Response::InvalidParams("Unrecognized response");
+}
+
+Response PageHandler::AddScriptToEvaluateOnLoad(const std::string& source,
+                                                std::string* identifier) {
+  return Response::FallThrough();
+}
+
+Response PageHandler::RemoveScriptToEvaluateOnLoad(
+    const std::string& identifier) {
+  return Response::FallThrough();
+}
+
+Response PageHandler::SetAutoAttachToCreatedPages(bool auto_attach) {
+  return Response::FallThrough();
+}
+
+Response PageHandler::GetResourceTree(scoped_refptr<FrameResourceTree>* tree) {
+  return Response::FallThrough();
+}
+
+Response PageHandler::GetResourceContent(DevToolsCommandId command_id,
+                                         const std::string& frame_id,
+                                         const std::string& url) {
+  return Response::FallThrough();
+}
+
+Response PageHandler::SearchInResource(DevToolsCommandId command_id,
+                                       const std::string& frame_id,
+                                       const std::string& url,
+                                       const std::string& query,
+                                       bool* case_sensitive,
+                                       bool* is_regex) {
+  return Response::FallThrough();
+}
+
+Response PageHandler::SetDocumentContent(const std::string& frame_id,
+                                         const std::string& html) {
+  return Response::FallThrough();
+}
+
+Response PageHandler::ConfigureOverlay(const bool* is_suspended,
+                                       const std::string* message) {
+  return Response::FallThrough();
+}
+
+Response PageHandler::GetAppManifest(
+    std::string* url,
+    std::vector<scoped_refptr<AppManifestError>>* errors,
+    std::string* data) {
+  return Response::FallThrough();
+}
+
+Response PageHandler::GetLayoutMetrics(
+    scoped_refptr<LayoutViewport>* layout_viewport,
+    scoped_refptr<VisualViewport>* visual_viewport) {
+  return Response::FallThrough();
+}
+
+std::unique_ptr<PageNavigationThrottle>
+PageHandler::CreateThrottleForNavigation(NavigationHandle* navigation_handle) {
+  if (!navigation_throttle_enabled_)
+    return nullptr;
+
+  std::unique_ptr<PageNavigationThrottle> throttle(new PageNavigationThrottle(
+      weak_factory_.GetWeakPtr(), next_navigation_id_, navigation_handle));
+  navigation_throttles_[next_navigation_id_++] = throttle.get();
+  return throttle;
+}
+
+void PageHandler::OnPageNavigationThrottleDisposed(int navigation_id) {
+  DCHECK(navigation_throttles_.find(navigation_id) !=
+         navigation_throttles_.end());
+  navigation_throttles_.erase(navigation_id);
+}
+
+void PageHandler::NavigationRequested(const PageNavigationThrottle* throttle) {
+  NavigationHandle* navigation_handle = throttle->navigation_handle();
+  client_->NavigationRequested(
+      NavigationRequestedParams::Create()
+          ->set_is_in_main_frame(navigation_handle->IsInMainFrame())
+          ->set_is_redirect(navigation_handle->WasServerRedirect())
+          ->set_navigation_id(throttle->navigation_id())
+          ->set_url(navigation_handle->GetURL().spec()));
+}
+
 WebContentsImpl* PageHandler::GetWebContents() {
   return host_ ?
       static_cast<WebContentsImpl*>(WebContents::FromRenderFrameHost(host_)) :
@@ -407,9 +526,9 @@ void PageHandler::InnerSwapCompositorFrame() {
       gfx::ScaleSize(gfx::SizeF(view->GetPhysicalBackingSize()),
                      1 / metadata.device_scale_factor);
 
-  blink::WebScreenInfo screen_info;
-  view->GetScreenInfo(&screen_info);
-  double device_scale_factor = screen_info.deviceScaleFactor;
+  content::ScreenInfo screen_info;
+  GetWebContents()->GetView()->GetScreenInfo(&screen_info);
+  double device_scale_factor = screen_info.device_scale_factor;
   double scale = 1;
 
   if (screencast_max_width_ > 0) {
@@ -484,7 +603,8 @@ void PageHandler::ScreencastFrameEncoded(cc::CompositorFrameMetadata metadata,
   scoped_refptr<ScreencastFrameMetadata> param_metadata =
       ScreencastFrameMetadata::Create()
           ->set_page_scale_factor(metadata.page_scale_factor)
-          ->set_offset_top(metadata.location_bar_content_translation.y())
+          ->set_offset_top(metadata.top_controls_height *
+                           metadata.top_controls_shown_ratio)
           ->set_device_width(screen_size_dip.width())
           ->set_device_height(screen_size_dip.height())
           ->set_scroll_offset_x(metadata.root_scroll_offset.x())
