@@ -11,6 +11,7 @@
 #include "base/bind_helpers.h"
 #include "base/files/file_path.h"
 #include "base/location.h"
+#include "base/memory/memory_coordinator_client_registry.h"
 #include "base/memory/ptr_util.h"
 #include "base/memory/weak_ptr.h"
 #include "base/single_thread_task_runner.h"
@@ -23,14 +24,16 @@
 #include "content/browser/dom_storage/dom_storage_task_runner.h"
 #include "content/browser/dom_storage/session_storage_namespace_impl.h"
 #include "content/browser/leveldb_wrapper_impl.h"
+#include "content/browser/memory/memory_coordinator.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/local_storage_usage_info.h"
 #include "content/public/browser/session_storage_usage_info.h"
+#include "content/public/common/content_features.h"
 #include "mojo/common/common_type_converters.h"
-#include "services/shell/public/cpp/connection.h"
-#include "services/shell/public/cpp/connector.h"
-#include "services/user/public/cpp/constants.h"
-#include "services/user/public/interfaces/user_service.mojom.h"
+#include "services/file/public/interfaces/constants.mojom.h"
+#include "services/file/public/interfaces/file_system.mojom.h"
+#include "services/service_manager/public/cpp/connection.h"
+#include "services/service_manager/public/cpp/connector.h"
 
 namespace content {
 namespace {
@@ -80,7 +83,8 @@ void GetSessionStorageUsageHelper(
 // for now).
 class DOMStorageContextWrapper::MojoState {
  public:
-  MojoState(shell::Connector* connector, const base::FilePath& subdirectory)
+  MojoState(service_manager::Connector* connector,
+            const base::FilePath& subdirectory)
       : connector_(connector),
         // TODO(michaeln): Enable writing to disk when db is versioned,
         // for now using an empty subdirectory to use an in-memory db.
@@ -99,8 +103,8 @@ class DOMStorageContextWrapper::MojoState {
   }
 
   void OnUserServiceConnectionComplete() {
-    CHECK_EQ(shell::mojom::ConnectResult::SUCCEEDED,
-             user_service_connection_->GetResult());
+    CHECK_EQ(service_manager::mojom::ConnectResult::SUCCEEDED,
+             file_service_connection_->GetResult());
   }
 
   void OnUserServiceConnectionError() {
@@ -120,7 +124,7 @@ class DOMStorageContextWrapper::MojoState {
   // Maps between an origin and its prefixed LevelDB view.
   std::map<url::Origin, std::unique_ptr<LevelDBWrapperImpl>> level_db_wrappers_;
 
-  shell::Connector* const connector_;
+  service_manager::Connector* const connector_;
   const base::FilePath subdirectory_;
 
   enum ConnectionState {
@@ -129,9 +133,9 @@ class DOMStorageContextWrapper::MojoState {
     CONNECTION_FINISHED
   } connection_state_;
 
-  std::unique_ptr<shell::Connection> user_service_connection_;
+  std::unique_ptr<service_manager::Connection> file_service_connection_;
 
-  user_service::mojom::UserServicePtr user_service_;
+  file::mojom::FileSystemPtr file_system_;
   filesystem::mojom::DirectoryPtr directory_;
 
   leveldb::mojom::LevelDBServicePtr leveldb_service_;
@@ -149,28 +153,27 @@ void DOMStorageContextWrapper::MojoState::OpenLocalStorage(
   // If we don't have a filesystem_connection_, we'll need to establish one.
   if (connection_state_ == NO_CONNECTION) {
     CHECK(connector_);
-    user_service_connection_ =
-        connector_->Connect(user_service::kUserServiceName);
+    file_service_connection_ =
+        connector_->Connect(file::mojom::kServiceName);
     connection_state_ = CONNECTION_IN_PROGRESS;
-    user_service_connection_->AddConnectionCompletedClosure(
+    file_service_connection_->AddConnectionCompletedClosure(
         base::Bind(&MojoState::OnUserServiceConnectionComplete,
                    weak_ptr_factory_.GetWeakPtr()));
-    user_service_connection_->SetConnectionLostClosure(
+    file_service_connection_->SetConnectionLostClosure(
         base::Bind(&MojoState::OnUserServiceConnectionError,
                    weak_ptr_factory_.GetWeakPtr()));
 
     if (!subdirectory_.empty()) {
       // We were given a subdirectory to write to. Get it and use a disk backed
       // database.
-      user_service_connection_->GetInterface(&user_service_);
-      user_service_->GetSubDirectory(
-          mojo::String::From(subdirectory_.AsUTF8Unsafe()),
-          GetProxy(&directory_),
-          base::Bind(&MojoState::OnDirectoryOpened,
-                     weak_ptr_factory_.GetWeakPtr()));
+      file_service_connection_->GetInterface(&file_system_);
+      file_system_->GetSubDirectory(subdirectory_.AsUTF8Unsafe(),
+                                    GetProxy(&directory_),
+                                    base::Bind(&MojoState::OnDirectoryOpened,
+                                               weak_ptr_factory_.GetWeakPtr()));
     } else {
       // We were not given a subdirectory. Use a memory backed database.
-      user_service_connection_->GetInterface(&leveldb_service_);
+      file_service_connection_->GetInterface(&leveldb_service_);
       leveldb_service_->OpenInMemory(
           GetProxy(&database_),
           base::Bind(&MojoState::OnDatabaseOpened,
@@ -200,7 +203,7 @@ void DOMStorageContextWrapper::MojoState::OnDirectoryOpened(
 
   // Now that we have a directory, connect to the LevelDB service and get our
   // database.
-  user_service_connection_->GetInterface(&leveldb_service_);
+  file_service_connection_->GetInterface(&leveldb_service_);
 
   leveldb_service_->Open(
       std::move(directory_), "leveldb", GetProxy(&database_),
@@ -216,11 +219,10 @@ void DOMStorageContextWrapper::MojoState::OnDatabaseOpened(
     leveldb_service_.reset();
   }
 
-  // We no longer need the user service; we've either transferred
-  // |directory_| to the leveldb service, or we got a file error and no more is
-  // possible.
+  // We no longer need the file service; we've either transferred |directory_|
+  // to the leveldb service, or we got a file error and no more is possible.
   directory_.reset();
-  user_service_.reset();
+  file_system_.reset();
 
   // |leveldb_| should be known to either be valid or invalid by now. Run our
   // delayed bindings.
@@ -245,13 +247,13 @@ void DOMStorageContextWrapper::MojoState::BindLocalStorage(
 
   auto found = level_db_wrappers_.find(origin);
   if (found == level_db_wrappers_.end()) {
-    level_db_wrappers_[origin] = base::WrapUnique(new LevelDBWrapperImpl(
+    level_db_wrappers_[origin] = base::MakeUnique<LevelDBWrapperImpl>(
         database_.get(), origin.Serialize(),
         kPerStorageAreaQuota + kPerStorageAreaOverQuotaAllowance,
-        base::TimeDelta::FromSeconds(kCommitDefaultDelaySecs),
-        kMaxBytesPerHour, kMaxCommitsPerHour,
+        base::TimeDelta::FromSeconds(kCommitDefaultDelaySecs), kMaxBytesPerHour,
+        kMaxCommitsPerHour,
         base::Bind(&MojoState::OnLevelDDWrapperHasNoBindings,
-                   base::Unretained(this), origin)));
+                   base::Unretained(this), origin));
     found = level_db_wrappers_.find(origin);
   }
 
@@ -260,7 +262,7 @@ void DOMStorageContextWrapper::MojoState::BindLocalStorage(
 }
 
 DOMStorageContextWrapper::DOMStorageContextWrapper(
-    shell::Connector* connector,
+    service_manager::Connector* connector,
     const base::FilePath& profile_path,
     const base::FilePath& local_partition_path,
     storage::SpecialStoragePolicy* special_storage_policy) {
@@ -283,11 +285,14 @@ DOMStorageContextWrapper::DOMStorageContextWrapper(
           worker_pool,
           worker_pool->GetNamedSequenceToken("dom_storage_primary"),
           worker_pool->GetNamedSequenceToken("dom_storage_commit"),
-          BrowserThread::GetMessageLoopProxyForThread(BrowserThread::IO)
-              .get()));
+          BrowserThread::GetTaskRunnerForThread(BrowserThread::IO).get()));
 
-  memory_pressure_listener_.reset(new base::MemoryPressureListener(
-      base::Bind(&DOMStorageContextWrapper::OnMemoryPressure, this)));
+  if (base::FeatureList::IsEnabled(features::kMemoryCoordinator)) {
+    base::MemoryCoordinatorClientRegistry::GetInstance()->Register(this);
+  } else {
+    memory_pressure_listener_.reset(new base::MemoryPressureListener(
+        base::Bind(&DOMStorageContextWrapper::OnMemoryPressure, this)));
+  }
 }
 
 DOMStorageContextWrapper::~DOMStorageContextWrapper() {}
@@ -312,6 +317,15 @@ void DOMStorageContextWrapper::GetSessionStorageUsage(
                  base::RetainedRef(context_), callback));
 }
 
+void DOMStorageContextWrapper::DeleteLocalStorageForPhysicalOrigin(
+    const GURL& origin) {
+  DCHECK(context_.get());
+  context_->task_runner()->PostShutdownBlockingTask(
+      FROM_HERE, DOMStorageTaskRunner::PRIMARY_SEQUENCE,
+      base::Bind(&DOMStorageContextImpl::DeleteLocalStorageForPhysicalOrigin,
+                 context_, origin));
+}
+
 void DOMStorageContextWrapper::DeleteLocalStorage(const GURL& origin) {
   DCHECK(context_.get());
   context_->task_runner()->PostShutdownBlockingTask(
@@ -367,6 +381,10 @@ void DOMStorageContextWrapper::Shutdown() {
       FROM_HERE,
       DOMStorageTaskRunner::PRIMARY_SEQUENCE,
       base::Bind(&DOMStorageContextImpl::Shutdown, context_));
+  if (base::FeatureList::IsEnabled(features::kMemoryCoordinator)) {
+    base::MemoryCoordinatorClientRegistry::GetInstance()->Unregister(this);
+  }
+
 }
 
 void DOMStorageContextWrapper::Flush() {
@@ -392,6 +410,33 @@ void DOMStorageContextWrapper::OnMemoryPressure(
       base::MemoryPressureListener::MEMORY_PRESSURE_LEVEL_CRITICAL) {
     purge_option = DOMStorageContextImpl::PURGE_AGGRESSIVE;
   }
+  PurgeMemory(purge_option);
+}
+
+void DOMStorageContextWrapper::OnMemoryStateChange(base::MemoryState state) {
+  // TODO(hajimehoshi): As OnMemoryStateChange changes the state, we should
+  // adjust the limitation to the amount of cache, DomStroageContextImpl doesn't
+  // have such limitation so far though.
+  switch (state) {
+    case base::MemoryState::NORMAL:
+      // Don't have to purge memory here.
+      break;
+    case base::MemoryState::THROTTLED:
+      // TOOD(hajimehoshi): We don't have throttling 'level' so far. When we
+      // have such value, let's change the argument accroding to the value.
+      PurgeMemory(DOMStorageContextImpl::PURGE_AGGRESSIVE);
+      break;
+    case base::MemoryState::SUSPENDED:
+      // Note that SUSPENDED never occurs in the main browser process so far.
+      // Fall through.
+    case base::MemoryState::UNKNOWN:
+      NOTREACHED();
+      break;
+  }
+}
+
+void DOMStorageContextWrapper::PurgeMemory(DOMStorageContextImpl::PurgeOption
+    purge_option) {
   context_->task_runner()->PostTask(
       FROM_HERE,
       base::Bind(&DOMStorageContextImpl::PurgeMemory, context_, purge_option));
