@@ -31,6 +31,11 @@
 
 namespace content {
 
+class ScopedAllowWaitForDebugURL {
+ private:
+  base::ThreadRestrictions::ScopedAllowWait wait;
+};
+
 namespace {
 
 // Define the Asan debug URLs.
@@ -51,7 +56,7 @@ const char kKaskoSendReport[] = "/send-report";
 
 void HandlePpapiFlashDebugURL(const GURL& url) {
 #if defined(ENABLE_PLUGINS)
-  bool crash = url == GURL(kChromeUIPpapiFlashCrashURL);
+  bool crash = url == kChromeUIPpapiFlashCrashURL;
 
   std::vector<PpapiPluginProcessHost*> hosts;
   PpapiPluginProcessHost::FindByName(
@@ -70,7 +75,7 @@ bool IsKaskoDebugURL(const GURL& url) {
 #if BUILDFLAG(ENABLE_KASKO)
   return (url.is_valid() && url.SchemeIs(kChromeUIScheme) &&
           url.DomainIs(kKaskoCrashDomain) &&
-          url.path() == kKaskoSendReport);
+          url.path_piece() == kKaskoSendReport);
 #else
   return false;
 #endif
@@ -122,14 +127,17 @@ bool IsAsanDebugURL(const GURL& url) {
     return false;
   }
 
-  if (url.path() == kAsanHeapOverflow || url.path() == kAsanHeapUnderflow ||
-      url.path() == kAsanUseAfterFree) {
+  if (url.path_piece() == kAsanHeapOverflow ||
+      url.path_piece() == kAsanHeapUnderflow ||
+      url.path_piece() == kAsanUseAfterFree) {
     return true;
   }
 
 #if defined(SYZYASAN)
-  if (url.path() == kAsanCorruptHeapBlock || url.path() == kAsanCorruptHeap)
+  if (url.path_piece() == kAsanCorruptHeapBlock ||
+      url.path_piece() == kAsanCorruptHeap) {
     return true;
+  }
 #endif
 
   return false;
@@ -140,21 +148,21 @@ bool HandleAsanDebugURL(const GURL& url) {
   if (!base::debug::IsBinaryInstrumented())
     return false;
 
-  if (url.path() == kAsanCorruptHeapBlock) {
+  if (url.path_piece() == kAsanCorruptHeapBlock) {
     base::debug::AsanCorruptHeapBlock();
     return true;
-  } else if (url.path() == kAsanCorruptHeap) {
+  } else if (url.path_piece() == kAsanCorruptHeap) {
     base::debug::AsanCorruptHeap();
     return true;
   }
 #endif
 
 #if defined(ADDRESS_SANITIZER) || defined(SYZYASAN)
-  if (url.path() == kAsanHeapOverflow) {
+  if (url.path_piece() == kAsanHeapOverflow) {
     base::debug::AsanHeapOverflow();
-  } else if (url.path() == kAsanHeapUnderflow) {
+  } else if (url.path_piece() == kAsanHeapUnderflow) {
     base::debug::AsanHeapUnderflow();
-  } else if (url.path() == kAsanUseAfterFree) {
+  } else if (url.path_piece() == kAsanUseAfterFree) {
     base::debug::AsanHeapUseAfterFree();
   } else {
     return false;
@@ -164,14 +172,15 @@ bool HandleAsanDebugURL(const GURL& url) {
   return true;
 }
 
+void HangCurrentThread() {
+  ScopedAllowWaitForDebugURL allow_wait;
+  base::WaitableEvent(base::WaitableEvent::ResetPolicy::AUTOMATIC,
+                      base::WaitableEvent::InitialState::NOT_SIGNALED)
+      .Wait();
+}
 
 }  // namespace
 
-class ScopedAllowWaitForDebugURL {
- private:
-  base::ThreadRestrictions::ScopedAllowWait wait;
-};
-
 bool HandleDebugURL(const GURL& url, ui::PageTransition transition) {
   // Ensure that the user explicitly navigated to this URL, unless
   // kEnableGpuBenchmarking is enabled by Telemetry.
@@ -192,41 +201,48 @@ bool HandleDebugURL(const GURL& url, ui::PageTransition transition) {
     return true;
   }
 
-  if (url == GURL(kChromeUIBrowserCrashURL)) {
+  if (url == kChromeUIBrowserCrashURL) {
     // Induce an intentional crash in the browser process.
     CHECK(false);
     return true;
   }
 
-  if (url == GURL(kChromeUIBrowserUIHang)) {
-    ScopedAllowWaitForDebugURL allow_wait;
-    base::WaitableEvent(false, false).Wait();
+  if (url == kChromeUIBrowserUIHang) {
+    HangCurrentThread();
     return true;
   }
 
-  if (url == GURL(kChromeUIGpuCleanURL)) {
+  if (url == kChromeUIDelayedBrowserUIHang) {
+    // Webdriver-safe url to hang the ui thread. Webdriver waits for the onload
+    // event in javascript which needs a little more time to fire.
+    BrowserThread::PostDelayedTask(BrowserThread::UI, FROM_HERE,
+                                   base::Bind(&HangCurrentThread),
+                                   base::TimeDelta::FromSeconds(2));
+    return true;
+  }
+
+  if (url == kChromeUIGpuCleanURL) {
     GpuProcessHostUIShim* shim = GpuProcessHostUIShim::GetOneInstance();
     if (shim)
       shim->SimulateRemoveAllContext();
     return true;
   }
 
-  if (url == GURL(kChromeUIGpuCrashURL)) {
+  if (url == kChromeUIGpuCrashURL) {
     GpuProcessHostUIShim* shim = GpuProcessHostUIShim::GetOneInstance();
     if (shim)
       shim->SimulateCrash();
     return true;
   }
 
-  if (url == GURL(kChromeUIGpuHangURL)) {
+  if (url == kChromeUIGpuHangURL) {
     GpuProcessHostUIShim* shim = GpuProcessHostUIShim::GetOneInstance();
     if (shim)
       shim->SimulateHang();
     return true;
   }
 
-  if (url == GURL(kChromeUIPpapiFlashCrashURL) ||
-      url == GURL(kChromeUIPpapiFlashHangURL)) {
+  if (url == kChromeUIPpapiFlashCrashURL || url == kChromeUIPpapiFlashHangURL) {
     BrowserThread::PostTask(BrowserThread::IO, FROM_HERE,
                             base::Bind(&HandlePpapiFlashDebugURL, url));
     return true;
@@ -242,12 +258,13 @@ bool IsRendererDebugURL(const GURL& url) {
   if (url.SchemeIs(url::kJavaScriptScheme))
     return true;
 
-  return url == GURL(kChromeUIBadCastCrashURL) ||
-         url == GURL(kChromeUICrashURL) ||
-         url == GURL(kChromeUIDumpURL) ||
-         url == GURL(kChromeUIKillURL) ||
-         url == GURL(kChromeUIHangURL) ||
-         url == GURL(kChromeUIShorthangURL);
+  return url == kChromeUIBadCastCrashURL ||
+         url == kChromeUICrashURL ||
+         url == kChromeUIDumpURL ||
+         url == kChromeUIKillURL ||
+         url == kChromeUIHangURL ||
+         url == kChromeUIShorthangURL ||
+         url == kChromeUIMemoryExhaustURL;
 }
 
 }  // namespace content
