@@ -14,14 +14,20 @@ NavigationRequestInfo::NavigationRequestInfo(
     const url::Origin& request_initiator,
     bool is_main_frame,
     bool parent_is_main_frame,
-    int frame_tree_node_id)
+    bool are_ancestors_secure,
+    int frame_tree_node_id,
+    bool is_for_guests_only,
+    bool report_raw_headers)
     : common_params(common_params),
       begin_params(begin_params),
       first_party_for_cookies(first_party_for_cookies),
       request_initiator(request_initiator),
       is_main_frame(is_main_frame),
       parent_is_main_frame(parent_is_main_frame),
-      frame_tree_node_id(frame_tree_node_id) {}
+      are_ancestors_secure(are_ancestors_secure),
+      frame_tree_node_id(frame_tree_node_id),
+      is_for_guests_only(is_for_guests_only),
+      report_raw_headers(report_raw_headers) {}
 
 NavigationRequestInfo::~NavigationRequestInfo() {}
 
