@@ -41,7 +41,7 @@ class RegistryTestMockFactory : public MockIndexedDBFactory {
   }
 
   bool CheckNoOriginsInUse() const {
-    return !duplicate_calls_ && !origins_.size();
+    return !duplicate_calls_ && origins_.empty();
   }
 
   bool CheckSingleOriginInUse(const url::Origin& origin) const {
@@ -107,7 +107,8 @@ class IndexedDBActiveBlobRegistryTest : public testing::Test {
         factory_(new RegistryTestMockFactory),
         backing_store_(
             new MockIDBBackingStore(factory_.get(), task_runner_.get())),
-        registry_(new IndexedDBActiveBlobRegistry(backing_store_.get())) {}
+        registry_(base::MakeUnique<IndexedDBActiveBlobRegistry>(
+            backing_store_.get())) {}
 
   void RunUntilIdle() { task_runner_->RunUntilIdle(); }
   RegistryTestMockFactory* factory() const { return factory_.get(); }
