@@ -13,6 +13,7 @@
 #include "base/files/scoped_temp_dir.h"
 #include "base/logging.h"
 #include "base/macros.h"
+#include "base/memory/ptr_util.h"
 #include "base/sequenced_task_runner.h"
 #include "base/strings/string16.h"
 #include "base/strings/utf_string_conversions.h"
@@ -38,6 +39,12 @@ namespace content {
 
 namespace {
 
+// Write |content| to |file|. Returns true on success.
+bool WriteFile(const base::FilePath& file, base::StringPiece content) {
+  int write_size = base::WriteFile(file, content.data(), content.length());
+  return write_size >= 0 && write_size == static_cast<int>(content.length());
+}
+
 class Comparator : public LevelDBComparator {
  public:
   int Compare(const base::StringPiece& a,
@@ -50,6 +57,7 @@ class Comparator : public LevelDBComparator {
 class DefaultLevelDBFactory : public LevelDBFactory {
  public:
   DefaultLevelDBFactory() {}
+
   leveldb::Status OpenLevelDB(const base::FilePath& file_name,
                               const LevelDBComparator* comparator,
                               std::unique_ptr<LevelDBDatabase>* db,
@@ -70,13 +78,14 @@ class TestableIndexedDBBackingStore : public IndexedDBBackingStore {
       IndexedDBFactory* indexed_db_factory,
       const Origin& origin,
       const base::FilePath& path_base,
-      net::URLRequestContext* request_context,
+      scoped_refptr<net::URLRequestContextGetter> request_context_getter,
       LevelDBFactory* leveldb_factory,
       base::SequencedTaskRunner* task_runner,
       leveldb::Status* status) {
     DCHECK(!path_base.empty());
 
-    std::unique_ptr<LevelDBComparator> comparator(new Comparator());
+    std::unique_ptr<LevelDBComparator> comparator =
+        base::MakeUnique<Comparator>();
 
     if (!base::CreateDirectory(path_base)) {
       *status = leveldb::Status::IOError("Unable to create base dir");
@@ -96,7 +105,7 @@ class TestableIndexedDBBackingStore : public IndexedDBBackingStore {
 
     scoped_refptr<TestableIndexedDBBackingStore> backing_store(
         new TestableIndexedDBBackingStore(indexed_db_factory, origin, blob_path,
-                                          request_context, std::move(db),
+                                          request_context_getter, std::move(db),
                                           std::move(comparator), task_runner));
 
     *status = backing_store->SetUpMetadata();
@@ -153,17 +162,18 @@ class TestableIndexedDBBackingStore : public IndexedDBBackingStore {
   }
 
  private:
-  TestableIndexedDBBackingStore(IndexedDBFactory* indexed_db_factory,
-                                const Origin& origin,
-                                const base::FilePath& blob_path,
-                                net::URLRequestContext* request_context,
-                                std::unique_ptr<LevelDBDatabase> db,
-                                std::unique_ptr<LevelDBComparator> comparator,
-                                base::SequencedTaskRunner* task_runner)
+  TestableIndexedDBBackingStore(
+      IndexedDBFactory* indexed_db_factory,
+      const Origin& origin,
+      const base::FilePath& blob_path,
+      scoped_refptr<net::URLRequestContextGetter> request_context_getter,
+      std::unique_ptr<LevelDBDatabase> db,
+      std::unique_ptr<LevelDBComparator> comparator,
+      base::SequencedTaskRunner* task_runner)
       : IndexedDBBackingStore(indexed_db_factory,
                               origin,
                               blob_path,
-                              request_context,
+                              request_context_getter,
                               std::move(db),
                               std::move(comparator),
                               task_runner),
@@ -186,19 +196,13 @@ class TestIDBFactory : public IndexedDBFactoryImpl {
 
   scoped_refptr<TestableIndexedDBBackingStore> OpenBackingStoreForTest(
       const Origin& origin,
-      net::URLRequestContext* url_request_context) {
-    blink::WebIDBDataLoss data_loss;
-    std::string data_loss_reason;
+      scoped_refptr<net::URLRequestContextGetter> url_request_context_getter) {
+    IndexedDBDataLossInfo data_loss_info;
     bool disk_full;
     leveldb::Status status;
-    scoped_refptr<IndexedDBBackingStore> backing_store =
-        OpenBackingStore(origin,
-                         context()->data_path(),
-                         url_request_context,
-                         &data_loss,
-                         &data_loss_reason,
-                         &disk_full,
-                         &status);
+    scoped_refptr<IndexedDBBackingStore> backing_store = OpenBackingStore(
+        origin, context()->data_path(), url_request_context_getter,
+        &data_loss_info, &disk_full, &status);
     scoped_refptr<TestableIndexedDBBackingStore> testable_store =
         static_cast<TestableIndexedDBBackingStore*>(backing_store.get());
     return testable_store;
@@ -210,15 +214,14 @@ class TestIDBFactory : public IndexedDBFactoryImpl {
   scoped_refptr<IndexedDBBackingStore> OpenBackingStoreHelper(
       const Origin& origin,
       const base::FilePath& data_directory,
-      net::URLRequestContext* request_context,
-      blink::WebIDBDataLoss* data_loss,
-      std::string* data_loss_message,
+      scoped_refptr<net::URLRequestContextGetter> request_context_getter,
+      IndexedDBDataLossInfo* data_loss_info,
       bool* disk_full,
       bool first_time,
       leveldb::Status* status) override {
     DefaultLevelDBFactory leveldb_factory;
     return TestableIndexedDBBackingStore::Open(
-        this, origin, data_directory, request_context, &leveldb_factory,
+        this, origin, data_directory, request_context_getter, &leveldb_factory,
         context()->TaskRunner(), status);
   }
 
@@ -232,16 +235,18 @@ class IndexedDBBackingStoreTest : public testing::Test {
   void SetUp() override {
     const Origin origin(GURL("http://localhost:81"));
     task_runner_ = new base::TestSimpleTaskRunner();
+    url_request_context_getter_ =
+        new net::TestURLRequestContextGetter(task_runner_);
     special_storage_policy_ = new MockSpecialStoragePolicy();
     quota_manager_proxy_ = new MockQuotaManagerProxy(nullptr, nullptr);
     special_storage_policy_->SetAllUnlimited(true);
     ASSERT_TRUE(temp_dir_.CreateUniqueTempDir());
     idb_context_ = new IndexedDBContextImpl(
-        temp_dir_.path(), special_storage_policy_.get(),
+        temp_dir_.GetPath(), special_storage_policy_.get(),
         quota_manager_proxy_.get(), task_runner_.get());
     idb_factory_ = new TestIDBFactory(idb_context_.get());
-    backing_store_ =
-        idb_factory_->OpenBackingStoreForTest(origin, &url_request_context_);
+    backing_store_ = idb_factory_->OpenBackingStoreForTest(
+        origin, url_request_context_getter_);
 
     // useful keys and values during tests
     m_value1 = IndexedDBValue("value1", std::vector<IndexedDBBlobInfo>());
@@ -298,12 +303,12 @@ class IndexedDBBackingStoreTest : public testing::Test {
     if (backing_store_->writes().size() != reads.size())
       return false;
     std::set<int64_t> ids;
-    for (size_t i = 0; i < backing_store_->writes().size(); ++i)
-      ids.insert(backing_store_->writes()[i].key());
+    for (const auto& write : backing_store_->writes())
+      ids.insert(write.key());
     if (ids.size() != backing_store_->writes().size())
       return false;
-    for (size_t i = 0; i < reads.size(); ++i) {
-      if (ids.count(reads[i].key()) != 1)
+    for (const auto& read : reads) {
+      if (ids.count(read.key()) != 1)
         return false;
     }
     return true;
@@ -341,8 +346,8 @@ class IndexedDBBackingStoreTest : public testing::Test {
   }
 
  protected:
-  // Must be initialized before url_request_context_
-  content::TestBrowserThreadBundle thread_bundle_;
+  // Must be initialized before url_request_context_getter_
+  TestBrowserThreadBundle thread_bundle_;
 
   base::ScopedTempDir temp_dir_;
   scoped_refptr<base::TestSimpleTaskRunner> task_runner_;
@@ -350,7 +355,7 @@ class IndexedDBBackingStoreTest : public testing::Test {
   scoped_refptr<MockQuotaManagerProxy> quota_manager_proxy_;
   scoped_refptr<IndexedDBContextImpl> idb_context_;
   scoped_refptr<TestIDBFactory> idb_factory_;
-  net::TestURLRequestContext url_request_context_;
+  scoped_refptr<net::URLRequestContextGetter> url_request_context_getter_;
 
   scoped_refptr<TestableIndexedDBBackingStore> backing_store_;
 
@@ -388,7 +393,7 @@ TEST_F(IndexedDBBackingStoreTest, PutGetConsistency) {
   {
     IndexedDBBackingStore::Transaction transaction1(backing_store_.get());
     transaction1.Begin();
-    ScopedVector<storage::BlobDataHandle> handles;
+    std::vector<std::unique_ptr<storage::BlobDataHandle>> handles;
     IndexedDBBackingStore::RecordIdentifier record;
     leveldb::Status s = backing_store_->PutRecord(
         &transaction1, 1, 1, m_key1, &m_value1, &handles, &record);
@@ -420,7 +425,7 @@ TEST_F(IndexedDBBackingStoreTest, PutGetConsistencyWithBlobs) {
   {
     IndexedDBBackingStore::Transaction transaction1(backing_store_.get());
     transaction1.Begin();
-    ScopedVector<storage::BlobDataHandle> handles;
+    std::vector<std::unique_ptr<storage::BlobDataHandle>> handles;
     IndexedDBBackingStore::RecordIdentifier record;
     EXPECT_TRUE(backing_store_->PutRecord(&transaction1,
                                           1,
@@ -507,7 +512,7 @@ TEST_F(IndexedDBBackingStoreTest, DeleteRange) {
       IndexedDBValue value3 = IndexedDBValue("value3", blob_info3);
       IndexedDBBackingStore::Transaction transaction1(backing_store_.get());
       transaction1.Begin();
-      ScopedVector<storage::BlobDataHandle> handles;
+      std::vector<std::unique_ptr<storage::BlobDataHandle>> handles;
       IndexedDBBackingStore::RecordIdentifier record;
       EXPECT_TRUE(backing_store_->PutRecord(&transaction1,
                                             1,
@@ -561,7 +566,7 @@ TEST_F(IndexedDBBackingStoreTest, DeleteRange) {
       EXPECT_TRUE(callback->called);
       EXPECT_TRUE(callback->succeeded);
       EXPECT_TRUE(transaction2.CommitPhaseTwo().ok());
-      EXPECT_EQ(2UL, backing_store_->removals().size());
+      ASSERT_EQ(2UL, backing_store_->removals().size());
       EXPECT_EQ(backing_store_->writes()[1].key(),
                 backing_store_->removals()[0]);
       EXPECT_EQ(backing_store_->writes()[2].key(),
@@ -601,7 +606,7 @@ TEST_F(IndexedDBBackingStoreTest, DeleteRangeEmptyRange) {
       IndexedDBValue value3 = IndexedDBValue("value3", blob_info3);
       IndexedDBBackingStore::Transaction transaction1(backing_store_.get());
       transaction1.Begin();
-      ScopedVector<storage::BlobDataHandle> handles;
+      std::vector<std::unique_ptr<storage::BlobDataHandle>> handles;
       IndexedDBBackingStore::RecordIdentifier record;
       EXPECT_TRUE(backing_store_->PutRecord(&transaction1,
                                             1,
@@ -663,7 +668,7 @@ TEST_F(IndexedDBBackingStoreTest, DeleteRangeEmptyRange) {
 TEST_F(IndexedDBBackingStoreTest, BlobJournalInterleavedTransactions) {
   IndexedDBBackingStore::Transaction transaction1(backing_store_.get());
   transaction1.Begin();
-  ScopedVector<storage::BlobDataHandle> handles1;
+  std::vector<std::unique_ptr<storage::BlobDataHandle>> handles1;
   IndexedDBBackingStore::RecordIdentifier record1;
   EXPECT_TRUE(backing_store_->PutRecord(&transaction1, 1, 1, m_key3, &m_value3,
                                         &handles1, &record1).ok());
@@ -677,7 +682,7 @@ TEST_F(IndexedDBBackingStoreTest, BlobJournalInterleavedTransactions) {
 
   IndexedDBBackingStore::Transaction transaction2(backing_store_.get());
   transaction2.Begin();
-  ScopedVector<storage::BlobDataHandle> handles2;
+  std::vector<std::unique_ptr<storage::BlobDataHandle>> handles2;
   IndexedDBBackingStore::RecordIdentifier record2;
   EXPECT_TRUE(backing_store_->PutRecord(&transaction2, 1, 1, m_key1, &m_value1,
                                         &handles2, &record2).ok());
@@ -700,7 +705,7 @@ TEST_F(IndexedDBBackingStoreTest, LiveBlobJournal) {
   {
     IndexedDBBackingStore::Transaction transaction1(backing_store_.get());
     transaction1.Begin();
-    ScopedVector<storage::BlobDataHandle> handles;
+    std::vector<std::unique_ptr<storage::BlobDataHandle>> handles;
     IndexedDBBackingStore::RecordIdentifier record;
     EXPECT_TRUE(backing_store_->PutRecord(&transaction1,
                                           1,
@@ -781,7 +786,7 @@ TEST_F(IndexedDBBackingStoreTest, HighIds) {
   {
     IndexedDBBackingStore::Transaction transaction1(backing_store_.get());
     transaction1.Begin();
-    ScopedVector<storage::BlobDataHandle> handles;
+    std::vector<std::unique_ptr<storage::BlobDataHandle>> handles;
     IndexedDBBackingStore::RecordIdentifier record;
     leveldb::Status s = backing_store_->PutRecord(&transaction1,
                                                   high_database_id,
@@ -871,7 +876,7 @@ TEST_F(IndexedDBBackingStoreTest, InvalidIds) {
   IndexedDBBackingStore::Transaction transaction1(backing_store_.get());
   transaction1.Begin();
 
-  ScopedVector<storage::BlobDataHandle> handles;
+  std::vector<std::unique_ptr<storage::BlobDataHandle>> handles;
   IndexedDBBackingStore::RecordIdentifier record;
   leveldb::Status s = backing_store_->PutRecord(&transaction1,
                                                 database_id,
@@ -1053,10 +1058,101 @@ TEST_F(IndexedDBBackingStoreTest, GetDatabaseNames) {
 
   std::vector<base::string16> names = backing_store_->GetDatabaseNames(&s);
   EXPECT_TRUE(s.ok());
-  EXPECT_EQ(names.size(), 1ULL);
-  EXPECT_EQ(names[0], db1_name);
+  ASSERT_EQ(1U, names.size());
+  EXPECT_EQ(db1_name, names[0]);
 }
 
 }  // namespace
 
+// Not in the anonymous namespace to friend IndexedDBBackingStore.
+TEST_F(IndexedDBBackingStoreTest, ReadCorruptionInfo) {
+  // No |path_base|.
+  std::string message;
+  EXPECT_FALSE(IndexedDBBackingStore::ReadCorruptionInfo(base::FilePath(),
+                                                         Origin(), &message));
+  EXPECT_TRUE(message.empty());
+  message.clear();
+
+  const base::FilePath path_base = temp_dir_.GetPath();
+  const Origin origin(GURL("http://www.google.com/"));
+  ASSERT_FALSE(path_base.empty());
+  ASSERT_TRUE(PathIsWritable(path_base));
+
+  // File not found.
+  EXPECT_FALSE(
+      IndexedDBBackingStore::ReadCorruptionInfo(path_base, origin, &message));
+  EXPECT_TRUE(message.empty());
+  message.clear();
+
+  const base::FilePath info_path =
+      path_base.AppendASCII("http_www.google.com_0.indexeddb.leveldb")
+          .AppendASCII("corruption_info.json");
+  ASSERT_TRUE(CreateDirectory(info_path.DirName()));
+
+  // Empty file.
+  std::string dummy_data;
+  ASSERT_TRUE(WriteFile(info_path, dummy_data));
+  EXPECT_FALSE(
+      IndexedDBBackingStore::ReadCorruptionInfo(path_base, origin, &message));
+  EXPECT_FALSE(PathExists(info_path));
+  EXPECT_TRUE(message.empty());
+  message.clear();
+
+  // File size > 4 KB.
+  dummy_data.resize(5000, 'c');
+  ASSERT_TRUE(WriteFile(info_path, dummy_data));
+  EXPECT_FALSE(
+      IndexedDBBackingStore::ReadCorruptionInfo(path_base, origin, &message));
+  EXPECT_FALSE(PathExists(info_path));
+  EXPECT_TRUE(message.empty());
+  message.clear();
+
+  // Random string.
+  ASSERT_TRUE(WriteFile(info_path, "foo bar"));
+  EXPECT_FALSE(
+      IndexedDBBackingStore::ReadCorruptionInfo(path_base, origin, &message));
+  EXPECT_FALSE(PathExists(info_path));
+  EXPECT_TRUE(message.empty());
+  message.clear();
+
+  // Not a dictionary.
+  ASSERT_TRUE(WriteFile(info_path, "[]"));
+  EXPECT_FALSE(
+      IndexedDBBackingStore::ReadCorruptionInfo(path_base, origin, &message));
+  EXPECT_FALSE(PathExists(info_path));
+  EXPECT_TRUE(message.empty());
+  message.clear();
+
+  // Empty dictionary.
+  ASSERT_TRUE(WriteFile(info_path, "{}"));
+  EXPECT_FALSE(
+      IndexedDBBackingStore::ReadCorruptionInfo(path_base, origin, &message));
+  EXPECT_FALSE(PathExists(info_path));
+  EXPECT_TRUE(message.empty());
+  message.clear();
+
+  // Dictionary, no message key.
+  ASSERT_TRUE(WriteFile(info_path, "{\"foo\":\"bar\"}"));
+  EXPECT_FALSE(
+      IndexedDBBackingStore::ReadCorruptionInfo(path_base, origin, &message));
+  EXPECT_FALSE(PathExists(info_path));
+  EXPECT_TRUE(message.empty());
+  message.clear();
+
+  // Dictionary, message key.
+  ASSERT_TRUE(WriteFile(info_path, "{\"message\":\"bar\"}"));
+  EXPECT_TRUE(
+      IndexedDBBackingStore::ReadCorruptionInfo(path_base, origin, &message));
+  EXPECT_FALSE(PathExists(info_path));
+  EXPECT_EQ("bar", message);
+  message.clear();
+
+  // Dictionary, message key and more.
+  ASSERT_TRUE(WriteFile(info_path, "{\"message\":\"foo\",\"bar\":5}"));
+  EXPECT_TRUE(
+      IndexedDBBackingStore::ReadCorruptionInfo(path_base, origin, &message));
+  EXPECT_FALSE(PathExists(info_path));
+  EXPECT_EQ("foo", message);
+}
+
 }  // namespace content
