@@ -12,7 +12,7 @@
 #include "base/files/file_enumerator.h"
 #include "base/files/file_util.h"
 #include "base/logging.h"
-#include "base/metrics/histogram.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/sequenced_task_runner.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
@@ -149,9 +149,9 @@ std::vector<IndexedDBInfo> IndexedDBContextImpl::GetAllOriginsInfo() {
   std::vector<IndexedDBInfo> result;
   for (const auto& origin : origins) {
     size_t connection_count = GetConnectionCount(origin);
-    result.push_back(
-        IndexedDBInfo(GURL(origin.Serialize()), GetOriginDiskUsage(origin),
-                      GetOriginLastModified(origin), connection_count));
+    result.push_back(IndexedDBInfo(origin.GetURL(), GetOriginDiskUsage(origin),
+                                   GetOriginLastModified(origin),
+                                   connection_count));
   }
   return result;
 }
@@ -166,14 +166,16 @@ base::ListValue* IndexedDBContextImpl::GetAllOriginsDetails() {
 
   std::sort(origins.begin(), origins.end(), HostNameComparator);
 
-  std::unique_ptr<base::ListValue> list(new base::ListValue());
+  std::unique_ptr<base::ListValue> list(base::MakeUnique<base::ListValue>());
   for (const auto& origin : origins) {
-    std::unique_ptr<base::DictionaryValue> info(new base::DictionaryValue());
+    std::unique_ptr<base::DictionaryValue> info(
+        base::MakeUnique<base::DictionaryValue>());
     info->SetString("url", origin.Serialize());
     info->SetString("size", ui::FormatBytes(GetOriginDiskUsage(origin)));
     info->SetDouble("last_modified", GetOriginLastModified(origin).ToJsTime());
     if (!is_incognito()) {
-      std::unique_ptr<base::ListValue> paths(new base::ListValue());
+      std::unique_ptr<base::ListValue> paths(
+          base::MakeUnique<base::ListValue>());
       for (const base::FilePath& path : GetStoragePaths(origin))
         paths->AppendString(path.value());
       info->Set("paths", paths.release());
@@ -189,31 +191,28 @@ base::ListValue* IndexedDBContextImpl::GetAllOriginsDetails() {
                 IndexedDBFactory::OriginDBMapIterator>
           range = factory_->GetOpenDatabasesForOrigin(origin);
       // TODO(jsbell): Sort by name?
-      std::unique_ptr<base::ListValue> database_list(new base::ListValue());
+      std::unique_ptr<base::ListValue> database_list(
+          base::MakeUnique<base::ListValue>());
 
       for (IndexedDBFactory::OriginDBMapIterator it = range.first;
            it != range.second;
            ++it) {
         const IndexedDBDatabase* db = it->second;
         std::unique_ptr<base::DictionaryValue> db_info(
-            new base::DictionaryValue());
+            base::MakeUnique<base::DictionaryValue>());
 
         db_info->SetString("name", db->name());
-        db_info->SetDouble("pending_opens", db->PendingOpenCount());
-        db_info->SetDouble("pending_upgrades", db->PendingUpgradeCount());
-        db_info->SetDouble("running_upgrades", db->RunningUpgradeCount());
-        db_info->SetDouble("pending_deletes", db->PendingDeleteCount());
-        db_info->SetDouble("connection_count",
-                           db->ConnectionCount() - db->PendingUpgradeCount() -
-                               db->RunningUpgradeCount());
+        db_info->SetDouble("connection_count", db->ConnectionCount());
+        db_info->SetDouble("active_open_delete", db->ActiveOpenDeleteCount());
+        db_info->SetDouble("pending_open_delete", db->PendingOpenDeleteCount());
 
         std::unique_ptr<base::ListValue> transaction_list(
-            new base::ListValue());
+            base::MakeUnique<base::ListValue>());
         std::vector<const IndexedDBTransaction*> transactions =
             db->transaction_coordinator().GetTransactions();
         for (const auto* transaction : transactions) {
           std::unique_ptr<base::DictionaryValue> transaction_info(
-              new base::DictionaryValue());
+              base::MakeUnique<base::DictionaryValue>());
 
           const char* const kModes[] =
               { "readonly", "readwrite", "versionchange" };
@@ -257,10 +256,10 @@ base::ListValue* IndexedDBContextImpl::GetAllOriginsDetails() {
           transaction_info->SetDouble(
               "tasks_completed", transaction->diagnostics().tasks_completed);
 
-          std::unique_ptr<base::ListValue> scope(new base::ListValue());
+          std::unique_ptr<base::ListValue> scope(
+              base::MakeUnique<base::ListValue>());
           for (const auto& id : transaction->scope()) {
-            IndexedDBDatabaseMetadata::ObjectStoreMap::const_iterator it =
-                db->metadata().object_stores.find(id);
+            const auto& it = db->metadata().object_stores.find(id);
             if (it != db->metadata().object_stores.end())
               scope->AppendString(it->second.name);
           }
@@ -336,7 +335,7 @@ void IndexedDBContextImpl::DeleteForOrigin(const Origin& origin) {
   } else {
     // LevelDB does not delete empty directories; work around this.
     // TODO(jsbell): Remove when upstream bug is fixed.
-    // https://code.google.com/p/leveldb/issues/detail?id=209
+    // https://github.com/google/leveldb/issues/215
     const bool kNonRecursive = false;
     base::DeleteFile(idb_directory, kNonRecursive);
   }
@@ -440,7 +439,7 @@ void IndexedDBContextImpl::ConnectionOpened(const Origin& origin,
                                             IndexedDBConnection* connection) {
   DCHECK(TaskRunner()->RunsTasksOnCurrentThread());
   quota_manager_proxy()->NotifyStorageAccessed(
-      storage::QuotaClient::kIndexedDatabase, GURL(origin.Serialize()),
+      storage::QuotaClient::kIndexedDatabase, origin.GetURL(),
       storage::kStorageTypeTemporary);
   if (AddToOriginSet(origin)) {
     // A newly created db, notify the quota system.
@@ -454,7 +453,7 @@ void IndexedDBContextImpl::ConnectionClosed(const Origin& origin,
                                             IndexedDBConnection* connection) {
   DCHECK(TaskRunner()->RunsTasksOnCurrentThread());
   quota_manager_proxy()->NotifyStorageAccessed(
-      storage::QuotaClient::kIndexedDatabase, GURL(origin.Serialize()),
+      storage::QuotaClient::kIndexedDatabase, origin.GetURL(),
       storage::kStorageTypeTemporary);
   if (factory_.get() && factory_->GetConnectionCount(origin) == 0)
     QueryDiskAndUpdateQuotaUsage(origin);
@@ -500,8 +499,7 @@ IndexedDBContextImpl::~IndexedDBContextImpl() {
 // static
 base::FilePath IndexedDBContextImpl::GetBlobStoreFileName(
     const Origin& origin) {
-  std::string origin_id =
-      storage::GetIdentifierFromOrigin(GURL(origin.Serialize()));
+  std::string origin_id = storage::GetIdentifierFromOrigin(origin.GetURL());
   return base::FilePath()
       .AppendASCII(origin_id)
       .AddExtension(kIndexedDBExtension)
@@ -510,8 +508,7 @@ base::FilePath IndexedDBContextImpl::GetBlobStoreFileName(
 
 // static
 base::FilePath IndexedDBContextImpl::GetLevelDBFileName(const Origin& origin) {
-  std::string origin_id =
-      storage::GetIdentifierFromOrigin(GURL(origin.Serialize()));
+  std::string origin_id = storage::GetIdentifierFromOrigin(origin.GetURL());
   return base::FilePath()
       .AppendASCII(origin_id)
       .AddExtension(kIndexedDBExtension)
@@ -552,7 +549,7 @@ void IndexedDBContextImpl::QueryDiskAndUpdateQuotaUsage(const Origin& origin) {
   if (difference) {
     origin_size_map_[origin] = current_disk_usage;
     quota_manager_proxy()->NotifyStorageModified(
-        storage::QuotaClient::kIndexedDatabase, GURL(origin.Serialize()),
+        storage::QuotaClient::kIndexedDatabase, origin.GetURL(),
         storage::kStorageTypeTemporary, difference);
   }
 }
@@ -561,7 +558,8 @@ std::set<Origin>* IndexedDBContextImpl::GetOriginSet() {
   if (!origin_set_) {
     std::vector<Origin> origins;
     GetAllOriginsAndPaths(data_path_, &origins, NULL);
-    origin_set_.reset(new std::set<Origin>(origins.begin(), origins.end()));
+    origin_set_ =
+        base::MakeUnique<std::set<Origin>>(origins.begin(), origins.end());
   }
   return origin_set_.get();
 }
