@@ -11,6 +11,7 @@
 #include "base/auto_reset.h"
 #include "base/logging.h"
 #include "base/macros.h"
+#include "base/memory/ptr_util.h"
 #include "base/run_loop.h"
 #include "base/strings/string16.h"
 #include "base/strings/utf_string_conversions.h"
@@ -26,6 +27,7 @@
 #include "content/browser/indexed_db/mock_indexed_db_callbacks.h"
 #include "content/browser/indexed_db/mock_indexed_db_database_callbacks.h"
 #include "content/browser/indexed_db/mock_indexed_db_factory.h"
+#include "content/public/test/test_browser_thread_bundle.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 using base::ASCIIToUTF16;
@@ -36,7 +38,12 @@ const int kFakeChildProcessId = 0;
 
 namespace content {
 
-TEST(IndexedDBDatabaseTest, BackingStoreRetention) {
+class IndexedDBDatabaseTest : public ::testing::Test {
+ private:
+  TestBrowserThreadBundle thread_bundle_;
+};
+
+TEST_F(IndexedDBDatabaseTest, BackingStoreRetention) {
   scoped_refptr<IndexedDBFakeBackingStore> backing_store =
       new IndexedDBFakeBackingStore();
   EXPECT_TRUE(backing_store->HasOneRef());
@@ -55,7 +62,7 @@ TEST(IndexedDBDatabaseTest, BackingStoreRetention) {
   EXPECT_TRUE(backing_store->HasOneRef());  // local
 }
 
-TEST(IndexedDBDatabaseTest, ConnectionLifecycle) {
+TEST_F(IndexedDBDatabaseTest, ConnectionLifecycle) {
   scoped_refptr<IndexedDBFakeBackingStore> backing_store =
       new IndexedDBFakeBackingStore();
   EXPECT_TRUE(backing_store->HasOneRef());  // local
@@ -75,10 +82,11 @@ TEST(IndexedDBDatabaseTest, ConnectionLifecycle) {
   scoped_refptr<MockIndexedDBDatabaseCallbacks> callbacks1(
       new MockIndexedDBDatabaseCallbacks());
   const int64_t transaction_id1 = 1;
-  IndexedDBPendingConnection connection1(
-      request1, callbacks1, kFakeChildProcessId, transaction_id1,
-      IndexedDBDatabaseMetadata::DEFAULT_VERSION);
-  db->OpenConnection(connection1);
+  std::unique_ptr<IndexedDBPendingConnection> connection1(
+      base::MakeUnique<IndexedDBPendingConnection>(
+          request1, callbacks1, kFakeChildProcessId, transaction_id1,
+          IndexedDBDatabaseMetadata::DEFAULT_VERSION));
+  db->OpenConnection(std::move(connection1));
 
   EXPECT_FALSE(backing_store->HasOneRef());  // db, connection count > 0
 
@@ -86,10 +94,11 @@ TEST(IndexedDBDatabaseTest, ConnectionLifecycle) {
   scoped_refptr<MockIndexedDBDatabaseCallbacks> callbacks2(
       new MockIndexedDBDatabaseCallbacks());
   const int64_t transaction_id2 = 2;
-  IndexedDBPendingConnection connection2(
-      request2, callbacks2, kFakeChildProcessId, transaction_id2,
-      IndexedDBDatabaseMetadata::DEFAULT_VERSION);
-  db->OpenConnection(connection2);
+  std::unique_ptr<IndexedDBPendingConnection> connection2(
+      base::MakeUnique<IndexedDBPendingConnection>(
+          request2, callbacks2, kFakeChildProcessId, transaction_id2,
+          IndexedDBDatabaseMetadata::DEFAULT_VERSION));
+  db->OpenConnection(std::move(connection2));
 
   EXPECT_FALSE(backing_store->HasOneRef());  // local and connection
 
@@ -107,7 +116,7 @@ TEST(IndexedDBDatabaseTest, ConnectionLifecycle) {
   db = NULL;
 }
 
-TEST(IndexedDBDatabaseTest, ForcedClose) {
+TEST_F(IndexedDBDatabaseTest, ForcedClose) {
   scoped_refptr<IndexedDBFakeBackingStore> backing_store =
       new IndexedDBFakeBackingStore();
   EXPECT_TRUE(backing_store->HasOneRef());
@@ -127,10 +136,11 @@ TEST(IndexedDBDatabaseTest, ForcedClose) {
       new MockIndexedDBDatabaseCallbacks());
   scoped_refptr<MockIndexedDBCallbacks> request(new MockIndexedDBCallbacks());
   const int64_t upgrade_transaction_id = 3;
-  IndexedDBPendingConnection connection(
-      request, callbacks, kFakeChildProcessId, upgrade_transaction_id,
-      IndexedDBDatabaseMetadata::DEFAULT_VERSION);
-  database->OpenConnection(connection);
+  std::unique_ptr<IndexedDBPendingConnection> connection(
+      base::MakeUnique<IndexedDBPendingConnection>(
+          request, callbacks, kFakeChildProcessId, upgrade_transaction_id,
+          IndexedDBDatabaseMetadata::DEFAULT_VERSION));
+  database->OpenConnection(std::move(connection));
   EXPECT_EQ(database.get(), request->connection()->database());
 
   const int64_t transaction_id = 123;
@@ -146,29 +156,34 @@ TEST(IndexedDBDatabaseTest, ForcedClose) {
   EXPECT_TRUE(callbacks->abort_called());
 }
 
-class MockDeleteCallbacks : public IndexedDBCallbacks {
+class MockCallbacks : public IndexedDBCallbacks {
  public:
-  MockDeleteCallbacks()
-      : IndexedDBCallbacks(NULL, 0, 0),
-        blocked_called_(false),
-        success_called_(false) {}
+  MockCallbacks() : IndexedDBCallbacks(NULL, 0, 0) {}
 
   void OnBlocked(int64_t existing_version) override { blocked_called_ = true; }
   void OnSuccess(int64_t result) override { success_called_ = true; }
+  void OnError(const IndexedDBDatabaseError& error) override {
+    error_called_ = true;
+  }
+  bool IsValid() const override { return valid_; }
 
   bool blocked_called() const { return blocked_called_; }
   bool success_called() const { return success_called_; }
+  bool error_called() const { return error_called_; }
+  void set_valid(bool valid) { valid_ = valid; }
 
  private:
-  ~MockDeleteCallbacks() override {}
+  ~MockCallbacks() override {}
 
-  bool blocked_called_;
-  bool success_called_;
+  bool blocked_called_ = false;
+  bool success_called_ = false;
+  bool error_called_ = false;
+  bool valid_ = true;
 
-  DISALLOW_COPY_AND_ASSIGN(MockDeleteCallbacks);
+  DISALLOW_COPY_AND_ASSIGN(MockCallbacks);
 };
 
-TEST(IndexedDBDatabaseTest, PendingDelete) {
+TEST_F(IndexedDBDatabaseTest, PendingDelete) {
   scoped_refptr<IndexedDBFakeBackingStore> backing_store =
       new IndexedDBFakeBackingStore();
   EXPECT_TRUE(backing_store->HasOneRef());  // local
@@ -188,29 +203,143 @@ TEST(IndexedDBDatabaseTest, PendingDelete) {
   scoped_refptr<MockIndexedDBDatabaseCallbacks> callbacks1(
       new MockIndexedDBDatabaseCallbacks());
   const int64_t transaction_id1 = 1;
-  IndexedDBPendingConnection connection(
-      request1, callbacks1, kFakeChildProcessId, transaction_id1,
-      IndexedDBDatabaseMetadata::DEFAULT_VERSION);
-  db->OpenConnection(connection);
+  std::unique_ptr<IndexedDBPendingConnection> connection(
+      base::MakeUnique<IndexedDBPendingConnection>(
+          request1, callbacks1, kFakeChildProcessId, transaction_id1,
+          IndexedDBDatabaseMetadata::DEFAULT_VERSION));
+  db->OpenConnection(std::move(connection));
 
+  EXPECT_EQ(db->ConnectionCount(), 1UL);
+  EXPECT_EQ(db->ActiveOpenDeleteCount(), 0UL);
+  EXPECT_EQ(db->PendingOpenDeleteCount(), 0UL);
   EXPECT_FALSE(backing_store->HasOneRef());  // local and db
 
-  scoped_refptr<MockDeleteCallbacks> request2(new MockDeleteCallbacks());
+  scoped_refptr<MockCallbacks> request2(new MockCallbacks());
   db->DeleteDatabase(request2);
+  EXPECT_EQ(db->ConnectionCount(), 1UL);
+  EXPECT_EQ(db->ActiveOpenDeleteCount(), 1UL);
+  EXPECT_EQ(db->PendingOpenDeleteCount(), 0UL);
 
   EXPECT_FALSE(request2->blocked_called());
   db->VersionChangeIgnored();
   EXPECT_TRUE(request2->blocked_called());
+  EXPECT_EQ(db->ConnectionCount(), 1UL);
+  EXPECT_EQ(db->ActiveOpenDeleteCount(), 1UL);
+  EXPECT_EQ(db->PendingOpenDeleteCount(), 0UL);
 
   EXPECT_FALSE(backing_store->HasOneRef());  // local and db
 
   db->Close(request1->connection(), true /* forced */);
+  EXPECT_EQ(db->ConnectionCount(), 0UL);
+  EXPECT_EQ(db->ActiveOpenDeleteCount(), 0UL);
+  EXPECT_EQ(db->PendingOpenDeleteCount(), 0UL);
 
   EXPECT_FALSE(db->backing_store());
   EXPECT_TRUE(backing_store->HasOneRef());  // local
   EXPECT_TRUE(request2->success_called());
 }
 
+TEST_F(IndexedDBDatabaseTest, ConnectionRequestsNoLongerValid) {
+  scoped_refptr<IndexedDBFakeBackingStore> backing_store =
+      new IndexedDBFakeBackingStore();
+
+  const int64_t transaction_id1 = 1;
+  scoped_refptr<MockIndexedDBFactory> factory = new MockIndexedDBFactory();
+  leveldb::Status s;
+  scoped_refptr<IndexedDBDatabase> db = IndexedDBDatabase::Create(
+      ASCIIToUTF16("db"), backing_store.get(), factory.get(),
+      IndexedDBDatabase::Identifier(), &s);
+
+  // Make a connection request. This will be processed immediately.
+  scoped_refptr<MockIndexedDBCallbacks> request1(new MockIndexedDBCallbacks());
+  {
+    std::unique_ptr<IndexedDBPendingConnection> connection(
+        base::MakeUnique<IndexedDBPendingConnection>(
+            request1, make_scoped_refptr(new MockIndexedDBDatabaseCallbacks()),
+            kFakeChildProcessId, transaction_id1,
+            IndexedDBDatabaseMetadata::DEFAULT_VERSION));
+    db->OpenConnection(std::move(connection));
+  }
+
+  EXPECT_EQ(db->ConnectionCount(), 1UL);
+  EXPECT_EQ(db->ActiveOpenDeleteCount(), 0UL);
+  EXPECT_EQ(db->PendingOpenDeleteCount(), 0UL);
+
+  // Make a delete request. This will be blocked by the open.
+  scoped_refptr<MockCallbacks> request2(new MockCallbacks());
+  db->DeleteDatabase(request2);
+
+  EXPECT_EQ(db->ConnectionCount(), 1UL);
+  EXPECT_EQ(db->ActiveOpenDeleteCount(), 1UL);
+  EXPECT_EQ(db->PendingOpenDeleteCount(), 0UL);
+
+  db->VersionChangeIgnored();
+
+  EXPECT_TRUE(request2->blocked_called());
+
+  // Make another delete request. This will be waiting in the queue.
+  scoped_refptr<MockCallbacks> request3(new MockCallbacks());
+  db->DeleteDatabase(request3);
+  EXPECT_FALSE(request3->HasOneRef());  // local, db
+
+  EXPECT_EQ(db->ConnectionCount(), 1UL);
+  EXPECT_EQ(db->ActiveOpenDeleteCount(), 1UL);
+  EXPECT_EQ(db->PendingOpenDeleteCount(), 1UL);
+
+  // Make another connection request. This will also be waiting in the queue.
+  scoped_refptr<MockCallbacks> request4(new MockCallbacks());
+  {
+    std::unique_ptr<IndexedDBPendingConnection> connection(
+        base::MakeUnique<IndexedDBPendingConnection>(
+            request4, make_scoped_refptr(new MockIndexedDBDatabaseCallbacks()),
+            kFakeChildProcessId, transaction_id1,
+            IndexedDBDatabaseMetadata::DEFAULT_VERSION));
+    db->OpenConnection(std::move(connection));
+  }
+
+  EXPECT_EQ(db->ConnectionCount(), 1UL);
+  EXPECT_EQ(db->ActiveOpenDeleteCount(), 1UL);
+  EXPECT_EQ(db->PendingOpenDeleteCount(), 2UL);
+
+  // Finally yet another delete request, also waiting in the queue.
+  scoped_refptr<MockCallbacks> request5(new MockCallbacks());
+  db->DeleteDatabase(request2);
+
+  EXPECT_EQ(db->ConnectionCount(), 1UL);
+  EXPECT_EQ(db->ActiveOpenDeleteCount(), 1UL);
+  EXPECT_EQ(db->PendingOpenDeleteCount(), 3UL);
+
+  // Simulate renderer going away.
+  request3->set_valid(false);
+  request4->set_valid(false);
+
+  // Close the blocking connection. First delete request should succeed.
+  EXPECT_FALSE(request2->success_called());
+  db->Close(request1->connection(), true /* forced */);
+  EXPECT_TRUE(request2->success_called());
+
+  // Delete requests that have lost dispatcher should still be processed so
+  // that e.g. a delete followed by a window close is not ignored.
+  EXPECT_FALSE(request3->blocked_called());
+  EXPECT_TRUE(request3->success_called());
+  EXPECT_FALSE(request3->error_called());
+  EXPECT_TRUE(request3->HasOneRef());  // local
+
+  // Open requests that have lost dispatcher should not be processed.
+  EXPECT_FALSE(request4->blocked_called());
+  EXPECT_FALSE(request4->success_called());
+  EXPECT_FALSE(request4->error_called());
+  EXPECT_TRUE(request4->HasOneRef());  // local
+
+  // Final delete request should also run.
+  EXPECT_TRUE(request2->success_called());
+
+  // And everything else should be complete.
+  EXPECT_EQ(db->ConnectionCount(), 0UL);
+  EXPECT_EQ(db->ActiveOpenDeleteCount(), 0UL);
+  EXPECT_EQ(db->PendingOpenDeleteCount(), 0UL);
+}
+
 void DummyOperation(IndexedDBTransaction* transaction) {
 }
 
@@ -233,14 +362,18 @@ class IndexedDBDatabaseOperationTest : public testing::Test {
     request_ = new MockIndexedDBCallbacks();
     callbacks_ = new MockIndexedDBDatabaseCallbacks();
     const int64_t transaction_id = 1;
-    db_->OpenConnection(IndexedDBPendingConnection(
-        request_, callbacks_, kFakeChildProcessId, transaction_id,
-        IndexedDBDatabaseMetadata::DEFAULT_VERSION));
+    std::unique_ptr<IndexedDBPendingConnection> connection(
+        base::MakeUnique<IndexedDBPendingConnection>(
+            request_, callbacks_, kFakeChildProcessId, transaction_id,
+            IndexedDBDatabaseMetadata::DEFAULT_VERSION));
+    db_->OpenConnection(std::move(connection));
     EXPECT_EQ(IndexedDBDatabaseMetadata::NO_VERSION, db_->metadata().version);
 
+    connection_ = base::MakeUnique<IndexedDBConnection>(db_, callbacks_);
     transaction_ = IndexedDBClassFactory::Get()->CreateIndexedDBTransaction(
-        transaction_id, callbacks_, std::set<int64_t>() /*scope*/,
-        blink::WebIDBTransactionModeVersionChange, db_.get(),
+        transaction_id, connection_->GetWeakPtr(),
+        std::set<int64_t>() /*scope*/,
+        blink::WebIDBTransactionModeVersionChange,
         new IndexedDBFakeBackingStore::FakeTransaction(commit_success_));
     db_->TransactionCreated(transaction_.get());
 
@@ -258,12 +391,13 @@ class IndexedDBDatabaseOperationTest : public testing::Test {
   scoped_refptr<MockIndexedDBCallbacks> request_;
   scoped_refptr<MockIndexedDBDatabaseCallbacks> callbacks_;
   scoped_refptr<IndexedDBTransaction> transaction_;
+  std::unique_ptr<IndexedDBConnection> connection_;
 
   leveldb::Status commit_success_;
 
  private:
-  base::MessageLoop message_loop_;
   scoped_refptr<MockIndexedDBFactory> factory_;
+  content::TestBrowserThreadBundle thread_bundle_;
 
   DISALLOW_COPY_AND_ASSIGN(IndexedDBDatabaseOperationTest);
 };
@@ -375,9 +509,9 @@ TEST_F(IndexedDBDatabaseOperationTest, CreatePutDelete) {
 
   // Put is asynchronous
   IndexedDBValue value("value1", std::vector<IndexedDBBlobInfo>());
-  ScopedVector<storage::BlobDataHandle> handles;
-  std::unique_ptr<IndexedDBKey> key(new IndexedDBKey("key"));
-  std::vector<IndexedDBDatabase::IndexKeys> index_keys;
+  std::vector<std::unique_ptr<storage::BlobDataHandle>> handles;
+  std::unique_ptr<IndexedDBKey> key(base::MakeUnique<IndexedDBKey>("key"));
+  std::vector<IndexedDBIndexKeys> index_keys;
   scoped_refptr<MockIndexedDBCallbacks> request(
       new MockIndexedDBCallbacks(false));
   db_->Put(transaction_->id(), store_id, &value, &handles, std::move(key),
