@@ -33,4 +33,8 @@ void MockIndexedDBCallbacks::OnSuccess(
   connection_ = std::move(connection);
 }
 
+bool MockIndexedDBCallbacks::IsValid() const {
+  return true;
+}
+
 }  // namespace content
