@@ -4,6 +4,7 @@
 
 #include "content/browser/loader/sync_resource_handler.h"
 
+#include "base/callback_helpers.h"
 #include "base/logging.h"
 #include "content/browser/loader/netlog_observer.h"
 #include "content/browser/loader/resource_dispatcher_host_impl.h"
@@ -20,25 +21,19 @@ namespace content {
 
 SyncResourceHandler::SyncResourceHandler(
     net::URLRequest* request,
-    IPC::Message* result_message,
+    const SyncLoadResultCallback& result_handler,
     ResourceDispatcherHostImpl* resource_dispatcher_host)
     : ResourceHandler(request),
       read_buffer_(new net::IOBuffer(kReadBufSize)),
-      result_message_(result_message),
+      result_handler_(result_handler),
       rdh_(resource_dispatcher_host),
       total_transfer_size_(0) {
   result_.final_url = request->url();
 }
 
 SyncResourceHandler::~SyncResourceHandler() {
-  if (result_message_) {
-    result_message_->set_reply_error();
-    ResourceMessageFilter* filter = GetFilter();
-    // If the filter doesn't exist at this point, the process has died and isn't
-    // waiting for the result message anymore.
-    if (filter)
-      filter->Send(result_message_);
-  }
+  if (result_handler_)
+    result_handler_.Run(nullptr);
 }
 
 bool SyncResourceHandler::OnRequestRedirected(
@@ -73,8 +68,8 @@ bool SyncResourceHandler::OnResponseStarted(
     return false;
 
   if (rdh_->delegate()) {
-    rdh_->delegate()->OnResponseStarted(
-        request(), info->GetContext(), response, info->filter());
+    rdh_->delegate()->OnResponseStarted(request(), info->GetContext(),
+                                        response);
   }
 
   NetLogObserver::PopulateResponseInfo(request(), response);
@@ -95,10 +90,6 @@ bool SyncResourceHandler::OnWillStart(const GURL& url, bool* defer) {
   return true;
 }
 
-bool SyncResourceHandler::OnBeforeNetworkStart(const GURL& url, bool* defer) {
-  return true;
-}
-
 bool SyncResourceHandler::OnWillRead(scoped_refptr<net::IOBuffer>* buf,
                                      int* buf_size,
                                      int min_size) {
@@ -117,21 +108,14 @@ bool SyncResourceHandler::OnReadCompleted(int bytes_read, bool* defer) {
 
 void SyncResourceHandler::OnResponseCompleted(
     const net::URLRequestStatus& status,
-    const std::string& security_info,
     bool* defer) {
-  ResourceMessageFilter* filter = GetFilter();
-  if (!filter)
-    return;
-
   result_.error_code = status.error();
 
   int total_transfer_size = request()->GetTotalReceivedBytes();
   result_.encoded_data_length = total_transfer_size_ + total_transfer_size;
+  result_.encoded_body_length = request()->GetRawBodyBytes();
 
-  ResourceHostMsg_SyncLoad::WriteReplyParams(result_message_, result_);
-  filter->Send(result_message_);
-  result_message_ = NULL;
-  return;
+  base::ResetAndReturn(&result_handler_).Run(&result_);
 }
 
 void SyncResourceHandler::OnDataDownloaded(int bytes_downloaded) {
