@@ -28,6 +28,7 @@
 #include "testing/gmock/include/gmock/gmock.h"
 
 using ::testing::_;
+using ::testing::ByRef;
 using ::testing::Eq;
 using ::testing::InvokeWithoutArgs;
 using ::testing::Mock;
@@ -38,22 +39,22 @@ namespace content {
 
 namespace {
 
-// Matches mojo structs.
+// Matches Mojo structs.
 MATCHER_P(Equals, expected, "") {
   return expected.Equals(arg);
 }
 
-const char *const kPresentationId = "presentationId";
-const char *const kPresentationUrl = "http://foo.com/index.html";
-
-bool ArePresentationSessionMessagesEqual(
-    const blink::mojom::SessionMessage* expected,
-    const blink::mojom::SessionMessage* actual) {
-  return expected->type == actual->type &&
-         expected->message == actual->message &&
-         expected->data.Equals(actual->data);
+// Matches PresentationSessionInfo passed by reference.
+MATCHER_P(SessionInfoEquals, expected, "") {
+  blink::mojom::PresentationSessionInfo& expected_value = expected;
+  return expected_value.Equals(arg);
 }
 
+const char kPresentationId[] = "presentationId";
+const char kPresentationUrl1[] = "http://foo.com/index.html";
+const char kPresentationUrl2[] = "http://example.com/index.html";
+const char kPresentationUrl3[] = "http://example.net/index.html";
+
 void DoNothing(blink::mojom::PresentationSessionInfoPtr info,
                blink::mojom::PresentationErrorPtr error) {}
 
@@ -80,29 +81,27 @@ class MockPresentationServiceDelegate : public PresentationServiceDelegate {
   MOCK_METHOD0(AddScreenAvailabilityListener, bool());
 
   MOCK_METHOD3(RemoveScreenAvailabilityListener,
-      void(
-          int render_process_id,
-          int routing_id,
-          PresentationScreenAvailabilityListener* listener));
+      void(int render_process_id,
+           int routing_id,
+           PresentationScreenAvailabilityListener* listener));
   MOCK_METHOD2(Reset,
-      void(
-          int render_process_id,
-          int routing_id));
-  MOCK_METHOD4(SetDefaultPresentationUrl,
+      void(int render_process_id,
+           int routing_id));
+  MOCK_METHOD4(SetDefaultPresentationUrls,
                void(int render_process_id,
                     int routing_id,
-                    const std::string& default_presentation_url,
+                    const std::vector<GURL>& default_presentation_urls,
                     const PresentationSessionStartedCallback& callback));
   MOCK_METHOD5(StartSession,
                void(int render_process_id,
                     int render_frame_id,
-                    const std::string& presentation_url,
+                    const std::vector<GURL>& presentation_urls,
                     const PresentationSessionStartedCallback& success_cb,
                     const PresentationSessionErrorCallback& error_cb));
   MOCK_METHOD6(JoinSession,
                void(int render_process_id,
                     int render_frame_id,
-                    const std::string& presentation_url,
+                    const std::vector<GURL>& presentation_urls,
                     const std::string& presentation_id,
                     const PresentationSessionStartedCallback& success_cb,
                     const PresentationSessionErrorCallback& error_cb));
@@ -152,7 +151,7 @@ class MockPresentationServiceClient
     : public blink::mojom::PresentationServiceClient {
  public:
   MOCK_METHOD2(OnScreenAvailabilityUpdated,
-      void(const mojo::String& url, bool available));
+               void(const GURL& url, bool available));
   void OnConnectionStateChanged(
       blink::mojom::PresentationSessionInfoPtr connection,
       blink::mojom::PresentationConnectionState new_state) override {
@@ -165,19 +164,19 @@ class MockPresentationServiceClient
   void OnConnectionClosed(
       blink::mojom::PresentationSessionInfoPtr connection,
       blink::mojom::PresentationConnectionCloseReason reason,
-      const mojo::String& message) override {
+      const std::string& message) override {
     OnConnectionClosed(*connection, reason, message);
   }
   MOCK_METHOD3(OnConnectionClosed,
                void(const blink::mojom::PresentationSessionInfo& connection,
                     blink::mojom::PresentationConnectionCloseReason reason,
-                    const mojo::String& message));
+                    const std::string& message));
 
-  MOCK_METHOD1(OnScreenAvailabilityNotSupported, void(const mojo::String& url));
+  MOCK_METHOD1(OnScreenAvailabilityNotSupported, void(const GURL& url));
 
   void OnSessionMessagesReceived(
       blink::mojom::PresentationSessionInfoPtr session_info,
-      mojo::Array<blink::mojom::SessionMessagePtr> messages) override {
+      std::vector<blink::mojom::SessionMessagePtr> messages) override {
     messages_received_ = std::move(messages);
     MessagesReceived();
   }
@@ -190,12 +189,22 @@ class MockPresentationServiceClient
   MOCK_METHOD1(OnDefaultSessionStarted,
                void(const blink::mojom::PresentationSessionInfo& session_info));
 
-  mojo::Array<blink::mojom::SessionMessagePtr> messages_received_;
+  void OnReceiverConnectionAvailable(
+      blink::mojom::PresentationSessionInfoPtr session_info) override {
+    OnReceiverConnectionAvailable(*session_info);
+  }
+  MOCK_METHOD1(OnReceiverConnectionAvailable,
+               void(const blink::mojom::PresentationSessionInfo& session_info));
+
+  std::vector<blink::mojom::SessionMessagePtr> messages_received_;
 };
 
 class PresentationServiceImplTest : public RenderViewHostImplTestHarness {
  public:
-  PresentationServiceImplTest() {}
+  PresentationServiceImplTest()
+      : presentation_url1_(GURL(kPresentationUrl1)),
+        presentation_url2_(GURL(kPresentationUrl2)),
+        presentation_url3_(GURL(kPresentationUrl3)) {}
 
   void SetUp() override {
     RenderViewHostImplTestHarness::SetUp();
@@ -213,6 +222,9 @@ class PresentationServiceImplTest : public RenderViewHostImplTestHarness {
         new mojo::Binding<blink::mojom::PresentationServiceClient>(
             &mock_client_, mojo::GetProxy(&client_ptr)));
     service_impl_->SetClient(std::move(client_ptr));
+
+    presentation_urls_.push_back(presentation_url1_);
+    presentation_urls_.push_back(presentation_url2_);
   }
 
   void TearDown() override {
@@ -224,8 +236,8 @@ class PresentationServiceImplTest : public RenderViewHostImplTestHarness {
     RenderViewHostImplTestHarness::TearDown();
   }
 
-  void ListenForScreenAvailabilityAndWait(
-      const mojo::String& url, bool delegate_success) {
+  void ListenForScreenAvailabilityAndWait(const GURL& url,
+                                          bool delegate_success) {
     base::RunLoop run_loop;
     // This will call to |service_impl_| via mojo. Process the message
     // using RunLoop.
@@ -255,15 +267,14 @@ class PresentationServiceImplTest : public RenderViewHostImplTestHarness {
     run_loop_quit_closure_.Reset();
   }
 
-  void SimulateScreenAvailabilityChangeAndWait(
-      const std::string& url, bool available) {
+  void SimulateScreenAvailabilityChangeAndWait(const GURL& url,
+                                               bool available) {
     auto listener_it = service_impl_->screen_availability_listeners_.find(url);
     ASSERT_TRUE(listener_it->second);
 
     base::RunLoop run_loop;
-    EXPECT_CALL(mock_client_,
-                OnScreenAvailabilityUpdated(mojo::String(url), available))
-      .WillOnce(InvokeWithoutArgs(&run_loop, &base::RunLoop::Quit));
+    EXPECT_CALL(mock_client_, OnScreenAvailabilityUpdated(url, available))
+        .WillOnce(InvokeWithoutArgs(&run_loop, &base::RunLoop::Quit));
     listener_it->second->OnScreenAvailabilityChanged(available);
     run_loop.Run();
   }
@@ -273,9 +284,9 @@ class PresentationServiceImplTest : public RenderViewHostImplTestHarness {
   }
 
   void ExpectCleanState() {
-    EXPECT_TRUE(service_impl_->default_presentation_url_.empty());
+    EXPECT_TRUE(service_impl_->default_presentation_urls_.empty());
     EXPECT_EQ(
-        service_impl_->screen_availability_listeners_.find(kPresentationUrl),
+        service_impl_->screen_availability_listeners_.find(presentation_url1_),
         service_impl_->screen_availability_listeners_.end());
     EXPECT_FALSE(service_impl_->on_session_messages_callback_.get());
   }
@@ -299,13 +310,11 @@ class PresentationServiceImplTest : public RenderViewHostImplTestHarness {
   }
 
   void ExpectSessionMessages(
-      const mojo::Array<blink::mojom::SessionMessagePtr>& expected_msgs,
-      const mojo::Array<blink::mojom::SessionMessagePtr>& actual_msgs) {
+      const std::vector<blink::mojom::SessionMessagePtr>& expected_msgs,
+      const std::vector<blink::mojom::SessionMessagePtr>& actual_msgs) {
     EXPECT_EQ(expected_msgs.size(), actual_msgs.size());
-    for (size_t i = 0; i < actual_msgs.size(); ++i) {
-      EXPECT_TRUE(ArePresentationSessionMessagesEqual(expected_msgs[i].get(),
-                                                      actual_msgs[i].get()));
-    }
+    for (size_t i = 0; i < actual_msgs.size(); ++i)
+      EXPECT_TRUE(expected_msgs[i].Equals(actual_msgs[i]));
   }
 
   void ExpectSendSessionMessageCallback(bool success) {
@@ -318,18 +327,18 @@ class PresentationServiceImplTest : public RenderViewHostImplTestHarness {
   void RunListenForSessionMessages(const std::string& text_msg,
                                    const std::vector<uint8_t>& binary_data,
                                    bool pass_ownership) {
-    mojo::Array<blink::mojom::SessionMessagePtr> expected_msgs(2);
+    std::vector<blink::mojom::SessionMessagePtr> expected_msgs(2);
     expected_msgs[0] = blink::mojom::SessionMessage::New();
     expected_msgs[0]->type = blink::mojom::PresentationMessageType::TEXT;
     expected_msgs[0]->message = text_msg;
     expected_msgs[1] = blink::mojom::SessionMessage::New();
     expected_msgs[1]->type =
         blink::mojom::PresentationMessageType::ARRAY_BUFFER;
-    expected_msgs[1]->data = mojo::Array<uint8_t>::From(binary_data);
+    expected_msgs[1]->data = binary_data;
 
     blink::mojom::PresentationSessionInfoPtr session(
         blink::mojom::PresentationSessionInfo::New());
-    session->url = kPresentationUrl;
+    session->url = presentation_url1_;
     session->id = kPresentationId;
 
     PresentationSessionMessageCallback message_cb;
@@ -338,7 +347,7 @@ class PresentationServiceImplTest : public RenderViewHostImplTestHarness {
     EXPECT_CALL(mock_delegate_, ListenForSessionMessages(_, _, _, _))
         .WillOnce(DoAll(InvokeWithoutArgs(&run_loop, &base::RunLoop::Quit),
                         SaveArg<3>(&message_cb)));
-    service_ptr_->ListenForSessionMessages(session.Clone());
+    service_ptr_->ListenForSessionMessages(std::move(session));
     run_loop.Run();
     }
 
@@ -374,18 +383,23 @@ class PresentationServiceImplTest : public RenderViewHostImplTestHarness {
       client_binding_;
 
   base::Closure run_loop_quit_closure_;
+
+  GURL presentation_url1_;
+  GURL presentation_url2_;
+  GURL presentation_url3_;
+  std::vector<GURL> presentation_urls_;
 };
 
 TEST_F(PresentationServiceImplTest, ListenForScreenAvailability) {
-  ListenForScreenAvailabilityAndWait(kPresentationUrl, true);
+  ListenForScreenAvailabilityAndWait(presentation_url1_, true);
 
-  SimulateScreenAvailabilityChangeAndWait(kPresentationUrl, true);
-  SimulateScreenAvailabilityChangeAndWait(kPresentationUrl, false);
-  SimulateScreenAvailabilityChangeAndWait(kPresentationUrl, true);
+  SimulateScreenAvailabilityChangeAndWait(presentation_url1_, true);
+  SimulateScreenAvailabilityChangeAndWait(presentation_url1_, false);
+  SimulateScreenAvailabilityChangeAndWait(presentation_url1_, true);
 }
 
 TEST_F(PresentationServiceImplTest, Reset) {
-  ListenForScreenAvailabilityAndWait(kPresentationUrl, true);
+  ListenForScreenAvailabilityAndWait(presentation_url1_, true);
 
   ExpectReset();
   service_impl_->Reset();
@@ -393,7 +407,7 @@ TEST_F(PresentationServiceImplTest, Reset) {
 }
 
 TEST_F(PresentationServiceImplTest, DidNavigateThisFrame) {
-  ListenForScreenAvailabilityAndWait(kPresentationUrl, true);
+  ListenForScreenAvailabilityAndWait(presentation_url1_, true);
 
   ExpectReset();
   service_impl_->DidNavigateAnyFrame(
@@ -404,7 +418,7 @@ TEST_F(PresentationServiceImplTest, DidNavigateThisFrame) {
 }
 
 TEST_F(PresentationServiceImplTest, DidNavigateOtherFrame) {
-  ListenForScreenAvailabilityAndWait(kPresentationUrl, true);
+  ListenForScreenAvailabilityAndWait(presentation_url1_, true);
 
   // TODO(imcheng): How to get a different RenderFrameHost?
   service_impl_->DidNavigateAnyFrame(
@@ -414,11 +428,11 @@ TEST_F(PresentationServiceImplTest, DidNavigateOtherFrame) {
 
   // Availability is reported and callback is invoked since it was not
   // removed.
-  SimulateScreenAvailabilityChangeAndWait(kPresentationUrl, true);
+  SimulateScreenAvailabilityChangeAndWait(presentation_url1_, true);
 }
 
 TEST_F(PresentationServiceImplTest, ThisRenderFrameDeleted) {
-  ListenForScreenAvailabilityAndWait(kPresentationUrl, true);
+  ListenForScreenAvailabilityAndWait(presentation_url1_, true);
 
   ExpectReset();
 
@@ -429,65 +443,76 @@ TEST_F(PresentationServiceImplTest, ThisRenderFrameDeleted) {
 }
 
 TEST_F(PresentationServiceImplTest, OtherRenderFrameDeleted) {
-  ListenForScreenAvailabilityAndWait(kPresentationUrl, true);
+  ListenForScreenAvailabilityAndWait(presentation_url1_, true);
 
   // TODO(imcheng): How to get a different RenderFrameHost?
   service_impl_->RenderFrameDeleted(nullptr);
 
   // Availability is reported and callback should be invoked since listener
   // has not been deleted.
-  SimulateScreenAvailabilityChangeAndWait(kPresentationUrl, true);
+  SimulateScreenAvailabilityChangeAndWait(presentation_url1_, true);
 }
 
 TEST_F(PresentationServiceImplTest, DelegateFails) {
-  ListenForScreenAvailabilityAndWait(kPresentationUrl, false);
+  ListenForScreenAvailabilityAndWait(presentation_url1_, false);
   ASSERT_EQ(
-      service_impl_->screen_availability_listeners_.find(kPresentationUrl),
+      service_impl_->screen_availability_listeners_.find(presentation_url1_),
       service_impl_->screen_availability_listeners_.end());
 }
 
-TEST_F(PresentationServiceImplTest, SetDefaultPresentationUrl) {
-  std::string url1("http://fooUrl");
-  EXPECT_CALL(mock_delegate_, SetDefaultPresentationUrl(_, _, Eq(url1), _))
+TEST_F(PresentationServiceImplTest, SetDefaultPresentationUrls) {
+  EXPECT_CALL(mock_delegate_,
+              SetDefaultPresentationUrls(_, _, presentation_urls_, _))
       .Times(1);
-  service_impl_->SetDefaultPresentationURL(url1);
-  EXPECT_EQ(url1, service_impl_->default_presentation_url_);
 
-  std::string url2("http://barUrl");
-  // Sets different DPU.
+  service_impl_->SetDefaultPresentationUrls(presentation_urls_);
+
+  // Sets different DPUs.
+  std::vector<GURL> more_urls = presentation_urls_;
+  more_urls.push_back(presentation_url3_);
+
   content::PresentationSessionStartedCallback callback;
-  EXPECT_CALL(mock_delegate_, SetDefaultPresentationUrl(_, _, Eq(url2), _))
+  EXPECT_CALL(mock_delegate_, SetDefaultPresentationUrls(_, _, more_urls, _))
       .WillOnce(SaveArg<3>(&callback));
-  service_impl_->SetDefaultPresentationURL(url2);
-  EXPECT_EQ(url2, service_impl_->default_presentation_url_);
+  service_impl_->SetDefaultPresentationUrls(more_urls);
 
   blink::mojom::PresentationSessionInfo session_info;
-  session_info.url = url2;
+  session_info.url = presentation_url2_;
   session_info.id = kPresentationId;
+
   base::RunLoop run_loop;
-  EXPECT_CALL(mock_client_, OnDefaultSessionStarted(Equals(session_info)))
+  EXPECT_CALL(mock_client_,
+              OnDefaultSessionStarted(SessionInfoEquals(ByRef(session_info))))
       .WillOnce(InvokeWithoutArgs(&run_loop, &base::RunLoop::Quit));
-  callback.Run(content::PresentationSessionInfo(url2, kPresentationId));
+  EXPECT_CALL(mock_delegate_, ListenForConnectionStateChange(_, _, _, _));
+  callback.Run(
+      content::PresentationSessionInfo(presentation_url2_, kPresentationId));
   run_loop.Run();
 }
 
-TEST_F(PresentationServiceImplTest, ListenForConnectionStateChange) {
-  content::PresentationSessionInfo connection(kPresentationUrl,
+TEST_F(PresentationServiceImplTest,
+       ListenForConnectionStateChangeAndChangeState) {
+  content::PresentationSessionInfo connection(presentation_url1_,
                                               kPresentationId);
   content::PresentationConnectionStateChangedCallback state_changed_cb;
-  EXPECT_CALL(mock_delegate_, ListenForConnectionStateChange(_, _, _, _))
-      .WillOnce(SaveArg<3>(&state_changed_cb));
-  service_impl_->ListenForConnectionStateChange(connection);
-
   // Trigger state change. It should be propagated back up to |mock_client_|.
   blink::mojom::PresentationSessionInfo presentation_connection;
-  presentation_connection.url = kPresentationUrl;
+  presentation_connection.url = presentation_url1_;
   presentation_connection.id = kPresentationId;
+
+  EXPECT_CALL(mock_delegate_, ListenForConnectionStateChange(_, _, _, _))
+      .WillOnce(SaveArg<3>(&state_changed_cb));
+  EXPECT_CALL(mock_client_,
+              OnConnectionStateChanged(
+                  SessionInfoEquals(ByRef(presentation_connection)),
+                  blink::mojom::PresentationConnectionState::CONNECTED));
+  service_impl_->ListenForConnectionStateChangeAndChangeState(connection);
+
   {
     base::RunLoop run_loop;
     EXPECT_CALL(mock_client_,
                 OnConnectionStateChanged(
-                    Equals(presentation_connection),
+                    SessionInfoEquals(ByRef(presentation_connection)),
                     blink::mojom::PresentationConnectionState::TERMINATED))
         .WillOnce(InvokeWithoutArgs(&run_loop, &base::RunLoop::Quit));
     state_changed_cb.Run(PresentationConnectionStateChangeInfo(
@@ -497,17 +522,17 @@ TEST_F(PresentationServiceImplTest, ListenForConnectionStateChange) {
 }
 
 TEST_F(PresentationServiceImplTest, ListenForConnectionClose) {
-  content::PresentationSessionInfo connection(kPresentationUrl,
+  content::PresentationSessionInfo connection(presentation_url1_,
                                               kPresentationId);
   content::PresentationConnectionStateChangedCallback state_changed_cb;
   EXPECT_CALL(mock_delegate_, ListenForConnectionStateChange(_, _, _, _))
       .WillOnce(SaveArg<3>(&state_changed_cb));
-  service_impl_->ListenForConnectionStateChange(connection);
+  service_impl_->ListenForConnectionStateChangeAndChangeState(connection);
 
   // Trigger connection close. It should be propagated back up to
   // |mock_client_|.
   blink::mojom::PresentationSessionInfo presentation_connection;
-  presentation_connection.url = kPresentationUrl;
+  presentation_connection.url = presentation_url1_;
   presentation_connection.id = kPresentationId;
   {
     base::RunLoop run_loop;
@@ -516,63 +541,57 @@ TEST_F(PresentationServiceImplTest, ListenForConnectionClose) {
     closed_info.close_reason = PRESENTATION_CONNECTION_CLOSE_REASON_WENT_AWAY;
     closed_info.message = "Foo";
 
-    EXPECT_CALL(mock_client_,
-                OnConnectionClosed(
-                    Equals(presentation_connection),
-                    blink::mojom::PresentationConnectionCloseReason::WENT_AWAY,
-                    mojo::String("Foo")))
+    EXPECT_CALL(
+        mock_client_,
+        OnConnectionClosed(
+            SessionInfoEquals(ByRef(presentation_connection)),
+            blink::mojom::PresentationConnectionCloseReason::WENT_AWAY, "Foo"))
         .WillOnce(InvokeWithoutArgs(&run_loop, &base::RunLoop::Quit));
     state_changed_cb.Run(closed_info);
     run_loop.Run();
   }
 }
 
-TEST_F(PresentationServiceImplTest, SetSameDefaultPresentationUrl) {
+TEST_F(PresentationServiceImplTest, SetSameDefaultPresentationUrls) {
   EXPECT_CALL(mock_delegate_,
-              SetDefaultPresentationUrl(_, _, Eq(kPresentationUrl), _))
+              SetDefaultPresentationUrls(_, _, presentation_urls_, _))
       .Times(1);
-  service_impl_->SetDefaultPresentationURL(kPresentationUrl);
+  service_impl_->SetDefaultPresentationUrls(presentation_urls_);
   EXPECT_TRUE(Mock::VerifyAndClearExpectations(&mock_delegate_));
-  EXPECT_EQ(kPresentationUrl, service_impl_->default_presentation_url_);
 
-  // Same URL as before; no-ops.
-  service_impl_->SetDefaultPresentationURL(kPresentationUrl);
+  // Same URLs as before; no-ops.
+  service_impl_->SetDefaultPresentationUrls(presentation_urls_);
   EXPECT_TRUE(Mock::VerifyAndClearExpectations(&mock_delegate_));
-  EXPECT_EQ(kPresentationUrl, service_impl_->default_presentation_url_);
 }
 
 TEST_F(PresentationServiceImplTest, StartSessionSuccess) {
   service_ptr_->StartSession(
-      kPresentationUrl,
-      base::Bind(
-          &PresentationServiceImplTest::ExpectNewSessionCallbackSuccess,
-          base::Unretained(this)));
+      presentation_urls_,
+      base::Bind(&PresentationServiceImplTest::ExpectNewSessionCallbackSuccess,
+                 base::Unretained(this)));
   base::RunLoop run_loop;
   base::Callback<void(const PresentationSessionInfo&)> success_cb;
-  EXPECT_CALL(mock_delegate_, StartSession(_, _, Eq(kPresentationUrl), _, _))
-      .WillOnce(DoAll(
-            InvokeWithoutArgs(&run_loop, &base::RunLoop::Quit),
-            SaveArg<3>(&success_cb)));
+  EXPECT_CALL(mock_delegate_, StartSession(_, _, presentation_urls_, _, _))
+      .WillOnce(DoAll(InvokeWithoutArgs(&run_loop, &base::RunLoop::Quit),
+                      SaveArg<3>(&success_cb)));
   run_loop.Run();
 
   EXPECT_CALL(mock_delegate_, ListenForConnectionStateChange(_, _, _, _))
       .Times(1);
-  success_cb.Run(PresentationSessionInfo(kPresentationUrl, kPresentationId));
+  success_cb.Run(PresentationSessionInfo(presentation_url1_, kPresentationId));
   SaveQuitClosureAndRunLoop();
 }
 
 TEST_F(PresentationServiceImplTest, StartSessionError) {
   service_ptr_->StartSession(
-      kPresentationUrl,
-      base::Bind(
-          &PresentationServiceImplTest::ExpectNewSessionCallbackError,
-          base::Unretained(this)));
+      presentation_urls_,
+      base::Bind(&PresentationServiceImplTest::ExpectNewSessionCallbackError,
+                 base::Unretained(this)));
   base::RunLoop run_loop;
   base::Callback<void(const PresentationError&)> error_cb;
-  EXPECT_CALL(mock_delegate_, StartSession(_, _, Eq(kPresentationUrl), _, _))
-      .WillOnce(DoAll(
-            InvokeWithoutArgs(&run_loop, &base::RunLoop::Quit),
-            SaveArg<4>(&error_cb)));
+  EXPECT_CALL(mock_delegate_, StartSession(_, _, presentation_urls_, _, _))
+      .WillOnce(DoAll(InvokeWithoutArgs(&run_loop, &base::RunLoop::Quit),
+                      SaveArg<4>(&error_cb)));
   run_loop.Run();
   error_cb.Run(PresentationError(PRESENTATION_ERROR_UNKNOWN, "Error message"));
   SaveQuitClosureAndRunLoop();
@@ -580,47 +599,42 @@ TEST_F(PresentationServiceImplTest, StartSessionError) {
 
 TEST_F(PresentationServiceImplTest, JoinSessionSuccess) {
   service_ptr_->JoinSession(
-      kPresentationUrl,
-      kPresentationId,
-      base::Bind(
-          &PresentationServiceImplTest::ExpectNewSessionCallbackSuccess,
-          base::Unretained(this)));
+      presentation_urls_, base::Optional<std::string>(kPresentationId),
+      base::Bind(&PresentationServiceImplTest::ExpectNewSessionCallbackSuccess,
+                 base::Unretained(this)));
   base::RunLoop run_loop;
   base::Callback<void(const PresentationSessionInfo&)> success_cb;
-  EXPECT_CALL(mock_delegate_, JoinSession(
-      _, _, Eq(kPresentationUrl), Eq(kPresentationId), _, _))
-      .WillOnce(DoAll(
-            InvokeWithoutArgs(&run_loop, &base::RunLoop::Quit),
-            SaveArg<4>(&success_cb)));
+  EXPECT_CALL(mock_delegate_,
+              JoinSession(_, _, presentation_urls_, kPresentationId, _, _))
+      .WillOnce(DoAll(InvokeWithoutArgs(&run_loop, &base::RunLoop::Quit),
+                      SaveArg<4>(&success_cb)));
   run_loop.Run();
 
   EXPECT_CALL(mock_delegate_, ListenForConnectionStateChange(_, _, _, _))
       .Times(1);
-  success_cb.Run(PresentationSessionInfo(kPresentationUrl, kPresentationId));
+  success_cb.Run(PresentationSessionInfo(presentation_url1_, kPresentationId));
   SaveQuitClosureAndRunLoop();
 }
 
 TEST_F(PresentationServiceImplTest, JoinSessionError) {
   service_ptr_->JoinSession(
-      kPresentationUrl,
-      kPresentationId,
-      base::Bind(
-          &PresentationServiceImplTest::ExpectNewSessionCallbackError,
-          base::Unretained(this)));
+      presentation_urls_, base::Optional<std::string>(kPresentationId),
+      base::Bind(&PresentationServiceImplTest::ExpectNewSessionCallbackError,
+                 base::Unretained(this)));
   base::RunLoop run_loop;
   base::Callback<void(const PresentationError&)> error_cb;
-  EXPECT_CALL(mock_delegate_, JoinSession(
-      _, _, Eq(kPresentationUrl), Eq(kPresentationId), _, _))
-      .WillOnce(DoAll(
-            InvokeWithoutArgs(&run_loop, &base::RunLoop::Quit),
-            SaveArg<5>(&error_cb)));
+  EXPECT_CALL(mock_delegate_,
+              JoinSession(_, _, presentation_urls_, kPresentationId, _, _))
+      .WillOnce(DoAll(InvokeWithoutArgs(&run_loop, &base::RunLoop::Quit),
+                      SaveArg<5>(&error_cb)));
   run_loop.Run();
   error_cb.Run(PresentationError(PRESENTATION_ERROR_UNKNOWN, "Error message"));
   SaveQuitClosureAndRunLoop();
 }
 
 TEST_F(PresentationServiceImplTest, CloseConnection) {
-  service_ptr_->CloseConnection(kPresentationUrl, kPresentationId);
+  service_ptr_->CloseConnection(presentation_url1_, kPresentationId);
+
   base::RunLoop run_loop;
   EXPECT_CALL(mock_delegate_, CloseConnection(_, _, Eq(kPresentationId)))
       .WillOnce(InvokeWithoutArgs(&run_loop, &base::RunLoop::Quit));
@@ -628,7 +642,7 @@ TEST_F(PresentationServiceImplTest, CloseConnection) {
 }
 
 TEST_F(PresentationServiceImplTest, Terminate) {
-  service_ptr_->Terminate(kPresentationUrl, kPresentationId);
+  service_ptr_->Terminate(presentation_url1_, kPresentationId);
   base::RunLoop run_loop;
   EXPECT_CALL(mock_delegate_, Terminate(_, _, Eq(kPresentationId)))
       .WillOnce(InvokeWithoutArgs(&run_loop, &base::RunLoop::Quit));
@@ -654,20 +668,16 @@ TEST_F(PresentationServiceImplTest, ListenForSessionMessagesWithEmptyMsg) {
 }
 
 TEST_F(PresentationServiceImplTest, StartSessionInProgress) {
-  std::string presentation_url1("http://fooUrl");
-  std::string presentation_url2("http://barUrl");
-  EXPECT_CALL(mock_delegate_, StartSession(_, _, Eq(presentation_url1), _, _))
+  EXPECT_CALL(mock_delegate_, StartSession(_, _, presentation_urls_, _, _))
       .Times(1);
-  service_ptr_->StartSession(presentation_url1,
-                             base::Bind(&DoNothing));
+  service_ptr_->StartSession(presentation_urls_, base::Bind(&DoNothing));
 
   // This request should fail immediately, since there is already a StartSession
   // in progress.
   service_ptr_->StartSession(
-      presentation_url2,
-      base::Bind(
-          &PresentationServiceImplTest::ExpectNewSessionCallbackError,
-          base::Unretained(this)));
+      presentation_urls_,
+      base::Bind(&PresentationServiceImplTest::ExpectNewSessionCallbackError,
+                 base::Unretained(this)));
   SaveQuitClosureAndRunLoop();
 }
 
@@ -676,7 +686,7 @@ TEST_F(PresentationServiceImplTest, SendStringMessage) {
 
   blink::mojom::PresentationSessionInfoPtr session(
       blink::mojom::PresentationSessionInfo::New());
-  session->url = kPresentationUrl;
+  session->url = presentation_url1_;
   session->id = kPresentationId;
   blink::mojom::SessionMessagePtr message_request(
       blink::mojom::SessionMessage::New());
@@ -714,12 +724,12 @@ TEST_F(PresentationServiceImplTest, SendArrayBuffer) {
 
   blink::mojom::PresentationSessionInfoPtr session(
       blink::mojom::PresentationSessionInfo::New());
-  session->url = kPresentationUrl;
+  session->url = presentation_url1_;
   session->id = kPresentationId;
   blink::mojom::SessionMessagePtr message_request(
       blink::mojom::SessionMessage::New());
   message_request->type = blink::mojom::PresentationMessageType::ARRAY_BUFFER;
-  message_request->data = mojo::Array<uint8_t>::From(data);
+  message_request->data = data;
   service_ptr_->SendSessionMessage(
       std::move(session), std::move(message_request),
       base::Bind(&PresentationServiceImplTest::ExpectSendSessionMessageCallback,
@@ -758,12 +768,12 @@ TEST_F(PresentationServiceImplTest, SendArrayBufferWithExceedingLimit) {
 
   blink::mojom::PresentationSessionInfoPtr session(
       blink::mojom::PresentationSessionInfo::New());
-  session->url = kPresentationUrl;
+  session->url = presentation_url1_;
   session->id = kPresentationId;
   blink::mojom::SessionMessagePtr message_request(
       blink::mojom::SessionMessage::New());
   message_request->type = blink::mojom::PresentationMessageType::ARRAY_BUFFER;
-  message_request->data = mojo::Array<uint8_t>::From(data);
+  message_request->data = data;
   service_ptr_->SendSessionMessage(
       std::move(session), std::move(message_request),
       base::Bind(&PresentationServiceImplTest::ExpectSendSessionMessageCallback,
@@ -789,12 +799,12 @@ TEST_F(PresentationServiceImplTest, SendBlobData) {
 
   blink::mojom::PresentationSessionInfoPtr session(
       blink::mojom::PresentationSessionInfo::New());
-  session->url = kPresentationUrl;
+  session->url = presentation_url1_;
   session->id = kPresentationId;
   blink::mojom::SessionMessagePtr message_request(
       blink::mojom::SessionMessage::New());
   message_request->type = blink::mojom::PresentationMessageType::BLOB;
-  message_request->data = mojo::Array<uint8_t>::From(data);
+  message_request->data = data;
   service_ptr_->SendSessionMessage(
       std::move(session), std::move(message_request),
       base::Bind(&PresentationServiceImplTest::ExpectSendSessionMessageCallback,
@@ -830,19 +840,17 @@ TEST_F(PresentationServiceImplTest, MaxPendingJoinSessionRequests) {
   EXPECT_CALL(mock_delegate_, JoinSession(_, _, _, _, _, _))
       .Times(num_requests);
   for (; i < num_requests; ++i) {
-    service_ptr_->JoinSession(
-        base::StringPrintf(presentation_url, i),
-        base::StringPrintf(presentation_id, i),
-        base::Bind(&DoNothing));
+    std::vector<GURL> urls = {GURL(base::StringPrintf(presentation_url, i))};
+    service_ptr_->JoinSession(urls, base::StringPrintf(presentation_id, i),
+                              base::Bind(&DoNothing));
   }
 
+  std::vector<GURL> urls = {GURL(base::StringPrintf(presentation_url, i))};
   // Exceeded maximum queue size, should invoke mojo callback with error.
   service_ptr_->JoinSession(
-        base::StringPrintf(presentation_url, i),
-        base::StringPrintf(presentation_id, i),
-        base::Bind(
-            &PresentationServiceImplTest::ExpectNewSessionCallbackError,
-            base::Unretained(this)));
+      urls, base::StringPrintf(presentation_id, i),
+      base::Bind(&PresentationServiceImplTest::ExpectNewSessionCallbackError,
+                 base::Unretained(this)));
   SaveQuitClosureAndRunLoop();
 }
 
@@ -850,9 +858,9 @@ TEST_F(PresentationServiceImplTest, ScreenAvailabilityNotSupported) {
   mock_delegate_.set_screen_availability_listening_supported(false);
   base::RunLoop run_loop;
   EXPECT_CALL(mock_client_,
-              OnScreenAvailabilityNotSupported(Eq(kPresentationUrl)))
+              OnScreenAvailabilityNotSupported(presentation_url1_))
       .WillOnce(InvokeWithoutArgs(&run_loop, &base::RunLoop::Quit));
-  ListenForScreenAvailabilityAndWait(kPresentationUrl, false);
+  ListenForScreenAvailabilityAndWait(presentation_url1_, false);
   run_loop.Run();
 }
 
