@@ -7,7 +7,7 @@
 #include "base/memory/ptr_util.h"
 #include "content/browser/renderer_host/input/synthetic_mouse_pointer.h"
 #include "content/browser/renderer_host/input/synthetic_touch_pointer.h"
-#include "third_party/WebKit/public/web/WebInputEvent.h"
+#include "third_party/WebKit/public/platform/WebInputEvent.h"
 
 namespace content {
 
@@ -18,9 +18,9 @@ SyntheticPointer::~SyntheticPointer() {}
 std::unique_ptr<SyntheticPointer> SyntheticPointer::Create(
     SyntheticGestureParams::GestureSourceType gesture_source_type) {
   if (gesture_source_type == SyntheticGestureParams::TOUCH_INPUT) {
-    return base::WrapUnique(new SyntheticTouchPointer());
+    return base::MakeUnique<SyntheticTouchPointer>();
   } else if (gesture_source_type == SyntheticGestureParams::MOUSE_INPUT) {
-    return base::WrapUnique(new SyntheticMousePointer());
+    return base::MakeUnique<SyntheticMousePointer>();
   } else {
     NOTREACHED() << "Invalid gesture source type";
     return std::unique_ptr<SyntheticPointer>();
