@@ -5,11 +5,16 @@
 #include "content/browser/renderer_host/media/audio_sync_reader.h"
 
 #include <algorithm>
+#include <string>
+#include <utility>
 
 #include "base/command_line.h"
+#include "base/format_macros.h"
+#include "base/memory/ptr_util.h"
 #include "base/memory/shared_memory.h"
-#include "base/metrics/histogram.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/strings/stringprintf.h"
+#include "base/trace_event/trace_event.h"
 #include "build/build_config.h"
 #include "content/browser/renderer_host/media/media_stream_manager.h"
 #include "content/public/common/content_switches.h"
@@ -38,11 +43,16 @@ void LogAudioGlitchResult(AudioGlitchResult result) {
 
 namespace content {
 
-AudioSyncReader::AudioSyncReader(base::SharedMemory* shared_memory,
-                                 const media::AudioParameters& params)
-    : shared_memory_(shared_memory),
+AudioSyncReader::AudioSyncReader(
+    const media::AudioParameters& params,
+    std::unique_ptr<base::SharedMemory> shared_memory,
+    std::unique_ptr<base::CancelableSyncSocket> socket,
+    std::unique_ptr<base::CancelableSyncSocket> foreign_socket)
+    : shared_memory_(std::move(shared_memory)),
       mute_audio_(base::CommandLine::ForCurrentProcess()->HasSwitch(
           switches::kMuteAudio)),
+      socket_(std::move(socket)),
+      foreign_socket_(std::move(foreign_socket)),
       packet_size_(shared_memory_->requested_size()),
       renderer_callback_count_(0),
       renderer_missed_callback_count_(0),
@@ -98,13 +108,37 @@ AudioSyncReader::~AudioSyncReader() {
   renderer_missed_callback_count_ > 0 ?
       LogAudioGlitchResult(AUDIO_RENDERER_AUDIO_GLITCHES) :
       LogAudioGlitchResult(AUDIO_RENDERER_NO_AUDIO_GLITCHES);
-  std::string log_string =
-      base::StringPrintf("ASR: number of detected audio glitches=%d",
-                         static_cast<int>(renderer_missed_callback_count_));
+  std::string log_string = base::StringPrintf(
+      "ASR: number of detected audio glitches: %" PRIuS " out of %" PRIuS,
+      renderer_missed_callback_count_, renderer_callback_count_);
   MediaStreamManager::SendMessageToNativeLog(log_string);
   DVLOG(1) << log_string;
 }
 
+// static
+std::unique_ptr<AudioSyncReader> AudioSyncReader::Create(
+    const media::AudioParameters& params) {
+  base::CheckedNumeric<size_t> memory_size =
+      sizeof(media::AudioOutputBufferParameters);
+  memory_size += AudioBus::CalculateMemorySize(params);
+
+  std::unique_ptr<base::SharedMemory> shared_memory(new base::SharedMemory());
+  std::unique_ptr<base::CancelableSyncSocket> socket(
+      new base::CancelableSyncSocket());
+  std::unique_ptr<base::CancelableSyncSocket> foreign_socket(
+      new base::CancelableSyncSocket());
+
+  if (!memory_size.IsValid() ||
+      !shared_memory->CreateAndMapAnonymous(memory_size.ValueOrDie()) ||
+      !base::CancelableSyncSocket::CreatePair(socket.get(),
+                                              foreign_socket.get())) {
+    return nullptr;
+  }
+  return base::WrapUnique(new AudioSyncReader(params, std::move(shared_memory),
+                                              std::move(socket),
+                                              std::move(foreign_socket)));
+}
+
 // media::AudioOutputController::SyncReader implementations.
 void AudioSyncReader::UpdatePendingBytes(uint32_t bytes,
                                          uint32_t frames_skipped) {
@@ -150,20 +184,8 @@ void AudioSyncReader::Close() {
   socket_->Close();
 }
 
-bool AudioSyncReader::Init() {
-  socket_.reset(new base::CancelableSyncSocket());
-  foreign_socket_.reset(new base::CancelableSyncSocket());
-  return base::CancelableSyncSocket::CreatePair(socket_.get(),
-                                                foreign_socket_.get());
-}
-
-bool AudioSyncReader::PrepareForeignSocket(
-    base::ProcessHandle process_handle,
-    base::SyncSocket::TransitDescriptor* descriptor) {
-  return foreign_socket_->PrepareTransitDescriptor(process_handle, descriptor);
-}
-
 bool AudioSyncReader::WaitUntilDataIsReady() {
+  TRACE_EVENT0("audio", "AudioSyncReader::WaitUntilDataIsReady");
   base::TimeDelta timeout = maximum_wait_time_;
   const base::TimeTicks start_time = base::TimeTicks::Now();
   const base::TimeTicks finish_time = start_time + timeout;
@@ -201,7 +223,8 @@ bool AudioSyncReader::WaitUntilDataIsReady() {
   // Receive timed out or another error occurred.  Receive can timeout if the
   // renderer is unable to deliver audio data within the allotted time.
   if (!bytes_received || renderer_buffer_index != buffer_index_) {
-    DVLOG(2) << "AudioSyncReader::WaitUntilDataIsReady() timed out.";
+    TRACE_EVENT_INSTANT0("audio", "AudioSyncReader::Read timed out",
+                         TRACE_EVENT_SCOPE_THREAD);
 
     base::TimeDelta time_since_start = base::TimeTicks::Now() - start_time;
     UMA_HISTOGRAM_CUSTOM_TIMES("Media.AudioOutputControllerDataNotReady",
