@@ -7,12 +7,14 @@
 
 #include <stdint.h>
 
+#include <string>
 #include <tuple>
 #include <vector>
 
 #include "content/common/p2p_messages.h"
 #include "ipc/ipc_sender.h"
 #include "net/base/net_errors.h"
+#include "net/log/net_log_with_source.h"
 #include "net/socket/stream_socket.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -58,7 +60,7 @@ class FakeSocket : public net::StreamSocket {
   bool IsConnectedAndIdle() const override;
   int GetPeerAddress(net::IPEndPoint* address) const override;
   int GetLocalAddress(net::IPEndPoint* address) const override;
-  const net::BoundNetLog& NetLog() const override;
+  const net::NetLogWithSource& NetLog() const override;
   void SetSubresourceSpeculation() override;
   void SetOmniboxSpeculation() override;
   bool WasEverUsed() const override;
@@ -90,7 +92,7 @@ class FakeSocket : public net::StreamSocket {
   net::IPEndPoint peer_address_;
   net::IPEndPoint local_address_;
 
-  net::BoundNetLog net_log_;
+  net::NetLogWithSource net_log_;
 };
 
 void CreateRandomPacket(std::vector<char>* packet);
