@@ -9,6 +9,7 @@
 #include "base/macros.h"
 #include "base/strings/utf_string_conversions.h"
 #include "build/build_config.h"
+#include "content/browser/bad_message.h"
 #include "content/browser/dom_storage/dom_storage_context_wrapper.h"
 #include "content/browser/dom_storage/session_storage_namespace_impl.h"
 #include "content/browser/frame_host/navigator.h"
@@ -17,6 +18,7 @@
 #include "content/browser/renderer_host/render_view_host_impl.h"
 #include "content/browser/web_contents/web_contents_impl.h"
 #include "content/common/frame_messages.h"
+#include "content/common/render_message_filter.mojom.h"
 #include "content/common/resource_messages.h"
 #include "content/common/resource_request.h"
 #include "content/common/view_messages.h"
@@ -24,6 +26,7 @@
 #include "content/public/browser/content_browser_client.h"
 #include "content/public/browser/interstitial_page.h"
 #include "content/public/browser/interstitial_page_delegate.h"
+#include "content/public/browser/resource_context.h"
 #include "content/public/browser/resource_dispatcher_host.h"
 #include "content/public/browser/storage_partition.h"
 #include "content/public/common/appcache_info.h"
@@ -88,7 +91,8 @@ RenderViewHostImpl* PrepareToDuplicateHosts(Shell* shell,
   WebContentsImpl* wc = static_cast<WebContentsImpl*>(shell->web_contents());
   wc->GetFrameTree()->root()->navigator()->RequestOpenURL(
       wc->GetFrameTree()->root()->current_frame_host(), extension_url, false,
-      nullptr, nullptr, Referrer(), CURRENT_TAB, false, true);
+      nullptr, std::string(), Referrer(), WindowOpenDisposition::CURRENT_TAB,
+      false, true);
 
   // Since the navigation above requires a cross-process swap, there will be a
   // speculative/pending RenderFrameHost. Ensure it exists and is in a different
@@ -111,6 +115,7 @@ ResourceRequest CreateXHRRequest(const char* url) {
   request.method = "GET";
   request.url = GURL(url);
   request.referrer_policy = blink::WebReferrerPolicyDefault;
+  request.request_initiator = url::Origin();
   request.load_flags = 0;
   request.origin_pid = 0;
   request.resource_type = RESOURCE_TYPE_XHR;
@@ -256,7 +261,6 @@ IN_PROC_BROWSER_TEST_F(SecurityExploitBrowserTest,
   // Since this test executes on the UI thread and hopping threads might cause
   // different timing in the test, let's simulate a CreateNewWindow call coming
   // from the IO thread.
-  ViewHostMsg_CreateWindow_Params params;
   DOMStorageContextWrapper* dom_storage_context =
       static_cast<DOMStorageContextWrapper*>(
           BrowserContext::GetStoragePartition(
@@ -270,7 +274,8 @@ IN_PROC_BROWSER_TEST_F(SecurityExploitBrowserTest,
   // RenderViewHostImpl has-a RenderWidgetHostImpl. https://crbug.com/545684
   int32_t main_frame_widget_routing_id = duplicate_routing_id;
   pending_rvh->CreateNewWindow(duplicate_routing_id, main_frame_routing_id,
-                               main_frame_widget_routing_id, params,
+                               main_frame_widget_routing_id,
+                               mojom::CreateNewWindowParams(),
                                session_storage.get());
 
   // If the above operation doesn't cause a crash, the test has succeeded!
@@ -401,15 +406,15 @@ IN_PROC_BROWSER_TEST_F(SecurityExploitBrowserTest,
   ASSERT_EQ("\"okay2\"", interstitial->last_command());
 }
 
-class IsolatedAppContentBrowserClient : public TestContentBrowserClient {
- public:
-  bool IsIllegalOrigin(content::ResourceContext* resource_context,
-                       int child_process_id,
-                       const GURL& origin) override {
-    // Simulate a case where an app origin is not in an app process.
-    return true;
-  }
-};
+// Intercepts the HTTP origin header and on being invoked once it is found
+// aborts the requeest.
+void OnHttpHeaderReceived(const std::string& header,
+                          const std::string& value,
+                          int child_process_id,
+                          content::ResourceContext* resource_context,
+                          OnHeaderProcessedCallback callback) {
+  callback.Run(false, content::bad_message::RDH_ILLEGAL_ORIGIN);
+}
 
 // Renderer processes should not be able to spoof Origin HTTP headers.
 IN_PROC_BROWSER_TEST_F(SecurityExploitBrowserTest, InvalidOriginHeaders) {
@@ -445,10 +450,9 @@ IN_PROC_BROWSER_TEST_F(SecurityExploitBrowserTest, InvalidOriginHeaders) {
   // ResourceRequest IPC can't be created in a test outside content/.
   NavigateToURL(shell(), web_url);
   {
-    // Set up a ContentBrowserClient that simulates an app URL in a non-app
-    // process.
-    IsolatedAppContentBrowserClient app_client;
-    ContentBrowserClient* old_client = SetBrowserClientForTesting(&app_client);
+    content::ResourceDispatcherHost::Get()->RegisterInterceptor(
+        "Origin", "", base::Bind(&OnHttpHeaderReceived));
+
     RenderProcessHostWatcher web_process_killed(
         web_rfh->GetProcess(),
         RenderProcessHostWatcher::WATCH_FOR_PROCESS_EXIT);
@@ -458,7 +462,6 @@ IN_PROC_BROWSER_TEST_F(SecurityExploitBrowserTest, InvalidOriginHeaders) {
                                         kRequestIdNotPreviouslyUsed,
                                         embedder_isolated_origin_msg));
     web_process_killed.Wait();
-    SetBrowserClientForTesting(old_client);
   }
 
   // Web processes cannot make XHRs with invalid Origin headers.
@@ -528,7 +531,6 @@ IN_PROC_BROWSER_TEST_F(SecurityExploitBrowserTest, MismatchedOriginOnCommit) {
   // Create commit params with different origins in params.url and
   // params.origin.
   FrameHostMsg_DidCommitProvisionalLoad_Params params;
-  params.page_id = 0;
   params.nav_entry_id = 0;
   params.did_create_new_entry = false;
   params.url = url;
