@@ -11,6 +11,7 @@
 #include <map>
 #include <set>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include "base/bind.h"
@@ -48,39 +49,46 @@ struct ServiceWorkerRegistrationInfo;
 
 // This class provides an interface to store and retrieve ServiceWorker
 // registration data. The lifetime is equal to ServiceWorkerContextCore that is
-// an owner of this class.
+// an owner of this class. When a storage operation fails, this is marked as
+// disabled and all subsequent requests are aborted until the context core is
+// restarted.
 class CONTENT_EXPORT ServiceWorkerStorage
     : NON_EXPORTED_BASE(public ServiceWorkerVersion::Listener) {
  public:
   typedef std::vector<ServiceWorkerDatabase::ResourceRecord> ResourceList;
   typedef base::Callback<void(ServiceWorkerStatusCode status)> StatusCallback;
-  typedef base::Callback<void(ServiceWorkerStatusCode status,
-                              const scoped_refptr<ServiceWorkerRegistration>&
-                                  registration)> FindRegistrationCallback;
   typedef base::Callback<void(
+      ServiceWorkerStatusCode status,
+      scoped_refptr<ServiceWorkerRegistration> registration)>
+      FindRegistrationCallback;
+  typedef base::Callback<void(
+      ServiceWorkerStatusCode status,
       const std::vector<scoped_refptr<ServiceWorkerRegistration>>&
-          registrations)> GetRegistrationsCallback;
-  typedef base::Callback<void(const std::vector<ServiceWorkerRegistrationInfo>&
-                                  registrations)> GetRegistrationsInfosCallback;
-  typedef base::Callback<
-      void(const std::string& data, ServiceWorkerStatusCode status)>
-          GetUserDataCallback;
+          registrations)>
+      GetRegistrationsCallback;
+  typedef base::Callback<void(
+      ServiceWorkerStatusCode status,
+      const std::vector<ServiceWorkerRegistrationInfo>& registrations)>
+      GetRegistrationsInfosCallback;
+  using GetUserDataCallback =
+      base::Callback<void(const std::vector<std::string>& data,
+                          ServiceWorkerStatusCode status)>;
   typedef base::Callback<void(
       const std::vector<std::pair<int64_t, std::string>>& user_data,
       ServiceWorkerStatusCode status)> GetUserDataForAllRegistrationsCallback;
 
   ~ServiceWorkerStorage() override;
 
-  static scoped_ptr<ServiceWorkerStorage> Create(
+  static std::unique_ptr<ServiceWorkerStorage> Create(
       const base::FilePath& path,
       const base::WeakPtr<ServiceWorkerContextCore>& context,
-      scoped_ptr<ServiceWorkerDatabaseTaskManager> database_task_manager,
+      std::unique_ptr<ServiceWorkerDatabaseTaskManager> database_task_manager,
       const scoped_refptr<base::SingleThreadTaskRunner>& disk_cache_thread,
       storage::QuotaManagerProxy* quota_manager_proxy,
       storage::SpecialStoragePolicy* special_storage_policy);
 
   // Used for DeleteAndStartOver. Creates new storage based on |old_storage|.
-  static scoped_ptr<ServiceWorkerStorage> Create(
+  static std::unique_ptr<ServiceWorkerStorage> Create(
       const base::WeakPtr<ServiceWorkerContextCore>& context,
       ServiceWorkerStorage* old_storage);
 
@@ -142,12 +150,14 @@ class CONTENT_EXPORT ServiceWorkerStorage
                           const GURL& origin,
                           const StatusCallback& callback);
 
-  scoped_ptr<ServiceWorkerResponseReader> CreateResponseReader(
+  // Creates a resource accessor. Never returns nullptr but an accessor may be
+  // associated with the disabled disk cache if the storage is disabled.
+  std::unique_ptr<ServiceWorkerResponseReader> CreateResponseReader(
       int64_t resource_id);
-  scoped_ptr<ServiceWorkerResponseWriter> CreateResponseWriter(
-      int64_t resource_id);
-  scoped_ptr<ServiceWorkerResponseMetadataWriter> CreateResponseMetadataWriter(
+  std::unique_ptr<ServiceWorkerResponseWriter> CreateResponseWriter(
       int64_t resource_id);
+  std::unique_ptr<ServiceWorkerResponseMetadataWriter>
+  CreateResponseMetadataWriter(int64_t resource_id);
 
   // Adds |resource_id| to the set of resources that are in the disk cache
   // but not yet stored with a registration.
@@ -159,21 +169,24 @@ class CONTENT_EXPORT ServiceWorkerStorage
   void DoomUncommittedResources(const std::set<int64_t>& resource_ids);
 
   // Provide a storage mechanism to read/write arbitrary data associated with
-  // a registration. Each registration has its own key namespace. Stored data
-  // is deleted when the associated registraton is deleted.
+  // a registration. Each registration has its own key namespace.
+  // GetUserData responds OK only if all keys are found; otherwise NOT_FOUND,
+  // and the callback's data will be empty.
   void GetUserData(int64_t registration_id,
-                   const std::string& key,
+                   const std::vector<std::string>& keys,
                    const GetUserDataCallback& callback);
-  void StoreUserData(int64_t registration_id,
-                     const GURL& origin,
-                     const std::string& key,
-                     const std::string& data,
-                     const StatusCallback& callback);
+  // Stored data is deleted when the associated registraton is deleted.
+  void StoreUserData(
+      int64_t registration_id,
+      const GURL& origin,
+      const std::vector<std::pair<std::string, std::string>>& key_value_pairs,
+      const StatusCallback& callback);
+  // Responds OK if all are successfully deleted or not found in the database.
   void ClearUserData(int64_t registration_id,
-                     const std::string& key,
+                     const std::vector<std::string>& keys,
                      const StatusCallback& callback);
-  // Returns all registrations that have user data with a particular key, as
-  // well as that user data.
+  // Responds with all registrations that have user data with a particular key,
+  // as well as that user data.
   void GetUserDataForAllRegistrations(
       const std::string& key,
       const GetUserDataForAllRegistrationsCallback& callback);
@@ -211,14 +224,16 @@ class CONTENT_EXPORT ServiceWorkerStorage
       ServiceWorkerRegistration* registration);
 
   void Disable();
-  bool IsDisabled() const;
 
   // |resources| must already be on the purgeable list.
   void PurgeResources(const ResourceList& resources);
 
  private:
+  friend class ServiceWorkerDispatcherHostTest;
   friend class ServiceWorkerHandleTest;
   friend class ServiceWorkerStorageTest;
+  friend class ServiceWorkerStorageOriginTrialsTest;
+  friend class ServiceWorkerRegistrationTest;
   friend class ServiceWorkerResourceStorageTest;
   friend class ServiceWorkerControlleeRequestHandlerTest;
   friend class ServiceWorkerContextRequestHandlerTest;
@@ -267,6 +282,7 @@ class CONTENT_EXPORT ServiceWorkerStorage
     StatusCallback callback;
 
     DidDeleteRegistrationParams();
+    DidDeleteRegistrationParams(const DidDeleteRegistrationParams& other);
     ~DidDeleteRegistrationParams();
   };
 
@@ -283,7 +299,7 @@ class CONTENT_EXPORT ServiceWorkerStorage
   typedef std::vector<ServiceWorkerDatabase::RegistrationData> RegistrationList;
   typedef std::map<int64_t, scoped_refptr<ServiceWorkerRegistration>>
       RegistrationRefsById;
-  typedef base::Callback<void(scoped_ptr<InitialData> data,
+  typedef base::Callback<void(std::unique_ptr<InitialData> data,
                               ServiceWorkerDatabase::Status status)>
       InitializeCallback;
   typedef base::Callback<void(ServiceWorkerDatabase::Status status)>
@@ -302,9 +318,9 @@ class CONTENT_EXPORT ServiceWorkerStorage
       const ServiceWorkerDatabase::RegistrationData& data,
       const ResourceList& resources,
       ServiceWorkerDatabase::Status status)> FindInDBCallback;
-  typedef base::Callback<void(
-      const std::string& data,
-      ServiceWorkerDatabase::Status)> GetUserDataInDBCallback;
+  typedef base::Callback<void(const std::vector<std::string>& data,
+                              ServiceWorkerDatabase::Status)>
+      GetUserDataInDBCallback;
   typedef base::Callback<void(
       const std::vector<std::pair<int64_t, std::string>>& user_data,
       ServiceWorkerDatabase::Status)>
@@ -316,7 +332,7 @@ class CONTENT_EXPORT ServiceWorkerStorage
   ServiceWorkerStorage(
       const base::FilePath& path,
       base::WeakPtr<ServiceWorkerContextCore> context,
-      scoped_ptr<ServiceWorkerDatabaseTaskManager> database_task_manager,
+      std::unique_ptr<ServiceWorkerDatabaseTaskManager> database_task_manager,
       const scoped_refptr<base::SingleThreadTaskRunner>& disk_cache_thread,
       storage::QuotaManagerProxy* quota_manager_proxy,
       storage::SpecialStoragePolicy* special_storage_policy);
@@ -326,7 +342,7 @@ class CONTENT_EXPORT ServiceWorkerStorage
 
   bool LazyInitialize(
       const base::Closure& callback);
-  void DidReadInitialData(scoped_ptr<InitialData> data,
+  void DidReadInitialData(std::unique_ptr<InitialData> data,
                           ServiceWorkerDatabase::Status status);
   void DidFindRegistrationForDocument(
       const GURL& document_url,
@@ -346,15 +362,15 @@ class CONTENT_EXPORT ServiceWorkerStorage
       const ServiceWorkerDatabase::RegistrationData& data,
       const ResourceList& resources,
       ServiceWorkerDatabase::Status status);
-  void DidGetRegistrations(const GetRegistrationsCallback& callback,
-                           RegistrationList* registration_data_list,
-                           std::vector<ResourceList>* resources_list,
-                           const GURL& origin_filter,
-                           ServiceWorkerDatabase::Status status);
-  void DidGetRegistrationsInfos(const GetRegistrationsInfosCallback& callback,
-                                RegistrationList* registration_data_list,
-                                const GURL& origin_filter,
-                                ServiceWorkerDatabase::Status status);
+  void DidGetRegistrationsForOrigin(const GetRegistrationsCallback& callback,
+                                    RegistrationList* registration_data_list,
+                                    std::vector<ResourceList>* resources_list,
+                                    const GURL& origin_filter,
+                                    ServiceWorkerDatabase::Status status);
+  void DidGetAllRegistrationsInfos(
+      const GetRegistrationsInfosCallback& callback,
+      RegistrationList* registration_data_list,
+      ServiceWorkerDatabase::Status status);
   void DidStoreRegistration(
       const StatusCallback& callback,
       const ServiceWorkerDatabase::RegistrationData& new_version,
@@ -377,10 +393,9 @@ class CONTENT_EXPORT ServiceWorkerStorage
   void DidStoreUserData(
       const StatusCallback& callback,
       ServiceWorkerDatabase::Status status);
-  void DidGetUserData(
-      const GetUserDataCallback& callback,
-      const std::string& data,
-      ServiceWorkerDatabase::Status status);
+  void DidGetUserData(const GetUserDataCallback& callback,
+                      const std::vector<std::string>& data,
+                      ServiceWorkerDatabase::Status status);
   void DidDeleteUserData(
       const StatusCallback& callback,
       ServiceWorkerDatabase::Status status);
@@ -470,7 +485,7 @@ class CONTENT_EXPORT ServiceWorkerStorage
       ServiceWorkerDatabase* database,
       scoped_refptr<base::SequencedTaskRunner> original_task_runner,
       int64_t registration_id,
-      const std::string& key,
+      const std::vector<std::string>& keys,
       const GetUserDataInDBCallback& callback);
   static void GetUserDataForAllRegistrationsInDB(
       ServiceWorkerDatabase* database,
@@ -481,6 +496,7 @@ class CONTENT_EXPORT ServiceWorkerStorage
       ServiceWorkerDatabase* database,
       const std::set<GURL>& origins);
 
+  bool IsDisabled() const;
   void ScheduleDeleteAndStartOver();
   void DidDeleteDatabase(
       const StatusCallback& callback,
@@ -518,14 +534,14 @@ class CONTENT_EXPORT ServiceWorkerStorage
   base::WeakPtr<ServiceWorkerContextCore> context_;
 
   // Only accessed using |database_task_manager_|.
-  scoped_ptr<ServiceWorkerDatabase> database_;
+  std::unique_ptr<ServiceWorkerDatabase> database_;
 
-  scoped_ptr<ServiceWorkerDatabaseTaskManager> database_task_manager_;
+  std::unique_ptr<ServiceWorkerDatabaseTaskManager> database_task_manager_;
   scoped_refptr<base::SingleThreadTaskRunner> disk_cache_thread_;
   scoped_refptr<storage::QuotaManagerProxy> quota_manager_proxy_;
   scoped_refptr<storage::SpecialStoragePolicy> special_storage_policy_;
 
-  scoped_ptr<ServiceWorkerDiskCache> disk_cache_;
+  std::unique_ptr<ServiceWorkerDiskCache> disk_cache_;
 
   std::deque<int64_t> purgeable_resource_ids_;
   bool is_purge_pending_;
