@@ -18,6 +18,8 @@
 #include "base/memory/ptr_util.h"
 #include "base/message_loop/message_loop.h"
 #include "base/run_loop.h"
+#include "base/single_thread_task_runner.h"
+#include "base/strings/string_split.h"
 #include "base/task_runner.h"
 #include "base/threading/thread.h"
 #include "base/threading/thread_task_runner_handle.h"
@@ -81,7 +83,12 @@ class LoggingFixedReceivedData final : public RequestPeer::ReceivedData {
     return data_.empty() ? nullptr : &data_[0];
   }
   int length() const override { return static_cast<int>(data_.size()); }
-  int encoded_length() const override { return static_cast<int>(data_.size()); }
+  int encoded_data_length() const override {
+    return static_cast<int>(data_.size());
+  }
+  int encoded_body_length() const override {
+    return static_cast<int>(data_.size());
+  }
 
  private:
   const std::string name_;
@@ -140,7 +147,7 @@ class ThreadedSharedMemoryDataConsumerHandleTest : public ::testing::Test {
     void ReadData() {
       if (!client_) {
         client_.reset(new ClientImpl(this));
-        reader_ = handle_->ObtainReader(client_.get());
+        reader_ = handle_->obtainReader(client_.get());
       }
 
       Result rv = kOk;
@@ -166,7 +173,7 @@ class ThreadedSharedMemoryDataConsumerHandleTest : public ::testing::Test {
 
       // The operation is done.
       reader_.reset();
-      main_message_loop_->PostTask(FROM_HERE, on_done_);
+      main_message_loop_->task_runner()->PostTask(FROM_HERE, on_done_);
     }
 
    private:
@@ -196,7 +203,8 @@ class SharedMemoryDataConsumerHandleTest
     handle_.reset(new SharedMemoryDataConsumerHandle(GetParam(), &writer_));
   }
   std::unique_ptr<FixedReceivedData> NewFixedData(const char* s) {
-    return base::WrapUnique(new FixedReceivedData(s, strlen(s), strlen(s)));
+    auto size = strlen(s);
+    return base::MakeUnique<FixedReceivedData>(s, size, size, size);
   }
 
   StrictMock<MockClient> client_;
@@ -215,7 +223,7 @@ void RunPostedTasks() {
 TEST_P(SharedMemoryDataConsumerHandleTest, ReadFromEmpty) {
   char buffer[4];
   size_t read = 88;
-  auto reader = handle_->ObtainReader(nullptr);
+  auto reader = handle_->obtainReader(nullptr);
   Result result = reader->read(buffer, 4, kNone, &read);
 
   EXPECT_EQ(kShouldWait, result);
@@ -227,7 +235,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, AutoClose) {
   size_t read = 88;
 
   writer_.reset();
-  auto reader = handle_->ObtainReader(nullptr);
+  auto reader = handle_->obtainReader(nullptr);
   Result result = reader->read(buffer, 4, kNone, &read);
 
   EXPECT_EQ(kDone, result);
@@ -239,7 +247,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, ReadSimple) {
 
   char buffer[4] = {};
   size_t read = 88;
-  auto reader = handle_->ObtainReader(nullptr);
+  auto reader = handle_->obtainReader(nullptr);
   Result result = reader->read(buffer, 3, kNone, &read);
 
   EXPECT_EQ(kOk, result);
@@ -267,7 +275,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, ReadAfterHandleIsGone) {
 
   char buffer[8] = {};
   size_t read = 88;
-  auto reader = handle_->ObtainReader(nullptr);
+  auto reader = handle_->obtainReader(nullptr);
 
   handle_.reset();
 
@@ -293,7 +301,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, ReObtainReader) {
 
   char buffer[4] = {};
   size_t read = 88;
-  auto reader = handle_->ObtainReader(nullptr);
+  auto reader = handle_->obtainReader(nullptr);
   Result result = reader->read(buffer, 3, kNone, &read);
 
   EXPECT_EQ(kOk, result);
@@ -301,7 +309,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, ReObtainReader) {
   EXPECT_STREQ("hel", buffer);
 
   reader.reset();
-  reader = handle_->ObtainReader(nullptr);
+  reader = handle_->obtainReader(nullptr);
 
   result = reader->read(buffer, 3, kNone, &read);
   EXPECT_EQ(kOk, result);
@@ -325,7 +333,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, CloseBeforeReading) {
 
   char buffer[20] = {};
   size_t read = 88;
-  auto reader = handle_->ObtainReader(nullptr);
+  auto reader = handle_->obtainReader(nullptr);
   Result result = reader->read(buffer, sizeof(buffer), kNone, &read);
 
   EXPECT_EQ(kOk, result);
@@ -342,7 +350,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, CloseWithDataBeforeZeroRead) {
   writer_->Close();
 
   size_t read = 88;
-  auto reader = handle_->ObtainReader(nullptr);
+  auto reader = handle_->obtainReader(nullptr);
   Result result = reader->read(nullptr, 0, kNone, &read);
 
   EXPECT_EQ(kOk, result);
@@ -353,7 +361,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, CloseWithoutDataBeforeZeroRead) {
   writer_->Close();
 
   size_t read = 88;
-  auto reader = handle_->ObtainReader(nullptr);
+  auto reader = handle_->obtainReader(nullptr);
   Result result = reader->read(nullptr, 0, kNone, &read);
 
   EXPECT_EQ(kDone, result);
@@ -373,7 +381,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, AddMultipleData) {
   size_t read;
   Result result;
 
-  auto reader = handle_->ObtainReader(nullptr);
+  auto reader = handle_->obtainReader(nullptr);
   std::fill(&buffer[0], &buffer[arraysize(buffer)], 0);
   result = reader->read(buffer, 6, kNone, &read);
   EXPECT_EQ(kOk, result);
@@ -417,7 +425,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, AddMultipleDataInteractively) {
   size_t read;
   Result result;
 
-  auto reader = handle_->ObtainReader(nullptr);
+  auto reader = handle_->obtainReader(nullptr);
   std::fill(&buffer[0], &buffer[arraysize(buffer)], 0);
   result = reader->read(buffer, 6, kNone, &read);
   EXPECT_EQ(kOk, result);
@@ -479,7 +487,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, RegisterClient) {
   EXPECT_CALL(checkpoint, Call(4));
 
   checkpoint.Call(0);
-  auto reader = handle_->ObtainReader(&client_);
+  auto reader = handle_->obtainReader(&client_);
   checkpoint.Call(1);
   RunPostedTasks();
   checkpoint.Call(2);
@@ -502,7 +510,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, RegisterClientWhenDataExists) {
   checkpoint.Call(0);
   writer_->AddData(NewFixedData("Once "));
   checkpoint.Call(1);
-  auto reader = handle_->ObtainReader(&client_);
+  auto reader = handle_->obtainReader(&client_);
   checkpoint.Call(2);
   RunPostedTasks();
   checkpoint.Call(3);
@@ -525,7 +533,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, AddDataWhenClientIsRegistered) {
   EXPECT_CALL(checkpoint, Call(5));
 
   checkpoint.Call(0);
-  auto reader = handle_->ObtainReader(&client_);
+  auto reader = handle_->obtainReader(&client_);
   checkpoint.Call(1);
   writer_->AddData(NewFixedData("Once "));
   checkpoint.Call(2);
@@ -550,7 +558,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, CloseWithClientAndData) {
   EXPECT_CALL(checkpoint, Call(3));
 
   checkpoint.Call(0);
-  auto reader = handle_->ObtainReader(&client_);
+  auto reader = handle_->obtainReader(&client_);
   checkpoint.Call(1);
   writer_->AddData(NewFixedData("Once "));
   checkpoint.Call(2);
@@ -567,7 +575,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, ReleaseReader) {
   EXPECT_CALL(checkpoint, Call(2));
 
   checkpoint.Call(0);
-  auto reader = handle_->ObtainReader(&client_);
+  auto reader = handle_->obtainReader(&client_);
   checkpoint.Call(1);
   reader.reset();
   writer_->AddData(NewFixedData("Once "));
@@ -579,7 +587,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, TwoPhaseReadShouldWait) {
   const void* buffer = &result;
   size_t size = 99;
 
-  auto reader = handle_->ObtainReader(nullptr);
+  auto reader = handle_->obtainReader(nullptr);
   result = reader->beginRead(&buffer, kNone, &size);
   EXPECT_EQ(kShouldWait, result);
   EXPECT_EQ(nullptr, buffer);
@@ -593,7 +601,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, TwoPhaseReadSimple) {
   const void* buffer = &result;
   size_t size = 99;
 
-  auto reader = handle_->ObtainReader(nullptr);
+  auto reader = handle_->obtainReader(nullptr);
   result = reader->beginRead(&buffer, kNone, &size);
   EXPECT_EQ(kOk, result);
   EXPECT_EQ(5u, size);
@@ -664,7 +672,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, CallOnClearWhenDestructed2) {
   handle_.reset(new SharedMemoryDataConsumerHandle(
       kApplyBackpressure,
       base::Bind(&DestructionTrackingFunction::Call, on_clear), &writer_));
-  auto reader = handle_->ObtainReader(nullptr);
+  auto reader = handle_->obtainReader(nullptr);
   handle_.reset();
   on_clear = nullptr;
   checkpoint.Call(1);
@@ -738,7 +746,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, TwoPhaseReadWithMultipleData) {
   const void* buffer = &result;
   size_t size = 99;
 
-  auto reader = handle_->ObtainReader(nullptr);
+  auto reader = handle_->obtainReader(nullptr);
   result = reader->beginRead(&buffer, kNone, &size);
   EXPECT_EQ(kOk, result);
   EXPECT_EQ(5u, size);
@@ -778,7 +786,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, ErrorRead) {
   Result result;
   char buffer[20] = {};
   size_t read = 99;
-  auto reader = handle_->ObtainReader(nullptr);
+  auto reader = handle_->obtainReader(nullptr);
 
   writer_->Fail();
   result = reader->read(buffer, sizeof(buffer), kNone, &read);
@@ -791,7 +799,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, ErrorTwoPhaseRead) {
   Result result;
   const void* pointer = &result;
   size_t size = 99;
-  auto reader = handle_->ObtainReader(nullptr);
+  auto reader = handle_->obtainReader(nullptr);
 
   writer_->Fail();
   result = reader->beginRead(&pointer, kNone, &size);
@@ -805,11 +813,11 @@ TEST_P(SharedMemoryDataConsumerHandleTest, FailWhileTwoPhaseReadIsInProgress) {
   Result result;
   const void* pointer = nullptr;
   size_t size = 0;
-  auto reader = handle_->ObtainReader(nullptr);
+  auto reader = handle_->obtainReader(nullptr);
 
   writer_->AddData(NewFixedData("Once "));
   result = reader->beginRead(&pointer, kNone, &size);
-  auto buffer = static_cast<const char*>(pointer);
+  auto* buffer = static_cast<const char*>(pointer);
 
   ASSERT_EQ(kOk, result);
   ASSERT_NE(nullptr, pointer);
@@ -841,7 +849,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, FailWithClient) {
   EXPECT_CALL(checkpoint, Call(3));
 
   checkpoint.Call(0);
-  auto reader = handle_->ObtainReader(&client_);
+  auto reader = handle_->obtainReader(&client_);
   checkpoint.Call(1);
   writer_->Fail();
   checkpoint.Call(2);
@@ -862,7 +870,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, FailWithClientAndData) {
   EXPECT_CALL(checkpoint, Call(4));
 
   checkpoint.Call(0);
-  auto reader = handle_->ObtainReader(&client_);
+  auto reader = handle_->obtainReader(&client_);
   checkpoint.Call(1);
   writer_->AddData(NewFixedData("Once "));
   checkpoint.Call(2);
@@ -885,7 +893,7 @@ TEST_P(SharedMemoryDataConsumerHandleTest, RecursiveErrorNotification) {
   EXPECT_CALL(checkpoint, Call(3));
 
   checkpoint.Call(0);
-  auto reader = handle_->ObtainReader(&client_);
+  auto reader = handle_->obtainReader(&client_);
   checkpoint.Call(1);
   writer_->AddData(NewFixedData("Once "));
   checkpoint.Call(2);
@@ -900,19 +908,19 @@ TEST(SharedMemoryDataConsumerHandleBackpressureTest, Read) {
   size_t size;
 
   std::unique_ptr<Writer> writer;
-  auto handle = base::WrapUnique(
-      new SharedMemoryDataConsumerHandle(kApplyBackpressure, &writer));
+  auto handle = base::MakeUnique<SharedMemoryDataConsumerHandle>(
+      kApplyBackpressure, &writer);
   scoped_refptr<Logger> logger(new Logger);
   writer->AddData(
-      base::WrapUnique(new LoggingFixedReceivedData("data1", "Once ", logger)));
+      base::MakeUnique<LoggingFixedReceivedData>("data1", "Once ", logger));
   writer->AddData(
-      base::WrapUnique(new LoggingFixedReceivedData("data2", "upon ", logger)));
+      base::MakeUnique<LoggingFixedReceivedData>("data2", "upon ", logger));
   writer->AddData(
-      base::WrapUnique(new LoggingFixedReceivedData("data3", "a ", logger)));
+      base::MakeUnique<LoggingFixedReceivedData>("data3", "a ", logger));
   writer->AddData(
-      base::WrapUnique(new LoggingFixedReceivedData("data4", "time ", logger)));
+      base::MakeUnique<LoggingFixedReceivedData>("data4", "time ", logger));
 
-  auto reader = handle->ObtainReader(nullptr);
+  auto reader = handle->obtainReader(nullptr);
   logger->Add("1");
   result = reader->read(buffer, 2, kNone, &size);
   EXPECT_EQ(kOk, result);
@@ -945,17 +953,17 @@ TEST(SharedMemoryDataConsumerHandleBackpressureTest, CloseAndReset) {
   size_t size;
 
   std::unique_ptr<Writer> writer;
-  auto handle = base::WrapUnique(
-      new SharedMemoryDataConsumerHandle(kApplyBackpressure, &writer));
+  auto handle = base::MakeUnique<SharedMemoryDataConsumerHandle>(
+      kApplyBackpressure, &writer);
   scoped_refptr<Logger> logger(new Logger);
   writer->AddData(
-      base::WrapUnique(new LoggingFixedReceivedData("data1", "Once ", logger)));
+      base::MakeUnique<LoggingFixedReceivedData>("data1", "Once ", logger));
   writer->AddData(
-      base::WrapUnique(new LoggingFixedReceivedData("data2", "upon ", logger)));
+      base::MakeUnique<LoggingFixedReceivedData>("data2", "upon ", logger));
   writer->AddData(
-      base::WrapUnique(new LoggingFixedReceivedData("data3", "a ", logger)));
+      base::MakeUnique<LoggingFixedReceivedData>("data3", "a ", logger));
 
-  auto reader = handle->ObtainReader(nullptr);
+  auto reader = handle->obtainReader(nullptr);
   logger->Add("1");
   result = reader->read(buffer, 2, kNone, &size);
   EXPECT_EQ(kOk, result);
@@ -967,30 +975,37 @@ TEST(SharedMemoryDataConsumerHandleBackpressureTest, CloseAndReset) {
   reader.reset();
   logger->Add("4");
 
-  EXPECT_EQ(
-      "1\n"
-      "2\n"
-      "3\n"
-      "data1 is destructed.\n"
-      "data2 is destructed.\n"
-      "data3 is destructed.\n"
-      "4\n",
-      logger->log());
+  std::vector<std::string> log = base::SplitString(
+      logger->log(), "\n", base::KEEP_WHITESPACE, base::SPLIT_WANT_ALL);
+
+  ASSERT_EQ(8u, log.size());
+  EXPECT_EQ("1", log[0]);
+  EXPECT_EQ("2", log[1]);
+  EXPECT_EQ("3", log[2]);
+  EXPECT_EQ("4", log[6]);
+  EXPECT_EQ("", log[7]);
+
+  // The destruction order doesn't matter in this case.
+  std::vector<std::string> destruction_entries = {log[3], log[4], log[5]};
+  std::sort(destruction_entries.begin(), destruction_entries.end());
+  EXPECT_EQ(destruction_entries[0], "data1 is destructed.");
+  EXPECT_EQ(destruction_entries[1], "data2 is destructed.");
+  EXPECT_EQ(destruction_entries[2], "data3 is destructed.");
 }
 
 TEST(SharedMemoryDataConsumerHandleWithoutBackpressureTest, AddData) {
   base::MessageLoop loop;
   std::unique_ptr<Writer> writer;
-  auto handle = base::WrapUnique(
-      new SharedMemoryDataConsumerHandle(kDoNotApplyBackpressure, &writer));
+  auto handle = base::MakeUnique<SharedMemoryDataConsumerHandle>(
+      kDoNotApplyBackpressure, &writer);
   scoped_refptr<Logger> logger(new Logger);
 
   logger->Add("1");
   writer->AddData(
-      base::WrapUnique(new LoggingFixedReceivedData("data1", "Once ", logger)));
+      base::MakeUnique<LoggingFixedReceivedData>("data1", "Once ", logger));
   logger->Add("2");
   writer->AddData(
-      base::WrapUnique(new LoggingFixedReceivedData("data2", "upon ", logger)));
+      base::MakeUnique<LoggingFixedReceivedData>("data2", "upon ", logger));
   logger->Add("3");
 
   EXPECT_EQ(
@@ -1004,28 +1019,28 @@ TEST(SharedMemoryDataConsumerHandleWithoutBackpressureTest, AddData) {
 
 TEST_F(ThreadedSharedMemoryDataConsumerHandleTest, Read) {
   base::RunLoop run_loop;
-  auto operation = base::WrapUnique(new ReadDataOperation(
-      std::move(handle_), &loop_, run_loop.QuitClosure()));
+  auto operation = base::MakeUnique<ReadDataOperation>(
+      std::move(handle_), &loop_, run_loop.QuitClosure());
   scoped_refptr<Logger> logger(new Logger);
 
   base::Thread t("DataConsumerHandle test thread");
   ASSERT_TRUE(t.Start());
 
-  t.message_loop()->PostTask(FROM_HERE,
-                             base::Bind(&ReadDataOperation::ReadData,
-                                        base::Unretained(operation.get())));
+  t.task_runner()->PostTask(FROM_HERE,
+                            base::Bind(&ReadDataOperation::ReadData,
+                                       base::Unretained(operation.get())));
 
   logger->Add("1");
   writer_->AddData(
-      base::WrapUnique(new LoggingFixedReceivedData("data1", "Once ", logger)));
+      base::MakeUnique<LoggingFixedReceivedData>("data1", "Once ", logger));
   writer_->AddData(
-      base::WrapUnique(new LoggingFixedReceivedData("data2", "upon ", logger)));
-  writer_->AddData(base::WrapUnique(
-      new LoggingFixedReceivedData("data3", "a time ", logger)));
-  writer_->AddData(base::WrapUnique(
-      new LoggingFixedReceivedData("data4", "there ", logger)));
+      base::MakeUnique<LoggingFixedReceivedData>("data2", "upon ", logger));
   writer_->AddData(
-      base::WrapUnique(new LoggingFixedReceivedData("data5", "was ", logger)));
+      base::MakeUnique<LoggingFixedReceivedData>("data3", "a time ", logger));
+  writer_->AddData(
+      base::MakeUnique<LoggingFixedReceivedData>("data4", "there ", logger));
+  writer_->AddData(
+      base::MakeUnique<LoggingFixedReceivedData>("data5", "was ", logger));
   writer_->Close();
   logger->Add("2");
 
