@@ -79,7 +79,7 @@ class ReadDataOperation : public ReadDataOperationBase {
   void ReadData() {
     if (!client_) {
       client_.reset(new ClientImpl(this));
-      reader_ = handle_->ObtainReader(client_.get());
+      reader_ = handle_->obtainReader(client_.get());
     }
 
     Result rv = kOk;
@@ -137,7 +137,7 @@ class TwoPhaseReadDataOperation : public ReadDataOperationBase {
   void ReadData() {
     if (!client_) {
       client_.reset(new ClientImpl(this));
-      reader_ = handle_->ObtainReader(client_.get());
+      reader_ = handle_->obtainReader(client_.get());
     }
 
     Result rv;
@@ -193,12 +193,15 @@ class WebDataConsumerHandleImplTest : public ::testing::Test {
     options.struct_size = sizeof(MojoCreateDataPipeOptions);
     options.flags = MOJO_CREATE_DATA_PIPE_OPTIONS_FLAG_NONE;
     options.element_num_bytes = 1;
-    options.capacity_num_bytes = 4;
+    options.capacity_num_bytes = kDataPipeCapacity;
 
     MojoResult result = mojo::CreateDataPipe(&options, &producer_, &consumer_);
     ASSERT_EQ(MOJO_RESULT_OK, result);
   }
 
+ protected:
+  static constexpr int kDataPipeCapacity = 4;
+
   // This function can be blocked if the associated consumer doesn't consume
   // the data.
   std::string ProduceData(size_t total_size) {
@@ -236,8 +239,8 @@ class WebDataConsumerHandleImplTest : public ::testing::Test {
 
 TEST_F(WebDataConsumerHandleImplTest, ReadData) {
   base::RunLoop run_loop;
-  auto operation = base::WrapUnique(new ReadDataOperation(
-      std::move(consumer_), &message_loop_, run_loop.QuitClosure()));
+  auto operation = base::MakeUnique<ReadDataOperation>(
+      std::move(consumer_), &message_loop_, run_loop.QuitClosure());
 
   base::Thread t("DataConsumerHandle test thread");
   ASSERT_TRUE(t.Start());
@@ -257,8 +260,8 @@ TEST_F(WebDataConsumerHandleImplTest, ReadData) {
 
 TEST_F(WebDataConsumerHandleImplTest, TwoPhaseReadData) {
   base::RunLoop run_loop;
-  auto operation = base::WrapUnique(new TwoPhaseReadDataOperation(
-      std::move(consumer_), &message_loop_, run_loop.QuitClosure()));
+  auto operation = base::MakeUnique<TwoPhaseReadDataOperation>(
+      std::move(consumer_), &message_loop_, run_loop.QuitClosure());
 
   base::Thread t("DataConsumerHandle test thread");
   ASSERT_TRUE(t.Start());
@@ -276,6 +279,29 @@ TEST_F(WebDataConsumerHandleImplTest, TwoPhaseReadData) {
   EXPECT_EQ(expected, operation->result());
 }
 
+TEST_F(WebDataConsumerHandleImplTest, ZeroSizeRead) {
+  ASSERT_GT(kDataPipeCapacity - 1, 0);
+  constexpr size_t data_size = kDataPipeCapacity - 1;
+  std::string expected = ProduceData(data_size);
+  producer_.reset();
+  std::unique_ptr<WebDataConsumerHandleImpl> handle(
+      new WebDataConsumerHandleImpl(std::move(consumer_)));
+  std::unique_ptr<WebDataConsumerHandle::Reader> reader(
+      handle->obtainReader(nullptr));
+
+  size_t read_size;
+  WebDataConsumerHandle::Result rv =
+      reader->read(nullptr, 0, WebDataConsumerHandle::FlagNone, &read_size);
+  EXPECT_EQ(WebDataConsumerHandle::Result::Ok, rv);
+
+  char buffer[16];
+  rv = reader->read(&buffer, sizeof(buffer), WebDataConsumerHandle::FlagNone,
+                    &read_size);
+  EXPECT_EQ(WebDataConsumerHandle::Result::Ok, rv);
+  EXPECT_EQ(data_size, read_size);
+  EXPECT_EQ(expected, std::string(buffer, read_size));
+}
+
 }  // namespace
 
 }  // namespace content
