@@ -15,15 +15,13 @@
 #include "base/memory/ptr_util.h"
 #include "base/memory/weak_ptr.h"
 #include "base/message_loop/message_loop.h"
+#include "base/run_loop.h"
 #include "base/single_thread_task_runner.h"
 #include "base/time/default_tick_clock.h"
 #include "base/time/time.h"
-#include "components/scheduler/child/scheduler_tqm_delegate_impl.h"
-#include "components/scheduler/child/web_task_runner_impl.h"
-#include "components/scheduler/child/worker_scheduler.h"
 #include "content/child/request_extra_data.h"
-#include "content/child/request_info.h"
 #include "content/child/resource_dispatcher.h"
+#include "content/child/sync_load_response.h"
 #include "content/public/child/fixed_received_data.h"
 #include "content/public/child/request_peer.h"
 #include "content/public/common/content_switches.h"
@@ -45,6 +43,7 @@ namespace content {
 namespace {
 
 const char kTestURL[] = "http://foo";
+const char kTestHTTPSURL[] = "https://foo";
 const char kTestData[] = "blah!";
 
 const char kFtpDirMimeType[] = "text/vnd.chromium.ftp-dir";
@@ -67,13 +66,28 @@ class TestResourceDispatcher : public ResourceDispatcher {
 
   // TestDispatcher implementation:
 
-  int StartAsync(const RequestInfo& request_info,
-                 ResourceRequestBodyImpl* request_body,
-                 std::unique_ptr<RequestPeer> peer) override {
+  void StartSync(std::unique_ptr<ResourceRequest> request,
+                 int routing_id,
+                 SyncLoadResponse* response,
+                 blink::WebURLRequest::LoadingIPCType ipc_type,
+                 mojom::URLLoaderFactory* url_loader_factory) override {
+    *response = sync_load_response_;
+  }
+
+  int StartAsync(
+      std::unique_ptr<ResourceRequest> request,
+      int routing_id,
+      scoped_refptr<base::SingleThreadTaskRunner> loading_task_runner,
+      const GURL& frame_origin,
+      std::unique_ptr<RequestPeer> peer,
+      blink::WebURLRequest::LoadingIPCType ipc_type,
+      mojom::URLLoaderFactory* url_loader_factory,
+      mojo::AssociatedGroup* associated_group) override {
     EXPECT_FALSE(peer_);
+    EXPECT_EQ(blink::WebURLRequest::LoadingIPCType::ChromeIPC, ipc_type);
     peer_ = std::move(peer);
-    url_ = request_info.url;
-    stream_url_ = request_info.resource_body_stream_url;
+    url_ = request->url;
+    stream_url_ = request->resource_body_stream_url;
     return 1;
   }
 
@@ -94,23 +108,25 @@ class TestResourceDispatcher : public ResourceDispatcher {
   }
   bool defers_loading() const { return defers_loading_; }
 
+  void set_sync_load_response(const SyncLoadResponse& sync_load_response) {
+    sync_load_response_ = sync_load_response;
+  }
+
  private:
   std::unique_ptr<RequestPeer> peer_;
   bool canceled_;
   bool defers_loading_;
   GURL url_;
   GURL stream_url_;
+  SyncLoadResponse sync_load_response_;
 
   DISALLOW_COPY_AND_ASSIGN(TestResourceDispatcher);
 };
 
 class TestWebURLLoaderClient : public blink::WebURLLoaderClient {
  public:
-  TestWebURLLoaderClient(ResourceDispatcher* dispatcher,
-                         scoped_refptr<scheduler::TaskQueue> task_runner)
-      : loader_(new WebURLLoaderImpl(
-            dispatcher,
-            base::WrapUnique(new scheduler::WebTaskRunnerImpl(task_runner)))),
+  TestWebURLLoaderClient(ResourceDispatcher* dispatcher)
+      : loader_(new WebURLLoaderImpl(dispatcher, nullptr, nullptr)),
         delete_on_receive_redirect_(false),
         delete_on_receive_response_(false),
         delete_on_receive_data_(false),
@@ -124,7 +140,7 @@ class TestWebURLLoaderClient : public blink::WebURLLoaderClient {
   ~TestWebURLLoaderClient() override {}
 
   // blink::WebURLLoaderClient implementation:
-  void willFollowRedirect(
+  bool willFollowRedirect(
       blink::WebURLLoader* loader,
       blink::WebURLRequest& newRequest,
       const blink::WebURLResponse& redirectResponse) override {
@@ -140,6 +156,8 @@ class TestWebURLLoaderClient : public blink::WebURLLoaderClient {
 
     if (delete_on_receive_redirect_)
       loader_.reset();
+
+    return true;
   }
 
   void didSendData(blink::WebURLLoader* loader,
@@ -172,7 +190,8 @@ class TestWebURLLoaderClient : public blink::WebURLLoaderClient {
   void didReceiveData(blink::WebURLLoader* loader,
                       const char* data,
                       int dataLength,
-                      int encodedDataLength) override {
+                      int encodedDataLength,
+                      int encodedBodyLength) override {
     EXPECT_TRUE(loader_);
     EXPECT_EQ(loader_.get(), loader);
     // The response should have started, but must not have finished, or failed.
@@ -262,32 +281,22 @@ class TestWebURLLoaderClient : public blink::WebURLLoaderClient {
 
 class WebURLLoaderImplTest : public testing::Test {
  public:
-  explicit WebURLLoaderImplTest()
-      : worker_scheduler_(scheduler::WorkerScheduler::Create(
-            scheduler::SchedulerTqmDelegateImpl::Create(
-                &message_loop_,
-                base::WrapUnique(new base::DefaultTickClock())))) {
-    worker_scheduler_->Init();
-    client_.reset(new TestWebURLLoaderClient(
-        &dispatcher_, worker_scheduler_->DefaultTaskRunner()));
+  WebURLLoaderImplTest() {
+    client_.reset(new TestWebURLLoaderClient(&dispatcher_));
   }
 
   ~WebURLLoaderImplTest() override {}
 
   void DoStartAsyncRequest() {
-    blink::WebURLRequest request;
-    request.initialize();
-    request.setURL(GURL(kTestURL));
+    blink::WebURLRequest request{GURL(kTestURL)};
     client()->loader()->loadAsynchronously(request, client());
     ASSERT_TRUE(peer());
   }
 
   void DoStartAsyncRequestWithPriority(
       blink::WebURLRequest::Priority priority) {
-    blink::WebURLRequest request;
-    request.initialize();
+    blink::WebURLRequest request{GURL(kTestURL)};
     request.setPriority(priority);
-    request.setURL(GURL(kTestURL));
     client()->loader()->loadAsynchronously(request, client());
     ASSERT_TRUE(peer());
   }
@@ -304,6 +313,18 @@ class WebURLLoaderImplTest : public testing::Test {
     EXPECT_TRUE(client()->did_receive_redirect());
   }
 
+  void DoReceiveHTTPSRedirect() {
+    EXPECT_FALSE(client()->did_receive_redirect());
+    net::RedirectInfo redirect_info;
+    redirect_info.status_code = 302;
+    redirect_info.new_method = "GET";
+    redirect_info.new_url = GURL(kTestHTTPSURL);
+    redirect_info.new_first_party_for_cookies = GURL(kTestHTTPSURL);
+    peer()->OnReceivedRedirect(redirect_info,
+                               content::ResourceResponseInfo());
+    EXPECT_TRUE(client()->did_receive_redirect());
+  }
+
   void DoReceiveResponse() {
     EXPECT_FALSE(client()->did_receive_response());
     peer()->OnReceivedResponse(content::ResourceResponseInfo());
@@ -313,14 +334,15 @@ class WebURLLoaderImplTest : public testing::Test {
   // Assumes it is called only once for a request.
   void DoReceiveData() {
     EXPECT_EQ("", client()->received_data());
-    peer()->OnReceivedData(base::WrapUnique(new FixedReceivedData(
-        kTestData, strlen(kTestData), strlen(kTestData))));
+    auto size = strlen(kTestData);
+    peer()->OnReceivedData(
+        base::MakeUnique<FixedReceivedData>(kTestData, size, size, size));
     EXPECT_EQ(kTestData, client()->received_data());
   }
 
   void DoCompleteRequest() {
     EXPECT_FALSE(client()->did_finish());
-    peer()->OnCompletedRequest(net::OK, false, false, "", base::TimeTicks(),
+    peer()->OnCompletedRequest(net::OK, false, false, base::TimeTicks(),
                                strlen(kTestData));
     EXPECT_TRUE(client()->did_finish());
     // There should be no error.
@@ -330,7 +352,7 @@ class WebURLLoaderImplTest : public testing::Test {
 
   void DoFailRequest() {
     EXPECT_FALSE(client()->did_finish());
-    peer()->OnCompletedRequest(net::ERR_FAILED, false, false, "",
+    peer()->OnCompletedRequest(net::ERR_FAILED, false, false,
                                base::TimeTicks(), strlen(kTestData));
     EXPECT_FALSE(client()->did_finish());
     EXPECT_EQ(net::ERR_FAILED, client()->error().reason);
@@ -346,8 +368,9 @@ class WebURLLoaderImplTest : public testing::Test {
   }
 
   void DoReceiveDataFtp() {
-    peer()->OnReceivedData(base::WrapUnique(new FixedReceivedData(
-        kFtpDirListing, strlen(kFtpDirListing), strlen(kFtpDirListing))));
+    auto size = strlen(kFtpDirListing);
+    peer()->OnReceivedData(
+        base::MakeUnique<FixedReceivedData>(kFtpDirListing, size, size, size));
     // The FTP delegate should modify the data the client sees.
     EXPECT_NE(kFtpDirListing, client()->received_data());
   }
@@ -359,9 +382,6 @@ class WebURLLoaderImplTest : public testing::Test {
 
  private:
   base::MessageLoop message_loop_;
-  // WorkerScheduler is needed because WebURLLoaderImpl needs a
-  // scheduler::TaskQueue.
-  std::unique_ptr<scheduler::WorkerScheduler> worker_scheduler_;
   TestResourceDispatcher dispatcher_;
   std::unique_ptr<TestWebURLLoaderClient> client_;
 };
@@ -444,23 +464,19 @@ TEST_F(WebURLLoaderImplTest, DeleteOnFail) {
 }
 
 TEST_F(WebURLLoaderImplTest, DeleteBeforeResponseDataURL) {
-  blink::WebURLRequest request;
-  request.initialize();
-  request.setURL(GURL("data:text/html;charset=utf-8,blah!"));
+  blink::WebURLRequest request(GURL("data:text/html;charset=utf-8,blah!"));
   client()->loader()->loadAsynchronously(request, client());
   client()->DeleteLoader();
-  message_loop()->RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   EXPECT_FALSE(client()->did_receive_response());
 }
 
 // Data URL tests.
 
 TEST_F(WebURLLoaderImplTest, DataURL) {
-  blink::WebURLRequest request;
-  request.initialize();
-  request.setURL(GURL("data:text/html;charset=utf-8,blah!"));
+  blink::WebURLRequest request(GURL("data:text/html;charset=utf-8,blah!"));
   client()->loader()->loadAsynchronously(request, client());
-  message_loop()->RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   EXPECT_EQ("blah!", client()->received_data());
   EXPECT_TRUE(client()->did_finish());
   EXPECT_EQ(net::OK, client()->error().reason);
@@ -468,45 +484,37 @@ TEST_F(WebURLLoaderImplTest, DataURL) {
 }
 
 TEST_F(WebURLLoaderImplTest, DataURLDeleteOnReceiveResponse) {
-  blink::WebURLRequest request;
-  request.initialize();
-  request.setURL(GURL("data:text/html;charset=utf-8,blah!"));
+  blink::WebURLRequest request(GURL("data:text/html;charset=utf-8,blah!"));
   client()->set_delete_on_receive_response();
   client()->loader()->loadAsynchronously(request, client());
-  message_loop()->RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   EXPECT_TRUE(client()->did_receive_response());
   EXPECT_EQ("", client()->received_data());
   EXPECT_FALSE(client()->did_finish());
 }
 
 TEST_F(WebURLLoaderImplTest, DataURLDeleteOnReceiveData) {
-  blink::WebURLRequest request;
-  request.initialize();
-  request.setURL(GURL("data:text/html;charset=utf-8,blah!"));
+  blink::WebURLRequest request(GURL("data:text/html;charset=utf-8,blah!"));
   client()->set_delete_on_receive_data();
   client()->loader()->loadAsynchronously(request, client());
-  message_loop()->RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   EXPECT_TRUE(client()->did_receive_response());
   EXPECT_EQ("blah!", client()->received_data());
   EXPECT_FALSE(client()->did_finish());
 }
 
 TEST_F(WebURLLoaderImplTest, DataURLDeleteOnFinish) {
-  blink::WebURLRequest request;
-  request.initialize();
-  request.setURL(GURL("data:text/html;charset=utf-8,blah!"));
+  blink::WebURLRequest request(GURL("data:text/html;charset=utf-8,blah!"));
   client()->set_delete_on_finish();
   client()->loader()->loadAsynchronously(request, client());
-  message_loop()->RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   EXPECT_TRUE(client()->did_receive_response());
   EXPECT_EQ("blah!", client()->received_data());
   EXPECT_TRUE(client()->did_finish());
 }
 
 TEST_F(WebURLLoaderImplTest, DataURLDefersLoading) {
-  blink::WebURLRequest request;
-  request.initialize();
-  request.setURL(GURL("data:text/html;charset=utf-8,blah!"));
+  blink::WebURLRequest request(GURL("data:text/html;charset=utf-8,blah!"));
   client()->loader()->loadAsynchronously(request, client());
 
   // setDefersLoading() might be called with either false or true in no
@@ -518,22 +526,22 @@ TEST_F(WebURLLoaderImplTest, DataURLDefersLoading) {
   client()->loader()->setDefersLoading(false);
   client()->loader()->setDefersLoading(true);
   client()->loader()->setDefersLoading(true);
-  message_loop()->RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   EXPECT_FALSE(client()->did_finish());
 
   client()->loader()->setDefersLoading(false);
   client()->loader()->setDefersLoading(true);
-  message_loop()->RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   EXPECT_FALSE(client()->did_finish());
 
   client()->loader()->setDefersLoading(false);
-  message_loop()->RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   EXPECT_TRUE(client()->did_finish());
 
   client()->loader()->setDefersLoading(true);
   client()->loader()->setDefersLoading(false);
   client()->loader()->setDefersLoading(false);
-  message_loop()->RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   EXPECT_TRUE(client()->did_finish());
 
   EXPECT_EQ("blah!", client()->received_data());
@@ -584,7 +592,7 @@ TEST_F(WebURLLoaderImplTest, FtpDeleteOnReceiveMoreData) {
   // Directory listings are only parsed once the request completes, so this will
   // cancel in DoReceiveDataFtp, before the request finishes.
   client()->set_delete_on_receive_data();
-  peer()->OnCompletedRequest(net::OK, false, false, "", base::TimeTicks(),
+  peer()->OnCompletedRequest(net::OK, false, false, base::TimeTicks(),
                               strlen(kTestData));
   EXPECT_FALSE(client()->did_finish());
 }
@@ -612,9 +620,7 @@ TEST_F(WebURLLoaderImplTest, BrowserSideNavigationCommit) {
   const GURL kNavigationURL = GURL(kTestURL);
   const GURL kStreamURL = GURL("http://bar");
   const std::string kMimeType = "text/html";
-  blink::WebURLRequest request;
-  request.initialize();
-  request.setURL(kNavigationURL);
+  blink::WebURLRequest request(kNavigationURL);
   request.setFrameType(blink::WebURLRequest::FrameTypeTopLevel);
   request.setRequestContext(blink::WebURLRequest::RequestContextFrame);
   std::unique_ptr<StreamOverrideParameters> stream_override(
@@ -629,7 +635,7 @@ TEST_F(WebURLLoaderImplTest, BrowserSideNavigationCommit) {
 
   client()->loader()->loadAsynchronously(request, client());
 
-  // The stream url should have been added to the RequestInfo.
+  // The stream url should have been added to the ResourceRequest.
   ASSERT_TRUE(peer());
   EXPECT_EQ(kNavigationURL, dispatcher()->url());
   EXPECT_EQ(kStreamURL, dispatcher()->stream_url());
@@ -669,11 +675,43 @@ TEST_F(WebURLLoaderImplTest, ResponseIPAddress) {
     content::ResourceResponseInfo info;
     info.socket_address = net::HostPortPair(test.ip, 443);
     blink::WebURLResponse response;
-    response.initialize();
     WebURLLoaderImpl::PopulateURLResponse(url, info, &response, true);
     EXPECT_EQ(test.expected, response.remoteIPAddress().utf8());
   };
 }
 
+// Verifies that the lengths used by the PerformanceResourceTiming API are
+// correctly assigned for sync XHR.
+TEST_F(WebURLLoaderImplTest, SyncLengths) {
+  static const char kBodyData[] =  "Today is Thursday";
+  const int kEncodedBodyLength = 30;
+  const int kEncodedDataLength = 130;
+  const GURL url(kTestURL);
+  blink::WebURLRequest request(url);
+
+  // Prepare a mock response
+  SyncLoadResponse sync_load_response;
+  sync_load_response.error_code = net::OK;
+  sync_load_response.url = url;
+  sync_load_response.data = kBodyData;
+  ASSERT_EQ(17u, sync_load_response.data.size());
+  sync_load_response.encoded_body_length = kEncodedBodyLength;
+  sync_load_response.encoded_data_length = kEncodedDataLength;
+  dispatcher()->set_sync_load_response(sync_load_response);
+
+  blink::WebURLResponse response;
+  blink::WebURLError error;
+  blink::WebData data;
+  int64_t encoded_data_length = 0;
+  client()->loader()->loadSynchronously(request, response, error, data,
+                                        encoded_data_length);
+
+  EXPECT_EQ(kEncodedBodyLength, response.encodedBodyLengthForTesting());
+  EXPECT_EQ(kEncodedDataLength, encoded_data_length);
+  int expected_decoded_body_length = strlen(kBodyData);
+  EXPECT_EQ(expected_decoded_body_length,
+            response.decodedBodyLengthForTesting());
+}
+
 }  // namespace
 }  // namespace content
