@@ -15,17 +15,18 @@
 #include "base/files/file_util.h"
 #include "base/logging.h"
 #include "base/memory/discardable_memory_allocator.h"
-#include "base/metrics/histogram.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/metrics/sparse_histogram.h"
 #include "base/rand_util.h"
+#include "base/single_thread_task_runner.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/threading/platform_thread.h"
 #include "base/time/time.h"
 #include "base/trace_event/trace_event.h"
 #include "build/build_config.h"
+#include "components/discardable_memory/client/client_discardable_shared_memory_manager.h"
 #include "content/child/browser_font_resource_trusted.h"
-#include "content/child/child_discardable_shared_memory_manager.h"
 #include "content/child/child_process.h"
 #include "content/common/child_process_messages.h"
 #include "content/ppapi_plugin/broker_process_dispatcher.h"
@@ -144,12 +145,11 @@ void PpapiThread::Shutdown() {
   if (plugin_entry_points_.shutdown_module)
     plugin_entry_points_.shutdown_module();
   blink_platform_impl_->Shutdown();
-  blink::Platform::shutdown();
 }
 
 bool PpapiThread::Send(IPC::Message* msg) {
   // Allow access from multiple threads.
-  if (base::MessageLoop::current() == message_loop())
+  if (message_loop()->task_runner()->BelongsToCurrentThread())
     return ChildThreadImpl::Send(msg);
 
   return sync_message_filter()->Send(msg);
@@ -488,10 +488,7 @@ bool PpapiThread::SetupChannel(base::ProcessId renderer_pid,
                                bool incognito,
                                IPC::ChannelHandle* handle) {
   DCHECK(is_broker_ == (connect_instance_func_ != NULL));
-  IPC::ChannelHandle plugin_handle;
-  plugin_handle.name = IPC::Channel::GenerateVerifiedChannelID(
-      base::StringPrintf(
-          "%d.r%d", base::GetCurrentProcId(), renderer_child_id));
+  mojo::MessagePipe pipe;
 
   ppapi::proxy::ProxyChannel* dispatcher = NULL;
   bool init_result = false;
@@ -500,10 +497,8 @@ bool PpapiThread::SetupChannel(base::ProcessId renderer_pid,
     BrokerProcessDispatcher* broker_dispatcher =
         new BrokerProcessDispatcher(plugin_entry_points_.get_interface,
                                     connect_instance_func_, peer_is_browser);
-    init_result = broker_dispatcher->InitBrokerWithChannel(this,
-                                                           renderer_pid,
-                                                           plugin_handle,
-                                                           false);
+    init_result = broker_dispatcher->InitBrokerWithChannel(
+        this, renderer_pid, pipe.handle0.release(), false);
     dispatcher = broker_dispatcher;
   } else {
     DCHECK_NE(base::kNullProcessId, renderer_pid);
@@ -511,10 +506,8 @@ bool PpapiThread::SetupChannel(base::ProcessId renderer_pid,
         new PluginProcessDispatcher(plugin_entry_points_.get_interface,
                                     permissions_,
                                     incognito);
-    init_result = plugin_dispatcher->InitPluginWithChannel(this,
-                                                           renderer_pid,
-                                                           plugin_handle,
-                                                           false);
+    init_result = plugin_dispatcher->InitPluginWithChannel(
+        this, renderer_pid, pipe.handle0.release(), false);
     dispatcher = plugin_dispatcher;
   }
 
@@ -522,16 +515,7 @@ bool PpapiThread::SetupChannel(base::ProcessId renderer_pid,
     delete dispatcher;
     return false;
   }
-
-  handle->name = plugin_handle.name;
-#if defined(OS_POSIX)
-  // On POSIX, transfer ownership of the renderer-side (client) FD.
-  // This ensures this process will be notified when it is closed even if a
-  // connection is not established.
-  handle->socket = base::FileDescriptor(dispatcher->TakeRendererFD());
-  if (handle->socket.fd == -1)
-    return false;
-#endif
+  *handle = pipe.handle1.release();
 
   // From here, the dispatcher will manage its own lifetime according to the
   // lifetime of the attached channel.
