@@ -5,11 +5,26 @@
 #ifndef CONTENT_PUBLIC_RENDERER_MEDIA_STREAM_RENDERER_FACTORY_H_
 #define CONTENT_PUBLIC_RENDERER_MEDIA_STREAM_RENDERER_FACTORY_H_
 
+#include <string>
+
 #include "base/callback.h"
 #include "base/memory/ref_counted.h"
 #include "content/public/renderer/media_stream_audio_renderer.h"
-#include "content/public/renderer/video_frame_provider.h"
-#include "url/gurl.h"
+#include "content/public/renderer/media_stream_video_renderer.h"
+#include "url/origin.h"
+
+namespace base {
+class SingleThreadTaskRunner;
+class TaskRunner;
+}  // namespace base
+
+namespace blink {
+class WebMediaStream;
+}  // namespace blink
+
+namespace media {
+class GpuVideoAcceleratorFactories;
+}  // namespace media
 
 namespace content {
 
@@ -21,14 +36,19 @@ class MediaStreamRendererFactory {
  public:
   virtual ~MediaStreamRendererFactory() {}
 
-  virtual scoped_refptr<VideoFrameProvider> GetVideoFrameProvider(
-      const GURL& url,
+  virtual scoped_refptr<MediaStreamVideoRenderer> GetVideoRenderer(
+      const blink::WebMediaStream& web_stream,
       const base::Closure& error_cb,
-      const VideoFrameProvider::RepaintCB& repaint_cb) = 0;
+      const MediaStreamVideoRenderer::RepaintCB& repaint_cb,
+      const scoped_refptr<base::SingleThreadTaskRunner>& media_task_runner,
+      const scoped_refptr<base::TaskRunner>& worker_task_runner,
+      media::GpuVideoAcceleratorFactories* gpu_factories) = 0;
 
   virtual scoped_refptr<MediaStreamAudioRenderer> GetAudioRenderer(
-      const GURL& url,
-      int render_frame_id) = 0;
+      const blink::WebMediaStream& web_stream,
+      int render_frame_id,
+      const std::string& device_id,
+      const url::Origin& security_origin) = 0;
 };
 
 }  // namespace content
