@@ -150,15 +150,15 @@ bool AudioMessageFilter::OnMessageReceived(const IPC::Message& message) {
   IPC_BEGIN_MESSAGE_MAP(AudioMessageFilter, message)
     IPC_MESSAGE_HANDLER(AudioMsg_NotifyDeviceAuthorized, OnDeviceAuthorized)
     IPC_MESSAGE_HANDLER(AudioMsg_NotifyStreamCreated, OnStreamCreated)
-    IPC_MESSAGE_HANDLER(AudioMsg_NotifyStreamStateChanged, OnStreamStateChanged)
+    IPC_MESSAGE_HANDLER(AudioMsg_NotifyStreamError, OnStreamError)
     IPC_MESSAGE_UNHANDLED(handled = false)
   IPC_END_MESSAGE_MAP()
   return handled;
 }
 
-void AudioMessageFilter::OnFilterAdded(IPC::Sender* sender) {
+void AudioMessageFilter::OnFilterAdded(IPC::Channel* channel) {
   DCHECK(io_task_runner_->BelongsToCurrentThread());
-  sender_ = sender;
+  sender_ = channel;
 }
 
 void AudioMessageFilter::OnFilterRemoved() {
@@ -222,16 +222,15 @@ void AudioMessageFilter::OnStreamCreated(
   delegate->OnStreamCreated(handle, socket_handle, length);
 }
 
-void AudioMessageFilter::OnStreamStateChanged(
-    int stream_id, media::AudioOutputIPCDelegateState state) {
+void AudioMessageFilter::OnStreamError(int stream_id) {
   DCHECK(io_task_runner_->BelongsToCurrentThread());
   media::AudioOutputIPCDelegate* delegate = delegates_.Lookup(stream_id);
   if (!delegate) {
-    DLOG(WARNING) << "Got OnStreamStateChanged() event for a non-existent or"
-                  << " removed audio renderer.  State: " << state;
+    DLOG(WARNING) << "Got OnStreamError() event for a non-existent or"
+                  << " removed audio renderer.";
     return;
   }
-  delegate->OnStateChanged(state);
+  delegate->OnError();
 }
 
 }  // namespace content
