@@ -16,25 +16,22 @@
 #include "media/base/key_systems.h"
 #include "media/base/media_keys.h"
 #include "media/cdm/aes_decryptor.h"
+#include "ppapi/features/features.h"
 #include "url/gurl.h"
-#if defined(ENABLE_PEPPER_CDMS)
+
+#if BUILDFLAG(ENABLE_PEPPER_CDMS)
 #include "content/renderer/media/cdm/ppapi_decryptor.h"
-#elif defined(ENABLE_BROWSER_CDMS)
-#include "content/renderer/media/cdm/proxy_media_keys.h"
-#endif  // defined(ENABLE_PEPPER_CDMS)
+#endif  // BUILDFLAG(ENABLE_PEPPER_CDMS)
 
 namespace content {
 
-#if defined(ENABLE_PEPPER_CDMS)
+#if BUILDFLAG(ENABLE_PEPPER_CDMS)
 RenderCdmFactory::RenderCdmFactory(
     const CreatePepperCdmCB& create_pepper_cdm_cb)
     : create_pepper_cdm_cb_(create_pepper_cdm_cb) {}
-#elif defined(ENABLE_BROWSER_CDMS)
-RenderCdmFactory::RenderCdmFactory(RendererCdmManager* manager)
-    : manager_(manager) {}
 #else
 RenderCdmFactory::RenderCdmFactory() {}
-#endif  // defined(ENABLE_PEPPER_CDMS)
+#endif  // BUILDFLAG(ENABLE_PEPPER_CDMS)
 
 RenderCdmFactory::~RenderCdmFactory() {
   DCHECK(thread_checker_.CalledOnValidThread());
@@ -46,7 +43,6 @@ void RenderCdmFactory::Create(
     const media::CdmConfig& cdm_config,
     const media::SessionMessageCB& session_message_cb,
     const media::SessionClosedCB& session_closed_cb,
-    const media::LegacySessionErrorCB& legacy_session_error_cb,
     const media::SessionKeysChangeCB& session_keys_change_cb,
     const media::SessionExpirationUpdateCB& session_expiration_update_cb,
     const media::CdmCreatedCB& cdm_created_cb) {
@@ -59,11 +55,8 @@ void RenderCdmFactory::Create(
   }
 
   if (media::CanUseAesDecryptor(key_system)) {
-    // TODO(sandersd): Address this now that prefixed EME has been removed.
-    // http://crbug.com/249976. The prefixed API always allowed distinctive
-    // identifiers and persistent state. Once that changes we can sanity check
-    // here that neither is allowed for AesDecryptor, since it does not support
-    // them and should never be configured that way. http://crbug.com/455271
+    DCHECK(!cdm_config.allow_distinctive_identifier);
+    DCHECK(!cdm_config.allow_persistent_state);
     scoped_refptr<media::MediaKeys> cdm(
         new media::AesDecryptor(security_origin, session_message_cb,
                                 session_closed_cb, session_keys_change_cb));
@@ -72,26 +65,19 @@ void RenderCdmFactory::Create(
     return;
   }
 
-#if defined(ENABLE_PEPPER_CDMS)
+#if BUILDFLAG(ENABLE_PEPPER_CDMS)
   DCHECK(!cdm_config.use_hw_secure_codecs);
   PpapiDecryptor::Create(
       key_system, security_origin, cdm_config.allow_distinctive_identifier,
       cdm_config.allow_persistent_state, create_pepper_cdm_cb_,
-      session_message_cb, session_closed_cb, legacy_session_error_cb,
-      session_keys_change_cb, session_expiration_update_cb, cdm_created_cb);
-#elif defined(ENABLE_BROWSER_CDMS)
-  DCHECK(cdm_config.allow_distinctive_identifier);
-  DCHECK(cdm_config.allow_persistent_state);
-  ProxyMediaKeys::Create(
-      key_system, security_origin, cdm_config.use_hw_secure_codecs, manager_,
-      session_message_cb, session_closed_cb, legacy_session_error_cb,
-      session_keys_change_cb, session_expiration_update_cb, cdm_created_cb);
+      session_message_cb, session_closed_cb, session_keys_change_cb,
+      session_expiration_update_cb, cdm_created_cb);
 #else
   // No possible CDM to create, so fail the request.
   base::ThreadTaskRunnerHandle::Get()->PostTask(
       FROM_HERE,
       base::Bind(cdm_created_cb, nullptr, "Key system not supported."));
-#endif  // defined(ENABLE_PEPPER_CDMS)
+#endif  // BUILDFLAG(ENABLE_PEPPER_CDMS)
 }
 
 }  // namespace content
