@@ -9,7 +9,7 @@
 #include "base/macros.h"
 #include "base/threading/thread_checker.h"
 #include "content/common/content_export.h"
-#include "media/audio/audio_parameters.h"
+#include "media/base/audio_parameters.h"
 #include "third_party/WebKit/public/platform/WebMediaStreamTrack.h"
 
 namespace content {
@@ -18,7 +18,7 @@ namespace content {
 // It is owned by blink::WebMediaStreamTrack as
 // blink::WebMediaStreamTrack::ExtraData.
 class CONTENT_EXPORT MediaStreamTrack
-    : NON_EXPORTED_BASE(public blink::WebMediaStreamTrack::ExtraData) {
+    : NON_EXPORTED_BASE(public blink::WebMediaStreamTrack::TrackData) {
  public:
   explicit MediaStreamTrack(bool is_local_track);
   ~MediaStreamTrack() override;
@@ -29,6 +29,9 @@ class CONTENT_EXPORT MediaStreamTrack
 
   virtual void Stop() = 0;
 
+  // TODO(hta): Make method pure virtual when all tracks have the method.
+  void getSettings(blink::WebMediaStreamTrack::Settings& settings) override {}
+
   bool is_local_track() const { return is_local_track_; }
 
  protected:
