@@ -18,8 +18,6 @@ namespace content {
 MockMediaStreamDispatcher::MockMediaStreamDispatcher()
     : MediaStreamDispatcher(NULL),
       audio_input_request_id_(-1),
-      audio_output_request_id_(-1),
-      video_request_id_(-1),
       request_stream_counter_(0),
       stop_audio_device_counter_(0),
       stop_video_device_counter_(0),
@@ -56,28 +54,6 @@ void MockMediaStreamDispatcher::CancelGenerateStream(
   EXPECT_EQ(request_id, audio_input_request_id_);
 }
 
-void MockMediaStreamDispatcher::EnumerateDevices(
-    int request_id,
-    const base::WeakPtr<MediaStreamDispatcherEventHandler>& event_handler,
-    MediaStreamType type,
-    const url::Origin& security_origin) {
-  if (type == MEDIA_DEVICE_AUDIO_CAPTURE) {
-    audio_input_request_id_ = request_id;
-    audio_input_array_.clear();
-    AddAudioInputDeviceToArray(true);
-    AddAudioInputDeviceToArray(false);
-  } else if (type == MEDIA_DEVICE_AUDIO_OUTPUT) {
-    audio_output_request_id_ = request_id;
-    audio_output_array_.clear();
-    AddAudioOutputDeviceToArray();
-  } else if (type == MEDIA_DEVICE_VIDEO_CAPTURE) {
-    video_request_id_ = request_id;
-    video_array_.clear();
-    AddVideoDeviceToArray(true);
-    AddVideoDeviceToArray(false);
-  }
-}
-
 void MockMediaStreamDispatcher::StopStreamDevice(
     const StreamDeviceInfo& device_info) {
   if (IsAudioInputMediaType(device_info.device.type)) {
@@ -124,16 +100,6 @@ void MockMediaStreamDispatcher::AddAudioInputDeviceToArray(
   audio_input_array_.push_back(audio);
 }
 
-void MockMediaStreamDispatcher::AddAudioOutputDeviceToArray() {
-  StreamDeviceInfo audio;
-  audio.device.id = kAudioOutputDeviceIdPrefix + base::IntToString(session_id_);
-  audio.device.name = "speaker";
-  audio.device.type = MEDIA_DEVICE_AUDIO_OUTPUT;
-  audio.device.video_facing = MEDIA_VIDEO_FACING_NONE;
-  audio.session_id = session_id_;
-  audio_output_array_.push_back(audio);
-}
-
 void MockMediaStreamDispatcher::AddVideoDeviceToArray(bool facing_user) {
   StreamDeviceInfo video;
   video.device.id = test_same_id_ ? "test_id" : "video_device_id";
